Package: gstat
Version: 2.1-6
Title: Spatial and Spatio-Temporal Geostatistical Modelling, Prediction
        and Simulation
Authors@R: c(person(given = "Edzer", 
        family = "Pebesma", 
        role = c("aut", "cre"),
        email = "edzer.pebesma@uni-muenster.de",
        comment = c(ORCID = "0000-0001-8049-7069")),
     person("Benedikt", "Graeler", role = "aut"))
Description: Variogram modelling; simple, ordinary and universal point or block (co)kriging; spatio-temporal kriging; sequential Gaussian or indicator (co)simulation; variogram and variogram map plotting utility functions; supports sf and stars.
Depends: R (>= 2.10)
Imports: utils, stats, graphics, methods, lattice, sp (>= 0.9-72), zoo,
        sf (>= 0.7-2), sftime, spacetime (>= 1.2-8), stars, FNN
Suggests: fields, maps, mapdata, xts, raster, future, future.apply,
        RColorBrewer, geoR, ggplot2
License: GPL (>= 2.0)
URL: https://github.com/r-spatial/gstat/,
        https://r-spatial.github.io/gstat/
Encoding: UTF-8
BugReports: https://github.com/r-spatial/gstat/issues/
NeedsCompilation: yes
RoxygenNote: 6.1.1
Packaged: 2026-03-29 20:06:42 UTC; edzer
Author: Edzer Pebesma [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8049-7069>),
  Benedikt Graeler [aut]
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Repository: CRAN
Date/Publication: 2026-03-30 05:40:07 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 11:19:08 UTC; unix
Archs: gstat.so.dSYM

useDynLib(gstat, 
	gstat_debug_level,
	gstat_exit,
	gstat_fit_variogram,
	gstat_get_variogram_models,
	gstat_init,
	gstat_load_ev,
	gstat_load_variogram,
	gstat_new_data,
	gstat_new_dummy_data,
	gstat_predict,
	gstat_set_merge,
	gstat_set_method,
	gstat_set_set,
	gstat_variogram,
	gstat_variogram_values
)

importFrom(utils, setTxtProgressBar, stack, txtProgressBar, head, tail)
importFrom(stats, as.formula, cor, cov2cor, delete.response, .getXlevels, 
	lm, median, model.extract, model.frame, model.matrix, na.exclude, 
	na.fail, na.omit, na.pass, optim, optimise, optimize, predict, 
	residuals, terms, 
	rnorm, fft, numericDeriv, runif, setNames)
importFrom(graphics, image.default, lines, locator, text)
importFrom(zoo, is.regular)
import(methods)
import(lattice)
importFrom(zoo, zoo, is.regular)
import(sp)
import(spacetime)
import(FNN)

export(as.vgm.variomodel,
	cross.name,
	fit.lmc,
	fit.variogram,
	fit.variogram.reml,
	fit.variogram.gls,
	fit.StVariogram,
	estiStAni,
	get.contr,
	get_gstat_progress,
	gstat,
	gstat.cv,
	"[.gstat",
	hscat,
	idw0,
	krigeTg,
	krige0, krigeST, krigeSTTg, krigeSimCE, krigeSTSimTB, vgmAreaST,
	map.to.lev,
	ossfim,
	show.vgms,
	spplot.vcov,
	variogram,
	variogramST,
	extractPar,
	extractParNames,
	variogramLine,
	variogramSurface,
	vgm, vgmArea, vgmST,
	vgm.panel.xyplot,
	panel.pointPairs,
	set_gstat_progress,
	xyz2img)

exportMethods(krige, idw, krige.cv)

S3method("[", gstat)

S3method(as.data.frame, variogramCloud)

S3method(image, data.frame)

S3method(plot, pointPairs)
S3method(plot, gstatVariogram)
S3method(plot, variogramMap)
S3method(plot, variogramCloud)
S3method(plot, variogramModel)
S3method(plot, StVariogram)

S3method(predict, gstat)

S3method(print, gstat)
S3method(print, gstatVariogram)
S3method(print, variogramCloud)
S3method(print, variogramModel)
S3method(print, StVariogramModel)

S3method(variogram, default)
S3method(variogram, formula)
S3method(variogram, gstat)

########################################################
# note that demo(stkrige) and demo(stkrige-prediction) #
# need to be run before this one                       #
########################################################

## cross-validation
crossStat <- function(var1, var2="PM10", STxDF=DE_RB_2005, digits=NA) {
  diff <- STxDF[,,var1,drop=F]@data[[1]] - STxDF[,,var2,drop=F]@data[[1]]
  RMSE <- sqrt(mean(diff^2))
  MAE <- mean(abs(diff))
  ME <- mean(diff)
  COR <- cor(STxDF[,,var1,drop=F]@data[[1]], STxDF[,,var2,drop=F]@data[[1]])
  res <- c(RMSE, MAE, ME, COR)
  names(res) <- c("RMSE", "MAE", "ME", "COR")
  if(is.na(digits))
    return(res)
  else
    return(round(res, digits))
}

# purely spatial:
DE_RB_2005 <- as(DE_RB_2005, "STSDF")

pureSp <- NULL
for(i in 1:365) { # i <- 1
  pureSp <- c(pureSp, krige.cv(PM10~1,DE_RB_2005[,i,"PM10"],
                               model=spVgmMod, nmax=10)$var1.pred)
}

DE_RB_2005@data$pureSp10Nghbr <- pureSp

pureSp <- NULL
for(i in 1:365) { # i <- 1
  pureSp <- c(pureSp, krige.cv(PM10~1,DE_RB_2005[,i,"PM10"],model=spVgmMod,nmax=50)$var1.pred)
}

DE_RB_2005@data$pureSp50Nghbr <- pureSp

## spatio-temporal LOOCV
target <- as(DE_RB_2005[,,"PM10"],"STFDF")

## seprable model
# 10 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) {
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitSepModel, nmax=10,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$sepModel10Nghbr <- as.vector(res)[!is.na(as.vector(res))]

# 50 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitSepModel, nmax=50,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$sepModel50Nghbr <- as.vector(res)[!is.na(as.vector(res))]

## product-sum model
# 10 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitProdSumModel, nmax=10,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$psModel10Nghbr <- as.vector(res)[!is.na(as.vector(res))]

# 50 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitProdSumModel, nmax=50,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$psModel50Nghbr <- as.vector(res)[!is.na(as.vector(res))]

## metric model
# 10 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitMetricModel, nmax=10,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$metricModel10Nghbr <- as.vector(res)[!is.na(as.vector(res))]

# 50 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitMetricModel, nmax=50,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$metricModel50Nghbr <- as.vector(res)[!is.na(as.vector(res))]

## sum-metric model
# 10 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitSumMetricModel, nmax=10,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$sumMetricModel10Nghbr <- as.vector(res)[!is.na(as.vector(res))]

# 50 neighbours
res <- array(NA, c(length(DE_RB_2005@sp), 365,2))
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 38
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"],] <- as.matrix(krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                                     newdata=DE_RB_2005[loc,,drop=F], 
                                                     fitSumMetricModel, nmax=50,
                                                     computeVar=T,
                                                     stAni=linStAni*1000/24/3600)@data[,c("var1.pred","var1.var")])
}

DE_RB_2005@data$sumMetricModel50Nghbr <- as.vector(res[,,1])[!is.na(target@data)]
DE_RB_2005@data$sumMetricModel50NghbrVar <- as.vector(res[,,2])[!is.na(target@data)]
DE_RB_2005@data$sumMetricModel50Nghbr95u <- apply(DE_RB_2005@data, 1, 
                                                  function(x) {
                                                    qnorm(0.975, x["sumMetricModel50Nghbr"],
                                                          sqrt(x["sumMetricModel50NghbrVar"]))
                                                  })
DE_RB_2005@data$sumMetricModel50Nghbr95l <- apply(DE_RB_2005@data, 1, 
                                                  function(x) {
                                                    qnorm(0.025, x["sumMetricModel50Nghbr"],
                                                          sqrt(x["sumMetricModel50NghbrVar"]))
                                                  })

## simple sum-metric model
# 10 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitSimpleSumMetricModel, nmax=10,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$simpleSumMetricModel10Nghbr <- as.vector(res)[!is.na(as.vector(res))]

# 50 neighbours
res <- matrix(NA, length(DE_RB_2005@sp), 365)
for(loc in 1:length(DE_RB_2005@sp)) { # loc <- 1
  cat("Location", loc, "\n")
  res[loc,!is.na(target[loc,])[,"PM10"]] <- krigeST(PM10~1, data=DE_RB_2005[(1:length(DE_RB_2005@sp))[-loc],], 
                                           newdata=DE_RB_2005[loc,,drop=F], 
                                           fitSimpleSumMetricModel, nmax=50,
                                           stAni=fitMetricModel$stAni/24/3600)$var1.pred
}

DE_RB_2005@data$simpleSumMetricModel50Nghbr <- as.vector(res)[!is.na(as.vector(res))]

###
# cross-stats

rbind(
crossStat("pureSp10Nghbr", digits=2),
crossStat("pureSp50Nghbr", digits=2),

crossStat("sepModel10Nghbr", digits=2),
crossStat("sepModel50Nghbr", digits=2),

crossStat("psModel10Nghbr", digits=2),
crossStat("psModel50Nghbr", digits=2),

crossStat("metricModel10Nghbr", digits=2),
crossStat("metricModel50Nghbr", digits=2),

crossStat("sumMetricModel10Nghbr", digits=2),
crossStat("sumMetricModel50Nghbr", digits=2))

if(paper) {
  texRow <- function(x) {
    paste(paste(x,collapse=" & ")," \\\\ \n")
  }
  
  cat(apply(round(rbind(crossStat("pureSp10Nghbr"),
                        
                        crossStat("sepModel10Nghbr"),
                        crossStat("psModel10Nghbr"),
                        crossStat("metricModel10Nghbr"),
                        crossStat("sumMetricModel10Nghbr"),
                        crossStat("simpleSumMetricModel50Nghbr"),
                        
                        crossStat("pureSp50Nghbr"),
                        
                        crossStat("sepModel50Nghbr"),
                        crossStat("psModel50Nghbr"),
                        crossStat("metricModel50Nghbr"),
                        crossStat("sumMetricModel50Nghbr"),
                        crossStat("simpleSumMetricModel50Nghbr")
                        ),
                  2),1,texRow))

  loc <- 38 # sample(68,1) # 15
  tw <- "2005-01-15/2005-04-15"
  
  png("vignettes/figures/singleStationTimeSeries.png", 9, 4, "in", bg="white", res = 149)
  plot(DE_RB_2005[loc,tw][,"sumMetricModel50Nghbr"], main=paste("Location", DE_RB_2005@sp@data$station_european_code[loc]),
       ylim=c(0,70))
  points(DE_RB_2005[loc,tw][,"PM10"], type="l", col="darkgreen", lty=1)
  points(DE_RB_2005[loc,tw][,"sumMetricModel50Nghbr95u"], type="l", col="darkgrey", lty=2)
  points(DE_RB_2005[loc,tw][,"sumMetricModel50Nghbr95l"], type="l", col="darkgrey", lty=2)
  legend("topright",legend = c("observed","sum-metric","95 % prediction band"), lty=c(1,1,2),
         col=c("darkgreen", "black", "darkgrey") )
  dev.off()
  
  DE_RB_2005@data$diffPM10 <- DE_RB_2005@data$sumMetricModel50Nghbr - DE_RB_2005@data$PM10

  stpl <- stplot(as(DE_RB_2005[,smplDays, "diffPM10"],"STFDF"),
                 col.regions=bpy.colors(5),
                 sp.layout = list("sp.polygons", DE_NUTS1), scales=list(draw=F), 
                 key.space="right", colorkey=T, cuts=c(-25,-15,-5,5,15,25),
                 main=NULL) #expression(paste("daily mean ","PM"[10]," concentration")))
  
  png("vignettes/figures/diffs_daily_means_PM10.png", width=9, height=6, "in", res=150)
  print(stpl)
  dev.off()
}

# $Id: gstat3D.R,v 1.5 2007-02-23 13:34:07 edzer Exp $
# simple demo of 3D interpolation of 50 points with random normal values,
# randomly located in the unit cube
library(sp)
library(gstat)
n <- 50

data3D <- data.frame(x = runif(n), y = runif(n), z = runif(n), v = rnorm(n))
coordinates(data3D) = ~x+y+z

range1D <- seq(from = 0, to = 1, length = 20)
grid3D <- expand.grid(x = range1D, y = range1D, z = range1D)
gridded(grid3D) = ~x+y+z

res3D <- krige(formula = v ~ 1, data3D, grid3D, model = vgm(1, "Exp", .2))

library(lattice)

levelplot(var1.pred ~ x + y | z, as.data.frame(res3D))
rm(n, data3D, range1D, grid3D, res3D)

# $Id: ugsim.R,v 1.2 2006-02-10 19:05:02 edzer Exp $
library(sp)
library(gstat)

# prediction grid:
data(meuse.grid)
gridded(meuse.grid) = ~x+y

# define variable as dummy data (parameters from log-zinc, meuse)
v = vgm(.55, "Sph", 900, .05)
g = gstat(NULL, "var1", lzn~1, beta = 5.9, nmax = 20, model = v, dummy = TRUE)

# simulation of a single variable
out = predict(g, meuse.grid, nsim = 20)
spplot(out)

# simulation of two negatively correlated variables:
v = vgm(.55, "Sph", 900, .05)
g = gstat(g, "var2", x~1, beta = 5.9, nmax = 20, model = v, dummy = TRUE)

v = vgm(-.3, "Sph", 900, 0.00001)
g = gstat(g, c("var1", "var2"), model = v)
out = predict(g, meuse.grid, nsim = 10)
spplot(out)

# $Id: cosimulation.R,v 1.5 2006-02-10 19:05:02 edzer Exp $
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y

# cosimulation the four heavy metal variables
meuse.g <- gstat(id="zn", formula=zinc~1, data=meuse, nmax = 10,
	set = list(zero = 1e-10))
meuse.g <- gstat(meuse.g, "cu", copper~1,  meuse, nmax = 10)
meuse.g <- gstat(meuse.g, "cd", cadmium~1, meuse, nmax = 10)
meuse.g <- gstat(meuse.g, "pb", lead~1,    meuse, nmax = 10)
meuse.g <- gstat(meuse.g, model=vgm(1, "Sph", 900, 1), fill.all=T)
x <- variogram(meuse.g, cutoff=1000)
meuse.fit = fit.lmc(x, meuse.g)
plot(x, model = meuse.fit)
z <- predict(meuse.fit, newdata = meuse.grid, nsim = 2)

library(lattice)

pl1 <- spplot(z, c(1,2), main = "zinc simulations")
pl2 <- spplot(z, c(3,4), main = "copper simulations")
pl3 <- spplot(z, c(5,6), main = "cadmium simulations")
pl4 <- spplot(z, c(7,8), main = "lead simulations")
print(pl1, split = c(1,1,2,2), more=TRUE)
print(pl2, split = c(1,2,2,2), more=TRUE)
print(pl3, split = c(2,1,2,2), more=TRUE)
print(pl4, split = c(2,2,2,2))

# indicator cosimulation for the 9 deciles of zinc:
q <- quantile(meuse$zinc, seq(.1,.9,.1))
meuse.i <- gstat(id = "zn1", formula = I(zinc < q[1])~1, 
	data = meuse, nmax = 7, beta = .1, set = list(order = 4, zero = 1e-5))
meuse.i <- gstat(meuse.i, "zn2", I(zinc < q[2])~1, meuse, 
	nmax = 7, beta=.2)
meuse.i <- gstat(meuse.i, "zn3", I(zinc < q[3])~1, meuse, 
	nmax = 7, beta=.3)
meuse.i <- gstat(meuse.i, "zn4", I(zinc < q[4])~1, meuse, 
	nmax = 7, beta=.4)
meuse.i <- gstat(meuse.i, "zn5", I(zinc < q[5])~1, meuse, 
	nmax = 7, beta=.5)
meuse.i <- gstat(meuse.i, "zn6", I(zinc < q[6])~1, meuse, 
	nmax = 7, beta=.6)
meuse.i <- gstat(meuse.i, "zn7", I(zinc < q[7])~1, meuse, 
	nmax = 7, beta=.7)
meuse.i <- gstat(meuse.i, "zn8", I(zinc < q[8])~1, meuse, 
	nmax = 7, beta=.8)
meuse.i <- gstat(meuse.i, "zn9", I(zinc < q[9])~1, meuse, 
	nmax = 7, beta=.9)

meuse.i <- gstat(meuse.i, model=vgm(1, "Sph", 900, 1), fill.all=T)
x <- variogram(meuse.i, cutoff=1000)
meuse.fit = fit.lmc(x, meuse.i)
plot(x, model = meuse.fit)

z <- predict(meuse.fit, newdata = meuse.grid, nsim = 2, indicators = TRUE)
spplot(z, main = "indicator simulations for 9 deciles")

#############################
## Example: Meuse data set ##
#############################
library(sp)
library(gstat)
data("meuse")
coordinates(meuse) <- ~x+y

data("meuse.grid")
coordinates(meuse.grid) <- ~x+y
gridded(meuse.grid) <- TRUE

# variography
empVgm <- variogram(zinc~1, meuse)
modVgm <- fit.variogram(empVgm, vgm(150000, "Sph", 1000, 25000))

# unconditional simulation
unconSim <- krigeSimCE(zinc~1, newdata = meuse.grid, model = modVgm, n=100)
unconSim@data$zinc.simMean <- apply(unconSim@data[,-c(1:5)], 1, mean)

spplot(unconSim[,6:20], main="15 out of 100 unconditional simulations")
spplot(unconSim, "zinc.simMean", main="mean of 100 unconditional simulations")

# conditional simulation
conSim <- krigeSimCE(zinc~1, meuse, meuse.grid, modVgm, n=100)
conSim@data$zinc.simMean <- apply(conSim@data[,-c(1:5)], 1, mean)

spplot(conSim[,6:20], main="15 out of 100 conditional simulations")

# compare with kriging predictor
simKrige <- krige(zinc~1, meuse, meuse.grid, modVgm)
spplot(simKrige, "var1.pred", main="interpolated zinc concentrations")
spplot(conSim, "zinc.simMean", main="mean of 100 conditional simulations")

################################################
## turning bands simulation in space and time ##
################################################

separableModel <- vgmST("separable",
                        space=vgm(0.85,"Exp", 831, 0.15),
                        time =vgm(0.9,"Exp", 3.25, 0.1),
                        sill=135000)
attr(separableModel,"temporal unit") <- "days"

library(spacetime)
stf <- STF(meuse, Sys.time()-20:0*24*3600)

stf_grid <- STF(geometry(meuse.grid), stf@time)

###################
## unconditional ##
###################

sTime <- Sys.time()
krigedSim <- krigeSTSimTB(newdata = stf_grid, modelList = separableModel, nsim = 100, nLyrs = 100)
Sys.time() - sTime

# plot one simulation along time
stplot(krigedSim[,,"sim1"], main="unconditional siumulation")

# plot one simulation along time as time series
stplot(krigedSim[1:21,,"sim1"], mode="ts", main="unconditional siumulation")
stplot(krigedSim[1:21,,"sim2"], mode="ts", main="unconditional siumulation")
stplot(krigedSim[1:21,,"sim3"], mode="ts", main="unconditional siumulation")
stplot(krigedSim[1:21,,"sim4"], mode="ts", main="unconditional siumulation")
stplot(krigedSim[1:21,,"sim5"], mode="ts", main="unconditional siumulation")
stplot(krigedSim[1:21,,"sim6"], mode="ts", main="unconditional siumulation")

# plot the ten simulations of the first day
spplot(krigedSim[,1], paste0("sim",1:10), as.table=TRUE, main="unconditional siumulation")

#################
## conditional ##
#################

sTime <- Sys.time()
krigedSim <- krigeSTSimTB(formula= zinc ~ 1, data = STFDF(geometry(meuse), stf@time, data.frame(zinc=rep(meuse$zinc, 21))),
                          newdata = stf_grid[1:500,], modelList = separableModel, nsim = 10, nLyrs = 500)
Sys.time() - sTime

# plot one simulation along time
stplot(krigedSim[,1:12], main="conditinal simulation")

# plot one simulation along time as time series
stplot(krigedSim[1:12,,"sim1"], mode="ts", main="conditinal simulation")

# plot the ten simulations of the first day
spplot(krigedSim[,1], paste0("sim",1:10), as.table=TRUE, main="conditinal simulation")

## FNN local prediction
########################
library(sp)
library(spacetime)
library(gstat)
library(lattice)

# create n space-time points over [0,1] x [0,1] x [Now, Now+some days]
t0 = Sys.time() # now
n = 1000
set.seed(13131) # fix outcomes
x = runif(n)
y = runif(n)
t = t0 + 1e6 * runif(n)
z = rnorm(n)
stidf = STIDF(SpatialPoints(cbind(x,y)), sort(t), data.frame(z=z))

stplot(stidf, number=21, main="random spatio-temporal noise")

library(sftime)
sft = st_as_sftime(stidf)

# create a regular 20 x 20 x 10 grid of prediction locations:
grd = as(SpatialGrid(GridTopology(c(0.025,0.025), c(.05, .05), c(20,20))), "SpatialPixels")
tgrd = seq(min(t)+10000, max(t)-10000, length.out = 10)

stf = STF(grd, tgrd)
#stf = STFDF(grd, tgrd, data.frame(x=rep(0,400*10)))

library(stars)
st = st_as_stars(stf)

# define a variogram model
sumMetricModel <- vgmST("sumMetric",
                        space=vgm(1/6, "Sph", 0.25, 1/60),
                        time =vgm(2/6, "Exp",  1e5, 1/60),
                        joint=vgm(0.4, "Exp", 0.3, 0.1),
                        stAni=1/1e6)
attr(sumMetricModel, "temporal unit") <- "secs"

dg <- data.frame(spacelag=rep(c(0.001,1:10)/10,6), 
                 timelag=rep(0:5*50e3, each=11))
#wireframe(model~spacelag+timelag,
#          variogramSurface(sumMetricModel, dist_grid = dg),
#          scales=list(arrows=F),
#          drape=T, col.regions=bpy.colors(),
#          zlim=c(0,1.2),
#          main="imposed sum-metric model")

locKrig_sft <- krigeST(z~1, sft, st, sumMetricModel, nmax=20, computeVar = T)
locKrig <- krigeST(z~1, stidf, stf, sumMetricModel, nmax=20, computeVar = T)
stplot(locKrig[,,"var1.pred"], col.regions=bpy.colors(), scales=list(draw=T))
plot(locKrig_sft[1], col = sf.colors(), breaks = "equal")
stplot(locKrig[,,"var1.var"], col.regions=bpy.colors(), scales=list(draw=T))
plot(locKrig_sft[2], col = sf.colors(), breaks = "equal")

st$foo = 0
st_as_sf(st, long = TRUE) |> st_as_sftime() -> st.sftime
locKrig_sft <- krigeST(z~1, sft, st.sftime, sumMetricModel, nmax=20, computeVar = T)
plot(locKrig_sft["var1.pred"])

# $Id: examples.R,v 1.6 2006-02-10 19:05:02 edzer Exp $
## ex01.cmd, ex02.cmd:
##
## Two variables with (initial estimates of) variograms,
## calcute sample variogram and plot fitted model
##

library(sp)
par(ask = TRUE)
data(meuse)
coordinates(meuse)=~x+y
x <- variogram(zinc ~ 1, meuse)
v <- vgm(140000, "Sph", 800, nug = 10000)
plot(x, model = v)
plot(x, model = fit.variogram(x, model = v))
x <- variogram(log(zinc) ~ 1, meuse)
v <- vgm(.5, "Sph", 800, nug = .1)
plot(x, model = v)
plot(x, model = fit.variogram(x, model = v))

##
## ex03.cmd:
## Inverse distance interpolation on a mask map
##
data(meuse.grid)
gridded(meuse.grid) = ~x+y
x <- krige(zinc ~ 1, meuse, meuse.grid, model = NULL)

library(lattice)

spplot(x[1])

##
## ex04.cmd 
## Local ordinary block kriging at non-gridded locations
##
## the gstat "classic" radius maps into the gstat "S" maxdist argument
##
new.locs <- SpatialPoints(cbind(x = c(181170, 180310, 180205, 178673, 178770, 178270),
	y = c(333250, 332189, 331707, 330066, 330675, 331075)))
krige(zinc ~ 1, meuse, new.locs, 
		model = vgm(1.34e5, "Sph", 800, nug = 2.42e4), 
		block = c(40,40), nmax = 40, nmin = 20, 
		maxdist = 1000)

##
## ex05.cmd 
##
## Local simple point kriging on a mask map
##
v <- vgm(0.581, "Sph", 900, nug = 0.0554)
x <- krige(log(zinc) ~ 1, meuse, meuse.grid, model = v, 
	nmax = 40, nmin = 20, maxdist = 1000, beta = 5.9)
spplot(x[1], main = "log(zinc) simple kriging prediction")
x$se = sqrt(x$var1.var)
spplot(x["se"], main = "log(zinc) simple kriging standard errors")

##
## ex06.cmd 
##
## Unconditional Gaussian simulation on a mask
## (local neigbourhoods, simple kriging)
##
x <- krige(log(zinc) ~ 1, locations = NULL, newdata = meuse.grid, 
	model = v, nmax = 20, beta = c(5.9), nsim = 5, dummy = TRUE)
spplot(x, main = "five unconditional realisations of a correlated Gaussian field")

##
## ex07.cmd 
##
## Gaussian simulation, conditional upon data
## (local neighbourhoods, simple and ordinary kriging)
##
x <- krige(log(zinc) ~ 1, meuse, meuse.grid, 
	model = v, nmax = 20, beta = c(5.9), nsim = 5)
spplot(x, main = "five conditional realisations of a correlated Gaussian field")

##
## ex08.cmd 
##
## Change of support: local ordinary block kriging on a mask
##
x <- krige(log(zinc) ~ 1, meuse, meuse.grid, 
	model = v, nmax = 40, nmin = 20, maxdist = 1000,
	block = c(40,40))
spplot(x[1], main = "ordinary block kriging predictions")
x$se = sqrt(x$var1.var)
spplot(x["se"], main = "ordinary block kriging prediction standard errors")

##
## ex09.cmd 
##
## Obtain map values at data() locations
## (Point-map overlay)
##
# we trick here by using inv.weighted distance interpolation, using the
# single nearest observation. It will not fail on points outside the grid.
# Note that we reversed meuse.grid and meuse to get these results.
x <- krige(part.a ~ 1, meuse.grid, meuse, model = NULL, nmax = 1)
meuse$part.a = x$var1.pred
x <- krige(part.b ~ 1, meuse.grid, meuse, model = NULL, nmax = 1)
meuse$part.b = x$var1.pred

##
## ex10.cmd 
##
## Multiple kriging: prediction on more than one variable
## (ordinary kriging of two variables)
## (note that zinc_map.eas wass obtained through ex09.gst)
##
x <- variogram(dist~1,meuse)
v.dist <- fit.variogram(x, model = vgm(1,"Gau",100))
plot(x, model = v.dist)
g <- gstat(id = "ln.zinc", form = log(zinc) ~ 1,
	data = meuse, nmax = 40, nmin = 20, maxdist = 1000, model = v)
g <- gstat(g, id = "dist", form = dist ~ 1, 
	data = meuse, nmax = 40, nmin = 20, maxdist = 1000,
	model = vgm(.01, "Nug", 0, add.to = v.dist))
# the added nugget variance is necessary to avoid near-singular covariances
x <- predict(g, meuse.grid)
spplot(x["ln.zinc.pred"], main = "log(zinc) ordinary kriging predictions")
x$ln.zinc.se = sqrt(x$ln.zinc.var)
spplot(x["ln.zinc.se"], main = "log(zinc) ordinary kriging prediction standard errors")
spplot(x["dist.pred"], main = "dist ordinary kriging predictions")
x$dist.se = sqrt(x$dist.var)
spplot(x["dist.se"], main = "dist ordinary kriging prediction standard errors")

##
## ex11.cmd 
##
## Multivariable kriging: ordinary local cokriging of two variables
## For examples of fitting an LMC: see demo(cokriging)
##
g <- gstat(id = "ln.zinc", form = log(zinc) ~ 1, 
	data = meuse, nmax = 40, nmin = 20, maxdist = 1000,
	model = vgm(0.581, "Sph", 900, 0.0554))
g <- gstat(g, id = "sq.dist", form = sqrt(dist) ~ 1, 
	data = meuse, nmax = 40, nmin = 20, maxdist = 1000,
	model = vgm(0.0631, "Sph", 900, 0.0001))
g <- gstat(g, id = c("ln.zinc", "sq.dist"), 
	model = vgm(-0.156, "Sph", 900, 1e-5))
# small nugget necessary to let gstat recognize LMC
x <- predict(g, meuse.grid)
spplot(x["ln.zinc.pred"], main = "log(zinc) ordinary cokriging predictions")
x$ln.zinc.se = sqrt(x$ln.zinc.var)
spplot(x["ln.zinc.se"], main = "log(zinc) ordinary cokriging prediction standard errors")
spplot(x["sq.dist.pred"], main = "dist ordinary cokriging predictions")
x$sq.dist.se = sqrt(x$sq.dist.var)
spplot(x["sq.dist.se"], main = "dist ordinary cokriging prediction standard errors")

##
## ex12.cmd 
##
## Stratified ordinary kriging (within-category ordinary kriging)
##

# find out in which part the data are:
meuse$part.a = krige(part.a~1, meuse.grid, meuse, nmax=1)$var1.pred
x1 = krige(log(zinc)~1, meuse[meuse$part.a == 0,], 
	meuse.grid[meuse.grid$part.a == 0,], model = vgm(.548, "Sph", 900, .0654), 
	nmin = 20, nmax = 40, maxdist = 1000)
x2 = krige(log(zinc)~1, meuse[meuse$part.a == 1,], 
	meuse.grid[meuse.grid$part.a == 1,], model = vgm(.716, "Sph", 900), 
	nmin = 20, nmax = 40, maxdist = 1000)
x = rbind(as.data.frame(x1), as.data.frame(x2))
gridded(x) = ~x+y
spplot(x["var1.pred"], main = "stratified kriging predictions")

##
## ex13.cmd 
##
## Local universal kriging, using one continuous variable
###
## the variogram should be that of the residual:
x <- krige(log(zinc) ~ sqrt(dist), meuse, meuse.grid, 
	model = vgm(.149, "Sph", 700, .0674), 
	nmax = 40, nmin = 20, maxdist = 1000)
spplot(x["var1.pred"], main = "universal kriging predictions")
x$var1.se = sqrt(x$var1.var)
spplot(x["var1.se"], main = "universal kriging prediction standard errors")

##
## ex14.cmd 
##
## Universal kriging, using one continuous and
## two binary variables.
##
x <- krige(log(zinc) ~ -1 + sqrt(dist)+ part.a + part.b, 
	meuse, meuse.grid, model = vgm(.149, "Sph", 700, .0674))
spplot(meuse.grid["part.a"], main = "the areas defining part.a (1) and part.b (0)")
spplot(x["var1.pred"], main = "universal kriging predictions")
x$var1.se = sqrt(x$var1.var)
spplot(x["var1.se"], main = "universal kriging prediction standard errors")

##
## ex14a.cmd 
## 
## stratified universal kriging: 
## (again: not implemented)
##

## ex15.cmd 
##
## Local linear model, using one continuous variable
##
x <- krige(log(zinc) ~ sqrt(dist), meuse, meuse.grid, 
	model = NULL, nmax = 40, nmin = 20, maxdist = 1000)
spplot(x["var1.pred"], main = "IID local linear model kriging predictions")
x$var1.se = sqrt(x$var1.var)
spplot(x["var1.se"], main = "IID local linear model prediction standard errors")

##
## ex16.cmd 
##
## Multivariable indicator cosimulation 
## ==>> see demo(cosimulation) for an extended example how to do this
##

##
## ex17.cmd 
##
## global coordinate polynomial trend surfaces
## trend orders 0-3 ==>> better use lm() for this
##

# $Id: krige.R,v 1.5 2007-02-27 22:09:31 edzer Exp $
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y

# ordinary kriging
v <- variogram(log(zinc)~1, meuse)
m <- fit.variogram(v, vgm(1, "Sph", 300, 1))
plot(v, model = m)
lzn.kr <- krige(formula = log(zinc)~1, meuse, meuse.grid, model = m)


pl1 <- spplot(lzn.kr[1], main = "ordinary kriging prediction of log-zinc")
lzn.kr$se = sqrt(lzn.kr$var1.var)
pl2 <- spplot(lzn.kr["se"], main = "ordinary kriging prediction error")

# universal kriging
v <- variogram(log(zinc)~sqrt(dist), meuse)
m <- fit.variogram(v, vgm(1, "Exp", 300, 1))
plot(v, model = m)
lzn.kr <- krige(log(zinc)~sqrt(dist), meuse, meuse.grid, model = m)
pl3 <- spplot(lzn.kr[1], main = "universal kriging prediction of log-zinc")
lzn.kr$se = sqrt(lzn.kr$var1.var)
pl4 <- spplot(lzn.kr["se"], main = "universal kriging prediction error")
print(pl1, split = c(1,1,2,2), more = T)
print(pl2, split = c(1,2,2,2), more = T)
print(pl3, split = c(2,1,2,2), more = T)
print(pl4, split = c(2,2,2,2))

# $Id: pcb.R,v 1.9 2008-02-01 22:39:44 edzer Exp $
# FIGURE 1:
library(sp)
library(gstat)
library(maptools)
library(lattice)
data(pcb)
coordinates(pcb) = ~x+y
data(ncp.grid)
gridded(ncp.grid) = ~x+y
wsv = readShapePoly(system.file("external/ncp.shp", package="gstat"))
classes = c(.2,1,2,5,10,20)
print(xyplot(y ~ x | as.factor(year), groups = sqrt(PCB138)/3, 
	data = as.data.frame(pcb),
    panel = function(x, y, groups, subscripts, ...) {
				 sp.polygons(wsv, col = grey(.5))
                 panel.xyplot(x, y, cex = groups[subscripts], ...)
			},
    xlab = "x-coordinate", ylab = "y-coordinate",
    key = list(corner = c(0,0), x=0.8, y=0.25, points = list(pch = 1, col = 1,
        cex = sqrt(classes)/3), text = list(as.character(classes))),
	aspect = "iso", as.table = T,
	xlim = c(464000, 739000), ylim = c(5696500, 6131500),
	scales = list(draw = F)))

# FIGURE 2:
pcb$yf = as.factor(pcb$year)
pcb$int = rep(NA, dim(pcb)[1])
c = lm(log(PCB138)~-1+depth+yf,pcb)$coefficients
i = 2
for (f in levels(pcb$yf)) {
	pcb$int[pcb$yf == f] = c[i]
	i = i+1
}
print(xyplot(log(PCB138)~depth | as.factor(year), groups = int, 
	data = as.data.frame(pcb), 
    panel = function(x, y, groups, subscripts, ...) {
                # panel.grid(h=-1, v= 2)
                panel.xyplot(x, y)
				y = c(groups[subscripts][1], groups[subscripts][1] + -0.0641*45)
				llines(c(0, 45), y, lty = 2)
            },
    scales = list(
		y = list(at=log(c(.2, .5, 1, 2, 5, 10, 20)), 
		labels=c(".2",".5","1","2","5","10","20"), alternating = F)
	), xlab = "water depth", ylab = "PCB138", as.table = T)
)

# FIGURE 3:
# ps.options(width=2, height=2)
pcb$res=residuals(lm(log(PCB138)~year+depth, pcb))
v3 = variogram(res ~ year, pcb, dX=.1, bound=c(0,1000,3000,5000,(1:16)*10000))
print(plot(v3, model = vgm(.224,"Exp",17247,.08), plot.numbers = TRUE))

# FIGURE 4:
# next
g.pcb = NULL
merge = list(c("P1986", 2, "P1991", 2), c("P1986", 2, "P1996", 2),
		c("P1986", 2, "P2000", 2))
for (f in levels(pcb$yf)[c(1,4,6,7)])
    g.pcb= gstat(g.pcb, 
		paste("P", as.character(f), sep = ""), 
		log(PCB138)~depth, pcb[pcb$yf == f,],
		merge = merge)
g.pcb = gstat(g.pcb, model = vgm(.224,"Exp",17247,.08), fill.all=T)
v = variogram(g.pcb, cutoff=1e5)
#plot(v, model = fit.lmc(v, g))
#plot(v, model = g,plot.numbers = TRUE)
PCB.cor = matrix(NA, 4,4)
i = 1
for (x in levels(pcb$yf)[c(1,4,6,7)]) {
    j = 1
    for (y in levels(pcb$yf)[c(1,4,6,7)]) {
        A = log(pcb[pcb$yf == x,]$PCB138)
        B.tmp = krige(PCB138~1, pcb[pcb$yf == y,], 
			pcb[pcb$yf == x,], nmax = 1, set=list(debug=0))
        B = log(B.tmp$var1.pred)
        # print(paste(x, y, cor(A,B)))
        PCB.cor[i,j] = cor(A,B)
        j = j + 1
    }
    i = i + 1
}
years=c(1986,1991,1996,2000)
dimnames(PCB.cor)=list(years,years)
PCB.cor.ns = PCB.cor
PCB.cor = 0.5 * (PCB.cor + t(PCB.cor))
i = 1
for (x in levels(pcb$yf)[c(1,4,6,7)]) {
    j = 1
    for (y in levels(pcb$yf)[c(1,4,6,7)]) {
        if (j > i) {
            name = paste(paste("P", x, sep=""), paste("P", y, sep=""),sep = ".")
			print(name)
            g.pcb$model[[name]]["psill"] = 
				g.pcb$model[[name]]["psill"]  * PCB.cor[i,j]
        }
        j = j + 1
    }
    i = i + 1
}
print(plot(v, model = g.pcb, plot.numbers = FALSE))

print(PCB.cor.ns, digits=3)

print(PCB.cor, digits=3)

# FIGURE 5:
pcb.cok = predict(g.pcb, newdata = ncp.grid, debug.level = 0)
levs = c(.1,.2,.5,1,2,5,10,20)

spplot(pcb.cok[c(1,3,5,7)],
	as.table=T, col.regions = bpy.colors(7),
	at = log(levs),
	colorkey = list(at = log(levs), labels = as.character(levs),
		col = bpy.colors(7)), 
	layout = c(4,1)
)

spplot(pcb.cok[c(4,11,6,12,13,8,14,15,16,10)-2],
	skip=c(F,T,T,T,F,F,T,T,F,F,F,T,F,F,F,F),
	as.table=T, layout=c(4,4),
	asp="iso", col.regions = bpy.colors())

X = cbind(rep(1,7), c(1986, 1987, 1989, 1991, 1993, 1996, 2000))
X2=X[c(1,4,6,7),]
lambda = solve(t(X2) %*% X2) %*% t(X2)
dimnames(lambda) = list(NULL, c(1986, 1991, 1996, 2000))
print(lambda[2, ], digits=3)

# FIGURE 7:
pcb.contr = get.contr(pcb.cok, g.pcb, X=lambda[2, ])
# copy coordinates
#pcb.contr$x = pcb.cok$x
#pcb.contr$y = pcb.cok$y

pl1 = spplot(pcb.contr["beta.1"],
	main = "log-PCB138: change estimate",
	col.regions = bpy.colors(100))
pcb.contr$sig = pcb.contr$beta.1 / sqrt(pcb.contr$var.beta.1)
pl2 = spplot(pcb.contr["sig"],
	main = "log-PCB138 change/SE",
	col.regions = bpy.colors(100))
print(pl1, position=c(0,0,0.5,1), more = TRUE)
print(pl2, position=c(0.5,0,1,1), more = FALSE)

cat("source:\n\nEdzer J. Pebesma, Richard N.M. Duin (2005) Spatio-temporal mapping of\nsea floor sediment pollution in the North Sea.  In: Ph. Renard, and\nR. Froidevaux, eds. Proceedings GeoENV 2004 -- Fifth European Conference\non Geostatistics for Environmental Applications; Springer.\n")

library(sp)
demo(meuse, ask = FALSE, echo = FALSE)

library(gstat)

# use collocated data:
g = gstat(NULL, "lzinc", log(zinc)~1, meuse)
g.coll = gstat(g,    "dist", dist~1, meuse, nmax = 1, merge = c("lzinc", "dist"))

g.fit = fit.lmc(variogram(g.coll), g.coll, vgm(1, "Sph", 900, 1),
	correct.diagonal = 1.01)

g.non_coll = gstat(g,  "dist", dist~1, meuse.grid, nmax = 1, merge = c("lzinc", "dist"))
g.non_coll$model = g.fit$model

# collocated cokriging:

pr = predict(g.non_coll, meuse.grid)
spplot(pr[c(1,3)])

# $Id: cokriging.R,v 1.4 2006-02-10 19:05:02 edzer Exp $
library(stars)
library(gstat)
data(meuse, package = "sp")
meuse = st_as_sf(meuse, coords = c("x", "y"))
data(meuse.grid, package = "sp")
meuse.grid = st_as_stars(meuse.grid)

# cokriging of the four heavy metal variables
# create gstat object, stepwise:
gstat(id="zn", formula=log(zinc)~1, data=meuse, nmax = 10) |>
  gstat("cu", log(copper)~1, meuse, nmax = 10) |>
  gstat("cd", log(cadmium)~1, meuse, nmax = 10) |>
  gstat("pb", log(lead)~1, meuse, nmax = 10) |>
  gstat(model=vgm(1, "Sph", 900, 1), fill.all=T) -> meuse.g

x <- variogram(meuse.g, cutoff=1000)
meuse.fit = fit.lmc(x, meuse.g)
plot(x, model = meuse.fit)
z <- predict(meuse.fit, newdata = meuse.grid)

z[c(1,3,5,7)] |> merge() |> plot()
# compute & plot standard errors:
z[c(2,4,6,9)] |> setNames(paste0(c("zn", "cu", "pb", "cd"), ": se")) |> 
   merge() |> sqrt() |> plot()

# old-style, with sp:
# indicator cokriging for the 9 percentiles of zinc:
q <- quantile(meuse$zinc, seq(.1,.9,.1))
gstat(id = "zn1", formula = I(zinc < q[1])~1, 
	data = meuse, nmax = 7, beta = .1, set = list(order = 4, zero = 1e-5)) |>
  gstat("zn2", I(zinc < q[2])~1, meuse, nmax = 7, beta=.2) |>
  gstat("zn3", I(zinc < q[3])~1, meuse, nmax = 7, beta=.3) |>
  gstat("zn4", I(zinc < q[4])~1, meuse, nmax = 7, beta=.4) |>
  gstat("zn5", I(zinc < q[5])~1, meuse, nmax = 7, beta=.5) |>
  gstat("zn6", I(zinc < q[6])~1, meuse, nmax = 7, beta=.6) |>
  gstat("zn7", I(zinc < q[7])~1, meuse, nmax = 7, beta=.7) |>
  gstat("zn8", I(zinc < q[8])~1, meuse, nmax = 7, beta=.8) |>
  gstat("zn9", I(zinc < q[9])~1, meuse, nmax = 7, beta=.9) |>
  gstat(model=vgm(1, "Sph", 900, 1), fill.all=T) -> meuse.i

x <- variogram(meuse.i, cutoff=1000)
meuse.fit = fit.lmc(x, meuse.i, correct.diagonal = 1.01)
plot(x, model = meuse.fit)
z <- predict(meuse.fit, newdata = meuse.grid)

z[c(1,3,5,7,9,11,13,15,17)] |> 
	setNames(paste("est.Pr(Zn < ", signif(q,4), ")", sep = "")) |> merge() |> plot()

Rprof()
# import NC SIDS data:
library(sp)
library(maptools)
fname = system.file("shapes/sids.shp", package="maptools")[1]
nc = readShapePoly(fname, proj4string = 
	CRS("+proj=longlat +datum=NAD27 +ellps=clrk66"))

# reproject to UTM17, so we can use Euclidian distances:
library(rgdal)
nc = spTransform(nc, CRS("+proj=utm +zone=17 +datum=WGS84 +ellps=WGS84"))

# create a target (newdata) grid, and plot:
grd = spsample(nc, "regular", n = 1000)
class(grd)
plot(nc, axes = TRUE)
points(grd, pch = 3)

library(gstat)

# area-to-point kriging:
kr = krige0(SID74 ~ 1, nc, grd, vgmArea, ndiscr = 9, 
	vgm = vgm(1, "Exp", 1e5, 0), # point variogram,
	verbose = TRUE)
out = SpatialPixelsDataFrame(grd, data.frame(pred = kr))

pl0 = spplot(nc["SID74"], main = "areas")
pl1 = spplot(out, sp.layout = list("sp.polygons", nc, first=F,col='grey'), 
    main = "points on a grid")
print(pl0, split = c(1,1,1,2), more = TRUE)
print(pl1, split = c(1,2,1,2), more = FALSE)

kriging.weights = function(x, formula, newdata, model) {
	weighti = function(x, i, formula,...) {
		ret =rep(0,nrow(x))
		ret[i]=1
		x[[1]]=ret
		krige(formula = formula,locations = x,...)
	}
	ret = sapply(1:nrow(x), weighti, x=x, newdata=newdata[1,], model=model,formula=formula)
	ret = t(sapply(ret, as.data.frame))
	unlist(ret[,3])
}
# example, at first cell of meuse.grid:
require(sp)
require(gstat)
data(meuse)
data(meuse.grid)
coordinates(meuse) = ~x+y
coordinates(meuse.grid) = ~x+y
meuse$wts = kriging.weights(meuse["zinc"], zinc~1, meuse.grid[1,], vgm(1, "Exp", 300))
summary(meuse$wts)
spplot(meuse["wts"], col.regions=bpy.colors(), cuts=(0:10)/20)

library(sp)
library(gstat)
data(meuse.grid)
gridded(meuse.grid) = ~x+y
data(meuse)
coordinates(meuse) = ~x+y

# choose arbitrary line over the grid:
image(meuse.grid["dist"],axes=T)
pp = rbind(c(180000,331000),c(180000,332000),c(181000,333500))
Sl = SpatialLines(list(Lines(list(Line(pp)), "a")))
plot(Sl,add=T,col='green')

# use the default spsample arguments of predict.gstat:
pts=spsample(Sl,n=500,'regular',offset=c(.5,.5))
plot(pts, pch=3, cex=.2, add=T)

v = vgm(.6, "Sph", 900, .06)
out1 = krige(log(zinc)~1, meuse, Sl, v)
out1

points(180333,332167,pch=3,cex=2)

# use the same line as block discretization, and predict for (0,0)
# (because the block discretizing points are not centered)
out2 = krige(log(zinc)~1, meuse, SpatialPoints(matrix(0,1,2)), v, block=coordinates(pts))
out2

compare.krigingLines = function(formula, data, newdata, model) {
	out1 = krige(formula, data, newdata, model)
	pts = spsample(newdata, n=500, 'regular', offset=.5)
	out2 = krige(formula, data, SpatialPoints(matrix(0,1,2)), model, block = coordinates(pts))
	print("difference:")
	as.data.frame(out1)[3:4]- as.data.frame(out2)[3:4]
}

compare.krigingLines(log(zinc)~1, meuse, Sl, v)

# one line, consisting of two line segments:
pp2 = rbind(c(181000,333500),c(181000,332500))
Sl2 = SpatialLines(list(Lines(list(Line(pp),Line(pp2)), "a")))
krige(log(zinc)~1, meuse, Sl2, v)
compare.krigingLines(log(zinc)~1, meuse, Sl2, v)

# two seperate line segments:
Sl3 = SpatialLines(list(Lines(list(Line(pp)), "a"),Lines(list(Line(pp2)),"b")))
krige(log(zinc)~1, meuse, Sl3, v)

library(sp)
library(gstat)
data(meuse)
data(meuse.grid)
coordinates(meuse)=~x+y
gridded(meuse.grid)=~x+y
v = variogram(I(zinc < 500)~1,meuse)
plot(v)
vm = fit.variogram(v, vgm(1, "Sph", 300, 1))
plot(v,vm)
vm
# possibly adjust sum of sill to be max. 0.25?
ik = krige(I(zinc>500)~1, meuse, meuse.grid, vm)
spplot(ik[1],col.regions=bpy.colors())
summary(ik[[1]])
# adjust values outside [0,1] to nearest limit:
ik[[1]][ik[[1]]<0] = 0
ik[[1]][ik[[1]]>1] = 1
summary(ik[[1]])
spplot(ik[1],col.regions=bpy.colors())

library(sp)
library(gstat)
data(meuse)
coordinates(meuse) = ~x + y
data(meuse.grid)
coordinates(meuse.grid) = ~x + y

# Variogram log Zn

lzn.vgm = variogram(log(zinc) ~ 1, meuse)
lzn.fit = fit.variogram(lzn.vgm, model = vgm(1, "Sph", 900, 1))

#Conditional simulation

nsim = 100
lzn.sim = krige(log(zinc) ~ 1, meuse, meuse.grid, model = lzn.fit,
	nmax = 30, nsim = nsim)

# Variogram of all relizations

m = out = list()
for (i in 1:nsim) {
	s = paste("sim", i, sep="")
	f = as.formula(paste(s, "~1"))
	v = variogram(f, lzn.sim)
	v$id = rep(s, nrow(v))
	out[[s]] = v
	m[[s]] = fit.variogram(v, lzn.fit)
}
plot(do.call(rbind, out), m, layout=c(10,10), skip = FALSE, 
	scales = list(y = list(relation = "same")))

#pdf("wind.pdf")
# PLEASE read the vignette of package spacetime for a more
# clever way to do all this!
library(sp)
library(gstat)
library(rgdal)
library(maptools)
# load wind data, run test:
example(wind)

m = map2SpatialLines(
	map("worldHires", xlim = c(-11,-5.4), ylim = c(51,55.5), plot=F))
proj4string(m) = "+proj=longlat +datum=WGS84 +ellps=WGS84"
m = spTransform(m, CRS("+proj=utm +zone=29 +datum=WGS84 +ellps=WGS84"))

# model temporal autocorrelation
acf(wind[7])
tdiscr = 0:40
lines(tdiscr, exp(- tdiscr/1.5))

# set up data, last year
years = 61
months = 1
jday = c(1,6,11,16,21,26)
sel = wind[wind$year %in% years & 
	wind$month %in% months &
	wind$jday %in% jday,]

#stations = 4:15
stations = 4:15

sels = stack(sel[stations])
sels$t = rep(sel$jday, length(stations))
sels$x = coordinates(wind.loc)[match(sels$ind, wind.loc$Code),1]
sels$y = coordinates(wind.loc)[match(sels$ind, wind.loc$Code),2]
summary(sels)

coordinates(sels) = ~x+y
proj4string(sels) = "+proj=longlat +datum=WGS84 +ellps=WGS84"
sels = spTransform(sels, CRS("+proj=utm +zone=29 +datum=WGS84 +ellps=WGS84"))
grd = makegrid(m, n = 1000)
grd$t = rep(1, nrow(grd))
coordinates(grd) = ~x1+x2
gridded(grd)=TRUE
proj4string(grd) = proj4string(sels)

#sels = as(sels, "data.frame")

# setup grid
covfn = function(x, y = x) { 
	u = spDists(coordinates(x), coordinates(y))
	t = abs(outer(x$t,y$t,"-"))
	0.6 * exp(-u/750000) * exp(-t/1.5)
}
for (i in 1:120) {
	grd$t = rep(i/4, nrow(grd))
	n = paste("t", i/4, sep="")
	grd[[n]] = krige0(sqrt(values)~1, sels, grd, covfn)
}
grd$t = NULL
#grd$pr = out$pred
#library(lattice)
#levelplot(pr~x1+x2|t,grd,col.regions=bpy.colors())
spl = list(list("sp.points", sels,first=F, cex=.5),
	list("sp.lines", m, col='grey'))
spplot(grd, sp.layout = spl, col.regions=bpy.colors())

# $Id: grass.R,v 1.4 2006-02-10 19:05:02 edzer Exp $
# this demo assumes quite a lot:
#  a. it assumes GRASS gis is running
#  b. it assumes that the meuse data zinc variable is available as a site list
#  c. it assumes that mask_map is present, and contains the mask map values
#     (i.e., the study area)

library(sp)
library(GRASS)           # load R GRASS interface

G = gmeta()              # retrieves active data base locations and topology
d = sites.get(G, "zinc") # retrieve zinc observations 
plot(d$east, d$north, asp=1)
names(d)[4] = "zinc"     # rename attribute
mask = rast.get

hist(d$zinc)
hist(log(d$zinc))

mask = rast.get(G, "mask_map")
plot(G, mask$mask.map)
points(d$east,d$north, pch="+")

library(gstat)           # load gstat library

bubble(d, zcol = "zinc", col=c(4,5), maxsize=2)

# explain S formulae: ~
v = variogram(log(zinc)~1, ~east+north, d)
plot(v)

v.mod = vgm(.6, "Sph", 900, .1)
plot(v, model = v.mod)

v.fit = fit.variogram(v, v.mod)
plot(v, model = v.fit)

zinc.g = gstat(NULL, "lzinc", log(zinc)~1, ~east+north, d, model = v.fit)
new.data = data.frame(east = east(G), north = north(G))
new.data[is.na(mask$mask.map), ] = c(NA,NA)

zinc.kr = predict(zinc.g, new.data)
image(zinc.kr)

library(lattice)

levelplot(lzinc.pred~east+north, zinc.kr, asp=1.34, col.regions=bpy.colors(100))

# push prediction and variances grids back into GRASS data base:
rast.put(G, "lzinc.pred", zinc.kr$lzinc.pred)
rast.put(G, "lzinc.var",  zinc.kr$lzinc.var)

# push cross validation residuals back to GRASS data base:
xv = krige.cv(log(zinc)~1, ~east+north, d, v.fit, nmax = 40, verb=F)
sites.put2(G, data = xv, dims = c("east", "north", "residual", "zscore"), 
	lname = "lzinc.xv")

## area2point in space and time
library(sp)
library(gstat)
data("meuse")
coordinates(meuse) <- ~x+y

data("meuse.grid")
coordinates(meuse.grid) <- ~x+y
gridded(meuse.grid) <- TRUE

meuse.coarse.grid <- SpatialGrid(GridTopology(meuse.grid@grid@cellcentre.offset, c(600,600), c(5, 7)))

separableModel <- vgmST("separable",
                        space=vgm(0.85,"Exp", 831, 0.15),
                        time =vgm(0.9,"Exp", 3.25, 0.1),
                        sill=135000)
attr(separableModel,"temporal unit") <- "days"

library(spacetime)
stf <- STF(meuse[sample(155, 25),], Sys.time()-2:0*24*3600)

stf_grid <- STF(geometry(meuse.coarse.grid), stf@time)

krigedSim <- krigeSTSimTB(newdata = stf_grid, modelList = separableModel, nsim = 1, nLyrs = 100)

# area-to-point kriging:
a2pST = krigeST(sim1 ~ 1, krigedSim, stf, modelList = vgmAreaST, ndiscr = 9, 
            model = separableModel, # point variogram,
            verbose = TRUE)

p1 <- stplot(krigedSim, color.key=F)
p2 <- stplot(a2pST, color.key=F)

print(p1, position=c(0,0.5,1,1), more=TRUE)
print(p2, position=c(0,0,1,0.5))

# how to get the BLUE trend coefficients out of a predict.gstat call?
# prepare data
library(sp)
library(gstat)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y

# create a manual, non-automatic intercept
meuse$Int = rep(1, length(meuse$zinc))
meuse.grid$Int = rep(1, length(meuse.grid$dist))

# create a gstat object without an automatic intercept:
g = gstat(formula = log(zinc)~-1+Int+sqrt(dist), data=meuse, model = vgm(1, "Exp", 300))
newdat = meuse.grid[1:2,c("Int", "dist")]
gridded(newdat) = FALSE
newdat$dist = c(0,1)
newdat$Int = c(1,0)
# (in the more general case of n predictors, make sure that the matrix with
# predictors, or design matrix, equals the identity matrix.)
newdat
out = as.data.frame(predict(g, newdat, BLUE = TRUE))[,3:4]
rownames(out) = c("Intercept", "slope")
colnames(out) = c("BLUE", "Variance")
out

# $Id: fulmar.R,v 1.3 2006-02-10 19:05:02 edzer Exp $
library(sp)
library(gstat)
data(fulmar)
data(ncp.grid)

glm98 <- glm(formula = fulmar ~ depth + coast, family = quasipoisson,
    data = fulmar[fulmar$year == 1998, ])
glm99 <- glm(formula = fulmar ~ depth + coast, family = quasipoisson,
    data = fulmar[fulmar$year == 1999, ])
fulmar98 = data.frame(fulmar[fulmar$year == 1998,], 
	pr98 = predict(glm98, type = "response"))
fulmar99 <- data.frame(fulmar[fulmar$year == 1999,], 
	pr99 = predict(glm99, type = "response"))
pr98.grd <- predict(glm98, newdata = ncp.grid, type = "response", se.fit=TRUE)
pr99.grd <- predict(glm99, newdata = ncp.grid, type = "response", se.fit=TRUE)
pr <- data.frame(ncp.grid, pr98=pr98.grd$fit, pr99=pr99.grd$fit,
	se98 = pr98.grd$se.fit, se99 = pr99.grd$se.fit)

# B.3 create gstat object
g <- gstat(id = "fulmar98", formula = fulmar~pr98, locations = ~x+y, 
	data = fulmar98, model = vgm(1.89629, "Exp", 50000, 0.852478), 
	beta = c(0,1), variance = "mu")
g <- gstat(g, id = "fulmar99", formula = fulmar~pr99, locations = ~x+y, 
	data = fulmar99, model = vgm(2.52259, "Exp", 50000, 1.76474), 
	beta = c(0,1), variance = "mu")
h <- g
h <- gstat(h, id = c("fulmar98","fulmar99"), 
	model = vgm(2.18, "Exp", 50000, 1.22))

# predict block means for blocks in ncp.grid$area (table 2; cokriging)
library(maptools)
areas.r = readShapePoly(system.file("external/ncp.shp", package="gstat"))
#areas.r <- as.SpatialRings.Shapes(areas.shp$Shapes, areas.shp$att.data$WSVGEB_)
coordinates(pr) = ~x+y
#pr.df = overlay(pr, areas.r, fn = mean)
pr.df = na.omit(as(aggregate(pr, areas.r, FUN = mean), "data.frame"))
# match non-empty (and relevant) areas:
#areas = SpatialPolygonsDataFrame(areas.r[c(2,3,4,16),"WSVGEB_"], pr.df[c(1,2,3,5),])#,match.ID=F)
areas = SpatialPolygonsDataFrame(areas.r[c(1,2,12,7),"WSVGEB_"], 
	pr.df[c(1,2,3,5),], match.ID=F)
# areas ID's 0 1 2 14
sk = predict(g, areas)
cok = predict(h, areas)
spplot(cok, c(3,5), names.attr = c("1998", "1999"), 
	main = "Fulmaris glacialis, density estimates\n(by irregular block cokriging)")
sk = as.data.frame(sk)
cok = as.data.frame(cok)
print(data.frame(area = c(1,2,3,16), 
		SK98 = sk$fulmar98.pred, SE98 = sqrt(sk$fulmar98.var),
		SK99 = sk$fulmar99.pred, SE99 = sqrt(sk$fulmar99.var),
		CK98 = cok$fulmar98.pred, SE98 = sqrt(cok$fulmar98.var),
		CK99 = cok$fulmar99.pred, SE99 = sqrt(cok$fulmar99.var)),
	digits=3)
print(data.frame(area = c(1,2,3,16), 
		dSK = sk$fulmar99.pred - sk$fulmar98.pred, 
		SEdSK = sqrt(sk$fulmar98.var+sk$fulmar99.var),
		dCOK = cok$fulmar99.pred - cok$fulmar98.pred, 
		SEdCOK = sqrt(cok$fulmar98.var+cok$fulmar99.var
			 - 2*cok$cov.fulmar98.fulmar99)),
	digits=3)

library(sp)
library(gstat)
data(meuse.grid)
gridded(meuse.grid) = ~x+y
data(meuse)
coordinates(meuse) = ~x+y

ncell = 1000000
# sample 1000000 points over meuse.grid:
newd = spsample(meuse.grid, ncell, type="regular")
ncell = dim(coordinates(newd))[1]
v = vgm(0.6, "Sph", 900, 0.05)

library(parallel)

nclus = detectCores()
clus <- c(rep("localhost", nclus))

# set up cluster and data
cl <- makeCluster(clus, type = "SOCK")
clusterEvalQ(cl, library(gstat))
clusterExport(cl, list("meuse", "meuse.grid", "v"))

# split prediction locations:

# either at random:
splt = sample(1:nclus, nrow(coordinates(newd)), replace = TRUE)

# or regular:
splt = rep(1:nclus, each = ceiling(ncell/nclus), length.out = ncell)
newdlst = lapply(as.list(1:nclus), function(w) newd[splt == w,])

# no cluster:
system.time(out.noclus <- krige(log(zinc)~1, meuse, newd, v))

# with cluster:
system.time(out.clus <- do.call("rbind", parLapply(cl, newdlst, function(lst) 
	krige(log(zinc)~1, meuse, lst, v)
)))
stopCluster(cl)
all.equal(out.clus, out.noclus)
gridded(out.clus) = TRUE
image(out.clus)

require(tools)
packages_to_check <- function(dep, which = c("Depends", "Imports", "LinkingTo", "Suggests"), recursive = FALSE){

download.file("https://cran.R-project.org/web/packages/packages.rds", "packages.rds", mode="wb")
    x <- readRDS("packages.rds")
    x <- x[!duplicated(x[,1]),]
    packages <- x[,1]
    rdeps <- package_dependencies(packages = dep, x,
                        which = which,
                        recursive = recursive, reverse = TRUE)
    paste(apply(x[x[,1] %in% rdeps[[1]], 1:2], 1, paste, collapse="_"), ".tar.gz", sep="")
}

#RCheck = function(x, URL = "http://ftp5.gwdg.de/pub/misc/cran/src/contrib/") {
RCheck = function(x, URL = "https://cran.r-project.org/src/contrib/") {
	if (!file.exists(x))
		download.file(paste(URL, x, sep=""), x)
	cmd = paste("R CMD check --as-cran ", x, " > ", x, ".log", sep = "")
	print(cmd)
	ret = system(cmd)
	print(ret)
	ret
}

result <-  packages_to_check("gstat")
result
sel = TRUE
library(parallel)
ncores_to_use = 2
cl <- makeCluster(getOption("cl.cores", ncores_to_use))
clusterExport(cl, c("RCheck", "sel", "result"))
out = parLapply(cl, result[sel], function(x) RCheck(x))
succ = unlist(out)
x = which(succ != 0)
result[x]
bla = lapply(result[x], function(y) {
		cat(paste(y, ":\n"))
		system(paste("tail -20 ",y,".log", sep=""))
	}
)

#result <-  packages_to_check("sp", recursive=TRUE) 

# Ben Graeler, 25 th March, 2016
#
# Script reproducing the fit of the vignette "spatio-temporal-kriging

# libraries
library(sp)
library(spacetime)
library(gstat)
library(rgdal)

# load data from package gstat
data(DE_RB_2005, package = "gstat")

paper <- FALSE
set.seed(123)
smplDays <- sort(sample(365,8))

# load German boundaries
data(air)
DE_NUTS1 <- spTransform(DE_NUTS1, CRS("+init=epsg:32632"))

if(!paper)
  plot(DE_NUTS1)

# station wise coverage
if(!paper)
  barplot(sort(table(DE_RB_2005@index[,1])),  
          main="reported days per station",
          ylab="number of days", xaxt="n")

# acf
if(!paper) {
  acf(DE_RB_2005[sample(68,1),,drop=F]@data)
  var(DE_RB_2005@data$PM10)
}

# a few daily snapshots
if(paper)
  png("vignettes/figures/daily_means_PM10.png", width=9, height=6, "in", res=150)
stplot(as(DE_RB_2005[,smplDays],"STFDF"),
       col.regions=bpy.colors(120)[-(1:20)],
       sp.layout = list("sp.polygons", DE_NUTS1), scales=list(draw=F), 
       key.space="right", colorkey=T, cuts=0:70,
       main=NULL)
if(paper)
  dev.off()

# number of stations
length(DE_RB_2005@sp)

# calculate the empirical variogram
empVgm <- variogramST(PM10~1, DE_RB_2005, tlags=0:6)
if(!paper) {
  plot(empVgm, wireframe=T, scales=list(arrows=F))
  plot(empVgm)
}

# fit of theoretical purely spatial models #
############################################
spEmpVgm <- empVgm[empVgm$timelag == 0,]
class(spEmpVgm) <- c("gstatVariogram", "data.frame")
spEmpVgm <- spEmpVgm[-1,1:3]
spEmpVgm$dir.hor <- 0
spEmpVgm$dir.ver <- 0
spVgmMod <- fit.variogram(spEmpVgm, vgm(80,"Exp",300000,20))
if(!paper)
  plot(spEmpVgm, spVgmMod)

# fit of theoretical spatio-temporal models #
#############################################

linStAni <- estiStAni(empVgm, c(50000,200000))

if(!paper) {
  plot(gamma~dist, empVgm[empVgm$timelag == 0,], ylim=c(0,100), xlim=c(0,800000))
  points(empVgm[empVgm$spacelag == 0,]$timelag*linStAni, empVgm[empVgm$spacelag == 0,]$gamma, col="red")
}

##
# rescale empVgm and linStAni to km for estimation
empVgm$dist  <- empVgm$dist/1000
empVgm$avgDist  <- empVgm$avgDist/1000
empVgm$spacelag <- empVgm$spacelag/1000

linStAni <- linStAni/1000

# separable
separableModel <- vgmST("separable", 
                        space=vgm(0.9,"Exp", 200, 0.1),
                        time =vgm(0.9,"Sph", 3.5, 0.1),
                        sill=120)
fitSepModel <- fit.StVariogram(empVgm, separableModel, fit.method = 7, 
                               stAni = linStAni, method = "L-BFGS-B", 
                               control = list(parscale=c(100,1,10,1,100)),
                               lower = c(10,0,.1,0,0.1), 
                               upper = c(2000,1,12,1,200))
attr(fitSepModel, "optim.output")$value
# Exp+Exp: 9.87, Exp+Sph: 6.82, Sph+Exp: 10.42, Sph+Sph: 7.50
if(!paper)
  plot(empVgm, fitSepModel, wireframe=T, all=T, scales=list(arrows=F), zlim=c(0,135))

# product-sum
prodSumModel <- vgmST("productSum",
                      space=vgm(10, "Exp", 200, 1),
                      time= vgm(10, "Sph",   2, 1), 
                      k=2)
fitProdSumModel <- fit.StVariogram(empVgm, prodSumModel, fit.method = 7, 
                                   stAni = linStAni, method = "L-BFGS-B", 
                                   control = list(parscale = c(1,10,1,1,0.1,1,10)),
                                   lower = rep(0.0001,7))
attr(fitProdSumModel, "optim.output")$value
# Exp+Exp: 10.09, Exp+Sph: 6.91, Sph+Exp: 10.64, Sph+Sph: 7.59
plot(empVgm, fitProdSumModel, wireframe=T, all=T, scales=list(arrows=F), zlim=c(0,135))

# metric
metricModel <- vgmST("metric",
                     joint = vgm(60, "Mat", 150, 10, kappa=0.6),
                     stAni = 60)
fitMetricModel <- fit.StVariogram(empVgm, metricModel, fit.method = 7,
                                  stAni = linStAni, method = "L-BFGS-B",
                                  control = list(parscale = c(10,20,5,10)),
                                  lower = c(80,50,5,50),
                                  upper = c(200,1500,60,300))
attr(fitMetricModel, "optim.output")$value 
# Exp: 10.25, Sph: 10.59,
# Gau: 21.32, Mat 5: 18.20, Mat 2: 14.43, Mat 1.25: 12.04,
# Mat 1: 11.07, Mat 0.75: 10.23, Mat 0.6: 10.05
if(!paper)
  plot(empVgm, fitMetricModel, wireframe=T, all=T, scales=list(arrows=F), zlim=c(0,135))

# simplified sumMetric model?
sumMetricFromsimpleSumMetric <- function(vgm) {
  vgmST("sumMetric",
        space=vgm(vgm$space$psill, vgm$space$model, vgm$space$range, vgm$nugget/3),
        time =vgm(vgm$time$psill, vgm$time$model, vgm$time$range, vgm$nugget/3),
        joint=vgm(vgm$joint$psill, vgm$joint$model, vgm$joint$range, vgm$nugget/3),
        stAni=vgm$stAni)
}

simpleSumMetricModel <- vgmST("simpleSumMetric",
                              space=vgm(120,"Sph", 150),
                              time =vgm(120,"Exp", 10),
                              joint=vgm(120,"Sph", 150),
                              nugget=10, stAni=150)
fitSimpleSumMetricModel <- fit.StVariogram(empVgm, simpleSumMetricModel,
                fit.method = 7, stAni=linStAni,
                method = "L-BFGS-B",
                lower = c(sill.s = 0, range.s = 10,
                          sill.t = 0, range.t = 0.1,
                          sill.st= 0, range.st= 10,
                          nugget=0, anis = 40),
                upper = c(sill.s = 200,  range.s = 500,
                          sill.t = 200,  range.t = 20,
                          sill.st= 200, range.st = 5000,
                          nugget = 100, anis = 1000),
                control = list(parscale = c(1,10,1,1,1,100,1,10)))
attr(fitSimpleSumMetricModel, "optim.output")$value
# Exp+Exp+Exp: 4.10 Exp+Sph+Exp: 3.60 Sph+Exp+Exp: 3.94 Sph+Sph+Exp: 3.32
# Exp+Exp+Sph: 3.74 Exp+Sph+Sph: 3.98 Sph+Exp+Sph: 3.31 Sph+Sph+Sph: 3.56
if(!paper)
  plot(empVgm,fitSimpleSumMetricModel, wireframe = T, scales = list(arrows = F), all = T , zlim=c(0,130))

# sum-metric
# sumMetricModel <- sumMetricFromsimpleSumMetric(fitSimpleSumMetricModel)

sumMetricModel <- vgmST("sumMetric",
                        space = vgm(20, "Sph", 150, 1),
                        time = vgm(10, "Exp", 2, 0.5),
                        joint = vgm(80, "Sph", 1500, 2.5),
                        stAni = 120)
fitSumMetricModel <- fit.StVariogram(empVgm, sumMetricModel, fit.method = 7, stAni=linStAni,
                method = "L-BFGS-B", 
                lower = c(sill.s = 0,  range.s = 10,  nugget.s = 0,
                          sill.t = 0,  range.t = 0.1,   nugget.t = 0,
                          sill.st= 0, range.st = 10, nugget.st = 0, 
                          anis = 40),
                upper = c(sill.s = 200,  range.s = 1E3,  nugget.s = 20,
                          sill.t = 200,  range.t = 75,   nugget.t = 20,
                          sill.st= 200, range.st = 5E3, nugget.st = 20,
                          anis = 500),
                control = list(parscale = c(1,100,1,1,0.5,1,1,100,1,100),
                               maxit=1e4))
attr(fitSumMetricModel, "optim.output")$value
# Exp+Exp+Exp: 4.10 Exp+Sph+Exp: 3.60 Sph+Exp+Exp: 3.89 Sph+Sph+Exp: 3.32
# Exp+Exp+Sph: 3.74 Exp+Sph+Sph: 3.73 Sph+Exp+Sph: 3.31 Sph+Sph+Sph: 3.36
if(!paper)
  plot(empVgm, fitSumMetricModel, wireframe=T, all=T, scales=list(arrows=F), zlim=c(0,130))

if(!paper)
  plot(empVgm,fitSumMetricModel, wireframe=T, all=T, scales=list(arrows=F), zlim=c(0,130))

if(paper)
  png("vignettes/figures/allVgmsWireframe.png", 9, 6, "in", bg="white", res = 150)
plot(empVgm, list(fitSepModel, fitProdSumModel, fitMetricModel,
                  fitSumMetricModel, fitSimpleSumMetricModel), 
     wireframe=T, all=T, zlim=c(0,140), ylim=c(0,6.1), xlim=c(0,300),
     scales=list(arrows = F, cex=.8,
                 x=list(at=0:3*100), 
                 y=list(at=0:6, labels=c("0 ","","2 ","","4 ","","6 ")), 
                 z=list(at=0:5*25, labels=c("0  ","","50   ","","100    ",""))),
     at=0:100*1.4,
     xlab=list("space [km]", rot=27, cex=0.8), 
     ylab=list("time [days]", rot=-40, cex=0.8),
     zlab=list(NULL, rot=94, cex=0.8))
if(paper)
  dev.off()

if(paper)
  png("vignettes/figures/allVgmsDiffWireframe.png", 9, 6, "in", bg="white", res = 150)
plot(empVgm, list(fitSepModel, fitProdSumModel, fitMetricModel,
                  fitSumMetricModel, fitSimpleSumMetricModel), 
     wireframe=T,  all=T, zlim=c(-10,25), ylim=c(0,6.1), xlim=c(0,300), 
     diff=TRUE,
     scales=list(arrows = F, cex=.8,
                 x=list(at=0:3*100), 
                 y=list(at=0:6, labels=c("0 ","","2 ","","4 ","","6 ")), 
                 z=list(at=-2:5*5, labels=c("-10  ","","0  ","","10  ","","20  ",""))),
     xlab=list("space [km]", rot=27, cex=0.8), 
     ylab=list("time [days]", rot=-40, cex=0.8),
     zlab=list(NULL, rot=94, cex=0.8))
if(paper)
  dev.off()

if(paper) {
  library(lattice)
  spacelag <- rep(0:300, 13)
  timelag <- rep(0:12/2,each=301)  
  
  cplot <- contourplot(model~spacelag+timelag|type, 
                       rbind(cbind(variogramSurface(fitSumMetricModel, 
                                                    data.frame(spacelag=spacelag,
                                                               timelag=timelag)),
                                   data.frame(type = rep("variogram of the sum-metric model", length(spacelag)))),
                             data.frame(spacelag=spacelag,
                                        timelag=timelag,
                                        gamma=sqrt(spacelag^2+116^2*timelag^2)/10,
                                        type="metric distance [10 km]")),
                       at=0:15*10,
                       xlab="space [km]",
                       ylab="timelag [days]")
  
  png("vignettes/figures/vgmVsMetricDist.png", 9, 4, "in", bg="white", res = 150)
  print(cplot)
  dev.off()
}

# $Id: uisim.R,v 1.5 2008-09-25 10:26:00 edzer Exp $
library(sp)
library(gstat)

# prediction grid:
data(meuse.grid)
gridded(meuse.grid) = ~x+y

# define variable as dummy data
v = vgm(.25, "Sph", 900)
g = gstat(NULL, "var1", x~1, beta = .5, nmax = 20, model = v, dummy = TRUE)

# simulation of a single variable
out = predict(g, meuse.grid, nsim = 20, indicators = TRUE)
spplot(out)

# simulation of two correlated variables:
v = vgm(.1, "Sph", 900)
g = gstat(g, "var2", x~1, beta = .25, nmax = 20, model = v, dummy = TRUE)

v = vgm(-.1, "Sph", 900)
g = gstat(g, c("var1", "var2"), model = v)
out = predict(g, meuse.grid, nsim = 10, indicators = TRUE, set = list(order = 2))
spplot(out)

# merge all 10 individual simulations into three-group factors:
for (i in 1:10) {
	v1 = paste("var1.sim", i, sep = "")
	v2 = paste("var2.sim", i, sep = "")
	m = cbind(out[[v1]], out[[v2]], 1 - (out[[v1]]+out[[v2]]))
	mout = factor(apply(m, 1, function(x) which(x == 1)))
	if (i == 1)
		out2 = SpatialPixelsDataFrame(as(out, "SpatialPixels"), data.frame(mout))
	else
		out2[[i]] = mout
}
names(out2) = paste("sim", 1:10, sep="")
spplot(out2)

require(RColorBrewer)
spplot(out2, col.regions=brewer.pal(3, "Set2"))

library(sp)
demo(meuse,ask=FALSE,echo=FALSE)
library(gstat)
v = variogram(log(zinc)~1, meuse, alpha = c(0,45,90,135))
vm = vgm(.25, "Sph", 1000, anis = c(45, 0.5))
plot(v, vm, main = "geometric")
zonal = vgm(.5, "Sph", 1e9, anis = c(45, 1/1e6))
# range is 1e9, effectively infinity, in 45 direction;
# it is 1e9/1e6 = 1000 in 135 direction.
vm = vgm(.25, "Sph", 1000, add.to = zonal)
plot(v, vm, main = "zonal")

###########################################################
# note that demo(stkrige) needs to be run before this one #
###########################################################

# libraries
library(sp)
library(spacetime)
library(gstat)
library(rgdal)

animate <- FALSE

# interpolation
# build a grid over Germany
gridDE <- SpatialGrid(GridTopology(DE_RB_2005@sp@bbox[,1]%/%10000*10000, c(10000,10000),
                                   cells.dim=ceiling(apply(DE_RB_2005@sp@bbox,1,diff)/10000)))
proj4string(gridDE) <- CRS("+init=epsg:32632")
fullgrid(gridDE) <- F

ind <- over(gridDE, as(DE_NUTS1,"SpatialPolygons"))
gridDE <- gridDE[!is.na(ind)]

# back scale vgms:
fitSepModel$space$range <- fitSepModel$space$range*1000

fitProdSumModel$space$range <- fitProdSumModel$space$range*1000

fitMetricModel$joint$range <- fitMetricModel$joint$range*1000
fitMetricModel$stAni <- fitMetricModel$stAni*1000

fitSimpleSumMetricModel$space$range <- fitSimpleSumMetricModel$space$range*1000
fitSimpleSumMetricModel$joint$range <- fitSimpleSumMetricModel$joint$range*1000
fitSimpleSumMetricModel$stAni <- fitSimpleSumMetricModel$stAni*1000

fitSumMetricModel$space$range <- fitSumMetricModel$space$range*1000
fitSumMetricModel$joint$range <- fitSumMetricModel$joint$range*1000
fitSumMetricModel$stAni <- fitSumMetricModel$stAni*1000

if(animate) {
  DE_pred <- STF(gridDE, DE_RB_2005@time)
  
  predMat <- matrix(NA,0,2)
  for (rd in 15:180) { # rd <- 15
    predMat <- rbind(predMat,
                     krigeST(PM10~1, data=DE_RB_2005[,rd+(-5:5)], # start: 12:24
                         newdata=DE_pred[,rd,drop=F], computeVar=T,
                         fitSumMetricModel, # nmax=50,
                         stAni=linStAni*1000/24/3600)@data)
  }
  
  DE_pred_winter <- DE_pred[,15:180]
  DE_pred_winter <- addAttrToGeom(DE_pred_winter, predMat)
  
  for(i in 1:length(DE_pred_winter@time)) { # i <- 1
    pnt <- spplot(DE_pred_winter[,i], "var1.pred", col.regions=bpy.colors(160)[-(1:9)], scales=list(draw=F),
                  at=c(-5,0:150),
                  sp.layout = list(list("sp.polygons", DE_NUTS1, first=FALSE, col=gray(0.5)),
                                   list("sp.points", DE_RB_2005[,i+14], col=gray(0.25), pch=3, cex=.5)),
                  main=as.character(index(DE_pred_winter[,i,drop=F]@time)))
  
    png(file=paste("vignettes/figures/animate/pred",i,".png", sep=""), width=6, height=6, "in", res=150)
    print(pnt)
    dev.off()
  }
}

DE_pred <- STF(gridDE, DE_RB_2005@time[smplDays])
tIDS <- unique(pmax(1,pmin(as.numeric(outer(-5:5, smplDays, "+")), 365)))

sepPred <- krigeST(PM10~1, data=DE_RB_2005[,tIDS], 
                   newdata=DE_pred, fitSepModel, nmax=50,
                   stAni=fitMetricModel$stAni/24/3600)
psPred <- krigeST(PM10~1, data=DE_RB_2005[,tIDS], 
                  newdata=DE_pred, fitProdSumModel, nmax=50,
                  stAni=fitMetricModel$stAni/24/3600)
metPred <- krigeST(PM10~1, data=DE_RB_2005[,tIDS], 
                   newdata=DE_pred, fitMetricModel, nmax=50,
                   stAni=fitMetricModel$stAni/24/3600)
sumPred <- krigeST(PM10~1, data=DE_RB_2005[,tIDS], 
                   newdata=DE_pred, fitSumMetricModel, nmax=50,
                   stAni=fitMetricModel$stAni/24/3600)
smplSumPred <- krigeST(PM10~1, data=DE_RB_2005[,tIDS], # start: 12:24
                       newdata=DE_pred,  fitSimpleSumMetricModel, nmax=50,
                       stAni=fitMetricModel$stAni/24/3600)

# pure spatial prediction
pureSpPred <- matrix(NA, nrow=length(gridDE), length(smplDays))
col <- 1
for(i in smplDays) { # i <- 1
  pureSpPred[,col] <- krige(PM10~1, as(DE_RB_2005, "STSDF")[,i],
                            gridDE, model=spVgmMod, nmax=50)$var1.pred
  col <- col+1
}

pureSpPred <- STFDF(gridDE, DE_RB_2005@time[smplDays], data.frame(var1.pred = as.numeric(pureSpPred)))

DE_RB_2005 <- as(DE_RB_2005, "STFDF")

if(paper) {
  stpl <- stplot(smplSumPred, col.regions=bpy.colors(120)[-(1:20)], scales=list(draw=F),
                 main=NULL, at=0:70, # "spatio-temporal sum-metric model"
                 sp.layout = list(list("sp.polygons", DE_NUTS1, first=FALSE, col=gray(0.5)),
                                  list("sp.points", DE_RB_2005@sp, col=gray(0.25), pch=3, cex=.5)))
  
  png(file="vignettes/figures/pred_daily_means_PM10.png", width=9, height=6, "in", res=150)
  print(stpl)
  dev.off()
} else {
  stplot(pureSpPred, col.regions=bpy.colors, scales=list(draw=F),
         main="pure spatial daily prediction")
  stplot(sepPred, col.regions=bpy.colors(), scales=list(draw=F),
         main="spatio-temporal separable model")
  stplot(psPred, col.regions=bpy.colors, scales=list(draw=F),
         main="spatio-temporal product-sum model")
  stplot(metPred, col.regions=bpy.colors, scales=list(draw=F),
         main="spatio-temporal metric model")
  stplot(sumPred, col.regions=bpy.colors, scales=list(draw=F),
         main="spatio-temporal sum-metric model")
  stplot(smplSumPred, col.regions=bpy.colors, scales=list(draw=F),
         main="spatio-temporal simple sum-metric  model")
}

library(sp)
library(gstat)

# roughly follows the case presented in:
# E.J. Pebesma and G.B.M. Heuvelink, 1999. Latin hypercube sampling 
# of Gaussian random fields. Technometrics 41 (4), pp. 303-312.

data(meuse)
data(meuse.grid)
coordinates(meuse) = ~x+y
gridded(meuse.grid) = ~x+y

x <- variogram(log(zinc) ~ 1, meuse)
v <- vgm(.5, "Sph", 800, nug = .1)
v.fit = fit.variogram(x, model = v)
plot(x, model = v.fit)

n = 100
ok = krige(log(zinc)~1, meuse, meuse.grid, v.fit, nmax=40)
sim = krige(log(zinc)~1, meuse, meuse.grid, v.fit, nsim = n, nmax=40)
simo = t(apply(as.data.frame(sim)[1:n], 1, order)) # rank order
nr = nrow(simo) # number of prediction locations
simo = (simo - 1.0 + matrix(runif(n * nr), nr, n))/n
summary(simo) # LHS on uniform [0,1] distribution; back to Gaussian:
lhs = t(apply(cbind(ok$var1.pred, sqrt(ok$var1.var), simo), 1,
	function(x) qnorm(x[-(1:2)], x[1], x[2])))
sim2 = sim
sim2@data = data.frame(lhs)
spplot(sim2[1:10], main = 'lhs', col.regions=bpy.colors())
# verify that simulated and true mean/var are close:
m = apply(lhs, 1, mean)
v = apply(lhs, 1, var)
summary(m - ok$var1.pred)
summary(v - ok$var1.var)

library(sp)
library(gstat)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y

# NLKrige: non-linear kriging (e.g. log-normal kriging), simulation based.
# arguments:
# formula, data, newdata, vgm: see ?krige
# trans: transformation and back-transformation function;
# summarize: optional, summarize the simulations, see example below;
# nmax, nsim: see ?krige
# density: number of points to discretize each block PROVIDED BLOCK SIZES
#   ARE CONSTANT (note that newdata can also be SpatialPolygons -- if newdata is
#   a grid, cell size is used as block size)
NLKrige = function(formula, data, newdata, vgm, trans = c(log,exp), summarize,
		nmax = 50, nsim = 100, density = 16, ...) {
	# transform target:
	target = as.character(as.list(formula)[[2]])
	data[[target]] = trans[[1]](data[[target]])
	# conditional simulation sample at finer grid:
	finegrid = spsample(newdata, n = density * length(newdata), 
		type = "regular", offset = c(.5,.5))
	sim = krige(formula, data, finegrid, vgm, nmax = nmax, nsim = nsim)
	# back transform ALL simulations:
	sim@data = trans[[2]](sim@data)
	# spatial aggregation of each simulation, taking block MEAN:
	aggr = aggregate(sim, newdata, mean)
	# aggregation to summary:
	if (!missing(summarize)) {
		ret = apply(aggr@data, 1, summarize, ...)
		if (is.matrix(ret))
			aggr@data = data.frame(t(ret))
		else
			aggr@data = data.frame(ret)
	}
	aggr
}

aggr = NLKrige(zinc~1, meuse, meuse.grid, vgm(.5, "Sph", 900, .1), nmax = 10, 
	summarize = mean)
spplot(aggr, main = "expected value of block means")

aggr = NLKrige(zinc~1, meuse, meuse.grid, vgm(.5, "Sph", 900, .1), nmax = 10, 
	summarize = quantile, probs = c(0.025, 0.975))
spplot(aggr, main = "95% CI for block means")

# $Id: block.R,v 1.5 2006-02-10 19:05:02 edzer Exp $
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y

vgm.fit = fit.variogram(variogram(zinc~1, meuse), vgm(1, "Sph", 800, 1))
bl0 = krige(zinc~1, meuse, meuse.grid, model = vgm.fit, block = c(0,0))
bl1 = krige(zinc~1, meuse, meuse.grid, model = vgm.fit, block = c(40,40))
bl2 = krige(zinc~1, meuse, meuse.grid, model = vgm.fit,block = c(100,100))
bl3 = krige(zinc~1, meuse, meuse.grid, model = vgm.fit,block = c(400,400))
bl0$"block=0x0" =     bl0$var1.pred
bl0$"block=40x40" =   bl1$var1.pred
bl0$"block=100x100" = bl2$var1.pred
bl0$"block=400x400" = bl3$var1.pred

plt1 = spplot(bl0, 3:6, layout=c(4,1), col.regions=bpy.colors(), main = "kriging predictions")

bl0$"block=0x0" =     bl0$var1.var
bl0$"block=40x40" =   bl1$var1.var
bl0$"block=100x100" = bl2$var1.var
bl0$"block=400x400" = bl3$var1.var

plt2 = spplot(bl0, 3:6, layout=c(4,1), col.regions=bpy.colors(), main = "kriging standard errors")

print(plt1, split = c(1, 1, 1, 2), more = T)
print(plt2, split = c(1, 2, 1, 2), more = F)
# block krige the full area:

bl = krige(zinc~1, meuse, newdata = SpatialPoints(data.frame(x=0,y=0)),
        model = vgm.fit, block = coordinates(meuse.grid))
bl
# block kriging standard error:
sqrt(bl$var1.var)
# classical statistical standard error of mean:
sqrt(var(meuse$zinc)/155)

# $Id: sic2004.R,v 1.4 2006-11-22 12:54:16 edzer Exp $
# compared to the original submission, two errors
# were found; one was corrected (dayx~x -> dayx~1), the other not.

# read csv files:
#sic.pred = read.csv("SIC2004_out.csv", header=F)
#names(sic.pred) = c("record", "x", "y")
#input = read.csv("SIC2004_input.csv", header=F)
#names(input) = c("record", "x", "y", "dayx")
#joker = read.csv("SIC2004_joker.csv", header=F)
#names(joker) = c("record", "x", "y", "dayx")

library(sp)
library(gstat)
library(lattice)
data(sic2004) # load directly from R data base
input = sic.test[,1:4]
names(input) = c("record", "x", "y", "dayx")
joker = sic.test[,c(1,2,3,5)]
names(joker) = c("record", "x", "y", "dayx")

do.sic = function(sic.input, sic.output) {
    require(gstat)
    # calculate omnidirectional sample variogram, cutoff 500000, intervals 20000
    sic.input = sic.input[sic.input$record != 838, ]
    v.sample = variogram(dayx~1, ~x+y, input, width=2e4, cutoff=5e5)
    #                                  ^^^^^ the error: this should be sic.input
	#                         ^ this error has been corrected
    # initial spherical model for fit: nug=100, p.sill=400,range=4.5e5
    initial.model = vgm(400, "Sph", 450000, 100)
    # fit the nugget, sill and range to the sample variogram:
    v.fitted = fit.variogram(v.sample, initial.model, fit.method = 2)
    # function returns the output from the call to krige():
    krige(dayx~1, ~x+y, sic.input, sic.output, model = v.fitted, nmax = 125)
}

# do the spatial interpolations:
output = do.sic(input, sic.pred)
joker.output = do.sic(joker, sic.pred)

# write csv files:
#write.table(output, "sic2004_output.csv", 
#    sep=",", col.names=FALSE, row.names=FALSE)
#write.table(joker.output, "sic2004_joker_output.csv",
#    sep=",", col.names=FALSE, row.names=FALSE)

# calculate output:

# read csv files:
sic.stats = function(obs, pred) {
	print("observed:")
	x = obs
	print(c(min=min(x), max=max(x), mean=mean(x), median=median(x), stddev=sqrt(var(x))))
	print("predicted:")
	x = pred
	print(c(min=min(x), max=max(x), mean=mean(x), median=median(x), stddev=sqrt(var(x))))
	print("error:")
	x = pred - obs
	print(c(mae=mean(abs(x)), me=mean(x), corr=cor(pred,obs), rmse=sqrt(mean(x^2))))

}

options(digits=4)
print("input data first")
sic.stats(sic.test[,4], output$var1.pred)
print("joker with error variogram")
sic.stats(sic.test[,4], joker.output$var1.pred)

##################

do.sic = function(sic.input, sic.output) {
    require(gstat)
    v.sample = variogram(dayx~1, ~x+y, input, width=2e4, cutoff=5e5)
    initial.model = vgm(400, "Sph", 450000, 100)
    v.fitted = fit.variogram(v.sample, initial.model, fit.method = 2)
    krige(dayx~1, ~x+y, sic.input, sic.output, model = v.fitted, nmax = 125)
}

# do the spatial interpolations on a grid:
grid1 = do.sic(input, sic.grid)
grid2 = do.sic(joker, sic.grid)

grid1$v2 = grid2$var1.pred
grid1$v2.var = grid2$var1.var

grid1$var1.pred[grid1$var1.pred>200] = 205 # mask larger values!
grid1$v2[grid1$v2>200] = 205               # mask larger values!

greys = c(255, 247, 240, 228, 217, 203, 189, 169, 150, 132, 115, 99, 82, 60, 37, 0)
panel.sic = function(...){
	panel.levelplot(...)
	lpoints(sic.train$x, sic.train$y, pch=22, cex=.5, col=1)
	lpoints(sic.pred$x, sic.pred$y, pch="+", cex=1, col=1)
}
cp1 = contourplot(z~x+y|name, map.to.lev(grid1,z=c(3,5),
    ns=c("data set 1","data set 2")), asp="iso",
    col.regions=rgb(greys/255,greys/255,greys/255),
    at=50+(0:16)*10,region=TRUE,labels=FALSE,
    xlab = "", ylab = "",scales=list(draw=F),panel=panel.sic)
cp2 = contourplot(sqrt(z)~x+y|name, map.to.lev(grid1,z=4,
    ns="standard error"), asp="iso",scales=list(draw=F),
    col.regions=rgb(greys/255,greys/255,greys/255),
    region=TRUE,labels=FALSE, xlab = "", ylab = "", panel=panel.sic)
print(cp1, c(0,0,0.625,1), more=T)
print(cp2, c(0.625,0,1,1), more=F)

wireframe(var1.pred~x+y,grid2,asp=c(diff(range(grid2$y))/diff(range(grid2$x)),0.5), 
    xlab="x",ylab="y",zlab="z",drape=T,
    col.regions=gray(sqrt(seq(from=1.0, to=0.0, length=100))))

library(sp)
library(gstat)
library(geoR)
xyz = data.frame(x = c(0,0,1), y = c(0, 1, 1), z = c(1,2,3))
coordinates(xyz)=~x+y
x0 = SpatialPoints(data.frame(x=0,y=.5))
kr1 = krige(z~1,xyz,x0,vgm(1, "Exp", 1))
kr2 = krige.conv(as.geodata(xyz), locations=coordinates(x0), 
	krige=list(cov.model="exponential", cov.par=c(1,1)))
kr1
c(kr2$predict, kr2$krige.var)
kr1[[1]] - kr2$predict
kr1[[2]] - kr2$krige.var

# $Id: gstat.formula.predict.q,v 1.14 2008-02-19 10:01:22 edzer Exp $

"gstat.formula.predict" <-
function (formula, newdata, na.action, BLUE.estimates = FALSE, xlev = NULL) 
{
	if (is(newdata, "SpatialPolygons")) {
		# locs = coordinates(getSpatialPolygonsLabelPoints(newdata)) -- deprecated, now use:

		locs = t(sapply(slot(newdata, "polygons"), function(x) slot(x, "labpt")))
		SpatialPoints(locs, slot(newdata, "proj4string"))
		locs = coordinates(locs)

		colnames(locs) = c("x", "y")
		if (is(newdata, "SpatialPolygonsDataFrame"))
			newdata = as.data.frame(newdata)
		else
			newdata = data.frame(a = rep(1, nrow(locs)))
	} else if (is(newdata, "SpatialLines")) {
		# locs = coordinates(getSpatialLinesMidPoints(newdata)) -- deprecated, now use:

		ret = lapply(newdata@lines,
	        	function(x) sapply(x@Lines,
				function(X) apply(X@coords, 2, mean)
			)
		)
		ret = t(sapply(ret, function(x) apply(x, 1, mean)))
		locs = coordinates(SpatialPoints(ret, slot(newdata, "proj4string")))
		colnames(locs) = c("x", "y")

		if (is(newdata, "SpatialLinesDataFrame"))
			newdata = as.data.frame(newdata)
		else
			newdata = data.frame(a = rep(1, nrow(locs)))
	} else {
		if (gridded(newdata))
			fullgrid(newdata) = FALSE
		locs = coordinates(newdata)
		newdata = as.data.frame(newdata)
	} 

	# resolve formula:
	terms.f = delete.response(terms(formula))
    mf.f = model.frame(terms.f, newdata, na.action = na.action, xlev = xlev)
    X = model.matrix(terms.f, mf.f)

	if (BLUE.estimates) { # fake the whole thing to get a matrix with BLUE parameter estimates:
		cnames = colnames(X)
		X = matrix(0, ncol(X), ncol(X))
		diag(X) = 1
		locs = locs[1,,drop=FALSE]
		if (ncol(X) > 1) {
			for (i in 2:ncol(X))
				locs = rbind(locs, locs[1,])
		}
		rownames(locs) = cnames
	}

	if (NROW(locs) != NROW(X)) { 
		# NA's were filtered in X, but not in coords:
    	mf.f =    model.frame(terms.f, newdata, na.action = na.pass)
		valid.pattern = !(apply(mf.f, 1, function(x) any(is.na(x))))
		X    = model.matrix(terms.f, mf.f[valid.pattern, , drop = FALSE])
		locs = locs[valid.pattern, ]
		if (NROW(locs) != NROW(X))
			stop("NROW(locs) != NROW(X): this should not occur")
	}
    list(locations = as.matrix(locs), X = as.matrix(X))
}

# $Id: variogram.formula.q,v 1.8 2006-02-10 19:01:07 edzer Exp $

"variogram.formula" <-
function (object, locations = coordinates(data), data, ...) 
{
	if ((missing(locations) && inherits(data, c("sf", "stars"))) || (inherits(locations, c("sf", "stars")))) {
		if (!requireNamespace("sf", quietly = TRUE))
       		stop("sf required: install that first") # nocov
		if (missing(locations))
			data = as(data, "Spatial")
		else
			locations = as(locations, "Spatial")
	}

	# gstat.formula takes care of the case where locations contains
	# both data and coordinates --- see there.
	## ret = gstat.formula(object, locations, data)
	## variogram(object = ret$y, locations = ret$locations, X = ret$X, ...)
	if ((missing(locations) && is(data, "ST")) || (is(locations, "ST")))
		variogramST(formula = object, locations = locations, data = data, ...)
	else {
		g = gstat(formula = object, locations = locations, data = data)
		variogram(g, ...)
	}
}

# $Id: image.data.frame.q,v 1.4 2006-02-10 19:01:07 edzer Exp $

"image.data.frame" <-
function (x, zcol = 3, xcol = 1, ycol = 2, asp = 1, ...)
{
    image.default(xyz2img(xyz = x, zcol = zcol, xcol = xcol, ycol = ycol),
		asp = asp,
		...)
}

# $Id: vgm.q,v 1.12 2007-02-27 22:09:32 edzer Exp $

warn.angle3 = TRUE

"vgm" <-
function(psill = NA, model, range = NA, nugget, add.to, anis, kappa = 0.5,
		..., covtable, Err = 0) {
	add.to.df = function(x, y) {
		x = rbind(x, y)
		row.names(x) = 1:nrow(x)
		return(x)
	}
	if (is.character(psill)) { # as of ``vgm("Sph")''
		if (length(psill) > 1) {
			ret = lapply(psill, function(x) vgm(x))
			class(ret) = c("variogramModelList", "list")
			return(ret)
		} # else:
		if (psill == "Nug")
			return(vgm(NA, "Nug", NA))
		else
			return(vgm(NA, psill, NA, NA))
	}
	stopifnot(length(psill) == 1)
	stopifnot(length(range) == 1)
	stopifnot(missing(nugget) || length(nugget) == 1)
	stopifnot(length(kappa) == 1)
	m = .Call(gstat_get_variogram_models, as.integer(0))
	n = length(m)
	mf = factor(m, levels = m)
	if (missing(model)) {
		ml = .Call(gstat_get_variogram_models, as.integer(1))
		mlf = factor(ml, levels = ml)
		return(data.frame(short = mf, long = mlf))
	}
	if (length(model) > 1) {
		ret = lapply(model, function(x) vgm(,x))
		class(ret) = c("variogramModelList", "list")
		return(ret)
	} 
	table = NULL
	if (model == "Tab" && !missing(covtable)) {
		table = as.matrix(covtable)
		if (NCOL(table) != 2)
			stop("covtable should be a 2-column matrix with distance and cov.")
		range = max(table[,1])
		if (min(table[,1]) != 0.0)
			stop("the first covariance value should be at distance 0.0")
		table = table[,2]
		mf = factor(c(m, "Tab"), levels = c(m, "Tab"))
		if (!missing(add.to) || !missing(nugget) || Err > 0)
			stop("cannot add submodels or nugget to covariance Table model")
	} else if (!any(m == model)) 
		stop(paste("variogram model", model, "unknown\n"))
	if (missing(anis))
		anis = c(0,0,0,1,1)
	if (length(anis) == 2)
		anis = c(anis[1], 0, 0, anis[2], 1)
	else if (length(anis) != 5)
		stop("anis vector should have length 2 (2D) or 5 (3D)")
	if (warn.angle3 && anis[3] != 0.0) {
		warn.angle3 = FALSE
		warning("you are using the third rotation angle; this code is based on the GSLIB2 code\nand must contain the bug described at the end of http://pangea.stanford.edu/ERE/research/scrf/software/gslib/bug/")
	}
	if (!is.na(range)) {
		if (model != "Nug") {
			if (model != "Lin" && model != "Err" && model != "Int")
				if (range <= 0.0) stop("range should be positive")
			else if(range < 0.0) stop("range should be non-negative")
			} else {
			if (range != 0.0) stop("Nugget should have zero range")
			if (anis[4] != 1.0 || anis[5] != 1.0)
				stop("Nugget anisotropy is not meaningful")
		}
	}
	if (!missing(nugget)) {
		ret = data.frame(model=mf[mf==model], psill=psill, range=range,
			kappa = kappa, ang1=anis[1], ang2=anis[2], ang3=anis[3], 
			anis1=anis[4], anis2=anis[5])
		n.vgm = data.frame(model=mf[mf=="Nug"], psill=nugget, range=0,
			kappa = 0.0, ang1=0.0, ang2=0.0, ang3=0.0, anis1=1.0, anis2=1.0)
		ret = add.to.df(n.vgm, ret)
	} else
		ret = data.frame(model=mf[mf==model], psill=psill, range=range,
			kappa = kappa, ang1=anis[1], ang2=anis[2], ang3=anis[3], 
			anis1=anis[4], anis2=anis[5])
	if (!missing(add.to))
		ret = add.to.df(data.frame(add.to), ret)
	if (Err > 0)
		ret = add.to.df(data.frame(model=mf[mf=="Err"], psill=Err,
			range=0.0, kappa = kappa, ang1=anis[1], ang2=anis[2],
			ang3=anis[3], anis1=anis[4], anis2=anis[5]), ret)
	if (!is.null(table))
		attr(ret, "table") = table
	class(ret) = c("variogramModel", "data.frame")
	ret
}

as.vgm.variomodel = function(m) {
	model = NULL
	if (m$cov.model == "exponential")
		model = "Exp"
	else if (m$cov.model == "circular")
		model = "Cir"
	else if (m$cov.model == "gaussian")
		model = "Gau"
	else if (m$cov.model == "linear")
		# model = "Lin"
		stop("no correct conversion available; use power model with power 1?")
	else if (m$cov.model == "matern")
		model = "Mat"
	else if (m$cov.model == "wave")
		model = "Wav"
	else if (m$cov.model == "power")
		model = "Pow"
	else if (m$cov.model == "spherical")
		model = "Sph"
	else if (m$cov.model == "pure.nugget")
		return(vgm(m$nugget + m$cov.pars[1], "Nug", 0))
	else
		stop("variogram model not supported")
# "cauchy",
#,"cubic",
#            "gneiting",
#			"gneiting.matern",
#			"powered.exponential",
#			"wave") ) {
	vgm(m$cov.pars[1], model, m$cov.pars[2], m$nugget, kappa = m$kappa)
}

#$lambda
#[1] 1

#$trend
#[1] "cte"

#$max.dist
#[1] 1441.83

#attr(,"class")
#[1] "variomodel"

plot.variogramModel = function(x, cutoff, ..., type = 'l') {
  if (missing(cutoff))
    stop("parameter cutoff needs to be specified")
  l = variogramLine(x, cutoff, ...)
  xyplot(gamma ~ dist, l, ylim = c(0, 1.04 * max(l$gamma)), xlim = c(0, max(l$dist)),
  	type = type, xlab = "distance", ylab = "semivariance")
}

# test -- load data:
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y

library(gstat)
# test -- idw
meuse.grid$idw <- idw0(zinc~1, meuse, meuse.grid)[,1]
x <- idw(zinc~1, meuse, meuse.grid)
all.equal(x$var1.pred, meuse.grid$idw)
spplot(meuse.grid["idw"],col.regions=bpy.colors())
v = vgm(1, "Exp", 500)
# test sk:
x0 <- krige0(zinc~1, meuse, meuse.grid, v, beta = 500, computeVar = TRUE)
rownames(x0$pred)=NULL
x <- krige(zinc~1, meuse, meuse.grid, v, beta = 500)
all.equal(x$var1.pred, x0$pred[,1])
all.equal(x$var1.var, x0$var)
# test ok:
x0 <- krige0(zinc~1, meuse, meuse.grid, v, computeVar = TRUE)
rownames(x0$pred)=NULL
names(x0$var)=NULL
x <- krige(zinc~1, meuse, meuse.grid, v)
all.equal(x$var1.pred, x0$pred[,1])
all.equal(x$var1.var, x0$var)
# test uk:
x0 <- krige0(zinc~sqrt(dist), meuse, meuse.grid, v, computeVar = TRUE)
rownames(x0$pred)=NULL
names(x0$var)=NULL
x <- krige(zinc~sqrt(dist), meuse, meuse.grid, v)
all.equal(x$var1.pred, x0$pred[,1])
all.equal(x$var1.var, x0$var)

# $Id: krigeTg.q,v 1.4 2009-07-07 15:42:39 edzer Exp $

phiInv <- function (x, lambda)
     if (lambda==0) log(x) else (x^lambda-1)/lambda
  
phi <- function(x, lambda)
     if (lambda==0) exp(x) else (x*lambda+1)^(1/lambda)
  
phiPrime <- function (x, lambda)
     if (lambda==0) exp(x) else (x*lambda+1)^(1/lambda-1)
  
phiDouble <- function (x, lambda)
     if (lambda==0) exp(x) else
        lambda * (1/lambda - 1) * (lambda * x + 1)^(1/lambda-2)

krigeTg <- function(formula, locations, newdata, model = NULL, ..., 
	nmax = Inf, nmin = 0, maxdist = Inf, block = numeric(0), 
	nsim = 0, na.action = na.pass, debug.level = 1,
	lambda = 1.0)
{
    m = model.frame(terms(formula), as.data.frame(locations))
    Y = model.extract(m, "response")
    if (length(Y) == 0)
        stop("no response variable present in formula")
    Terms = attr(m, "terms")
    X = model.matrix(Terms, m)
    has.intercept = attr(Terms, "intercept")
	if (ncol(X) > 1)
		stop("only formula with intercept allowed, e.g. y ~ 1")
	locations$value = phiInv(Y, lambda)
	locations$value1 = rep(1, length(locations$value))

    OK = krige(value ~ 1, locations, newdata, model, 
		nmax = nmax, nmin = nmin, 
		maxdist = maxdist, block = block, nsim = nsim,
		na.action = na.action, debug.level = debug.level, ...)

   if (nsim > 0) {
     OK@data = as.data.frame(phi(OK@data,lambda))
     return(OK)
   } 

	# else:
	# estimate mu:
    g = gstat(formula = value ~ 1, # locations = locations, 
		data = locations, model = model, nmax = nmax, nmin = nmin, 
		maxdist = maxdist, ...)
    mu = predict(g, newdata = newdata, block = block, nsim = nsim,
		na.action = na.action, debug.level = debug.level, BLUE = TRUE)
	OK$muhat = mu$var1.pred

    SK = krige(value1 ~ 1, locations, newdata,
		model = model, beta = 0.0, nmax = nmax, nmin = nmin, 
		maxdist = maxdist, block = block, nsim = nsim,
		na.action = na.action, debug.level = debug.level, ...)
	
	# find m:
	OK$m = (OK$var1.var - SK$var1.var)/(1 - SK$var1.pred)

	# copy SK output:
	OK$var1SK.pred = SK$var1.pred
	OK$var1SK.var = SK$var1.var

	# compute transGaussian kriging estimate & variance:
	OK$var1TG.pred = phi(OK$var1.pred, lambda) + 
		phiDouble(mu$var1.pred, lambda) * (OK$var1.var/2 - OK$m)
	OK$var1TG.var = phiPrime(mu$var1.pred, lambda)^2 * OK$var1.var
	OK
}

spplot.vcov = function(x, ...) {
    basenames = sub(".pred", "", names(x)[grep(".pred",names(x))])
    n = length(basenames)
    names = NULL
    skip = NULL
    for (i in 1:n) {
        skp = rep(TRUE, n)
        pos = 1
        if (i > 1) {
            for (j in 1:(i-1)) {
                names = c(names, paste("cov", basenames[j], basenames[i], sep = "."))
                skp[pos] = FALSE
                pos = pos + 1
            }
        }
        names = c(names, paste(basenames[i], ".var", sep = ""))
        skp[pos] = FALSE
        skip = c(skip, skp)
    }
    spplot(x, names, skip = skip, layout = c(n,n), as.table = TRUE, ...)
}

library(sp)

data(meuse)
data(meuse.grid)

set.seed(13131) # reproduce results

# select 10 random rows;
# create two missing values in the coordinates:
m = meuse.grid[sample(nrow(meuse.grid), 10), ]
m[c(2,8), "x"] = NA

library(gstat)
## this is not allowed anymore:
try(krige(log(zinc)~1,~x+y,meuse,m, na.action = na.pass))
try(krige(log(zinc)~1,~x+y,meuse,m, na.action = na.omit))
try(krige(log(zinc)~1,~x+y,meuse,m, na.action = na.exclude))
try(krige(log(zinc)~1,~x+y,meuse,m, na.action = na.fail))

# select 10 random rows;
# create two missing values in the regressor variable:
m = meuse.grid[sample(nrow(meuse.grid), 10), ]
m[c(3,7), "dist"] = NA
krige(log(zinc)~dist,~x+y,meuse,m, na.action = na.pass)
krige(log(zinc)~dist,~x+y,meuse,m, na.action = na.omit)
krige(log(zinc)~dist,~x+y,meuse,m, na.action = na.exclude)
try(krige(log(zinc)~dist,~x+y,meuse,m, na.action = na.fail))

# Sytze de Bruin's post to r-sig-geo, Nov 16, 2015:
library(sp)
library(gstat)

# some data
x <- c(215, 330, 410, 470, 545)
y <- c(230, 310, 330, 340, 365)
fc <- c(0.211, 0.251, 0.281, 0.262, 0.242)
por <- c(0.438, 0.457, 0.419, 0.430, 0.468)
Allier <- data.frame(x, y, fc, por)
coordinates(Allier) = ~x+y

# gstat object for co-kriging using linear model of co-regionalization
g <- gstat(id=c("fc"), formula=fc~1, data=Allier,
           model=vgm(0.00247, "Sph", 480, 0.00166))
g <- gstat(g, id="por", formula=por~1, data=Allier,
           model=vgm(0.00239, "Sph", 480, 0.00118))
g <- gstat(g, id=c("fc", "por"),
           model=vgm(0.00151, "Sph", 480, -0.00124))

# predict at single point
g$set = list(choleski = 0) # LDL'
A <- predict(g, SpatialPoints(data.frame(x=450, y=350)), debug = 32)
g$set = list(choleski = 1) # Choleski
B <- predict(g, SpatialPoints(data.frame(x=450, y=350)), debug = 32)
all.equal(A,B)

# TRUE

# $Id: plot.pointPairs.q,v 1.4 2006-02-10 19:01:07 edzer Exp $

"plot.pointPairs" <-
function(x, data, xcol = data$x, ycol = data$y, xlab = "x coordinate",
	ylab = "y coordinate", col.line = 2, line.pch = 0, 
	main = "selected point pairs", ...) {

	if (inherits(data, "sf"))
		data = as(data, "Spatial")

	if (is(data, "SpatialPoints")) {
		cc = coordinates(data) 
		xcol = cc[,1]
		ycol = cc[,2]
		xlab = colnames(cc)[1]
		ylab = colnames(cc)[2]
		asp = mapasp(data)
	} else
		asp = "iso"
	xyplot(ycol ~ xcol, aspect = asp,
		panel = panel.pointPairs, xlab = xlab, ylab = ylab, pairs = x,
		col.line = col.line, line.pch = line.pch, main = main, ...)
}

# $Id: ossfim.q,v 1.3 2006-02-10 19:01:07 edzer Exp $

"ossfim" <-
function(spacings = 1:5, block.sizes = 1:5, model, nmax = 25, debug = 0)
{
	n = floor(sqrt(nmax)) + 1
	x = 0:(n-1) + .5
	x = sort(c(-x, x))
	ret = matrix(NA, length(spacings) * length(block.sizes), 3)
	r = 1
	for (sp in spacings) {
		for (bl in block.sizes) {
			data.grid = data.frame(expand.grid(x * sp, x * sp),
				z = rep(1, length(x)^2))
			names(data.grid) = c("x", "y", "z")
			gridded(data.grid) = c("x", "y")
			x0 = SpatialPoints(matrix(0, 1, 2))
			kr = krige(z~1, data.grid, x0,
				block = c(bl, bl), model = model, nmax = nmax,
				set = list(debug = debug))
			ret[r, ] = c(sp, bl, sqrt(kr[["var1.var"]][1]))
			r = r + 1
		}
	}
	ret = data.frame(ret)
	names(ret) = c("spacing", "block.size", "kriging.se")
	ret
}

# $Id: zzz.q,v 1.10 2006-02-10 19:01:07 edzer Exp $

### NAMESPACE VERSION:

.onLoad <- function(lib, pkg) {
	# remove the require() call for 2.0.0:
	# require(lattice)
	# .Call(gstat_init, as.integer(1))
}

### pre-NAMESPACE VERSION:
## ".First.lib" <-
## function(lib, pkg) {
## 	require(lattice)
## 	library.dynam("gstat", pkg, lib)
## 	.Call(gstat_init, as.integer(1))
## }
 
variogram <- function(object, ...) UseMethod("variogram")

library(gstat)
d=expand.grid(x=c(-.5,.5), y=c(-.5,.5))
d$z=1:4
vv=vgm(model = "Tab",  covtable = 
	variogramLine(vgm(1, "Sph", 1), 1, n=1e4,min = 0, covariance = TRUE))
vv
krige(z~1,~x+y,d,data.frame(x=0,y=0),vgm(1, "Sph", 1))
krige(z~1,~x+y,d,data.frame(x=0,y=0),vv)
krige(z~1,~x+y,d[1:2,],data.frame(x=0,y=0),vgm(1, "Sph", 1))
krige(z~1,~x+y,d[1:2,],data.frame(x=0,y=0),vv)

.gstatOptions <- new.env(FALSE, globalenv())

assign("gstat_progress", TRUE, envir = .gstatOptions)

get_gstat_progress <- function() {
    get("gstat_progress", envir = .gstatOptions)
}

set_gstat_progress <- function(value) {
        stopifnot(is.logical(value))
        stopifnot(length(value) == 1)
        assign("gstat_progress", value, envir = .gstatOptions)
        get_gstat_progress()
}

# 
# library(gstat)
# # separable model: spatial and temporal sill will be ignored
# # and kept constant at 1-nugget respectively. A joint sill is used.
# separableModel <- vgmST("separable", 
#                         space=vgm(0.9,"Exp", 147, 0.1),
#                         time =vgm(0.9,"Exp", 3.5, 0.1),
#                         sill=40)
# 
# covSTMat <- ceWrapSpaceTimeOnTorusCalcCovRow1(c(250,20), c(1,16), separableModel)
# dim(covSTMat)
# 
# image(matrix(ceSim(covSTMat, 1, c(20, 16)),
#              nrow=20, ncol=16), asp=1, ylab="time", xlab="space")
# 
# # product sum model: spatial and temporal nugget will be ignored and kept
# # constant at 0. Only a joint nugget is used.
# prodSumModel <- vgmST("productSum",
#                       space=vgm(39, "Sph", 343, 0),
#                       time= vgm(36, "Exp",   3, 0), 
#                       k=15)
# 
# covSTMat <- ceWrapSpaceTimeOnTorusCalcCovRow1(c(250,20), c(1,16), prodSumModel)
# dim(covSTMat)
# 
# image(matrix(ceSim(covSTMat, 1, c(20, 16)),
#              nrow=20, ncol=16), asp=1, ylab="time", xlab="space")
# 
# # sum metric model: spatial, temporal and joint nugget will be estimated
# sumMetricModel <- vgmST("sumMetric",
#                         space=vgm( 6.9, "Lin", 200, 3.0),
#                         time =vgm(10.3, "Lin",  15, 3.6),
#                         joint=vgm(37.2, "Exp",  84,11.7),
#                         stAni=77.7)
# 
# covSTMat <- ceWrapSpaceTimeOnTorusCalcCovRow1(c(250,20), c(1,16), sumMetricModel)
# dim(covSTMat)
# 
# image(matrix(ceSim(covSTMat, 1, c(20, 16)),
#              nrow=20, ncol=16), asp=1, ylab="time", xlab="space")
# 
# # simplified sumMetric model, only a overall nugget is fitted. The spatial, 
# # temporal and jont nuggets are set to 0.
# simpleSumMetricModel <- vgmST("simpleSumMetric",
#                               space=vgm(20,"Lin", 150, 0),
#                               time =vgm(20,"Lin", 10,  0),
#                               joint=vgm(20,"Exp", 150, 0),
#                               nugget=1, stAni=15)
# 
# covSTMat <- ceWrapSpaceTimeOnTorusCalcCovRow1(c(250,20), c(1,16), simpleSumMetricModel)
# dim(covSTMat)
# 
# image(matrix(ceSim(covSTMat, 1, c(20, 16)),
#              nrow=20, ncol=16), asp=1, ylab="time", xlab="space")
# 
# # metric model
# metricModel <- vgmST("metric",
#                      joint=vgm(60, "Exp", 150, 10),
#                      stAni=60)
# 
# covSTMat <- ceWrapSpaceTimeOnTorusCalcCovRow1(c(250,20), c(1,16), metricModel, turningLayers = FALSE)
# dim(covSTMat)
# 
# image(matrix(ceSim(covSTMat, 1, c(20, 16)),
#              nrow=20, ncol=16), asp=1, ylab="time", xlab="space")

## turning bands:
# for cov-functions valid in 3-dim 

# adjust the covariance matrix (Schlahter, Chap. 2, eq 2.25)
# simulate several layers (up-right hyper-planes in the 3D+T-cube) with random directions on a unit hemi-sphere
# orthogonally project target points onto rndm lyrs and average the values for all rndm lyrs

# -> one simulation of a 3d+T Gaussian random field, for 2D use a hyperplane in 3D

# the tunring bands operator (numerical, 3-dimensional case)
# @input:
#  model target covariance function 
#  dist_grid: data.frame wtih columns spacelag and timelag for which the covariances are calculated
#
# @value: data.frame with columns spacelag, timelag and gamma where the latter contains the covariances for the 1-dim spatial and temporal layers

# model <- separableModel
# dist_grid <- as.data.frame(cbind("spacelag" = rep(1:150*1., each=4),
#                                  "timelag" = rep(1:4, 150)))

# CAVE: distgrid must ahve the correct spatial and temporal metrics

tbOperator <- function(model, dist_grid) {
  r <- dist_grid$spacelag
  
  derFun <- function(r) r * variogramSurface(model, dist_grid = data.frame(spacelag=r, timelag=dist_grid$timelag), 
                                             covariance = TRUE)$gamma
  
  cbind(dist_grid, "gamma" = diag(attr(numericDeriv(quote(derFun(r)), "r"), "gradient")))
}

randomDirections <- function(n) {
  u <- runif(n, 0, 2*pi)
  v <- runif(n, 0, 1)
  s <- sqrt(1-v^2)
  
  cbind(s*cos(u), s*sin(u), v)
}

# library(rgl)
# plot3d(randomDirections(100), aspect = c(1,1,0.5))


# nLyrs <- 500
# coordMat <- matrix(c(1,5,5,
#                      1,7,7,
#                      1,5,7,
#                      1,7,5), byrow=T, ncol=3)
# 
# coordMat <- matrix(runif(3*1001,1,5), byrow=T, ncol=3)
# 
# # random directions in 3D
# rndDir <- randomDirections(nLyrs)
# 
# # how much does each direction contribute to the point
# cntrbtn <- rndDir %*% t(coordMat)
# # -> each column corresponds to one location, each row i to the "contribution" of the i-th rndDir
# 
# cl_cntrbtn <- ceiling(cntrbtn)+10 # centre for grid; mind spatial index
# fl_cntrbtn <- floor(cntrbtn) +10
# 
# covRow1 <- ceWrapSpaceTimeOnTorusCalcCovRow1(c(250,20), c(1,16), metricModel, turningLayers = TRUE)
# 
# sTime <- Sys.time()
# simLyrs <- ceSim(covRow1, nLyrs, c(20,16))
# simLyrs <- lapply(1:nLyrs, function(col) matrix(simLyrs[,col], nrow=20, ncol=16))
# 
# lambda <- cl_cntrbtn - cntrbtn -10
# 
# cntrbSngSimLyr <- function(lyrId) {
#   simLyrs[[lyrId]][cl_cntrbtn[lyrId,],] * (1-lambda)[lyrId,] + simLyrs[[lyrId]][fl_cntrbtn[lyrId,],] * lambda[lyrId,]
# }
# 
# # reduce to the one realisation based on the combination of nLyrs turning bands
# simTs <- Reduce('+', lapply(1:nLyrs, cntrbSngSimLyr))/sqrt(nLyrs)
# eTime <- Sys.time()
# 
# eTime - sTime # 0.7 secs/simulation of 100 random points
# 
# image(simTs)
# 
# dim(simTs)
# # 1001 locations at 16 time stamps
# 
# plot(simTs[500,])

# computes the covariance matrixes and weights once, applied to series of
# variables/simulations where each variable/simulation is stored in one column of
# the multiVarMatrix copied from krigeST to avoid repeted calls to krige with
# multiple, identical inversions of the weights matrix

# TODO: add functionality for temporal sandwich-wise processing: i.e. use the 
# +/- nmaxTime time slices to predict one time slice

krigeSTMultiple <- function(formula, from, to, modelList, multiVarMatrix, nmaxTime=Inf) {
  lst = extractFormula(formula, from, to)

  separate <- length(from) > 1 && length(to) > 1 &&
    inherits(from, "STF") && inherits(to, "STF")
  
  X = lst$X
  x0 = lst$x0
  
  V = covfn.ST(from, model = modelList, separate=separate)
  v0 = covfn.ST(from, to, modelList)
  
  if (modelList$stModel == "separable" & separate)
    skwts <- STsolve(V, v0, X) # use Kronecker trick
  else 
    skwts <- CHsolve(V, cbind(v0, X))

  npts = length(to)
  ViX = skwts[,-(1:npts)]
  skwts = skwts[,1:npts]
  
  idPredFun <- function(sim) {
    sim <- matrix(sim, ncol = 1)
    beta = solve(t(X) %*% ViX, t(ViX) %*% sim)
    x0 %*% beta + t(skwts) %*% (sim - X %*% beta)
  }
  
  apply(multiVarMatrix, 2, idPredFun)
}



# unconcitional
## STFDF
# library(sp)
# library(spacetime)
# library(zoo)
# library(xts)
# data(meuse, package = "sp")
# coordinates(meuse) <- ~x+y
# proj4string(meuse) <- CRS("+init=epsg:28992")

# krigeST <- function(formula, data, newdata, modelList, y, beta, nmax=Inf, stAni=NULL,
#                     computeVar = FALSE, fullCovariance = FALSE,
#                     bufferNmax=2, progress=TRUE)

## 
krigeSTSimTB <- function(formula, data, newdata, modelList, nsim, 
                            progress=TRUE, nLyrs=500, tGrid=NULL, sGrid=NULL, ceExt=2,
                            nmax=Inf) {
  stopifnot(zoo::is.regular(newdata@time))
  
  condSim <- TRUE
  if (missing(data)) {
    condSim <- FALSE
    message("[No data provided: performing unconditional simulation.]")
  } else {
    message("[Performing conditional simulation.]")
  }
  
  pb <- txtProgressBar(0,nsim,style=3)
  
  # ST-simulation grid
  if (is.null(tGrid)) {
    tDis <- diff(c(index(newdata@time[1]), newdata@endTime[1]))
    if (!is.null(attr(modelList, "temporal unit"))) {
      units(tDis) <- attr(modelList, "temporal unit")
    } else {
      message("[The spatio-temporal variogram model does not carry a time unit attribute: krigeST cannot check whether the temporal distance metrics coincide.]")
    }
    
    tGrid <- c(as.numeric(tDis), length(newdata@time))
    attr(tGrid, "units") <- c("", units(tDis))
    
    debug_time_unit(units(tDis))
  }
  
  if (is.null(sGrid)) {
    if (gridded(newdata@sp)) {
      # SpatialPixels/SpatialGrid:
      # based on GridTopology: use minimal cellsize of both directions; take enough to cover the diagonal
      sDis <- min(newdata@sp@grid@cellsize)
      sDim <- ceiling(sqrt(sum((newdata@sp@grid@cellsize * newdata@sp@grid@cells.dim)^2))/sDis)
      sGrid <-  c(sDis, sDim)
    } else {
      # treat (as) SpatialPoints:
      # Average area per location --assuming-a-regular-squared-outline-taking-the-sqrt--> length/location --take-later-twice-as-many-->
      bboxExt <- apply(newdata@sp@bbox, 1, diff)
      sDis <- sqrt(prod(bboxExt)/length(newdata@sp))
      sDim <- ceiling(sqrt(sum((bboxExt/sDis)^2)))
      sGrid <- c(sDis, sDim)
    }
  }
    
  # random directions in 3D
  rndDir <- randomDirections(nLyrs)
  
  # coordinates 
  coordMat <- coordinates(newdata@sp)

  # coordinates (embedded in 3D) shifted + scaled to the grid index of the spatial gridding sGrid
  coordMat[,1] <- (coordMat[,1] - newdata@sp@bbox[1,1])/sGrid[1]
  coordMat[,2] <- (coordMat[,2] - newdata@sp@bbox[2,1])/sGrid[1]
  
  if (ncol(coordMat) == 2) {
    coordMat <- cbind(coordMat,1)
  } else {
    coordMat[,3] <- (coordMat[,3] - newdata@sp@bbox[3,1])/sGrid[1]
  }

  # how much does each direction contribute to the point
  cntrbtn <- rndDir %*% t(coordMat)
  # -> each column corresponds to one location, each row i to the "contribution" of the i-th rndDir
  
  # ceiling and floor + shifted to avoid negative indices
  cl_cntrbtn <- ceiling(cntrbtn) + sGrid[2]
  fl_cntrbtn <- floor(cntrbtn) + sGrid[2]
  
  covRow1 <- ceWrapSpaceTimeOnTorusCalcCovRow1(c(sGrid[1], 2*sGrid[2]), tGrid, modelList, turningLayers = TRUE, ext=ceExt)
  
  origDim <- c(2*sGrid[2], tGrid[2])
  sims <- list()

  ##
  for (i in 1:nsim) {
    setTxtProgressBar(pb, i)
    simLyrs <- ceSim(covRow1, nLyrs, origDim)
    simLyrs <- lapply(1:nLyrs, function(col) matrix(simLyrs[,col], 
                                                    nrow=origDim[1], 
                                                    ncol=origDim[2]))
    lambda <- cl_cntrbtn - cntrbtn - sGrid[2]
    
    cntrbSngSimLyr <- function(lyrId) {
      simLyrs[[lyrId]][cl_cntrbtn[lyrId,],] * (1-lambda)[lyrId,] + simLyrs[[lyrId]][fl_cntrbtn[lyrId,],] * lambda[lyrId,]
    }
    
    # reduce to the one realisation based on the combination of nLyrs turning bands
    sims[[paste0("sim",i)]] <- Reduce('+', lapply(1:nLyrs, cntrbSngSimLyr))/sqrt(nLyrs)
  }
  close(pb)
  
  sims <- do.call(cbind, lapply(sims, as.numeric))

  # bind simulations to newdata geometry
  if (!condSim) {
    if ("data" %in% slotNames(newdata))
      newdata@data <- cbind(newdata@data, sims)
    else
      newdata <- addAttrToGeom(newdata, as.data.frame(sims))
    return(newdata)
  }
  
  # function call ends here if no data has been provided -> unconditional case
  varName <- all.vars(formula[[2]])
  
  ## conditioning
  # interpolate the observations to the simulation grid
  obsMeanField <- krigeST(formula=formula, data=data, newdata=newdata, modelList=modelList)
  
  # interpolate to observation locations from the simulated grids for each simulation
  simMeanObsLoc <- krigeSTMultiple(as.formula(paste0("var1.pred ~", formula[[3]])),
                                   obsMeanField, data, modelList, sims)
  
  # interpolate from kriged mean sim at observed locations back to the grid for mean surface of the simulations
  simMeanFields <- krigeSTMultiple(as.formula(paste0(varName, "~", formula[[3]])),
                                 data, newdata, modelList, simMeanObsLoc)
  
  # add up the mean field and the corrected data
  sims <- obsMeanField@data$var1.pred + sims - simMeanFields
  
  # bind simulations to newdata geometry
  if ("data" %in% slotNames(newdata)) {
    newdata@data <- cbind(newdata@data, sims)
    return(newdata)
  }
  
  addAttrToGeom(newdata, as.data.frame(sims))
}


# 
# sTime <- Sys.time()
# krigedSim <- krigeSTUncSimTB(stf, metricModel, 100)
# Sys.time() - sTime
# 
# # 27 secs for 100 simulated ST fields of 155 locations and 21 time steps: 325500 values
# 
# # plot one simulation along time
# stplot(krigedSim[,1:12])
# 
# # plot one simulation along time
# stplot(krigedSim[1:12,,"sim1"], mode="ts")
# 
# # plot the ten simulations of the first day
# spplot(krigedSim[,1], paste0("sim",1:10), as.table=TRUE)

# $Id: get.contr.q,v 1.8 2006-03-20 15:18:14 edzer Exp $

"get.contr" <-
function (data, gstat.object, X, ids = names(gstat.object$data)) 
{
	contr.fun <- function(x, n, pr.idx, cov.idx, contr) {
		y = matrix(x[pr.idx], n, 1)
		V = matrix(x[cov.idx], n, n)
		beta = t(contr) %*% y
		Vbeta = t(contr) %*% V %*% contr
		ret = c(beta, diag(Vbeta))
		for (j in 1:nrow(Vbeta)) {
	       	if (j > 1)
       			for (k in 1:(j - 1))
        			ret = c(ret, Vbeta[j, k])
		}
		ret
	}
    lti <- function(i, j) { # lower triangular matrix index, when repr as array
        mx = max(i, j) - 1
        mn = min(i, j) - 1
        ((mx) * (mx - 1))/2 + mn + 1
    }
    n = length(ids)
    if (!is.matrix(X)) 
        X = as.matrix(X)
    if (n != nrow(X)) 
        stop("length(ids) should equal nrow(X) or length(X)")
    gstat.names = create.gstat.names(ids)
    names.pr = gstat.names[seq(1, 2 * n, 2)]
    names.cov = matrix("", n, n)
    for (i in 1:n) 
		for (j in 1:n) 
			names.cov[i, j] = ifelse(i == j, gstat.names[2 * i], 
				gstat.names[2 * n + lti(i, j)])
	pr.idx = match(names.pr, names(data))
	cov.idx = match(names.cov, names(data))
	if (any(is.na(pr.idx)) || any(is.na(cov.idx)))
		stop("colunn names in data not matched")

	res = data.frame(t(apply(as.data.frame(data)[names(data)], 1, 
		contr.fun, n = n, pr.idx = pr.idx, cov.idx = cov.idx,
		contr = X)))

	col.names = NULL
    for (j in 1:NCOL(X))
    	col.names = c(col.names, paste("beta", j, sep = "."))
    for (j in 1:NCOL(X))
    	col.names = c(col.names, paste("var.beta", j, sep = "."))
	for (j in 1:NCOL(X)) {
		if (j > 1) {
			for (k in 1:(j - 1)) {
				col.names = c(col.names, paste("cov.beta", 
					k, j, sep = "."))
			}
		}
	}
    names(res) = col.names
    if (is(data, "data.frame"))
		row.names(res) = row.names(data)
	else if (is(data, "SpatialPolygonsDataFrame")) {
		rownames(res) = sapply(data@polygons, function(x) slot(x, "ID"))
		res = SpatialPolygonsDataFrame(as(data, "SpatialPolygons"), res,
				match.ID = TRUE)
	} else if (is(data, "Spatial")) {
		coordinates(res) = coordinates(data)
		gridded(res) = gridded(data)
	}
    res
}

# $Id: plot.gstatVariogram.q,v 1.15 2007-06-08 18:03:25 edzer Exp $

"plot.gstatVariogram" <-
function (x, model = NULL, ylim, xlim, xlab = "distance", 
	ylab = attr(x, "what"), 
	panel = vgm.panel.xyplot, multipanel = TRUE, plot.numbers = FALSE, 
	scales = list(), ids = x$id, group.id = TRUE, skip, layout, ...) 
{
    if (missing(ylim)) {
        ylim = c(min(0, 1.04 * min(x$gamma)), 1.04 * max(x$gamma))
		ylim.set = FALSE
	} else
		ylim.set = TRUE
    if (missing(xlim)) 
        xlim = c(0, 1.04 * max(x$dist))
    labels = NULL
	shift = 0.03
	if (is.numeric(plot.numbers)) {
		shift = plot.numbers
		plot.numbers = TRUE
	} 
    if (plot.numbers == TRUE) 
       	labels = as.character(x$np)
    if (length(unique(x$dir.ver)) > 1 || any(x$dir.ver != 0))
		warning("vertical directions are not dealt with -- yet!")
    if (length(unique(x$dir.hor)) > 1 && group.id == TRUE) { 
	# directional, grouped:
        if (multipanel) {
            if (length(levels(ids)) > 1) { # multivariate directional:
				xyplot(gamma ~ dist | as.factor(dir.hor), data = x, 
					type = c("p", "l"), xlim = xlim, ylim = ylim, xlab = xlab, 
					ylab = ylab, groups = ids, scales = scales, ...)
			} else # univariate directional, multipanel:
				xyplot(gamma ~ dist | as.factor(dir.hor), subscripts = TRUE, 
                	panel = panel, data = x, xlim = xlim, 
                	ylim = ylim, xlab = xlab, ylab = ylab, direction = x$dir.hor, 
                	labels = labels, model = model, shift = shift, 
					mode = "directional", scales = scales, ...)
        } else { # univariate directional, using symbol/color to distinguish
            pch = as.integer(as.factor(x$dir.hor))
            xyplot(gamma ~ dist, data = x, type = c("p", "l"), 
                groups = pch, xlim = xlim, ylim = ylim, xlab = xlab, 
                ylab = ylab, pch = pch, scales = scales, ...)
        }
    } else if (length(unique(ids)) > 1) { # multivariable:
        n = floor(sqrt(2 * length(unique(ids))))
		if (missing(skip)) {
        	skip = NULL
        	for (row in n:1) 
				for (col in 1:n) 
					skip = c(skip, row < col)
		}
		if (missing(layout))
			layout = c(n,n)
        if (missing(scales)) 
            scales = list(y = list(relation = "free"))
		else
			if (!is.null(scales$relation) && scales$relation == "same")
				ylim.set = TRUE
    	if (length(unique(x$dir.hor)) > 1) { # multiv.; directional groups
			if (ylim.set) {
            	xyplot(gamma ~ dist | id, data = x, type = c("p", 
                	"l"), xlim = xlim, ylim = ylim, xlab = xlab, 
                	ylab = ylab, groups = as.factor(x$dir.hor), layout = layout,
                  	skip = skip, scales = scales, ...)
			} else {
            	xyplot(gamma ~ dist | id, data = x, type = c("p", 
                	"l"), xlim = xlim, xlab = xlab, 
                	ylab = ylab, groups = as.factor(x$dir.hor), layout = layout,
                  	skip = skip, scales = scales, ...)
			}
		} else { # non-multi-directional, multivariable
			if (ylim.set) {
        		xyplot(gamma ~ dist | id, data = x, xlim = xlim, 
            		ylim = ylim, xlab = xlab, ylab = ylab, ids = ids, 
            		panel= panel, labels = labels, scales = scales, 
            		layout = layout, skip = skip, prepanel = function(x, y) 
					list(ylim = c(min(0, y), max(0, y))), model = model, 
					direction = c(x$dir.hor[1], x$dir.ver[1]), shift = shift, 
					mode = "cross", ...)
			} else {
        		xyplot(gamma ~ dist | id, data = x, xlim = xlim, 
            		xlab = xlab, ylab = ylab, ids = ids, 
            		panel = panel, labels = labels, scales = scales, 
            		layout = layout, skip = skip, prepanel = function(x, 
                		y) list(ylim = c(min(0, y), max(0, y))), 
					model = model, direction = c(x$dir.hor[1], x$dir.ver[1]), 
					shift = shift, mode = "cross", ...)
			}
		}
    } else  # non multi-directional, univariable -- mostly used of all:
		xyplot(gamma ~ dist, data = x, panel = panel, xlim = xlim, 
			ylim = ylim, xlab = xlab, ylab = ylab, labels = labels, model = model, 
			direction = c(x$dir.hor[1], x$dir.ver[1]), shift = shift, 
			mode = "direct", scales = scales, ...)
}

"plot.variogramMap" <-
function(x, np = FALSE, skip, threshold, ...) {
	x = x$map
	if (!is(x, "SpatialPixelsDataFrame"))
		stop("x should be of class, or extend, SpatialPixelsDataFrame")
	if (np)
		start = 2 
	else
		start = 1 
	idx = seq(start, ncol(x@data), by=2)
	n = floor(sqrt(length(idx) * 2))
    if (missing(skip)) {
        skip = NULL
        for (row in n:1)
            for (col in 1:n)
                skip = c(skip, row < col)
    }
	if (!(missing(threshold)))
		x = x[x[[2]] >= threshold, ]

	levelplot(values ~ dx + dy | ind, as.data.frame(stack(x, select = idx)),
		asp = mapasp(x), layout = c(n, n), skip = skip, ...)
}

# $Id: print.variogramCloud.q,v 1.4 2007-10-18 10:13:13 edzer Exp $

as.data.frame.variogramCloud = function(x, row.names, optional, ...) {
	.BigInt = attr(x, ".BigInt")
	x$left = x$np %% .BigInt + 1
	x$right = x$np %/% .BigInt + 1
	x$np = NULL
	class(x) = "data.frame"
	x
}

print.variogramCloud <- function (x, ...) {
    print(as.data.frame(x), ...)
}

# $Id: vgm.panel.q,v 1.7 2007-06-08 06:45:52 edzer Exp $

"get.direction.unitv" <- function(alpha, beta) {
	cb = cos(beta)
	c(cb * sin(alpha), cb * cos(alpha), sin(beta))
}

"vgm.panel.xyplot" <-
function (x, y, subscripts, type = "p", pch = plot.symbol$pch, 
    col, col.line = plot.line$col, col.symbol = plot.symbol$col, 
    lty = plot.line$lty, cex = plot.symbol$cex, ids, lwd = plot.line$lwd, 
    model = model, direction = direction, labels, shift = shift, mode = mode, ...) 
{
    x <- as.numeric(x)
    y <- as.numeric(y)
    if (length(x) > 0) {
        if (!missing(col)) {
            if (missing(col.line)) 
                col.line <- col
            if (missing(col.symbol)) 
                col.symbol <- col
        }
        plot.symbol <- trellis.par.get("plot.symbol")
        plot.line <- trellis.par.get("plot.line")
        lpoints(x = x, y = y, cex = cex, col = col.symbol, pch = pch, type = type, ...)
        if (!is.null(labels)) 
            ltext(x = x + shift * max(x), y = y, labels = labels[subscripts])

		if (mode == "direct") {
        	if (!missing(model) && !is.null(model)) {
            	ang.hor <- pi * (direction[1]/180)
				ang.ver <- pi * (direction[2]/180)
            	dir <- get.direction.unitv(ang.hor, ang.ver)
            	ret <- variogramLine(model, max(x), dir = dir)
            	llines(x = ret$dist, y = ret$gamma, lty = lty, col = col.line, lwd = lwd)
        	}
		} else if (mode == "cross") {
    		id <- as.character(ids[subscripts][1])
        	if (!missing(model) && !is.null(model)) {
				if (inherits(model, "gstat"))
					m = model$model
				else
					m = model
				if (!is.list(m))
					stop("model argument not of class gstat or list")
				if (is.list(m) && !is.null(m[[id]])) {
                	ang.hor <- pi * (direction[1]/180)
					ang.ver <- pi * (direction[2]/180)
                	dir <- get.direction.unitv(ang.hor, ang.ver)
					ret <- variogramLine(m[[id]], max(x), dir = dir)
					llines(x = ret$dist, y = ret$gamma, lty = lty, col = col.line, lwd = lwd)
				}
			}
        } else if (mode == "directional") {
        	if (!missing(model) && !is.null(model)) {
            	dir <- c(1, 0, 0)
            	if (!missing(direction)) {
                	ang.hor <- pi * (direction[subscripts][1]/180.0)
                	dir <- get.direction.unitv(ang.hor, 0)
            	}
            	ret <- variogramLine(model, max(x), dir = dir)
            	llines(x = ret$dist, y = ret$gamma, lty = lty, col = col.line, lwd = lwd)
        	}
		}

    }
}

Sys.setenv(TZ = "UTC")

Sys.unsetenv("KMP_DEVICE_THREAD_LIMIT")
Sys.unsetenv("KMP_ALL_THREADS")
Sys.unsetenv("KMP_TEAMS_THREAD_LIMIT")
Sys.unsetenv("OMP_THREAD_LIMIT")

# 0. using sp:

suppressPackageStartupMessages(library(sp))
demo(meuse, ask = FALSE)
suppressPackageStartupMessages(library(gstat))
v = variogram(log(zinc)~1, meuse)
(v.fit = fit.variogram(v, vgm(1, "Sph", 900, 1)))
k_sp = krige(log(zinc)~1, meuse[-(1:5),], meuse[1:5,], v.fit)
k_sp_grd = krige(log(zinc)~1, meuse, meuse.grid, v.fit)

# 1. using sf:
suppressPackageStartupMessages(library(sf))
demo(meuse_sf, ask = FALSE, echo = FALSE)
# reloads meuse as data.frame, so
demo(meuse, ask = FALSE)

v = variogram(log(zinc)~1, meuse_sf)
(v.fit = fit.variogram(v, vgm(1, "Sph", 900, 1)))
k_sf = krige(log(zinc)~1, meuse_sf[-(1:5),], meuse_sf[1:5,], v.fit)

all.equal(k_sp, as(k_sf, "Spatial"), check.attributes = FALSE)
all.equal(k_sp, as(k_sf, "Spatial"), check.attributes = TRUE)

# 2. using stars for grid:

suppressPackageStartupMessages(library(stars))
st = st_as_stars(meuse.grid)
st_crs(st)

# compare inputs:
sp = as(st, "Spatial")
fullgrid(meuse.grid) = TRUE
all.equal(sp, meuse.grid["dist"], check.attributes = FALSE)
all.equal(sp, meuse.grid["dist"], check.attributes = TRUE, use.names = FALSE)

# kriging:
st_crs(st) = st_crs(meuse_sf) = NA # GDAL roundtrip messes them up!
k_st = if (Sys.getenv("USER") == "travis") {
	try(krige(log(zinc)~1, meuse_sf, st, v.fit))
} else {
	krige(log(zinc)~1, meuse_sf, st, v.fit)
}
k_st

# handle factors, when going to stars?
k_sp_grd$cls = cut(k_sp_grd$var1.pred, c(0, 5, 6, 7, 8, 9))
st_as_stars(k_sp_grd)
if (require(raster, quietly = TRUE)) {
 print(st_as_stars(raster::stack(k_sp_grd))) # check
 print(all.equal(st_redimension(st_as_stars(k_sp_grd)), st_as_stars(raster::stack(k_sp_grd)), check.attributes=FALSE))
}

suppressPackageStartupMessages(library(spacetime))

tm = as.POSIXct("2019-02-25 15:37:24 CET")
n = 4
s = stars:::st_stars(list(foo = array(1:(n^3), rep(n,3))),
stars:::create_dimensions(list(
  x = stars:::create_dimension(from = 1, to = n, offset = 10, delta = 0.5),
  y = stars:::create_dimension(from = 1, to = n, offset = 0, delta = -0.7),
  time = stars:::create_dimension(values = tm + 1:n)),
  raster = stars:::get_raster(dimensions = c("x", "y")))
  )
s

as.data.frame(s)
plot(s, col = sf.colors(), axes = TRUE)
(s.stfdf = as(s, "STFDF"))
stplot(s.stfdf, scales = list(draw = TRUE))

(s2 = st_as_stars(s.stfdf))
plot(s2, col = sf.colors(), axes = TRUE)
all.equal(s, s2, check.attributes = FALSE)

# multiple simulations:
data(meuse, package = "sp")
data(meuse.grid, package = "sp")
coordinates(meuse.grid) <- ~x+y
gridded(meuse.grid) <- TRUE
meuse.grid = st_as_stars(meuse.grid)
meuse_sf = st_as_sf(meuse, coords = c("x", "y"))
g = gstat(NULL, "zinc", zinc~1, meuse_sf, model = vgm(1, "Exp", 300), nmax = 10)
g = gstat(g, "lead", lead~1, meuse_sf, model = vgm(1, "Exp", 300), nmax = 10, fill.cross = TRUE)
set.seed(123)
## IGNORE_RDIFF_BEGIN
(p = predict(g, meuse.grid, nsim = 5))
## IGNORE_RDIFF_END

# $Id: print.variogramModel.q,v 1.5 2009-02-20 13:53:38 edzer Exp $

"print.variogramModel" =
function (x, ...) 
{
    df = data.frame(x)
	shape.models = c("Mat", "Exc", "Cau", "Ste")
	if (!any(match(df[, "model"], shape.models, nomatch=0)))
		df$kappa = NULL
    if (!any(df[, "anis2"] != 1))  {
		df$anis2 = NULL
		df$ang2 = NULL
		df$ang3 = NULL
		if (!any(df[, "anis1"] != 1))  {
			df$anis1 = NULL
			df$ang1 = NULL
		}
	} 
	if (any(match(df[, "model"], "Tab", nomatch=0))) {
		df$maxdist = df$range
		df$range = NULL
    	print(df, ...)
		cat("covariance table:\n")
		tab = attr(x, "table")
		idx = round(seq(1, length(tab), length=6))
		print(tab[idx])
	} else
    	print(df, ...)
	invisible(x)
}

#############################
## spatio-temporal kriging ##
#############################

debug_time_unit = function(tUnit) {
    # message("Using the following time unit: ", tUnit)
}

STsolve = function(A, b, X) {
  # V = A$T %x% A$S -- a separable covariance; solve A x = b for x
  # kronecker: T %x% S vec(L) = vec(c0) <-->  S L T = c0
  # solve for L: use Y = L T 
  # S Y = c0 -> Y = solve(S, c0)
  # L T = Y -> Tt Lt = Yt -> Lt = solve(Tt, Yt)
  #Tm = chol(A$Tm, LINPACK=TRUE)
  Tm = chol(A$Tm)
  #Sm = chol(A$Sm, LINPACK=TRUE)
  Sm = chol(A$Sm)
  STbacksolve = function(Tm, Cm, Sm) { 
    MyChSolve = function(A, b)
      backsolve(A, forwardsolve(A, b, upper.tri = TRUE, transpose = TRUE))
    # Y = MyChSolve(Sm, Cm)
    # L = MyChSolve(Tm, t(Y))
    # as.vector(t(L))
    as.vector(t(MyChSolve(Tm, t(MyChSolve(Sm, Cm)))))
  }
  # b comes separated:
  ret1 = apply(b$T, 2, function(x1) 
    apply(b$S, 2, function(x2)
      STbacksolve(Tm, matrix(x1 %x% x2, nrow(Sm), nrow(Tm)), Sm)))
  d = dim(ret1)
  dim(ret1) = c(d[1] / ncol(b$S), d[2] * ncol(b$S))
  # X comes full:
  ret2 = apply(X, 2, function(x) 
    STbacksolve(Tm, matrix(x, nrow(Sm), nrow(Tm)), Sm))
  cbind(ret1, ret2)
}

covfn.ST = function(x, y = x, model, ...) {
  switch(strsplit(model$stModel, "_")[[1]][1],
         separable=covSeparable(x, y, model, ...),
         productSumOld=covProdSumOld(x, y, model),
         productSum=covProdSum(x, y, model),
         sumMetric=covSumMetric(x, y, model),
         simpleSumMetric=covSimpleSumMetric(x, y, model),
         metric=covMetric(x, y, model),
         stop(paste("Provided spatio-temporal model (",model$stModel,") is not supported.",sep="")))
}


## krigeST
krigeST <- function(formula, data, newdata, modelList, beta, y, ...,
                    nmax=Inf, stAni=NULL,
                    computeVar = FALSE, fullCovariance = FALSE,
                    bufferNmax=2, progress=TRUE) {
  stopifnot(inherits(modelList, "StVariogramModel") || is.function(modelList))
  to_sftime = FALSE
  return_stars = if (inherits(data, c("stars"))) {
    if (!requireNamespace("sf", quietly = TRUE))
      stop("sf required: install that first") # nocov
    if (!requireNamespace("stars", quietly = TRUE))
      stop("stars required: install that first") # nocov
    if (sf::st_crs(data) != sf::st_crs(newdata))
      warning("CRS for data and newdata are not identical; assign CRS or use st_transform to correct")
    data = as(data, "STFDF")
    newdata = as(newdata, "STFDF")
    TRUE
  } else if (inherits(data, "sftime")) {
    if (sf::st_crs(data) != sf::st_crs(newdata))
      warning("CRS for data and newdata are not identical; assign CRS or use st_transform to correct")
    data = as(data, "STIDF")
    if (inherits(newdata, "stars")) {
		if (length(newdata) == 0)
			newdata$._dummy = 0.
		newdata = as(newdata, "STFDF")
	}
	if (inherits(newdata, "sftime")) {
  		to_sftime = TRUE
		newdata = as(newdata, "STIDF")
	}
    TRUE
  } else {
    if (!identical(data@sp@proj4string@projargs, newdata@sp@proj4string@projargs))
  	  message("please verify that the CRSs of data and newdata are identical, or transform them first to make them identical")
    FALSE
  }
  stopifnot(inherits(data, c("STF", "STS", "STI", "sftime")) && 
			inherits(newdata, c("STF", "STS", "STI", "sftime"))) 
  if (inherits(data, "sftime"))
    data = as(data, "STIDF")
  if (inherits(newdata, "sftime"))
    data = as(data, "STI")
  stopifnot(class(data@time) == class(newdata@time))
  stopifnot(nmax > 0)
  
  tUnitModel <- attr(modelList, "temporal unit")
  tUnitData <- units(abs(outer(index(data@time[1]), index(newdata@time[1]), "-")))
  
  if (is.null(tUnitModel)) {
    warning("The spatio-temporal variogram model does not carry the strongly recommended attribute 'temporal unit'.\n The unit '", tUnitData,
            "' has been assumed. krigeST could not check whether the temporal distances between locations and in the variogram coincide.")
    tUnit <- tUnitData
    attr(modelList, "temporal unit") <- tUnit
  } else {
    tUnit <- tUnitModel
    debug_time_unit(tUnit)
  }
  
  if(nmax < Inf) { # local neighbourhood ST kriging:
    ret = krigeST.local( formula = formula, data = data, 
                         newdata = newdata, modelList = modelList, beta=beta, # y=y, # for later use
                         nmax = nmax, stAni = stAni, 
                         computeVar = computeVar, fullCovariance = fullCovariance, 
                         bufferNmax = bufferNmax, progress = progress)

    if (return_stars) {
	  ret$._dummy = NULL
  	  if (to_sftime)
		  sftime::st_as_sftime(ret)
	  else 
		  stars::st_as_stars(as(ret, "STFDF"))
	} else
      ret
  } else {
    df <- krigeST.df(formula = formula, data = data, newdata = newdata, 
                   modelList = modelList, beta = beta, y = y, 
                   ..., 
                   nmax=nmax, stAni=stAni,
                   computeVar = computeVar, fullCovariance = fullCovariance,
                   bufferNmax = bufferNmax, progress = progress)
  
    # wrapping the predictions in ST*DF again
    if (!fullCovariance) {
      ret = addAttrToGeom(geometry(newdata), df)
      if (return_stars) {
	    ret$._dummy = NULL
  	  	if (to_sftime)
		  ret = sftime::st_as_sftime(ret)
  		else 
		  ret = stars::st_as_stars(as(ret, "STFDF"))
	  }
      ret
    } else
      df
  }
}

krigeST.df <- function(formula, data, newdata, modelList, beta, y, ...,
                       nmax = Inf, stAni = NULL,
                       computeVar = FALSE, fullCovariance = FALSE,
                       bufferNmax = 2, progress = TRUE) {
  
  separate <- length(data) > 1 && length(newdata) > 1 && inherits(data, "STF") && inherits(newdata, "STF")
  
  lst = extractFormula(formula, data, newdata)
  X = lst$X
  x0 = lst$x0
  if (missing(y))
    y = lst$y
  
  if (inherits(modelList, "StVariogramModel")) {
    V = covfn.ST(data, model = modelList, separate=separate)
    v0 = covfn.ST(data, newdata, modelList)
  
    if (is(data,"STSDF"))
      d0 <- data[data@index[1,1], data@index[1,2], drop = FALSE]
    else
      d0 = data[1, 1, drop=FALSE]
    
    c0 = as.numeric(covfn.ST(d0, d0, modelList, separate = FALSE))
  } else {
    V = modelList(data, data, ...)
    v0 = modelList(data, newdata, ...)
    
    if (computeVar) {
      if (is(newdata@sp, "SpatialLines") || is(newdata@sp, "SpatialPolygons"))
        stop("Varying target support (SpatialLines, SpatialPolygons) for kriging variance is not implemented.")
      c0 = as.numeric(modelList(newdata[1, drop=FALSE],
                                newdata[1, drop=FALSE]))
    }
  }
  
  if (!missing(beta)) { # sk:
    skwts = CHsolve(V, v0)
    npts = length(newdata)
    ViX = skwts[,-(1:npts)]
    skwts = skwts[,1:npts]
    if (computeVar)
      var <- c0 - apply(v0*skwts, 2, sum)
  } else {
    if (!is.function(modelList) && (modelList$stModel == "separable" & separate))
      skwts <- STsolve(V, v0, X) # use Kronecker trick
    else 
      skwts <- CHsolve(V, cbind(v0, X))
    npts = length(newdata)
    ViX = skwts[,-(1:npts)]
    skwts = skwts[,1:npts]
    beta = solve(t(X) %*% ViX, t(ViX) %*% y)
    if (computeVar) {
      # get (x0-X'C-1 c0)'(X'C-1X)-1 (x0-X'C-1 c0) -- precompute term 1+3:
      if(is.list(v0)) # in the separable case
        v0 = v0$Tm %x% v0$Sm
      Q = t(x0) - t(ViX) %*% v0
      # suggested by Marius Appel
      var = c0 - apply(v0 * skwts, 2, sum) + apply(Q * CHsolve(t(X) %*% ViX, Q), 2, sum)
      if (fullCovariance) {
        corMat <- cov2cor(covfn.ST(newdata, newdata, modelList))
        var <- corMat*matrix(sqrt(var) %x% sqrt(var), nrow(corMat), ncol(corMat))
        # var = c0 - t(v0) %*% skwts + t(Q) %*% CHsolve(t(X) %*% ViX, Q)
        # return(list(pred=pred, var=var))
      }
    }
  }
  
  pred = x0 %*% beta + t(skwts) %*% (y - X %*% beta)
  
  if(computeVar) {
    if (fullCovariance)
      list(pred=pred, var=var)
    else
      data.frame(var1.pred = pred, var1.var = var)
  } else
    data.frame(var1.pred = pred)
}

# local spatio-temporal kriging
krigeST.local <- function(formula, data, newdata, modelList, beta, nmax, stAni=NULL,
                          computeVar=FALSE, fullCovariance=FALSE, 
                          bufferNmax=2, progress=TRUE) {
  dimGeom <- ncol(coordinates(data))
  if (fullCovariance)
    stop("fullCovariance cannot be returned for local ST kriging")
  
  if(is.null(stAni) & !is.null(modelList$stAni)) {
    stAni <- modelList$stAni
    
    # scale stAni [spatial/temporal] to seconds
    if(!is.null(attr(modelList,"temporal unit")))
      stAni <- stAni/switch(attr(modelList, "temporal unit"),
                            secs=1,
                            mins=60,
                            hours=3600,
                            days=86400,
                            stop("Temporal unit",attr(modelList, "temporal unit"),"not implemented."))
  }
  
  if(is.null(stAni))
    stop("The spatio-temporal model does not provide a spatio-temporal 
         anisotropy scaling nor is the parameter stAni provided. One of 
         these is necessary for local spatio-temporal kriging.")
  
  # check whether the model meets the coordinates' unit
  if(!is.null(attr(modelList, "spatial unit")))
    stopifnot((is.projected(data) & (attr(modelList, "spatial unit") %in% c("km","m"))) | (!is.projected(data) & !(attr(modelList, "spatial unit") %in% c("km","m"))))
  
  if (inherits(data, c("STFDF", "STSDF", "sftime")))
    data <- as(data, "STIDF")
  
  clnd <- class(newdata)
  
  if(inherits(newdata, c("STFDF", "STSDF", "sftime")))
    newdata <- as(newdata, "STIDF")
  if(inherits(newdata, c("STF", "STS")))
    newdata <- as(newdata, "STI")
  
  # from here on every data set is assumed to be STI*
  
  if(dimGeom == 2) {
    df = as(data, "data.frame")[,c(1,2,4)]
    df$time = as.numeric(df$time)*stAni
    
    query = as(newdata, "data.frame")[,c(1,2,4)]
    query$time = as.numeric(query$time)*stAni
  } else {
    df = as(data, "data.frame")[,c(1,2,3,5)]
    df$time = as.numeric(df$time)*stAni
    
    query = as(newdata, "data.frame")[,c(1,2,3,5)]
    query$time = as.numeric(query$time)*stAni
  }
  
  df <- as.matrix(df)
  query <- as.matrix(query)
  
  if (computeVar) {
    res <- data.frame(var1.pred = rep(NA, nrow(query)),
                      var1.var = rep(NA, nrow(query)))
  } else {
    res <- data.frame(var1.pred = rep(NA, nrow(query)))
  }
  
  if(progress)
    pb = txtProgressBar(style = 3, max = nrow(query))
  
  nb <- t(apply(get.knnx(df, query, ceiling(bufferNmax*nmax))[[1]],1,sort))
  
  for (i in 1:nrow(query)) {
    nghbrData <- data[nb[i, ], , drop = FALSE]
    
    if(bufferNmax > 1) {
      nghbrCov <- covfn.ST(nghbrData, newdata[i, , drop = FALSE], modelList)
      nghbrData <- nghbrData[sort(order(nghbrCov, decreasing=T)[1:nmax]), , drop = FALSE]
    }
    
    res[i,] <- krigeST.df(formula, nghbrData, newdata[i, , drop = FALSE],
                          modelList, computeVar=computeVar, beta = beta,
                          fullCovariance=fullCovariance)
    if(progress)
      setTxtProgressBar(pb, i)  
  }
  if(progress)
    close(pb)
  
  if (clnd %in% c("STI", "STS", "STF")) {
    newdata <- addAttrToGeom(newdata, as.data.frame(res))
    newdata <- switch(clnd, 
                      STI = as(newdata, "STIDF"),
                      STS = as(newdata, "STSDF"),
                      STF = as(newdata, "STFDF"))
  } else {
    newdata@data <- cbind(newdata@data, res)
    newdata <- as(newdata, clnd)
  }
  newdata
}

## Area to point kriging

# define a spatio-temporal variogram model FUNCTION that can deal with x and y
# being of ST* with @sp of class SpatialPolygons OR SpatialPoints;
# SpatialGrid/Pixels are coerced to SpatialPolygons -- as in the so case, see
# krige0.R

# sts <- STS(meuse.grid, stf@time, cbind(sort(sample(length(meuse.grid), 500, replace = T)),
#                                        sample(21,500, replace=TRUE)))
# 
# x <- sts
# y <- stf
# model <- separableModel

vgmAreaST = function(x, y = x, model, ndiscrSpace = 16, verbose = FALSE, covariance = TRUE) {
  if (gridded(x@sp))
    x@sp = as(x@sp, "SpatialPolygons")
  if (gridded(y@sp))
    y@sp = as(y@sp, "SpatialPolygons")
  stopifnot(is(x@sp, "SpatialPolygons") || is(x@sp, "SpatialPoints"))
  stopifnot(is(y@sp, "SpatialPolygons") || is(y@sp, "SpatialPoints"))
  stopifnot(is(model, "StVariogramModel"))
  
  # make x and y both of type STS/STSDF
  if ("data" %in% slotNames(x)) {
    x <- as(x, "STSDF") 
  } else {
    x <- as(x, "STS") 
  }

  if ("data" %in% slotNames(y)) {
    y <- as(y, "STSDF") 
  } else {
    y <- as(y, "STS") 
  }
  
  nx = length(x)
  ny = length(y)
  
  V <- matrix(NA, nx, ny)
  
  # switch cases for polygons in x and y 
  if (is(x@sp, "SpatialPolygons")) {
    # x contains polygons -> loop and sample
    
    indexX <- x@index
    if (verbose)
      pb <- txtProgressBar(style = 3, max = nx)
    for (ix in 1:nx) { # ix <- 1
      px <- x[indexX[ix,,drop=F],drop=F]
      ptsx = STF(spsample(px@sp, ndiscrSpace, "regular", offset = c(.5,.5)), 
                 px@time, px@endTime)
      
      # does y also contain polygons?
      if (is(y@sp, "SpatialPolygons")) {
        # yes, also y contains polygons -> loop and sample
        
        indexY <- y@index
        for (iy in 1:ny) { # iy <- 1
          py <- y[indexY[iy,,drop=F],drop=F]
          ptsy = STF(spsample(py@sp, ndiscrSpace, "regular", offset = c(.5,.5)), 
                     py@time, py@endTime)
          
          suppressMessages(subV <- covfn.ST(ptsx, ptsy, model, separate=FALSE))
          
          V[ix, iy] <- mean(subV)
        }
      } else {
        # no, y contains points -> no second loop, calc covariance
        suppressMessages(subV <- covfn.ST(ptsx, y, model, separate=FALSE))
        V[ix, ] <- apply(subV, 2, mean)
      }
      
      if (verbose)
        setTxtProgressBar(pb, ix)
    }
    if (verbose)
      close(pb)
  } else {
    # none of x and y contain polygons -> no loops, calc covariance
    suppressMessages(V <- covfn.ST(x, y, model))
  }

  V
}

####################
## trans Gaussian ##
####################

krigeSTTg <- function(formula, data, newdata, modelList, y, nmax=Inf, stAni=NULL,
                      bufferNmax=2, progress=TRUE, lambda = 0) {
  if(!is.infinite(nmax))
    return(krigeSTTg.local(formula, data, newdata, modelList, y, nmax, stAni,
                           bufferNmax, progress, lambda))
  
  lst <- extractFormula(formula, data, newdata)
  
  Y <- lst$y
  X <- lst$X
  
  if (ncol(X) > 1)
    stop("only formula with intercept allowed, e.g. y ~ 1")
  
  data$value = phiInv(Y, lambda)
  data$value1 = rep(1, length(data$value))
  
  OK = krigeST(value ~ 1, data, newdata, modelList, 
               nmax = nmax, stAni = stAni, 
               computeVar=TRUE, bufferNmax = bufferNmax, progress = progress)
  
  separate <- length(data) > 1 && length(newdata) > 1 && 
    inherits(data, "STF") && inherits(newdata, "STF")
  
  V <- covfn.ST(data, model = modelList, separate=separate)
  Vi <- solve(V)
  
  muhat <- sum(Vi %*% data$value)/sum(Vi)
  
  # find m:
  v0 <- covfn.ST(data, newdata, model = modelList, separate=separate)
  m <- (1 - apply(Vi %*% v0,2,sum))/sum(Vi)
  
  # compute transGaussian kriging estimate & variance:
  OK$var1TG.pred = phi(OK$var1.pred, lambda) + 
    phiDouble(muhat, lambda) * (OK$var1.var/2 - m)
  OK$var1TG.var = phiPrime(muhat, lambda)^2 * OK$var1.var
  OK
}



krigeSTTg.local <- function(formula, data, newdata, modelList, y, nmax=Inf, stAni=NULL,
                            bufferNmax=2, progress=TRUE, lambda = 0) {
  stopifnot(!is.infinite(nmax))
  dimGeom <- ncol(coordinates(data))
  
  if(is.null(stAni) & !is.null(modelList$stAni)) {
    stAni <- modelList$stAni
    
    # scale stAni [spatial/temporal] to seconds
    if(!is.null(attr(modelList,"temporal unit")))
      stAni <- stAni/switch(attr(modelList, "temporal unit"),
                            secs=1,
                            mins=60,
                            hours=3600,
                            days=86400,
                            stop("Temporal unit",attr(modelList, "temporal unit"),"not implemented."))
  }
  
  if(is.null(stAni))
    stop("The spatio-temporal model does not provide a spatio-temporal 
         anisotropy scaling nor is the parameter stAni provided. One of 
         these is necessary for local spatio-temporal kriging.")
  
  # check whether the model meets the coordinates' unit
  if(!is.null(attr(modelList, "spatial unit")))
    stopifnot((is.projected(data) & (attr(modelList, "spatial unit") %in% c("km","m"))) | (!is.projected(data) & !(attr(modelList, "spatial unit") %in% c("km","m"))))
  
  if(is(data, "STFDF") || is(data, "STSDF"))
    data <- as(data, "STIDF")
  
  clnd <- class(newdata)
  
  if(is(newdata, "STFDF") || is(newdata, "STSDF"))
    newdata <- as(newdata, "STIDF")
  if(is(newdata, "STF") || is(newdata, "STS"))
    newdata <- as(newdata, "STI")
  
  # from here on every data set is assumed to be STI*
  
  if(dimGeom == 2) {
    df = as(data, "data.frame")[,c(1,2,4)]
    df$time = as.numeric(df$time)*stAni
    
    query = as(newdata, "data.frame")[,c(1,2,4)]
    query$time = as.numeric(query$time)*stAni
  } else {
    df = as(data, "data.frame")[,c(1,2,3,5)]
    df$time = as.numeric(df$time)*stAni
    
    query = as(newdata, "data.frame")[,c(1,2,3,5)]
    query$time = as.numeric(query$time)*stAni
  }
  
  df <- as.matrix(df)
  query <- as.matrix(query)
  
  res <- data.frame(var1.pred = rep(NA, nrow(query)),
                    var1.var = rep(NA, nrow(query)),
                    var1TG.pred = rep(NA, nrow(query)),
                    var1TG.var = rep(NA, nrow(query)))
  
  if(progress)
    pb = txtProgressBar(style = 3, max = nrow(query))
  
  nb <- t(apply(get.knnx(df, query, ceiling(bufferNmax*nmax))[[1]],1,sort))
  
  for (i in 1:nrow(query)) {
    nghbrData <- data[nb[i, ], , drop = FALSE]
    
    if(bufferNmax > 1) {
      nghbrCov <- covfn.ST(nghbrData, newdata[i, , drop = FALSE], modelList)
      nghbrData <- nghbrData[sort(order(nghbrCov, decreasing=T)[1:nmax]), , drop = FALSE]
    }
    
    res[i,] <- krigeSTTg(formula, nghbrData, newdata[i, , drop = FALSE],
                         modelList, lambda)@data
    if(progress)
      setTxtProgressBar(pb, i)  
  }
  if(progress)
    close(pb)
  
  if (clnd %in% c("STI", "STS", "STF")) {
    newdata <- addAttrToGeom(newdata, as.data.frame(res))
    newdata <- switch(clnd, 
                      STI = as(newdata, "STIDF"),
                      STS = as(newdata, "STSDF"),
                      STF = as(newdata, "STFDF"))
  } else {
    newdata@data <- cbind(newdata@data, res)
    newdata <- as(newdata, clnd)
  }
  newdata
}

# $Id: fit.lmc.q,v 1.8 2009-10-06 07:52:00 edzer Exp $

"fit.lmc" <-
function (v, g, model, fit.ranges = FALSE, fit.lmc = !fit.ranges, 
    correct.diagonal = 1.0, ...) 
{
    posdef = function(X) {
        q = eigen(X)
        d = q$values
        d[d < 0] = 0
        q$vectors %*% diag(d, nrow = length(d)) %*% t(q$vectors)
    }
    if (!inherits(v, "gstatVariogram"))
        stop("v should be of class gstatVariogram")
    if (!inherits(g, "gstat"))
        stop("g should be of class gstat")
    if (!missing(model)) {
        if (!inherits(model, "variogramModel"))
            stop("model should be of class variogramModel")
    }
    n = names(g$data)
    for (i in 1:length(n)) {
        for (j in i:length(n)) {
            name = ifelse(i == j, n[i], cross.name(n[i], n[j]))
            x = v[v$id == name, ]
            if (nrow(x) == 0) 
                stop(paste("gstatVariogram", name, "not present"))
            m = g$model[[name]]
            if (!missing(model)) 
                m = model
            g$model[[name]] = fit.variogram(x, m, fit.ranges = fit.ranges, 
                ...)
        }
    }
    if (fit.lmc) {
        m = g$model[[n[1]]]
        for (k in 1:nrow(m)) {
            psill = matrix(NA, nrow = length(n), ncol = length(n))
            for (i in 1:length(n)) {
                for (j in i:length(n)) {
                  name = ifelse(i == j, n[i], cross.name(n[i], n[j]))
                  psill[i, j] = psill[j, i] = g$model[[name]][k, 
                    "psill"]
                }
            }
            psill = posdef(psill)
			diag(psill) = diag(psill) * correct.diagonal 
            for (i in 1:length(n)) {
                for (j in i:length(n)) {
                  name = ifelse(i == j, n[i], cross.name(n[i], n[j]))
                  g$model[[name]][k, "psill"] = psill[i, j]
                }
            }
        }
    }
    g
}

hscat = function(formula, data, breaks, pch = 3, cex = .6, mirror = FALSE, 
		variogram.alpha = 0, as.table = TRUE, ...) {
	stopifnot(!missing(breaks))
	x = variogram(formula, data, cloud = TRUE, cutoff = max(breaks), 
		alpha = variogram.alpha, ...)
	x = as.data.frame(x)
	x$class = cut(x$dist, breaks = breaks)
	y = model.frame(formula, data)[[1]]
	x$xx = y[x$left]
	x$yy = y[x$right]
	if (mirror) 
		x = data.frame(
			xx = c(x$yy, y[x$left]),
			yy = c(x$xx, y[x$right]),
			class = c(x$class, x$class))
	lab = as.character(formula)[2]
	panel = function(x,y,subscripts, ...) {
		xr = c(min(x),max(x))
		llines(xr, xr)
		lpoints(x,y,...)
		ltext(min(x), max(y), paste("r =", signif(cor(x,y),3)), adj=c(0,0.5))
	}
	xyplot(xx~yy|class, x, panel = panel,
		main = "lagged scatterplots", xlab = lab, ylab = lab, 
		as.table = as.table, ...)
}

# $Id: fit.variogram.reml.q,v 1.14 2009-11-02 21:33:17 edzer Exp $

"fit.variogram.reml" <-
function (formula, locations, data, model, debug.level = 1, set, degree = 0)
{
    if (missing(formula)) 
        stop("nothing to fit to")
    if (!inherits(formula, "formula"))
		stop("first argument should be of class formula")
	if (!missing(locations)) {
    	if (inherits(locations, "formula"))
			coordinates(data) = locations
		else if (is(locations, "Spatial"))
			data = as(locations, "SpatialPointsDataFrame")
	}
    if (!is(data, "SpatialPointsDataFrame"))
        stop("data should (now) be of class SpatialPointsDataFrame")
    if (missing(model)) 
        stop("no model to fit")
    if (!inherits(model, "variogramModel"))
        stop("model should be of class variogramModel (use vgm)")
    fit.sills = rep(TRUE, length(model$model))
    fit.ranges = rep(FALSE, length(model$model))
    .Call(gstat_init, as.integer(debug.level))
    ret = gstat.formula(formula, data)
    ret$y <- residuals(lm(formula, data))
    .Call(gstat_new_data, as.double(ret$y), as.double(ret$locations),
		as.double(ret$X), as.integer(1), double(0), as.integer(-1), 
		as.integer(0), as.double(-1), as.integer(0), as.integer(1), 
		double(0), double(0), as.integer(degree),
		as.integer(is.projected(data)), as.integer(0),
		as.double(1.0), as.integer(0))
    load.variogram.model(model)
    if (!missing(set))
    	gstat.load.set(set)
    ret = .Call(gstat_fit_variogram, as.integer(5), 
        as.integer(fit.sills), as.integer(fit.ranges))
    .Call(gstat_exit, 0) 
    model$psill = ret[[1]]
    model$range = ret[[2]]
    model
}

# $Id: print.gstat.q,v 1.7 2006-02-10 19:01:07 edzer Exp $

"print.gstat" <-
function (x, ...) 
{
    if (missing(x) || !inherits(x, "gstat"))
        stop("wrong call")
    data.names <- names(x$data)
    if (length(data.names)) 
        cat("data:\n")
    for (n in data.names) {
        fstr = paste(x$data[[n]]$formula[c(2, 1, 3)], collapse = "")
        #lstr = paste(x$data[[n]]$locations[c(1, 2)], collapse = "")
        cat(n, ": formula =", fstr, ";")
        if (!is.null(x$data[[n]]$data)) {
            data.dim = dim(x$data[[n]]$data)
            cat(" data dim =", data.dim[1], "x", data.dim[2])
        }
        else {
            if (x$data[[n]]$dummy) 
                cat(" dummy data")
            else cat(" NULL data")
        }
		if (x$data[[n]]$nmax != Inf)
			cat(" nmax =", x$data[[n]]$nmax)
		if (x$data[[n]]$nmin > 0)
			cat(" nmin =", x$data[[n]]$nmin)
		if (x$data[[n]]$maxdist < Inf)
			cat(" radius =", x$data[[n]]$maxdist)
		if (x$data[[n]]$vfn > 1)
			cat(" variance function =", 
				c("identity", "mu", "mu(1-mu)", "mu^2", "mu^3")[x$data[[n]]$vfn])
		if (length(x$data[[n]]$beta) > 0)
			cat(" beta =", x$data[[n]]$beta)
		if (x$data[[n]]$degree > 0)
			cat(" degree =", x$data[[n]]$degree)
        cat("\n")
    }
    xx.names = xx = NULL
    for (n in data.names) {
        m = x$model[[n]]
        if (!is.null(m)) {
            xx = rbind(xx, m)
            if (nrow(m) == 1) 
                xx.names = c(xx.names, n)
            else xx.names = c(xx.names, paste(n, "[", 1:nrow(m), 
                "]", sep = ""))
        }
    }
    if (length(data.names) > 1) {
        for (j in 2:length(data.names)) {
            for (i in 1:(j - 1)) {
                n = cross.name(data.names[i], data.names[j])
                m = x$model[[n]]
                if (!is.null(m)) {
                  xx = rbind(xx, m)
                  if (nrow(m) == 1) 
                    xx.names = c(xx.names, n)
                  else xx.names = c(xx.names, paste(n, "[", 1:nrow(m), 
                    "]", sep = ""))
                }
            }
        }
    }
    if (!is.null(xx)) {
        cat("variograms:\n")
        row.names(xx) = xx.names
        print(xx, ...)
    }
    if (!is.null(x$set)) {
        s = gstat.set(x$set)
        for (i in 1:length(s)) cat(s[i], "\n")
    }
    if (!is.null(x$locations))
		print(x$locations)
    invisible(x)
}

fit.variogram.gls <-
function(formula, data, model, maxiter = 30, 
		eps = .01, trace = TRUE, ignoreInitial = TRUE, cutoff = Inf,
		plot = FALSE) {
	v = as.data.frame(variogram(formula, data, cloud = TRUE, cutoff = cutoff))
	i = v$left
	j = v$right
	y = v$gamma
	h0 = v$dist
	dists = spDists(data)
	n = length(i)
	iter = 0
	converged = FALSE
	if (model$model[1] == 'Nug') {
		if (ignoreInitial)
			init = c(mean(y)/2, mean(y)/2, median(h0)/4)
		else
			init = c(model$psill, model$range[2])
		gamfn0 = function(h, th, m = as.character(model$model[2]))
	    	variogramLine(vgm(th[2], m, th[3], th[1]), dist_vector=h)$gamma
		minfuncols0 = function(theta=rbind(1,1,1)) {
			res = y-gamfn0(h0,theta)
			sum(res^2)
		}
	
		# If variogram is Power model, maximum range should be 2. 
		# Also if ignoreInitial change initial value of range to 1
		if (any(model$model == 'Pow')) { 
	  		upperOptim <- c(max(y), max(y), 2)
	  		if (ignoreInitial)
				init[3] <- 1
		} else 
			upperOptim = c(max(y), max(y), max(h0)) 
	
		th = th0 = th.ols = optim(init, minfuncols0, gr=NULL, method="L-BFGS-B",
			lower=c(0,1e-9,1e-9), upper=upperOptim)$par
		if (trace)
			print(th)
		while (!converged && iter < maxiter) {
			comb = function(i,j) cbind(rep(i,length(j)), rep(j,each=length(i)))
			cov = matrix(
				variogramLine(model, dist_vector = dists[comb(i,i)])$gamma
				+ variogramLine(model, dist_vector = dists[comb(j,j)])$gamma
				- variogramLine(model, dist_vector = dists[comb(i,j)])$gamma
				- variogramLine(model, dist_vector = dists[comb(j,i)])$gamma,
				length(j), length(j))
			cov = 0.5 * cov ^ 2
			#cov = solve(cov)
			cov = qr(cov)
			minfuncrange = function(range) {
				res = y-gamfn0(h0,c(th[1],th[2],range))
				t(res) %*% solve(cov, res)
			}
			minfuncsill = function(sill) {
				res = y-gamfn0(h0, c(th[1],sill,th[3]))
				t(res) %*% solve(cov, res)
			}
			minfuncnugget = function(nugget) {
				res = y - gamfn0(h0, c(nugget, th[2], th[3]))
				t(res) %*% solve(cov, res)
			}
			th0 = th
			# Avoid calculating max every time
			th[1] = optimize(minfuncnugget,lower=0,upper=upperOptim[1])$minimum
			th[2] = optimize(minfuncsill,lower=1e-9,upper=upperOptim[2])$minimum
			th[3] = optimize(minfuncrange,lower=1e-9,upper=upperOptim[3])$minimum
			converged = sum(abs((th - th0)/th0)) < eps
			iter = iter + 1
			if (trace)
				print(th)
			model$psill = c(th[1], th[2])
			model$range[2] = th[3]
		}
		if (th[3] / max(h0) > .99)
			warning("range parameter at search space boundary")
		if (!converged) {
			warning("no convergence, returning OLS solution")
			th = th.ols
			model$psill = c(th[1], th[2])
			model$range[2] = th[3]
		}
	} else {
	  # No Nugget component. Use only psill = th[1] and range = th[2] parameters
	  if (ignoreInitial)
	    init = c(mean(y)/2, median(h0)/4)
	  else
	    init = c(model$psill, model$range)
	  gamfn = function(h, th, m = as.character(model$model))
	    variogramLine(vgm(psill = th[1], model = m, range = th[2]), dist_vector=h)$gamma
	  minfuncols = function(theta=rbind(1,1)) {
	    res = y-gamfn(h0,theta)
	    sum(res^2)
	  }
	  
	  # If variogram is Power model, maximum range should be 2. 
	  # Also if ignoreInitial change initial value of range to 1
	  if (any(model$model == 'Pow')) { 
	    upperOptim <- c(max(y), 2)
	    if (ignoreInitial) init[2] <- 1
	  } else { upperOptim = c(max(y), max(h0)) }
	  
	  th = th0 = th.ols = optim(init, minfuncols,gr=NULL, method="L-BFGS-B",
	                            lower=c(1e-9,1e-9), upper=upperOptim)$par
	  if (trace)
	    print(th)
	  while (!converged && iter < maxiter) {
	    comb = function(i,j) cbind(rep(i,length(j)), rep(j,each=length(i)))
	    cov = matrix(
	      variogramLine(model, dist_vector = dists[comb(i,i)])$gamma
	      + variogramLine(model, dist_vector = dists[comb(j,j)])$gamma
	      - variogramLine(model, dist_vector = dists[comb(i,j)])$gamma
	      - variogramLine(model, dist_vector = dists[comb(j,i)])$gamma,
	      length(j), length(j))
	    cov = 0.5 * cov ^ 2
	    #cov = solve(cov)
	    cov = qr(cov)
	    minfuncrange = function(range) {
	      res = y-gamfn(h0,c(th[1],range))
	      t(res) %*% solve(cov, res)
	    }
	    minfuncsill = function(sill) {
	      res = y-gamfn(h0, c(sill,th[2]))
	      t(res) %*% solve(cov, res)
	    }
	    th0 = th
	    th[1] = optimize(minfuncsill,lower=1e-9,upper=upperOptim[1])$minimum
	    th[2] = optimize(minfuncrange,lower=1e-9,upper=upperOptim[2])$minimum
	    converged = sum(abs((th - th0)/th0)) < eps
	    iter = iter + 1
	    if (trace)
	      print(th)
	    model$psill = th[1]
	    model$range = th[2]
	  }
	  if (th[2] / max(h0) > .99)
	    warning("range parameter at search space boundary")
	  if (!converged) {
	    warning("no convergence, returning OLS solution")
	    th = th.ols
	    model$psill = th[1]
	    model$range = th[2]
	  }
	}
	
	if (plot)
		plot(variogram(formula, data, cloud = TRUE, cutoff = cutoff), 
			model = model)
	else
		model
}

library(sp)
data(meuse)
coordinates(meuse) = c("x", "y")
new.locs <- SpatialPoints(data.frame(
	x = c(181170, 180310, 180205, 178673, 178770, 178270),
	y = c(333250, 332189, 331707, 330066, 330675, 331075)))
library(gstat)
krige(zinc ~ 1,  meuse, new.locs, vgm(1.34e5, "Sph", 800, nug = 2.42e4), 
		block = c(40,40), nmax = 40)

new.locs <- SpatialPoints(data.frame(x = c(181170), y = c(333250)))

disc = c(-15,-5,5,15)
block.irreg <- data.frame(expand.grid(x = disc, y = disc))
block.irreg

# first disable default Gaussian quadrature used for block integration, by
# setting nblockdiscr explicitly:
krige(zinc ~ 1, meuse, new.locs, model = vgm(1.34e5, "Sph", 800, nug = 2.42e4), 
		block = c(40,40), nmax = 40, set = list(nblockdiscr=4))
# now pass the same block discretization as block.irreg:
krige(zinc ~ 1, meuse, new.locs, vgm(1.34e5, "Sph", 800, nug = 2.42e4), 
		block = block.irreg, nmax = 40)
# check weights argument:
block.irreg <- data.frame(expand.grid(x = disc, y = disc), weights = rep(1/16, 16))
krige(zinc ~ 1, meuse, new.locs, vgm(1.34e5, "Sph", 800, nug = 2.42e4), 
		block = block.irreg, nmax = 40)

# $Id: panel.pointPairs.q,v 1.3 2008-03-10 10:00:10 edzer Exp $

"panel.pointPairs" <-
function (x, y, type = "p", pch = plot.symbol$pch, col, col.line = 
	plot.line$col, col.symbol = plot.symbol$col, lty = plot.line$lty, 
	cex = plot.symbol$cex, lwd = plot.line$lwd, pairs = pairs, 
	line.pch = line.pch, ...) 
{
    x = as.numeric(x)
    y = as.numeric(y)
    if (length(x) > 0) {
        if (!missing(col)) {
            if (missing(col.line)) 
                col.line = col
            if (missing(col.symbol)) 
                col.symbol = col
        }
        plot.symbol = trellis.par.get("plot.symbol")
        plot.line = trellis.par.get("plot.line")
        lpoints(x = x, y = y, cex = cex, col = col.symbol, pch = pch, ...)
        if (!missing(pairs)) {
			for (i in seq(along.with = pairs[,1])) {
				xx = c(x[pairs[i,1]], x[pairs[i,2]])
				yy = c(y[pairs[i,1]], y[pairs[i,2]])
            	llines(x = xx, y = yy, lty = lty, col = col.line, lwd = lwd)
				if (line.pch > 0)
					lpoints(mean(xx), mean(yy), pch = line.pch, col = col.line)
			}
        }
    }
}

VgmFillNA <- function(x, boundaries) {
  # pads the sample variogram with NA rows where no data are available.
	n = length(boundaries) - 1
	ix = rep(NA, n)
	#ix[which(1:n %in% findInterval(x$dist, boundaries))] = 1:nrow(x)
	ix[ findInterval(x$dist, boundaries, rightmost.closed = TRUE) ] = 1:nrow(x)
	# x$b = boundaries[-1]
	x[ix,]
}

VgmAverage = function(ret, boundaries) {
	# take out NULL variograms:
	ret = ret[!sapply(ret, is.null)]
	# take care of missing rows...
	ret = lapply(ret, VgmFillNA, 
			boundaries = c(0, 1e-6 * boundaries[2], boundaries[-1]))
	# weighted average:
  # sum np, weighted sum of gamma and dist devided by summed np
	np = apply(do.call(cbind, lapply(ret, function(x) x$np)), 1, sum,
	           na.rm = TRUE)
	gamma = apply(do.call(cbind, lapply(ret, function(x) x$gamma*x$np)), 1, sum,
		na.rm = TRUE)/np
	dist = apply(do.call(cbind, lapply(ret, function(x) x$dist*x$np)), 1, sum,
		na.rm = TRUE)/np
	v = data.frame(np = np, dist = dist, gamma = gamma)
	class(v) = class(ret[[1]])
	attr(v, "boundaries") = attr(ret[[1]], "boundaries")
	v[is.na(v)] = NA
	v
}

StVgmLag = function(formula, data, dt, pseudo, ...) {
  dotLst <- list(...)
	.ValidObs = function(formula, data)
		!is.na(data[[as.character(as.list(formula)[[2]])]])
	d = dim(data)
	ret = vector("list", d[2] - dt)
	if (dt == 0) {
		for (i in 1:d[2]) {
			d0 = data[,i]
			valid = .ValidObs(formula, d0)
			if(sum(valid) <= 1)
				ret[[i]] <- NULL
			else {
				d0 = d0[valid,]
				ret[[i]] = variogram(formula, d0, ...)
			}
		}
	} else {
		for (i in 1:(d[2] - dt)) {
			d1 = data[, i]
			valid1 = .ValidObs(formula, d1)
			d2 = data[, i + dt]
			valid2 = .ValidObs(formula, d2)
			if(sum(valid1)==0 || sum(valid2)==0)
				ret[[i]] <- NULL
			else {
				d1 = d1[valid1,]
				d2 = d2[valid2,]
				obj = gstat(NULL, paste("D", i, sep=""), formula, d1, 
					set = list(zero_dist = 3), beta = 0)
				obj = gstat(obj, paste("D", i+dt, sep=""), formula, d2, 
					beta = 0)
				ret[[i]] = variogram(obj, cross = "ONLY", pseudo = pseudo, ...)
			}
		}
	}
  if(!is.null(dotLst$cloud)) {
    if(dotLst$cloud)
      ret <- do.call("rbind", ret)
      ret$id <- "var1"
      return(ret)
  } else {
	   return(VgmAverage(ret, dotLst$boundaries))
  }
}

variogramST = function(formula, locations, data, ..., tlags = 0:15, cutoff, 
                       width = cutoff/15, boundaries=seq(0,cutoff,width),
                       progress = interactive(), pseudo = TRUE, 
                       assumeRegular=FALSE, na.omit=FALSE, cores = 1) {
	if (missing(data))
		data = locations

	if (inherits(data, "stars")) {
		if (!requireNamespace("stars", quietly = TRUE))
			stop("stars required: install that first") # nocov
		data = as(data, "STFDF")
	}

	if(missing(cutoff)) {
		ll = !is.na(is.projected(data@sp)) && !is.projected(data@sp)
		cutoff <- spDists(t(data@sp@bbox), longlat = ll)[1,2]/3
	}
  
	if (formula[[3]] != 1) { # there is a regression model:
		data$resid = residuals(lm(formula, data, na.action = na.exclude))
		formula = resid ~ 1
	}
	if(inherits(data, c("STIDF", "sftime")))
		return(variogramST.STIDF(formula, as(data, "STIDF"), tlags, cutoff, width, 
                             boundaries, progress, cores = cores, ...))
  
	stopifnot(is(data, "STFDF") || is(data, "STSDF"))
	it = index(data@time)
	if (assumeRegular || is.regular(zoo(matrix(1:length(it)), order.by = it),
                                  strict = TRUE)) {
		twidth = diff(it)[1]
		tlags = tlags[tlags <= min(max(tlags), length(unique(it)) - 1)]
	} else {
		warning("strictly irregular time steps were assumed to be regular")
		twidth = mean(diff(it))
	}
	obj = NULL
	t = twidth * tlags
	if (progress)
	  pb = txtProgressBar(style = 3, max = length(tlags))
	if (cores == 1) {
		ret = vector("list", length(tlags))
		for (dt in seq(along.with = tlags)) {
	  		ret[[dt]] = StVgmLag(formula, data, tlags[dt], pseudo = pseudo, 
						  	boundaries = boundaries, ...)
	  		ret[[dt]]$id = paste("lag", dt - 1, sep="")
	  		if (progress)
				setTxtProgressBar(pb, dt)
		}
	} else {
		if (!requireNamespace("future", quietly = TRUE) || 
				!requireNamespace("future.apply", quietly = TRUE))
	  		stop("For parallelization, future and future.apply packages are required")

	  	future::plan('multicore', workers = cores)
  		ret <- split(seq(along.with = tlags), seq(along.with = tlags))
		ret <- future.apply::future_lapply(X = ret,
				FUN = function(x){
					xx <- StVgmLag(formula, data, tlags[x], pseudo = pseudo, 
					boundaries = boundaries, ...)
					xx$id <- paste("lag", x - 1, sep="")
					if (progress)
						setTxtProgressBar(pb, x)
					return(xx)
				},
				future.seed = NULL # silence warning
			)
	}
	
	if (progress)
		close(pb)
	# add time lag:
	v = do.call(rbind, ret)
	v$timelag = rep(t, sapply(ret, nrow))
	if (is(t, "yearmon"))
		class(v$timelag) = "yearmon"

	b = attr(ret[[min(length(tlags),2)]], "boundaries")
	b = c(0, b[2]/1e6, b[-1])
	# ix = findInterval(v$dist, b) will use all spacelags
	b = b[-2]
	# spacelags = c(0, b[-length(b)] + diff(b)/2) will use all spacelags
	v$spacelag = c(0, b[-length(b)] + diff(b)/2) # spacelags[ix] will use all spacelags

	v$avgDist <- v$dist * v$np
	for (lagId in unique(v$spacelag)) {
	  bool <- v$spacelag == lagId
	  v$avgDist[bool] <- sum(v$avgDist[bool], na.rm = TRUE) / sum(v$np[bool], na.rm = TRUE)
	}

	class(v) = c("StVariogram", "data.frame")
	if(na.omit)
	v <- na.omit(v)

	# setting attributes to allow krigeST to check units
	attr(v$timelag, "units") <- attr(twidth,"units")
	if (isTRUE(!is.projected(data)))
		attr(v$spacelag, "units") = "km"
  
	return(v)
}

## very irregular data
variogramST.STIDF <- function (formula, data, tlags, cutoff, 
                               width, boundaries, progress, 
                               twindow, tunit, cores = 1) {
  ll = !is.na(is.projected(data@sp)) && !is.projected(data@sp)
  
  if (missing(cutoff))
    cutoff <- spDists(t(data@sp@bbox), longlat = ll)[1, 2]/3
  
  m = model.frame(terms(formula), as(data, "data.frame"))
  
  diffTime <- diff(index(data))
  if (inherits(diffTime, "difftime")) {
    timeScale <- units(diffTime)
    if(missing(tunit))
      warning(paste("The argument 'tunit' is missing: tlags are assumed to be given in ", timeScale, ".",sep=""))
    else {
      stopifnot(tunit %in% c("secs", "mins", "hours", "days", "weeks"))
      units(diffTime) <- tunit
      timeScale <- tunit
    }
    diffTime <- as.numeric(diffTime)
  } else if (!missing(tunit))
    stop("'tunit' ignored, as time values are not of class POSIXct or Date")
  
  if (missing(twindow))
    twindow <- round(2 * max(tlags, na.rm=TRUE) / mean(diffTime, na.rm = TRUE), 0)
    
  nData <- nrow(data)
  
  # re-using the order propertie of the time slot to only store the next "twindow" distances
  numTime <- as.numeric(index(data))
  diffTimeMat <- matrix(NA, nData, twindow)
  
  for (i in 1:nData) { # i <- 1
    diffTimeMat[i,1:min(nData,twindow)] <- cumsum(diffTime[i+0:(min(nData,twindow)-1)])
  }
  
  nSp <- length(boundaries)
  nTp <- length(tlags)
  
  distTp <- matrix(NA, nSp-1, nTp-1)
  distSp <- matrix(NA, nSp-1, nTp-1)
  gamma  <- matrix(NA, nSp-1, nTp-1)
  np     <- matrix(NA, nSp-1 ,nTp-1)
  
  # temporal selection
  if(progress)
    pb <- txtProgressBar(0, nTp-1, 0, style=3)
  for (i in 1:(nTp-1)) { # i <- 1
    ind <- which(diffTimeMat >= tlags[i] & diffTimeMat < tlags[i+1])
    if (length(ind) < 1) 
      next
    tmpInd <- matrix(NA,nrow=length(ind),4)
    tmpInd[,1] <- ind %% nData  # row number
    tmpInd[,2] <- (ind %/% nData)+1 # col number
    if (cores == 1){
      tmpInd[,3] <- apply(tmpInd[,1:2,drop=FALSE], 1, function(x) spDists(data@sp[x[1]], data@sp[x[2]+x[1],]))
    } else {
      if(!requireNamespace("future", quietly = TRUE) || !requireNamespace("future.apply", quietly = TRUE))
        stop("For parallelization, future and future.apply packages are required")
      future::plan("multicore", workers = cores)
      tmpInd[,3] <- future.apply::future_apply(X = tmpInd[,1:2,drop=FALSE], MARGIN = 1, 
                                 FUN = function(x) spDists(data@sp[x[1]], data@sp[x[2]+x[1],]),
                                 future.seed = NULL)
    }
    tmpInd[,4] <- diffTimeMat[tmpInd[,1:2, drop=FALSE]]
    
    # spatial selection
    for (j in 1:(nSp-1)) { # j <- 3
      indSp <- which(tmpInd[,3] >= boundaries[j] & tmpInd[,3] < boundaries[j+1])
      if (length(indSp) < 1)
        next
      distSp[j,i] <- mean(tmpInd[indSp,3])
      distTp[j,i] <- mean(tmpInd[indSp,4])
      
      indSp <- cbind(ind[indSp] %% nData, (ind[indSp] %/% nData)+1)
      np[j,i] <- nrow(indSp) # Issue #7, Thanks to Roelof.
      gamma[j,i] <- 0.5*mean((data[,,colnames(m)[1]]@data[indSp[,1],1] - data[,,colnames(m)[1]]@data[indSp[,1]+indSp[,2],1])^2,
                             na.rm=TRUE)
    }
    if(progress)
      setTxtProgressBar(pb, value=i)
  }
  if(progress)
    close(pb)
  
  res <- data.frame(np=as.vector(np),
                    dist=as.vector(distSp),
                    gamma=as.vector(gamma),
                    id=paste("lag",rep(1:(nTp-1),each=nSp-1), sep=""),
                    timelag=rep(tlags[-nTp]+diff(tlags)/2,each=nSp-1),
                    spacelag=rep(boundaries[-nSp]+diff(boundaries)/2, nTp-1))
  
  res$avgDist <- res$dist * res$np
  for (lagId in unique(res$spacelag)) {
    bool <- res$spacelag == lagId
    res$avgDist[bool] <- sum(res$avgDist[bool], na.rm = TRUE) / sum(res$np[bool], na.rm = TRUE)
  }
  
  attr(res$timelag, "units") <- timeScale
  attr(res$spacelag, "units") <- ifelse(ll, "km", "m")
  class(res) <- c("StVariogram", "data.frame")
  
  return(res)
}



## plotting
plot.StVariogram = function(x, model=NULL, ..., col = bpy.colors(), xlab, ylab,
                            map = TRUE, convertMonths = FALSE, as.table = TRUE,
                            wireframe = FALSE, diff = FALSE, all=FALSE) {
	lst = list(...)
	if (!is.null(lst$col.regions))
		col = lst$col.regions
	if (is(x$timelag, "yearmon")) {
		if (convertMonths) {
			x$timelag = as.numeric(x$timelag) * 12
			attr(x$timelag, "units") = "months"
		} else
			attr(x$timelag, "units") = "years"
	}
	if (missing(xlab)) {
		xlab = "distance"
		u =  attr(x$spacelag, "units")
		if (!is.null(u))
			xlab = paste(xlab, " (", u, ")", sep="")
	}
	if (missing(ylab)) {
		ylab = "time lag"
		u = attr(x$timelag, "units")
		if (!is.null(u))
			ylab = paste(ylab, " (", u, ")", sep="")
	}
	x$timelag = as.numeric(x$timelag)
  
  # check for older spatio-temporal variograms and compute avgDist on demand
  if(is.null(x$avgDist)) {
    x$avgDist <- x$dist * x$np
    for (lagId in unique(x$spacelag)) {
      bool <- x$spacelag == lagId
      x$avgDist[bool] <- sum(x$avgDist[bool] / sum(x$np[bool], na.rm = TRUE), na.rm = TRUE)
    }
    
  }
  
  if(!is.null(model)) {
    if (is(model,"StVariogramModel"))
      model <- list(model)
    for (mod in model) {
      slag <- x$avgDist
      slag[slag == 0 & x$timelag == 0] <- sqrt(.Machine$double.eps)
      x[[mod$stModel]] <- variogramSurface(mod, data.frame(spacelag = slag,
                                                           timelag = x$timelag))$gamma
      if (diff)
        x[[mod$stModel]] <- x[[mod$stModel]] - x$gamma
    }
  }
	x0 = x # needed by wireframe()
	
	if (!is.null(model)) {
    modelNames  <- sapply(model, function(x) x$stModel)
    
    if (all && !diff)
      v0 <- x[,c("dist", "id", "avgDist", "timelag")]
    else
      v0 <- NULL
    
    for (i in modelNames)
      v0 <- rbind(v0, x[,c("dist", "id", "avgDist", "timelag")])

    if(all & !diff) {# we also need the sample
      v0$what = factor(c(rep("sample", nrow(x)), rep(modelNames, each=nrow(x))),
                       levels=c("sample", modelNames), ordered = TRUE)
      v0$gamma = c(x$gamma, unlist(x[,modelNames]))
    } else {
      v0$what = factor(rep(modelNames, each=nrow(x)),
                       levels=modelNames, ordered=TRUE)
      v0$gamma = c(unlist(x[,modelNames]))
    }
		
		x = v0
	}
	if (wireframe) { 
		if (!is.null(model)) {
      if (length(model) > 1 || all)
        wireframe(gamma ~ avgDist*timelag | what, 
                  x, drape = TRUE, col.regions = col, 
                  xlab = xlab, ylab = ylab, as.table=as.table, ...)
      else 
        wireframe(as.formula(paste(model[[1]]$stModel,"~ avgDist*timelag")), 
                  x0, drape = TRUE, col.regions = col, 
                  xlab = xlab, ylab = ylab, as.table=as.table, ...)
		} else # without a model, plot only the sample variogram as a wireframe
			wireframe(gamma ~ avgDist * timelag, x0, drape = TRUE, col.regions = col,
				xlab = xlab, ylab = ylab, ...)
	} else if (map) {
		if (!is.null(model))
			f = gamma ~ avgDist + timelag | what
		else
			f = gamma ~ avgDist + timelag
		levelplot(f, x, xlab = xlab, ylab = ylab, col.regions = col, as.table=as.table, ...)
	} else { # not map, not wireplot
		if (!is.null(model))
			f = gamma ~ dist | what
		else
			f = gamma ~ dist
		x$id = factor(x$id, levels=unique(x$id))
		bp = bpy.colors(length(levels(x$id)))
		ps = list(superpose.line=list(col=bp), superpose.symbol=list(col=bp))
		xlim = c(0, max(x$dist) * 1.04)
		if (diff) {
		  xyplot(f, x, groups = x$id, type='b', xlim = xlim,
		         auto.key = list(space = "right"), xlab = xlab, 
		         par.settings = ps, as.table=as.table, ...)
		} else {
  		ylim = c(0, max(x$gamma) * 1.04)
  		xyplot(f, x, groups = x$id, type='b', ylim = ylim, xlim = xlim,
  		       auto.key = list(space = "right"), xlab = xlab, 
  		       par.settings = ps, as.table=as.table, ...)
    }
	}
}

print.StVariogramModel <- function(x, ...) {
  possComp <- c("space", "time", "joint")
  for(comp in possComp[possComp %in% names(x)]) {
    rownames(x[[comp]]) <- 1:nrow(x[[comp]])
    cat(paste(comp,"component: \n"))
    print(x[[comp]], ...)
  }
  
  possAddPar <- c("sill", "nugget", "stAni", "k")
  for(addPar in possAddPar[possAddPar %in% names(x)]) {
    cat(paste(addPar, ": ",x[[addPar]],"\n", sep=""))
  }
}


## guess the spatio-temporal anisotropy without spatio-temporal models
######################################################################

estiStAni <- function(empVgm, interval, method="linear", spatialVgm, temporalVgm, s.range=NA, t.range=NA) {
  if (!is.na(s.range))
    empVgm <- empVgm[empVgm$dist <= s.range,]
  if (!is.na(t.range))
    empVgm <- empVgm[empVgm$timelag <= t.range,]
  
  empVgm$timelag = as.numeric(empVgm$timelag) # in case it is of class difftime, messes up on R 4.1
  switch(method,
         linear = estiStAni.lin(empVgm, interval),
         range = estiAni.range(empVgm, spatialVgm, temporalVgm),
         vgm = estiAni.vgm(empVgm, spatialVgm, interval),
         metric = estiAni.metric(empVgm, spatialVgm, interval),
         stop(paste("Method", method,"is not implemented.")))
}

# linear
estiStAni.lin <- function(empVgm, interval) {
  lmSp <- lm(gamma~dist, empVgm[empVgm$timelag == 0,])
  
  optFun <- function(stAni, empVgm) {
    sqrt(mean((predict(lmSp, newdata = data.frame(dist=empVgm[empVgm$spacelag == 0,]$timelag*stAni)) - empVgm[empVgm$spacelag == 0,]$gamma)^2, na.rm=TRUE))
  }
  
  optimise(optFun, interval, empVgm = empVgm)$minimum  
}

# range
estiAni.range <- function(empVgm, spatialVgm, temporalVgm) {
  spEmpVgm <- empVgm[empVgm$timelag == 0,]
  class(spEmpVgm) <- c("gstatVariogram","data.frame")
  spEmpVgm <- spEmpVgm[-1,1:3]
  spEmpVgm$dir.hor <- 0
  spEmpVgm$dir.ver <- 0
  
  spatialVgm <- fit.variogram(spEmpVgm, spatialVgm)
  
  tmpEmpVgm <- empVgm[empVgm$spacelag == 0,]
  class(tmpEmpVgm) <- c("gstatVariogram","data.frame")
  tmpEmpVgm <- tmpEmpVgm[-1,c("np","timelag","gamma")]
  colnames(tmpEmpVgm) <- c("np", "dist", "gamma")
  tmpEmpVgm$dir.hor <- 0
  tmpEmpVgm$dir.ver <- 0
  
  temporalVgm <- fit.variogram(tmpEmpVgm, temporalVgm)
  
  spatialVgm$range[2]/temporalVgm$range[2]
}

# variograms
estiAni.vgm <- function(empVgm, spatialVgm, interval) {
  spEmpVgm <- empVgm[empVgm$timelag == 0,]
  class(spEmpVgm) <- c("gstatVariogram","data.frame")
  spEmpVgm <- spEmpVgm[-1,1:3]
  spEmpVgm$dir.hor <- 0
  spEmpVgm$dir.ver <- 0
  
  spatialVgm <- fit.variogram(spEmpVgm, spatialVgm)
  
  optFun <- function(stAni) {
    sqrt(mean((variogramLine(spatialVgm, dist_vector = empVgm[empVgm$spacelag == 0,]$timelag*stAni)$gamma - empVgm[empVgm$spacelag == 0,]$gamma)^2, na.rm=TRUE))
  }
  
  optimise(optFun, interval)$minimum
}

# metric variogram
estiAni.metric <- function(empVgm, spatialVgm, interval) {
  fit.StVariogram(empVgm, vgmST("metric", joint=spatialVgm, stAni=mean(interval)))$stAni[[1]]
}

# $Id: predict.gstat.q,v 1.35 2009-11-02 21:33:17 edzer Exp $

predict.gstat <-
function (object, newdata, block = numeric(0), nsim = 0, indicators = FALSE,
	BLUE = FALSE, debug.level = 1, mask, na.action = na.pass, sps.args = 
	list(n = 500, type = "regular", offset = c(.5, .5)), ...) 
{
	if (missing(object) || length(object$data) < 1) 
		stop("no data available")
	if (!inherits(object, "gstat"))
		stop("first argument should be of class gstat")
	to_stars = FALSE
	to_sf = if (inherits(newdata, c("sf", "sfc", "stars"))) {
			to_stars = inherits(newdata, "stars")
			newdata = as(newdata, "Spatial")
			TRUE
		} else
			FALSE

	if (!is.null(object$locations) && inherits(object$locations, "formula") 
			&& !(is(newdata, "Spatial"))) {
		coordinates(newdata) = object$locations
		return.sp = FALSE
	} else
		return.sp = TRUE
	
	max_dist = getMaxDist(object$data, newdata)
	
	.Call(gstat_init, as.integer(debug.level))
	bl_weights = numeric(0)
	if (!missing(mask)) {
		cat("argument mask is deprecated:")
		stop("use a missing value pattern in newdata instead")
	}
	nvars = length(object$data)
	new.X = NULL
	for (i in 1:length(object$data)) {
		name = names(object$data)[i]
		d = object$data[[i]]
		if (!is.null(d$data)) {
			if (!identical(d$data@proj4string@projargs, newdata@proj4string@projargs)) {
				print(proj4string(d$data))
				print(proj4string(newdata))
				stop(paste(name, ": data item in gstat object and newdata have different coordinate reference systems"))
			}
		}
		if (d$nmax == Inf) 
			nmax = as.integer(-1)
		else nmax = as.integer(d$nmax)
		nmin = as.integer(max(0, d$nmin))
		if (d$maxdist == Inf)
			maxdist = as.numeric(-1)
		else maxdist = d$maxdist
		if (d$dummy) {
			# tr = terms(d$locations)
			if (is.null(d$beta) || length(d$beta) == 0)
				stop("dummy data should have beta defined")
			if (d$degree != 0)
				stop("dummy data cannot have non-zero degree arg; use formula")
			# loc.dim = length(attr(tr, "term.labels"))
			loc.dim = dim(coordinates(newdata))[[2]]
			.Call(gstat_new_dummy_data, as.integer(loc.dim), 
				as.integer(d$has.intercept), as.double(d$beta), 
				nmax, nmin, maxdist, as.integer(d$vfn), 
				as.integer(is.projected(newdata)), as.integer(d$vdist))
			raw = list(xlevels = NULL)
		} else {
			if (is.null(d$weights))
				w = numeric(0)
			else
				w = d$weights
			raw = gstat.formula(d$formula, d$data)
			.Call(gstat_new_data, as.double(raw$y), 
				as.double(raw$locations), as.double(raw$X),
				as.integer(raw$has.intercept), as.double(d$beta),
				nmax, nmin, maxdist, as.integer(d$force), as.integer(d$vfn),
				as.numeric(w), double(0.0), as.integer(d$degree),
				as.integer(is.projected(d$data)), as.integer(d$vdist),
				as.double(d$lambda), as.integer(d$omax))
		}
		if (!is.null(object$model[[name]])) 
			load.variogram.model(object$model[[name]], c(i - 1, i - 1), max_dist = max_dist)
		raw = gstat.formula.predict(d$formula, newdata, na.action = na.action,
			(length(BLUE) == 2 && BLUE[2]), xlev = raw$xlevels)
		if (is.null(new.X)) 
			new.X = raw$X
		else new.X = cbind(new.X, raw$X)
		if (i > 1) {
			for (j in 1:(i - 1)) {
				cross = cross.name(names(object$data)[j], name)
				if (!is.null(object$model[[cross]])) 
					load.variogram.model(object$model[[cross]], 
						c(i - 1, j - 1), max_dist = max_dist)
			}
		}
	}
	if (!is.null(object$set)) 
		gstat.load.set(object$set)
	if (!is.null(object$merge)) 
		gstat.load.merge(object)
	if (is(newdata, "SpatialPolygons")) {
		pol = newdata@polygons
		if (length(pol) != nrow(raw$locations))
			stop("polygons and center points length mismatch")
		block = matrix(NA, 0, 2)
		nd = as(newdata, "SpatialPolygons")
		block.cols = rep(as.numeric(NA), length(pol))
		for (i in seq(along.with = pol)) {
			sps.args$x = nd[i]
			cc = coordinates(do.call("spsample", sps.args))
			cc[,1] = cc[,1] - raw$locations[i,1]
			cc[,2] = cc[,2] - raw$locations[i,2]
			block.cols[i] = nrow(block) + 1
			block = rbind(block, cc)
		}
		if (length(pol) == 1)
			block.cols = 2
	} else if (is(newdata, "SpatialLines")) {
		lin = newdata@lines
		if (length(lin) != nrow(raw$locations))
			stop("lines and line midpoints length mismatch")
		block = matrix(NA, 0, 2)
		nd = as(newdata, "SpatialLines")
		block.cols = rep(as.numeric(NA), length(lin))
		for (i in seq(along.with = lin)) {
			sps.args$x = nd[i]
			cc = coordinates(do.call("spsample", sps.args))
			cc[,1] = cc[,1] - raw$locations[i,1]
			cc[,2] = cc[,2] - raw$locations[i,2]
			block.cols[i] = nrow(block) + 1
			block = rbind(block, cc)
		}
		if (length(lin) == 1)
			block.cols = 2
	} else if (!is.null(dim(block))) { # i.e., block is data.frame or matrix
		if (is.data.frame(block) && !is.null(block$weights)) {
			bl_weights = block$weights
			block$weights = NULL
		} 
		block = data.matrix(block) # converts to numeric
		block.cols = ncol(block)
	} else {
		block = as.numeric(block) # make sure it's not integer
		block.cols = numeric(0)
	} 
	# handle NA's in the parts of newdata used:
	valid.pattern = NULL
	if (any(is.na(raw$locations)) || any(is.na(new.X))) {
		valid.pattern = !(apply(cbind(raw$locations, new.X), 1, 
				function(x) any(is.na(x))))
		raw$locations.all = raw$locations
		raw$locations = as.matrix(raw$locations[valid.pattern, ])
		new.X = as.matrix(new.X[valid.pattern, ])
	} 
	if (nsim) {
		if (indicators == TRUE)
			nsim = -abs(nsim)
	# random path: randomly permute row indices
		perm = sample(seq(along.with = new.X[, 1]))
		ret = .Call(gstat_predict, as.integer(nrow(as.matrix(new.X))),
			as.double(as.vector(raw$locations[perm, ])),
			as.double(as.vector(new.X[perm,])),
			as.integer(block.cols), as.vector(block), as.vector(bl_weights),
			as.integer(nsim), as.integer(BLUE))[[1]]
		if (abs(nsim) == 1)
			colsel = seq(1, by=2, length.out=nvars) # pred1 var1 pred2 var2 ...
		else
			colsel = TRUE
		ret = data.frame(cbind(raw$locations, 
			matrix(ret[order(perm), colsel], nrow(as.matrix(new.X)), abs(nsim) * nvars)))
	}
	else {
		ret = .Call(gstat_predict, as.integer(nrow(as.matrix(new.X))),
			as.double(as.vector(raw$locations)), as.vector(new.X), as.integer(block.cols), 
			as.vector(block), as.vector(bl_weights), as.integer(nsim), as.integer(BLUE))[[1]]
		ret = data.frame(cbind(raw$locations, ret))
	}
	.Call(gstat_exit, NULL)
	if (!is.null(valid.pattern) && any(valid.pattern)) {
		ret.all = data.frame(matrix(NA, length(valid.pattern), ncol(ret)))
		ret.all[, 1:ncol(raw$locations.all)] = raw$locations.all
		ret.all[valid.pattern, ] = ret
		ret = ret.all
	}
	if (abs(nsim) > 0) {
		names.vars = names(object$data)
		if (length(names.vars) > 1) 
			names.vars = paste(rep(names.vars, each = abs(nsim)), 
				paste("sim", 1:abs(nsim), sep = ""), sep = ".")
		else
			names.vars = paste("sim", 1:abs(nsim), sep = "")
	} else
		names.vars = create.gstat.names(names(object$data))
	names(ret) = c(dimnames(raw$locations)[[2]], names.vars)

	if (return.sp) {
		if (is(newdata, "SpatialPolygons")) {
			row.names(ret) = sapply(newdata@polygons, function(x) slot(x, "ID"))
			ret = SpatialPolygonsDataFrame(as(newdata, "SpatialPolygons"), ret,
				match.ID = TRUE)
		} else if (is(newdata, "SpatialLines")) {
			row.names(ret) = sapply(newdata@lines, function(x) slot(x, "ID"))
			ret = SpatialLinesDataFrame(as(newdata, "SpatialLines"), ret,
				match.ID = TRUE)
		} else {
			coordinates(ret) = dimnames(raw$locations)[[2]]
			if (gridded(newdata)) {
				returnFullGrid = fullgrid(newdata)
				fullgrid(newdata) = FALSE
				ret = new("SpatialPixelsDataFrame", 
					new("SpatialPixels", as(ret, "SpatialPoints"),
					    grid = newdata@grid, grid.index = newdata@grid.index,
					    bbox = newdata@bbox),
					data = ret@data, coords.nrs = ret@coords.nrs)
				fullgrid(ret) = returnFullGrid
			}
		}
		slot(ret, "proj4string") = slot(newdata, "proj4string")
		if (to_sf) {
			ret = if (to_stars) {
					if (!requireNamespace("stars", quietly = TRUE))
						stop("stars required: install that first") # nocov
					if (nsim) 
						sim_to_dimension(stars::st_as_stars(ret), nsim)
					else
						stars::st_as_stars(ret)
				} else {
					if (gridded(ret) && fullgrid(ret))
						fullgrid(ret) = FALSE
					if (!requireNamespace("sf", quietly = TRUE))
						stop("sf required: install that first") # nocov
					sf::st_as_sf(ret)
				}
		}
	}

	return(ret)
}

# call with: create.gstat.names(names(object$data))
# creates the names of the output columns in case of (multivariable) prediction
create.gstat.names <- function(ids, names.sep = ".") {
	nvars = length(ids)
	names.vars = character(nvars * 2 + nvars * (nvars - 1)/2)
	pos = 1
	for (i in 1:length(ids)) {
		name = ids[i]
		names.vars[1 + (i - 1) * 2] = paste(name, "pred", sep = names.sep)
		names.vars[2 + (i - 1) * 2] = paste(name, "var", sep = names.sep)
		if (i > 1) {
			for (j in 1:(i - 1)) {
				cross = paste(ids[j], name, sep = names.sep)
				names.vars[nvars * 2 + pos] = paste("cov", cross, 
					sep = names.sep)
				pos = pos + 1
			}
		}
	}
	return(names.vars)
}

getMaxDist = function(dataLst, newdata) {
  
  spBbox <- SpatialPoints(cbind(newdata@bbox[cbind(c(1,1,1,1), c(1,2,1,1))],
                                newdata@bbox[cbind(c(2,2,2,2), c(1,1,1,2))]), 
                          proj4string = newdata@proj4string)
  d <- c(spDists(spBbox)[cbind(c(1,3),c(2,4))])
  
	# d = apply(bbox(newdata), 1, diff)
	if (!is.null(dataLst[[1]]$data)) {
	  spBbox2 <- SpatialPoints(cbind(dataLst[[1]]$data@bbox[cbind(c(1,1,1,1), c(1,2,1,1))],
	                                 dataLst[[1]]$data@bbox[cbind(c(2,2,2,2), c(1,1,1,2))]), 
	                          proj4string = dataLst[[1]]$data@proj4string)
	  d2 <- c(spDists(spBbox2)[cbind(c(1,3),c(2,4))])
		# d2 = apply(bbox(dataLst[[1]]$data), 1, diff)
		d = apply(rbind(d,d2), 2, max) 
		# there are pathetic cases where this would not be sufficient 
	}
	if (length(d) == 2)
		d = c(d, 0)
	stopifnot(length(d) == 3)
	d
}

# $Id: load.variogram.model.q,v 1.7 2008-11-12 10:04:22 edzer Exp $

"load.variogram.model" <- function(model, ids = c(0, 0), max_dist = rep(-1.0, 3)) {
	if (missing(model))
		stop("model is missing");
	if (!inherits(model, "variogramModel"))
		stop("model should be of mode variogramModel (use function vgm)")
	if (any(model$range < 0.0)) {
		print(model)
		stop("variogram range can never be negative")
	}
	stopifnot(length(max_dist) == 3)
	anis = c(model$ang1, model$ang2, model$ang3, model$anis1, model$anis2)
	if (is.null(attr(model, "table")))
		covtable = numeric(0)
	else  {
		covtable = attr(model, "table")
		if (dim(model)[1] > 1 || model$model != "Tab")
			stop("table can only have one single model")
	}
# max_dist hack here for Lin(0) models:
#	if (max_dist > 0) {
#		w = which(model$model %in% c("Lin") & model$range == 0)
#		if (length(w) > 0) {
#			model[w,"psill"] = max_dist * model[w,"psill"]
#			model[w,"range"] = max_dist
#			cat("Conversion into equivalent model:\n")
#			print(model)
#		}
#	}
	if (!any(model$model %in% c("Lin", "Pow")))
		max_dist = rep(-1.0, 3) # ignore
	.Call(gstat_load_variogram, 
		as.integer(ids),
		as.character(model$model),
		as.numeric(model$psill),
		as.numeric(model$range),
		as.numeric(model$kappa),
		as.numeric(anis),
		covtable,
		as.numeric(max_dist))
}

# $Id: xyz2img.q,v 1.4 2006-02-10 19:01:07 edzer Exp $

"xyz2img" <-
function (xyz, zcol = 3, xcol = 1, ycol = 2, tolerance = 10 * .Machine$double.eps) 
{
    if (ncol(xyz) < 3) 
        stop("xyz object should have at least three columns")
    z = xyz[, zcol]
    x = xyz[, xcol]
    y = xyz[, ycol]
    xx = sort(unique(x))
    yy = sort(unique(y))
    nx = length(xx)
    ny = length(yy)
    nmax = max(nx, ny)
    difx = diff(xx)
    if (diff(range(unique(difx))) > tolerance) 
        stop("x intervals are not constant")
    dify = diff(yy)
    if (diff(range(unique(dify))) > tolerance) 
        stop("y intervals are not constant")
    dx = mean(difx)
    dy = mean(dify)
    xmin = min(xx)
    xmax = max(xx)
    xrange = xmax - xmin
    ymin = min(yy)
    ymax = max(yy)
    yrange = ymax - ymin
    row = round((x - xmin)/dx) + 1
    col = round((y - ymin)/dy) + 1
	zz = rep(as.numeric(NA), nx * ny)
	zz[row + nx * (col - 1)] = z
	zz = matrix(zz, nrow = nx, ncol = ny)
    list(x = seq(xmin, xmax, dx), y = seq(ymin, ymax, dy), z = zz)
}

# constructiong spatio-temporal variogram models
vgmST <- function(stModel, ..., space, time, joint, sill, k, nugget, stAni, 
                  temporalUnit) {
  stopifnot(is.character(stModel) && length(stModel)==1)
  
  old.stModel <- stModel
  stModel <- strsplit(stModel, "_")[[1]][1]
  
  if (stModel == "productSum" && !missing(sill))
    stop("The sill argument for the product-sum model has been removed 
due a change in notation of the spatio-temporal models. This 
affects as well how the spatial and temporal variograms are parameterised. 
Re-fit your model or use \"productSumOld\" instead.")
  
  if(!missing(sill))
    if(sill <= 0) stop("\"sill\" must be positive.")
  if(!missing(k))
    if(k <= 0) stop("\"k\" must be positive.")
  if(!missing(nugget))
    if(nugget < 0) stop("\"nugget\" must be non-negative.")
  if(!missing(stAni))
    if(stAni <= 0) stop("\"stAni\" must be positive.")
  
  vgmModel <- switch(stModel,
                     separable = list(space = space, time = time, sill = sill),
                     productSum = list(space = space, time = time, k = k),
                     productSumOld = list(space = space, time = time,
                                          sill = sill, nugget = nugget),
                     sumMetric = list(space = space, time = time, 
                                      joint = joint, stAni = stAni),
                     simpleSumMetric = list(space = space, time = time, 
                                            joint = joint, nugget = nugget, 
                                            stAni = stAni),
                     metric = list(joint = joint, stAni = stAni),
                     stop(paste("model", stModel, "unknown")))
  
  vgmModel$stModel <- old.stModel
  
  if (!missing(temporalUnit))
    attr(vgmModel, "temporal unit") = temporalUnit
  
  class(vgmModel) <- c("StVariogramModel", "list")
  
  vgmModel
}

# calculating spatio-temporal variogram surfaces
variogramSurface <- function(model, dist_grid, covariance=FALSE) {
  stopifnot(inherits(model, "StVariogramModel"))
  stopifnot(all(c("spacelag", "timelag") %in% colnames(dist_grid)))
  
  if (covariance) {
    switch(strsplit(model$stModel, "_")[[1]][1],
           separable=covSurfSeparable(model, dist_grid),
           productSum=covSurfProdSum(model, dist_grid),
           productSumOld=covSurfProdSumOld(model, dist_grid),
           sumMetric=covSurfSumMetric(model, dist_grid),
           simpleSumMetric=covSurfSimpleSumMetric(model, dist_grid),
           metric=covSurfMetric(model, dist_grid),
           stop("Only \"separable\", \"productSum\", \"sumMetric\", \"simpleSumMetric\" and \"metric\" are implemented."))
  } else {
    switch(strsplit(model$stModel, "_")[[1]][1],
           separable=vgmSeparable(model, dist_grid),
           productSum=vgmProdSum(model, dist_grid),
           productSumOld=vgmProdSumOld(model, dist_grid),
           sumMetric=vgmSumMetric(model, dist_grid),
           simpleSumMetric=vgmSimpleSumMetric(model, dist_grid),
           metric=vgmMetric(model, dist_grid),
           stop("Only \"separable\", \"productSum\", \"sumMetric\", \"simpleSumMetric\" and \"metric\" are implemented."))
  }
}

################################
## separable model: C_s * C_t ##
################################

vgmSeparable <- function(model, dist_grid) {
  vs = variogramLine(model$space, dist_vector=dist_grid$spacelag)$gamma
  vt = variogramLine(model$time,  dist_vector=dist_grid$timelag)$gamma
  
  cbind(dist_grid, "gamma" = model$sill*(vs+vt-vs*vt))
}

covSeparable <- function(x, y, model, separate) {  
  if(missing(separate))
    separate <- inherits(x, "STF") && inherits(y, "STF") && length(x) > 1 && length(y) > 1
  
  # the STF case
  if (inherits(x, "STF") && inherits(y, "STF")) {
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    Sm = variogramLine(model$space, covariance = TRUE, dist_vector = ds)*model$sill
    Tm = variogramLine(model$time, covariance = TRUE, dist_vector = dt)
    
    if (separate)
      return(list(Sm = Sm, Tm = Tm))
    else
      return(Tm %x% Sm) # kronecker product
  } 
  
  # separate makes only sense if both of x and y inherit STF
  if (separate)
    stop("An efficient inversion by separating the covarinace model is only possible if both of \"x\" and \"y\" inherit \"STF\"")
  
  # the STI case
  if (inherits(x, c("STI", "sftime")) || inherits(y, c("STI", "sftime"))) {
    # make sure that now both are of type STI
    x <- as(x, "STI")
    y <- as(y, "STI")
    
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    Sm = variogramLine(model$space, covariance = TRUE, dist_vector = ds)*model$sill
    Tm = variogramLine(model$time, covariance = TRUE, dist_vector = dt)
    
    return(Sm * Tm)
  }
  
  # the remaining cases, none of x and y is STI nor are both STF
  # make sure both are of type STS
  x <- as(x, "STS")
  y <- as(y, "STS")
  
  # calculate all spatial and temporal distances
  ds = spDists(x@sp, y@sp)
  dt = abs(outer(index(x@time), index(y@time), "-"))
  if(!is.null(attr(model,"temporal unit")))
    units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
  debug_time_unit(units(dt))
  dt <- as(dt, "matrix")
  
  # re-arrange the spatial and temporal distances
  sMat <- matrix(NA, nrow(x@index), nrow(y@index))
  tMat <- matrix(NA, nrow(x@index), nrow(y@index))
  for(r in 1:nrow(x@index)) {
    sMat[r,] <- ds[x@index[r,1], y@index[,1]]
    tMat[r,] <- dt[x@index[r,2], y@index[,2]]
  }
  
  # compose the cov-matrix
  Sm = variogramLine(model$space, covariance = TRUE, dist_vector = sMat)*model$sill
  Tm = variogramLine(model$time, covariance = TRUE, dist_vector = tMat)
  
  return(Sm * Tm)  
}

# covariance for the circulant embedding in ST
covSurfSeparable <- function(model, dist_grid) {
  Sm = variogramLine(model$space, covariance = TRUE, dist_vector = dist_grid$spacelag)$gamma*model$sill
  Tm = variogramLine(model$time, covariance = TRUE, dist_vector = dist_grid$timelag)$gamma
  
  cbind(dist_grid, "gamma" = Tm * Sm)
}

###########################################
## productSum model: C_s*C_t + C_s + C_t ##
###########################################

vgmProdSumOld <- function(model, dist_grid) {
  .Deprecated("vgmProdSum", package = "gstat", 
              msg="The former product-sum model is dprecited, consider to refit the new model specification",
              old = "vgmProdSumOld")
  
  vs = variogramLine(model$space, dist_vector=dist_grid$spacelag)$gamma
  vt = variogramLine(model$time, dist_vector=dist_grid$timelag)$gamma
  vn <- rep(model$nugget, length(vs))
  vn[vs == 0 & vt == 0] <- 0
  
  k <- (sum(model$space$psill)+sum(model$time$psill)-(model$sill+model$nugget))/(sum(model$space$psill)*sum(model$time$psill))
  
  if (k <= 0 || k > 1/max(rev(model$space$psill)[1], rev(model$time$psill)[1])) 
    k <- 10^6*abs(k) # distorting the model to let optim "hopefully" find suitable parameters
  
  cbind(dist_grid, "gamma" = as.vector(vs+vt-k*vs*vt+vn))
}

covProdSumOld <- function(x, y, model) {
  .Deprecated("covProdSum", package = "gstat", 
              msg="The former product-sum model is dprecited, consider to refit the new model specification",
              old = "covProdSumOld")
  
  stopifnot(inherits(x, c("STF", "STS", "STI", "sftime")) &&
			inherits(y, c("STF", "STS", "STI", "sftime")))
  
  # double check model for validity, i.e. k:
  k <- (sum(model$space$psill)+sum(model$time$psill)-model$sill)/(sum(model$space$psill)*sum(model$time$psill))
  if (k <= 0 || k > 1/max(model$space$psill[model$space$model!="Nug"], 
                         model$time$psill[model$time$model!="Nug"]))
    stop(paste("k (",k,") is non-positive or too large: no valid model!",sep=""))
  
  # the STF case
  if (inherits(x, "STF") && inherits(y, "STF")) {
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    vs = variogramLine(model$space, dist_vector = ds, covariance = TRUE)
    vt = variogramLine(model$time, dist_vector = dt, covariance = TRUE)
    
    return(model$sill-(vt %x% matrix(1,nrow(vs),ncol(vs)) + matrix(1,nrow(vt),ncol(vt)) %x% vs - k * vt %x% vs))
  } 
  
  # the STI case
  if(inherits(x, c("STI", "sftime")) || inherits(y, c("STI", "sftime"))) {
    # make sure that now both are of type STI
    x <- as(x, "STI")
    y <- as(y, "STI")
    
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    vs = variogramLine(model$space, dist_vector = ds, covariance = TRUE)
    vt = variogramLine(model$time, dist_vector = dt, covariance = TRUE)
    
    return(model$sill-(vt + vs - k * vt * vs))
  }
  
  # the remaining cases, none of x and y is STI nor are both STF
  # make sure both are of type STS
  x <- as(x, "STS")
  y <- as(y, "STS")
  
  # calculate all spatial and temporal distances
  ds = spDists(x@sp, y@sp)
  dt = abs(outer(index(x@time), index(y@time), "-"))
  if(!is.null(attr(model,"temporal unit")))
    units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
  debug_time_unit(units(dt))
  dt <- as(dt, "matrix")
  
  # re-arrange the spatial and temporal distances
  sMat <- matrix(NA, nrow(x@index), nrow(y@index))
  tMat <- matrix(NA, nrow(x@index), nrow(y@index))
  for(r in 1:nrow(x@index)) {
    sMat[r,] <- ds[x@index[r,1], y@index[,1]]
    tMat[r,] <- dt[x@index[r,2], y@index[,2]]
  }
  
  # compose the cov-matrix
  vs = variogramLine(model$space, dist_vector = sMat, covariance = TRUE)
  vt = variogramLine(model$time, dist_vector = tMat, covariance = TRUE)
  
  return(model$sill-(vt + vs - k * vt * vs))
}

# covariance for the circulant embedding in ST
covSurfProdSumOld <- function(model, dist_grid) {
  .Deprecated("covSurfProdSum", package = "gstat", 
              msg="The former product-sum model is dprecited, consider to refit the new model specification",
              old = "covSurfProdSumOld")
  
  vs = variogramLine(model$space, dist_vector = dist_grid$spacelag, covariance = TRUE)$gamma
  vt = variogramLine(model$time,  dist_vector = dist_grid$timelag, covariance = TRUE)$gamma
  
  k <- (sum(model$space$psill)+sum(model$time$psill)-(model$sill+model$nugget))/(sum(model$space$psill)*sum(model$time$psill))
  
  cbind(dist_grid, "gamma" = model$sill-(vt + vs - k * vt * vs))
}

vgmProdSum <- function(model, dist_grid) {
  if(!is.null(model$sill)) # backwards compatibility
    vgmProdSumOld(model, dist_grid)
  vs = variogramLine(model$space, dist_vector=dist_grid$spacelag)$gamma
  vt = variogramLine(model$time, dist_vector=dist_grid$timelag)$gamma
  
  sill_s <- sum(model$space$psill)
  sill_t <- sum(model$time$psill)
  k <- model$k
  
  cbind(dist_grid, "gamma" = as.vector((k*sill_t+1)*vs + (k*sill_s+1)*vt-k*vs*vt))
}

covProdSum <- function(x, y, model) {
  stopifnot(inherits(x, c("STF", "STS", "STI", "sftime")) &&
			inherits(y, c("STF", "STS", "STI", "sftime")))
  if(!is.null(model$sill)) # backwards compatibility
    covProdSumOld(x, y, model)
  
  # the STF case
  if (inherits(x, "STF") && inherits(y, "STF")) {
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    vs = variogramLine(model$space, dist_vector = ds, covariance =TRUE)
    vt = variogramLine(model$time, dist_vector = dt, covariance =TRUE)
    
    return(vt %x% matrix(1,nrow(vs),ncol(vs)) + matrix(1,nrow(vt),ncol(vt)) %x% vs + model$k * vt %x% vs)
  } 
  
  # the STI case
  if(inherits(x, c("STI", "sftime")) || inherits(y, c("STI", "sftime"))) {
    # make sure that now both are of type STI
    x <- as(x, "STI")
    y <- as(y, "STI")
    
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    vs = variogramLine(model$space, dist_vector = ds, covariance=TRUE)
    vt = variogramLine(model$time, dist_vector = dt, covariance=TRUE)
    
    return(vt + vs + model$k * vt * vs)
  }
  
  # the remaining cases, none of x and y is STI nor are both STF
  # make sure both are of type STS
  x <- as(x, "STS")
  y <- as(y, "STS")
  
  # calculate all spatial and temporal distances
  ds = spDists(x@sp, y@sp)
  dt = abs(outer(index(x@time), index(y@time), "-"))
  if(!is.null(attr(model,"temporal unit")))
    units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
  debug_time_unit(units(dt))
  dt <- as(dt, "matrix")
  
  # re-arrange the spatial and temporal distances
  sMat <- matrix(NA, nrow(x@index), nrow(y@index))
  tMat <- matrix(NA, nrow(x@index), nrow(y@index))
  for(r in 1:nrow(x@index)) {
    sMat[r,] <- ds[x@index[r,1], y@index[,1]]
    tMat[r,] <- dt[x@index[r,2], y@index[,2]]
  }
  
  # compose the cov-matrix
  vs = variogramLine(model$space, dist_vector = sMat, covariance = TRUE)
  vt = variogramLine(model$time, dist_vector = tMat, covariance = TRUE)
  
  return(vt + vs + model$k * vt * vs)
}

# covariance for the circulant embedding in ST
covSurfProdSum <- function(model, dist_grid) {
  vs = variogramLine(model$space, dist_vector = dist_grid$spacelag, covariance = TRUE)$gamma
  vt = variogramLine(model$time, dist_vector = dist_grid$timelag, covariance = TRUE)$gamma
  
  cbind(dist_grid, "gamma" = vt + vs + model$k * vt * vs)
}

#########################################################
# sumMetric model: C_s + C_t + C_st (Gerard Heuvelink) ##
#########################################################

vgmSumMetric <- function(model, dist_grid) {
  vs = variogramLine(model$space, dist_vector=dist_grid$spacelag)$gamma
  vt = variogramLine(model$time,  dist_vector=dist_grid$timelag)$gamma
  h = sqrt(dist_grid$spacelag^2 + (model$stAni * as.numeric(dist_grid$timelag))^2)
  vst = variogramLine(model$joint, dist_vector=h)$gamma
  
  cbind(dist_grid, "gamma" = vs + vt + vst)
}

covSumMetric <- function(x, y, model) {
  stopifnot(inherits(x, c("STF", "STS", "STI", "sftime")) &&
			inherits(y, c("STF", "STS", "STI", "sftime")))
  
  # the STF case
  if (inherits(x, "STF") && inherits(y, "STF")) {
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    Sm = variogramLine(model$space, covariance = TRUE, dist_vector = ds)
    Tm = variogramLine(model$time, covariance = TRUE, dist_vector = dt)
    
    h  = sqrt((matrix(1,nrow(dt),ncol(dt)) %x% ds)^2 
              + (model$stAni * dt %x% matrix(1,nrow(ds),ncol(ds)))^2)
    Mm = variogramLine(model$joint, covariance = TRUE, dist_vector = h)
    
    return(matrix(1,nrow(Tm),ncol(Tm)) %x% Sm + Tm %x% matrix(1,nrow(Sm),ncol(Sm)) + Mm)
  } 
  
  # the STI case
  if(inherits(x, c("STI", "sftime")) || inherits(y, c("STI", "sftime"))) {
    # make sure that now both are of type STI
    x <- as(x, "STI")
    y <- as(y, "STI")
    
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    Sm = variogramLine(model$space, covariance = TRUE, dist_vector = ds)
    Tm = variogramLine(model$time, covariance = TRUE, dist_vector = dt)
    
    h  = sqrt(ds^2 + (model$stAni * dt)^2)
    Mm = variogramLine(model$joint, covariance = TRUE, dist_vector = h)
    
    return(Sm + Tm + Mm)
  }
  
  # the remaining cases, none of x and y is STI nor are both STF
  # make sure both are of type STS
  x <- as(x, "STS")
  y <- as(y, "STS")
  
  # calculate all spatial and temporal distances
  ds = spDists(x@sp, y@sp)
  dt = abs(outer(index(x@time), index(y@time), "-"))
  if(!is.null(attr(model,"temporal unit")))
    units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
  debug_time_unit(units(dt))
  dt <- as(dt, "matrix")
  
  # re-arrange the spatial and temporal distances
  sMat <- matrix(NA, nrow(x@index), nrow(y@index))
  tMat <- matrix(NA, nrow(x@index), nrow(y@index))
  for(r in 1:nrow(x@index)) {
    sMat[r,] <- ds[x@index[r,1], y@index[,1]]
    tMat[r,] <- dt[x@index[r,2], y@index[,2]]
  }
  
  # compose the cov-matrix
  Sm = variogramLine(model$space, covariance = TRUE, dist_vector = sMat)
  Tm = variogramLine(model$time, covariance = TRUE, dist_vector = tMat)
  
  h  = sqrt(sMat^2 + (model$stAni * tMat)^2)
  Mm = variogramLine(model$joint, covariance = TRUE, dist_vector = h)
  
  return(Sm + Tm + Mm)
}

# covariance for the circulant embedding in ST
covSurfSumMetric <- function(model, dist_grid) {
  Sm = variogramLine(model$space, covariance = TRUE, dist_vector = dist_grid$spacelag)$gamma
  Tm = variogramLine(model$time, covariance = TRUE, dist_vector = dist_grid$timelag)$gamma
  
  h  = sqrt(dist_grid$spacelag^2 + (model$stAni * dist_grid$timelag)^2)
  Mm = variogramLine(model$joint, covariance = TRUE, dist_vector = h)$gamma
  
  cbind(dist_grid, "gamma" = Sm + Tm + Mm)
}

################################
## simplified sumMetric model ##
################################

vgmSimpleSumMetric <- function(model, dist_grid) {
  vs = variogramLine(model$space, dist_vector=dist_grid$spacelag)$gamma
  vt = variogramLine(model$time,  dist_vector=dist_grid$timelag)$gamma
  
  h = sqrt(dist_grid$spacelag^2 + (model$stAni * as.numeric(dist_grid$timelag))^2)
  
  vm = variogramLine(model$joint, dist_vector=h)$gamma
  vn <- variogramLine(vgm(model$nugget, "Nug", 0), dist_vector=h)$gamma
  
  cbind(dist_grid, "gamma" = vs + vt + vm + vn)
}

covSimpleSumMetric <- function(x, y, model) {
  modelNew <- vgmST("sumMetric", 
                    space=model$space, 
                    time=model$time,
                    joint=vgm(model$joint$psill[model$joint$model != "Nug"],
                              model$joint$model[model$joint$model != "Nug"],
                              model$joint$range[model$joint$model != "Nug"], 
                              model$nugget),
                    stAni=model$stAni)
  if (!is.null(attr(model,"temporal unit")))
    attr(modelNew,"temporal unit") <- attr(model,"temporal unit")
  covSumMetric(x, y, modelNew) 
}

# covariance for the circulant embedding in ST
covSurfSimpleSumMetric <- function(model, dist_grid) {
  modelNew <- vgmST("sumMetric", 
                    space=model$space, 
                    time=model$time,
                    joint=vgm(model$joint$psill[model$joint$model != "Nug"],
                              model$joint$model[model$joint$model != "Nug"],
                              model$joint$range[model$joint$model != "Nug"], 
                              model$nugget),
                    stAni=model$stAni)
  
  if (!is.null(attr(model,"temporal unit")))
    attr(modelNew,"temporal unit") <- attr(model,"temporal unit")
  
  covSurfSumMetric(modelNew, dist_grid) 
}

##################
## metric model ##
##################

vgmMetric <- function(model, dist_grid) {
  h = sqrt(dist_grid$spacelag^2 + (model$stAni * as.numeric(dist_grid$timelag))^2)

  cbind(dist_grid, "gamma" = variogramLine(model$joint, dist_vector=h)$gamma)
}

covMetric <- function(x, y, model) {
  stopifnot(inherits(x, c("STF", "STS", "STI", "sftime")) &&
			inherits(y, c("STF", "STS", "STI", "sftime")))
  
  # the STF case
  if (inherits(x, "STF") && inherits(y, "STF")) {
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    h  = sqrt((matrix(1,nrow(dt),ncol(dt)) %x% ds)^2
              + (model$stAni * dt %x% matrix(1,nrow(ds),ncol(ds)))^2)
    Mm = variogramLine(model$joint, covariance = TRUE, dist_vector = h)
    
    return(Mm)
  } 
  
  # the STI case
  if (inherits(x, c("STI", "sftime")) || inherits(y, c("STI", "sftime"))) {
    # make sure that now both are of type STI
    x <- as(x, "STI")
    y <- as(y, "STI")
    
    # calculate all spatial and temporal distances
    ds = spDists(x@sp, y@sp)
    dt = abs(outer(index(x@time), index(y@time), "-"))
    if(!is.null(attr(model,"temporal unit")))
      units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
    debug_time_unit(units(dt))
    dt <- as(dt, "matrix")
    
    # compose the cov-matrix
    h  = sqrt(ds^2 + (model$stAni * dt)^2)
    Mm = variogramLine(model$joint, covariance = TRUE, dist_vector = h)
    
    return(Mm)
  }
  
  # the remaining cases, none of x and y is STI nor are both STF
  # make sure both are of type STS
  x <- as(x, "STS")
  y <- as(y, "STS")
  
  # calculate all spatial and temporal distances
  ds = spDists(x@sp, y@sp)
  dt = abs(outer(index(x@time), index(y@time), "-"))
  if(!is.null(attr(model,"temporal unit")))
    units(dt) <- attr(model, "temporal unit") # ensure the same temporal metric as in the variogram definition
  debug_time_unit(units(dt))
  dt <- as(dt, "matrix")
  
  # re-arrange the spatial and temporal distances
  sMat <- matrix(NA, nrow(x@index), nrow(y@index))
  tMat <- matrix(NA, nrow(x@index), nrow(y@index))
  for(r in 1:nrow(x@index)) {
    sMat[r,] <- ds[x@index[r,1], y@index[,1]]
    tMat[r,] <- dt[x@index[r,2], y@index[,2]]
  }
  
  # compose the cov-matrix
  h  = sqrt(sMat^2 + (model$stAni * tMat)^2)
  Mm = variogramLine(model$joint, covariance = TRUE, dist_vector = h)
  
  return(Mm)
}

# covariance for the circulant embedding in ST
covSurfMetric <- function(model, dist_grid) {
  h  = sqrt(dist_grid$spacelag^2 + (model$stAni * dist_grid$timelag)^2)
  
  cbind(dist_grid, "gamma" = variogramLine(model$joint, covariance = TRUE, dist_vector = h)$gamma)
}

###########################
## fitting ST variograms ##
###########################

fit.StVariogram <- function(object, model, ..., method = "L-BFGS-B", lower, upper, fit.method = 6, 
                            stAni=NA, wles) {
  stopifnot(inherits(object, "StVariogram"), 
			inherits(model, "StVariogramModel"))
  
  sunit <- attr(object$spacelag, "units")
  tunit <- attr(object$timelag, "units")
  tu.obj = attr(model, "temporal unit")
  if (!is.null(tu.obj))
    stopifnot(identical(tunit, tu.obj))
  
  object$timelag = as.numeric(object$timelag) # needed for R 4.1
  object <- na.omit(object)
  
  ret <- model
  
  if(!missing(wles)) {
    if (wles)
      fit.method = 1
    else
      fit.method = 6
  }
  
  if (fit.method == 0) {
    attr(ret,"optim.output") <- "no fit"
    attr(ret, "MSE") <- mean((object$gamma - variogramSurface(model,
                                                              data.frame(spacelag=object$dist, timelag=object$timelag))$gamma)^2)
    attr(ret, "spatial unit")  <- sunit
    attr(ret, "temporal unit") <- tunit
    
    return(ret)
  }
  
  if ((fit.method == 7 || fit.method == 11) && is.null(model$stAni) && is.na(stAni)) {
    message("[An uninformed spatio-temporal anisotropy value of '1 (spatial unit)/(temporal unit)' is automatically selected. Consider providing a sensible estimate for stAni or using a different fit.method.]")
    stAni <- 1
  }
  
  weightingFun <- switch(fit.method,
                         function(obj, ...) obj$np, # 1
                         function(obj, gamma, ...) obj$np/gamma^2, # 2
                         function(obj, ...) obj$np, # 3
                         function(obj, gamma, ...) obj$np/gamma^2, # 4
                         function(obj, ...) stop("fit.method = 5 (REML), is not yet implemented"), # 5
                         function(obj, ...) 1, # 6
                         function(obj, curStAni, ...) 
                           if(is.na(stAni))
                             obj$np/(obj$dist^2+(curStAni*obj$timelag)^2)
                         else
                           obj$np/(obj$dist^2+(stAni*obj$timelag)^2), # 7
                         function(obj, ...) {
                           dist <- obj$dist
                           dist[dist == 0] <- min(dist[dist != 0], na.rm = TRUE)
                           obj$np/dist^2 # 8, pure space, 0 dist = min (dist > 0)
                         },
                         function(obj, ...) {
                           dist <- obj$timelag
                           dist[dist == 0] <- min(dist[dist != 0], na.rm = TRUE)
                           obj$np/dist^2
                         }, # 9, pure time
                         function(obj, gamma, ...) 1/gamma^2, # 10
                         function(obj, curStAni, ...) {
                           if(is.na(stAni))
                             1/(obj$dist^2+(curStAni*obj$timelag)^2)
                           else
                             1/(obj$dist^2+(stAni*obj$timelag)^2)
                         }, # 11
                         function(obj, ...) {
                           dist <- obj$dist
                           dist[dist == 0] <- min(dist[dist != 0], na.rm = TRUE)
                           1/(obj$dist^2) # 12, pure space
                         },
                         function(obj, ...) {
                           dist <- obj$timelag
                           dist[dist == 0] <- min(dist[dist != 0], na.rm = TRUE)
                           1/(obj$timelag^2)
                         }) # 13, pure time
  
  if(is.null(weightingFun))
    stop(paste("fit.method =", fit.method, "is not implementend"))
  
  fitFun = function(par, trace = FALSE, ...) {
    curModel <- insertPar(par, model)
    gammaMod <- variogramSurface(curModel,
                                 data.frame(spacelag=object$dist,
                                            timelag=object$timelag))$gamma
    resSq <- (object$gamma - gammaMod)^2
    resSq <- resSq * weightingFun(object, gamma=gammaMod, curStAni=curModel$stAni)
    if (trace)
      print(c(par, MSE = mean(resSq)))
    mean(resSq) # seems numerically more well behaved
  }
  
  if(missing(lower)) {
    min.s <- min(object$dist[object$dist>0])*0.05 # 5 % of the minimum distance larger 0
    min.t <- min(object$dist[object$timelag>0])*0.05 # 5 % of the minimum time lag 0),
    pos <- sqrt(.Machine$double.eps) # at least positive
    lower <- switch(strsplit(model$stModel, "_")[[1]][1],
                    separable=c(min.s, 0, min.t, 0, 0),
                    productSum=c(0, min.s, 0, 
                                 0, min.t, 0,
                                 pos),
                    productSumOld=c(0, min.s, 0, 
                                    0, min.t, 0, 0),
                    sumMetric=c(0, min.s, 0, 
                                0, min.t, 0,
                                0, pos, 0, pos),
                    simpleSumMetric=c(0, min.s,
                                      0, min.t,
                                      0, pos, 0, 0, pos),
                    metric=c(0, pos, 0, pos),
                    stop("Only \"separable\", \"productSum\", \"sumMetric\", \"simpleSumMetric\" and \"metric\" are implemented."))
  }
  if(missing(upper))
    upper <- switch(strsplit(model$stModel, "_")[[1]][1],
                    separable=c(Inf, 1, Inf, 1, Inf),
                    productSum=Inf,
                    productSumOld=Inf,
                    sumMetric=Inf,
                    simpleSumMetric=Inf,
                    metric=Inf,
                    stop("Only \"separable\", \"productSum\", \"sumMetric\", \"simpleSumMetric\" and \"metric\" are implemented."))
  
  
  
  pars.fit <- optim(extractPar(model), fitFun, ..., method = method, lower = lower, upper = upper)
  
  ret <- insertPar(pars.fit$par, model)
  attr(ret,"optim.output") <- pars.fit
  attr(ret, "MSE") <- mean((object$gamma - variogramSurface(insertPar(pars.fit$par, model),
                                                            data.frame(spacelag=object$dist, timelag=object$timelag))$gamma)^2)
  attr(ret, "spatial unit")  <- sunit
  attr(ret, "temporal unit") <- tunit
  
  return(ret)
}

###########
## tools ##
###########

# insert parameters into models
insertPar <- function(par, model) {
  switch(strsplit(model$stModel, "_")[[1]][1],
         separable=insertParSeparable(par, model),
         productSum=insertParProdSum(par, model),
         productSumOld=insertParProdSumOld(par, model),
         sumMetric=insertParSumMetric(par, model),
         simpleSumMetric=insertParSimpleSumMetric(par,model),
         metric=insertParMetric(par,model),
         stop("Only \"separable\", \"productSum\", \"sumMetric\", \"simpleSumMetric\" and \"metric\" are implemented."))
}

# extract parameters from models
extractPar <- function(model) {
  switch(strsplit(model$stModel, "_")[[1]][1],
         separable=c(range.s=model$space$range[2], nugget.s=model$space$psill[1],
                     range.t=model$time$range[2],  nugget.t=model$time$psill[1],
                     sill= model$sill[[1]]),
         productSumOld=c(sill.s = rev(model$space$psill)[1], range.s = rev(model$space$range)[1],
                         sill.t = rev(model$time$psill)[1],  range.t = rev(model$time$range)[1], 
                         sill=model$sill[[1]], nugget=model$nugget[[1]]),
         productSum=c(sill.s = model$space$psill[2], range.s = model$space$range[2], nugget.s = model$space$psill[1],
                      sill.t = model$time$psill[2],  range.t = model$time$range[2],  nugget.t = model$time$psill[1],
                      k=model$k),
         sumMetric=c(sill.s = model$space$psill[2], range.s = model$space$range[2], nugget.s = model$space$psill[1], 
                     sill.t = model$time$psill[2], range.t = model$time$range[2], nugget.t = model$time$psill[1],
                     sill.st = model$joint$psill[2], range.st = model$joint$range[2], nugget.st = model$joint$psill[1],
                     anis = model$stAni[[1]]),
         # simplified sumMetric model
         simpleSumMetric=c(sill.s = rev(model$space$psill)[1], range.s = rev(model$space$range)[1], 
                           sill.t = rev(model$time$psill)[1], range.t = rev(model$time$range)[1],
                           sill.st = rev(model$joint$psill)[1], range.st = rev(model$joint$range)[1], 
                           nugget = model$nugget[[1]], anis = model$stAni[[1]]),
         metric=c(sill = model$joint$psill[2], range = model$joint$range[2], nugget = model$joint$psill[1],
                  anis = model$stAni[[1]]),
         stop("Only \"separable\", \"productSum\", \"sumMetric\", \"simpleSumMetric\" and \"metric\" are implemented."))
}

# extract names
extractParNames <- function(model) {
  names(extractPar(model))
}

## dedicated insertion functions
################################

# separable model
insertParSeparable <- function(par, model) {
  vgmST("separable",
        space=vgm(1-par[2],as.character(model$space$model[2]),par[1],par[2],
                  kappa=model$space$kappa[2]),
        time= vgm(1-par[4],as.character(model$time$model[2]),par[3],par[4],
                  kappa=model$time$kappa[2]),
        sill=par[5])
}

# product sum model
insertParProdSumOld <- function(par, model) {
  vgmST("productSumOld",
        space=vgm(par[1],as.character(rev(model$space$model)[1]),par[2],
                  kappa=rev(model$space$kappa)[1]),
        time= vgm(par[3],as.character(rev(model$time$model)[1]),par[4],
                  kappa=rev(model$time$kappa)[1]),
        sill=par[5], nugget=par[6])
}

insertParProdSum <- function(par, model) {
  vgmST("productSum",
        space=vgm(par[1],as.character(model$space$model[2]),par[2],par[3],
                  kappa=model$space$kappa[2]),
        time= vgm(par[4],as.character(model$time$model[2]),par[5], par[6],
                  kappa=model$time$kappa[2]),
        k=par[7])
}

# sum metric model
insertParSumMetric <- function(par, model) {
  vgmST("sumMetric",
        space=vgm(par[1],as.character(model$space$model[2]),par[2],par[3],
                  kappa=model$space$kappa[2]),
        time= vgm(par[4],as.character(model$time$model[2]),par[5],par[6],
                  kappa=model$time$kappa[2]),
        joint=vgm(par[7],as.character(model$joint$model[2]),par[8],par[9],
                  kappa=model$joint$kappa[2]),
        stAni=par[10])
}

# simplified sum metric model
insertParSimpleSumMetric <- function(par, model) {
  vgmST("simpleSumMetric",
        space=vgm(par[1],as.character(rev(model$space$model)[1]),par[2],
                  kappa=rev(model$space$kappa)[1]),
        time= vgm(par[3],as.character(rev(model$time$model)[1]),par[4],
                  kappa=rev(model$time$kappa)[1]),
        joint=vgm(par[5],as.character(rev(model$joint$model)[1]),par[6],
                  kappa=rev(model$joint$kappa)[1]),
        nugget=par[7], stAni=par[8])
}

# metric model
insertParMetric <- function(par, model) {
  vgmST("metric",
        joint=vgm(par[1], as.character(model$joint$model[2]), par[2], par[3],
                  kappa=model$joint$kappa[2]),
        stAni=par[4])
}

# $Id: variogram.default.q,v 1.29 2009-11-02 21:33:17 edzer Exp $

"variogram.default" <-
function(object, locations, X, cutoff, width = cutoff/15.0, alpha = 0, 
    beta = 0, tol.hor = 90/length(alpha), tol.ver = 90/length(beta), 
    cressie = FALSE, dX = numeric(0), boundaries = numeric(0), 
    cloud = FALSE, trend.beta = NULL, debug.level = 1, cross = TRUE, 
	grid, map = FALSE, g = NULL, ..., projected = TRUE, lambda = 1.0,
	verbose = FALSE, covariogram = FALSE, PR = FALSE, pseudo = -1) 
{
	dots = list(...)
	if (length(dots) > 0) {
		warning(paste("the following arguments are ignored:",	
		              paste(dots, collapse = ", ")))
	  
	}
    id1 = id2 = 0
    ret = NULL
    if (missing(cutoff)) {
		if (is.logical(map) && map == TRUE)
			stop("for variogram maps, supply at least a cutoff")
        cutoff = numeric(0)
	}
    else if (width <= 0) 
        stop("argument width should be positive")
    if (missing(width)) 
        width = numeric(0)
    if (cloud == TRUE) 
        width = 0
	if (is.logical(map) && map == TRUE) {
		cells.dim = length(seq(-cutoff, cutoff, by = width))
		grid.topology = GridTopology(rep(-cutoff, 2), rep(width, 2), rep(cells.dim, 2))
		map = SpatialGrid(grid = grid.topology)
	}
	if (any(is.na(boundaries)))
		stop("no NA values allowed in boundaries")
    .Call(gstat_init, as.integer(debug.level))
	if (!is.null(g) && !is.null(g$set))
		gstat.load.set(g$set)
    id.names = NULL
    if (is.list(object) && is.list(locations)) {
        nvars = length(object)
        if (!is.null(names(object))) 
            id.names = names(object)
        for (i in 1:nvars) {
            if (missing(X)) 
                Xloc = rep(1, length(object[[i]]))
            else Xloc = X[[i]]
            t.beta = numeric(0)
            if (!is.null(trend.beta) && length(trend.beta) > 0) 
                t.beta = trend.beta[[i]]
            else t.beta = numeric(0)
			if (missing(grid) || !is.list(grid)) {
				if (!is.null(g) && gridded(g$data[i]$data))
					grd = unlist(gridparameters(g$data[i]$data))
				else 
					grd = numeric(0)
			} else
				grd = grid[[i]]
            .Call(gstat_new_data, as.double(object[[i]]), 
				as.double(locations[[i]]), as.double(Xloc), 
				as.integer(1), as.double(t.beta), as.integer(-1),
				as.integer(0), as.double(-1), as.integer(0), as.integer(1),
				double(0), grd, as.integer(0), as.integer(projected),
				as.integer(0), as.double(lambda), as.integer(0))
			if (!is.null(g) && !is.null(g$model[[id.names[i]]])) 
				load.variogram.model(g$model[[id.names[i]]], c(i - 1, i - 1))
        }
    } else 
		stop("argument object and locations should be lists")

	if (inherits(map, "SpatialGrid"))
		map = as.double(unlist(gridparameters(as(map, "SpatialGrid"))))

    pos = 0
    ids = NULL
	bnd = NULL
	is.direct = NULL
	if (PR) {
		if (any(object[[1]] <= 0))
			warning("pairwise relative variogram assumes non-negative data")
		covariogram = 2
	}
    if (cross == "ONLY") {
		stopifnot(nvars >= 2)
		id.range = 2:nvars
	} else if (cross == TRUE)
		id.range = nvars:1
	else
		id.range = 1:nvars
    for (id1 in id.range) {
		if (cross == "ST")
			id2range = 1
        else if (cross == "ONLY")
			id2range = 1:(id1-1)
		else
			id2range = ifelse(cross, 1, id1):id1
        for (id2 in id2range) {
			if (verbose) cat(".")
            if (is.null(id.names)) 
                id = ifelse(id1 == id2, paste(id1), cross.name(id2, id1))
            else id = ifelse(id1 == id2, paste(id.names[id1]), 
                cross.name(id.names[id2], id.names[id1]))
            for (a in alpha) {
                for (b in beta) {
                  direction = as.numeric(c(a, b, tol.hor, tol.ver))
				  # boundaries nees to be passed into the c code only ONCE:
				  if (is.null(bnd))
				  	bnd = boundaries 
				  else # NOT the first time we're here, so...
				    bnd = numeric(0)
                  ret.call = .Call(gstat_variogram, 
				  		as.integer(c(id1 - 1, id2 - 1)), 
						as.numeric(cutoff), as.numeric(width), 
                    	as.numeric(direction), as.integer(cressie), 
                    	as.numeric(dX), as.numeric(bnd), map, 
						as.integer(covariogram),
						as.integer(pseudo))
                  if (is.logical(map) && map == FALSE) {
                    np = ret.call[[1]]
                    sel = np > 0
                    n.dir = length(sel[sel])
                    if (n.dir > 0) {
                      dist = ret.call[[2]]
                      gamma = ret.call[[3]]
					  ret_cw = ret.call[[4]] # Sat Jul 16 16:19:59 CEST 2011
                      dir.a = rep(a, n.dir)
                      dir.b = rep(b, n.dir)
                      ids = c(ids, rep(id, n.dir))
					  is.direct = c(is.direct, id1 == id2)
                      df = data.frame(np = np[sel], dist = dist[sel], 
                        gamma = gamma[sel], dir.hor = dir.a, dir.ver = dir.b)
                      if (pos > 0) 
                        ret[(pos + 1):(pos + n.dir), ] = df
                      else ret = df
                      pos = pos + n.dir
                    }
				  } else {
				  	if (is.null(ret)) {
					  ret = data.frame(ret.call[[1]], ret.call[[2]],
							ret.call[[4]], ret.call[[3]])
					  names = c("dx", "dy", id, paste("np", id, sep="."))
					} else { 
					  ret = data.frame(ret, ret.call[[4]], ret.call[[3]])
					  names = c(names, id, paste("np", id, sep="."))
					}
					names(ret) = names
				  }
                }
            }
        }
    }
	if (verbose) cat("\n")
    .Call(gstat_exit, NULL)
	if (is.logical(map) && map == FALSE) {
    	if (!is.null(ids)) {
			ret$id = factor(ids, levels = unique(ids))
			attr(ret, "direct") = data.frame(id = unique(ids), is.direct = is.direct)
    		if (cloud) {
        		class(ret) = c("variogramCloud", "data.frame")
				attr(ret, ".BigInt") = 2^(4 * .Machine$sizeof.long)
    		} else {
				class(ret) = c("gstatVariogram", "data.frame")
				if (length(boundaries) == 0) {
					cutoff = ret_cw[1]
					width = ret_cw[2]
					boundaries = seq(0, cutoff, width)
					if (!identical(max(boundaries), cutoff))
						boundaries = c(boundaries, cutoff)
				}
				attr(ret, "boundaries") = boundaries
				attr(ret, "pseudo") = ret_cw[3]
				row.names(ret) = NULL
			}
		}
	} else {
		coordinates(ret) = c("dx", "dy")
		gridded(ret) = TRUE
		ret = list(map = ret)
		class(ret) = c("variogramMap", "list")
	}
	if (!is.null(ret)) {
		if (PR)
			attr(ret, "what") = "pairwise relative semivariance"
		else if (covariogram)
			attr(ret, "what") = "covariance"
		else if (cressie)
			attr(ret, "what") = "Cressie's semivariance"
		else
			attr(ret, "what") = "semivariance"
	}
    ret
}

library(sp)
library(gstat)
set.seed(13331)

x = runif(10)
y = runif(10)
z = rnorm(10)
d = data.frame(x=x,y=y,z=z)
bl = c(1,1)

nd = data.frame(x=.5, y=.5)

# single block:
library(sp)
coordinates(d) = ~x+y
nd = SpatialPoints(nd)
xx = krige(z~1, d, nd, model=vgm(1, "Exp", 1), block = bl,
	set = list(nb=4))

ring = cbind(c(0,bl[1],bl[1],0,0), c(0,0,bl[2],bl[2],0))
r1 = SpatialPolygons(list(Polygons(list(Polygon(ring)), ID = "xx")))
a = data.frame(a = 1, b = 2)
rownames(a) = "xx"
r1df = SpatialPolygonsDataFrame(r1, a)

g = gstat(formula=z~1, data=d, model=vgm(1, "Exp", 1))
args = list(type = "regular", n=16, offset=c(0.5,0.5))
yy = predict(g, r1df, block = bl, sps.args = args)

xx = as.data.frame(xx)
yy = as.data.frame(yy)
row.names(xx) = row.names(yy)
print(all.equal(xx,yy))

## multiple blocks of equal size:
nd = data.frame(x= 0:4 + .5, y=rep(.5,5))
nd = SpatialPoints(nd)
xx = krige(z~1, d, nd, model=vgm(1, "Exp", 1), block = bl,
	set = list(nb=4))
ring0 = cbind(c(0,bl[1],bl[1],0,0), c(0,0,bl[2],bl[2],0))
ring1 = cbind(c(1+0,1+bl[1],1+bl[1],1+0,1+0), c(0,0,bl[2],bl[2],0))
ring2 = cbind(c(2+0,2+bl[1],2+bl[1],2+0,2+0), c(0,0,bl[2],bl[2],0))
ring3 = cbind(c(3+0,3+bl[1],3+bl[1],3+0,3+0), c(0,0,bl[2],bl[2],0))
ring4 = cbind(c(4+0,4+bl[1],4+bl[1],4+0,4+0), c(0,0,bl[2],bl[2],0))
r1 = SpatialPolygons(list(
	Polygons(list(Polygon(ring0)), ID = "x0"),
	Polygons(list(Polygon(ring1)), ID = "x1"),
	Polygons(list(Polygon(ring2)), ID = "x2"),
	Polygons(list(Polygon(ring3)), ID = "x3"),
	Polygons(list(Polygon(ring4)), ID = "x4")
	))
df = data.frame(a=rep(1,5), b= rep(1,5))
rownames(df) = c("x0", "x1", "x2", "x3", "x4")
r1df = SpatialPolygonsDataFrame(r1, df)

yy = predict(g, r1, block = bl, sps.args = args)
xx = as.data.frame(xx)
yy = as.data.frame(yy)
row.names(xx) = row.names(yy)
all.equal(xx, yy)

## multiple blocks of equal size:
args = list(type = "regular", cellsize=.25, offset=c(0.5,0.5), n=16)
yy = predict(g, r1, block = bl, sps.args = args)
xx = as.data.frame(xx)
yy = as.data.frame(yy)
row.names(xx) = row.names(yy)
print(all.equal(xx, yy))

## multiple blocks of varying size:
nd = data.frame(x=c(0.5, 2, 4.5), y=c(0.5, 1, 1.5))
nd = SpatialPoints(nd)
bl = c(1,1)
ring0 = cbind(c(0,bl[1],bl[1],0,0), c(0,0,bl[2],bl[2],0))
xx1 = krige(z~1, d, nd[1], model=vgm(1, "Exp", 1), block = bl,
	set = list(nb=4))
bl = c(2,2)
ring1 = cbind(c(1+0,1+bl[1],1+bl[1],1+0,1+0), c(0,0,bl[2],bl[2],0))
xx2 = krige(z~1, d, nd[2], model=vgm(1, "Exp", 1), block = bl,
	set = list(nb=4))
bl = c(3,3)
ring2 = cbind(c(3+0,3+bl[1],3+bl[1],3+0,3+0), c(0,0,bl[2],bl[2],0))
xx3 = krige(z~1, d, nd[3], model=vgm(1, "Exp", 1), block = bl,
	set = list(nb=4))
r1 = SpatialPolygons(list(
	Polygons(list(Polygon(ring0)), ID = "x0"),
	Polygons(list(Polygon(ring1)), ID = "x1"),
	Polygons(list(Polygon(ring2)), ID = "x2")
	))
df = data.frame(a = rep(1,3), b = rep(1,3))
rownames(df) = c("x0", "x1", "x2")
r1df = SpatialPolygonsDataFrame(r1, df)

args = list(type = "regular", n=16, offset=c(0.5,0.5))
yy = predict(g, r1df, block = bl, sps.args = args)
xx = rbind(as.data.frame(xx1), as.data.frame(xx2), as.data.frame(xx3))
row.names(xx) = 1:3
xx = as.data.frame(xx)
yy = as.data.frame(yy)
row.names(xx) = row.names(yy)
print(all.equal(xx, yy))

suppressPackageStartupMessages(library(sp))
suppressPackageStartupMessages(library(spacetime))
suppressPackageStartupMessages(library(gstat))
suppressPackageStartupMessages(library(stars))

data(wind)
wind.loc$y = as.numeric(char2dms(as.character(wind.loc[["Latitude"]])))
wind.loc$x = as.numeric(char2dms(as.character(wind.loc[["Longitude"]])))
coordinates(wind.loc) = ~x+y
proj4string(wind.loc) = "+proj=longlat +datum=WGS84 +ellps=WGS84"

wind$time = ISOdate(wind$year+1900, wind$month, wind$day)
wind$jday = as.numeric(format(wind$time, '%j'))
stations = 4:15
windsqrt = sqrt(0.5148 * wind[stations]) # knots -> m/s
Jday = 1:366
daymeans = colMeans(
	sapply(split(windsqrt - colMeans(windsqrt), wind$jday), colMeans))
meanwind = lowess(daymeans ~ Jday, f = 0.1)$y[wind$jday]
velocities = apply(windsqrt, 2, function(x) { x - meanwind })
# match order of columns in wind to Code in wind.loc;
# convert to utm zone 29, to be able to do interpolation in
# proper Euclidian (projected) space:
pts = coordinates(wind.loc[match(names(wind[4:15]), wind.loc$Code),])
pts = SpatialPoints(pts)
if (require(rgdal, quietly = TRUE) && require(maps, quietly = TRUE)) {
proj4string(pts) = "+proj=longlat +datum=WGS84 +ellps=WGS84"
utm29 = CRS("+proj=utm +zone=29 +datum=WGS84 +ellps=WGS84")
pts = spTransform(pts, utm29)
# note the t() in:
w = STFDF(pts, wind$time, data.frame(values = as.vector(t(velocities))))

library(mapdata)
library(maptools)
m = map2SpatialLines(
    map("worldHires", xlim = c(-11,-5.4), ylim = c(51,55.5), plot=F))
proj4string(m) = "+proj=longlat +datum=WGS84 +ellps=WGS84"
m = spTransform(m, utm29)

# setup grid
grd = SpatialPixels(SpatialPoints(makegrid(m, n = 300)),
	proj4string = m@proj4string)
# grd$t = rep(1, nrow(grd))
#coordinates(grd) = ~x1+x2
#gridded(grd)=TRUE

# select april 1961:
w = w[, "1961-04"]

covfn = function(x, y = x) { 
	du = spDists(coordinates(x), coordinates(y))
	t1 = as.numeric(index(x)) # time in seconds
	t2 = as.numeric(index(y)) # time in seconds
	dt = abs(outer(t1, t2, "-"))
	# separable, product covariance model:
	0.6 * exp(-du/750000) * exp(-dt / (1.5 * 3600 * 24))
}

n = 10
tgrd = seq(min(index(w)), max(index(w)), length=n)
pred = krige0(sqrt(values)~1, w, STF(grd, tgrd), covfn)
layout = list(list("sp.points", pts, first=F, cex=.5),
	list("sp.lines", m, col='grey'))
wind.pr0 = STFDF(grd, tgrd, data.frame(var1.pred = pred))

v = vgmST("separable",
          space = vgm(1, "Exp", 750000), 
          time = vgm(1, "Exp", 1.5 * 3600 * 24),
          sill = 0.6)
wind.ST = krigeST(sqrt(values)~1, w, STF(grd, tgrd), v)

all.equal(wind.pr0, wind.ST)

# stars:
df = data.frame(a = rep(NA, 324*10))
s = STF(grd, tgrd)
newd = addAttrToGeom(s, df)
wind.sta = krigeST(sqrt(values)~1, st_as_stars(w), st_as_stars(newd), v)
# 1
plot(stars::st_as_stars(wind.ST), breaks = "equal", col = sf.colors())
# 2
stplot(wind.ST)
# 3
plot(wind.sta, breaks = "equal", col = sf.colors())
st_as_stars(wind.ST)[[1]][1:3,1:3,1]
(wind.sta)[[1]][1:3,1:3,1]
st_bbox(wind.sta)
bbox(wind.ST)
all.equal(wind.sta, stars::st_as_stars(wind.ST), check.attributes = FALSE)

# 4: roundtrip wind.sta->STFDF->stars
rt = stars::st_as_stars(as(wind.sta, "STFDF"))
plot(rt, breaks = "equal", col = sf.colors())
# 5:
stplot(as(wind.sta, "STFDF"))
st_bbox(rt)

# 6:
stplot(as(st_as_stars(wind.ST), "STFDF"))
}

library(sp)
data(meuse)
library(gstat)
v=variogram(log(zinc)~1,~x+y,meuse,cutoff=500,wi=100)
v$gamma=c(0.5,1,2,3,3)
fit.variogram(v, vgm(1, "Sph", 300, 0),warn.if.neg=TRUE)
fit.variogram(v, vgm(1, "Sph", 300, 0),warn.if.neg=FALSE)

# $Id: variogramLine.q,v 1.4 2008-08-18 16:32:42 edzer Exp $

"variogramLine" <-
function(object, maxdist, n = 200, min=1.0e-6 * maxdist, dir = c(1,0,0), 
	covariance = FALSE,	..., dist_vector = numeric(0), debug.level = 0)
{
	if (missing(object))
		stop("model is missing");
	if (!inherits(object, "variogramModel"))
		stop("model should be of mode variogramModel (use function vgm)")
	if (length(dist_vector) > 0)
		maxdist = 0.0
	else if (missing(maxdist))
		stop("maxdist or dist_vector needs to be set");
	if (length(dir) != 3)
		stop("dir should be numeric vector of length 3")
	.Call(gstat_init, as.integer(debug.level))
	pars = c(min,maxdist,n,dir)
	load.variogram.model(object, c(0,0)) # loads object into gstat 
	ret = .Call(gstat_variogram_values, as.integer(c(0,0)),
		as.numeric(pars), as.integer(covariance), as.numeric(dist_vector))
	.Call(gstat_exit, 0);
	if (is.matrix(dist_vector))
		matrix(ret[[2]], nrow(dist_vector), ncol(dist_vector))
	else
		data.frame(dist=ret[[1]], gamma=ret[[2]])
}

# $Id: set.q,v 1.7 2008-12-16 14:59:22 edzer Exp $
gstat.set <- function(set) {
	if(!is.list(set))
		stop("set should be a list")
	if (length(set) == 0)
		return(NULL)
	ret = NULL
	n = names(set)
	for (i in (1:length(set))) {
		val = set[[i]]
		if (n[i] == "method")
			str = paste("method: ", val, ";", sep="")
		else {
			if (is.character(val))
				val = paste("'", val, "'", sep = "")
			str = paste("set ", n[i], " = ", val, ";", sep="")
		}
		ret = c(ret, str)
	}
	ret
}

gstat.load.set <- function(set) {
	if(!is.list(set))
		stop("set should be a list")
	if (length(set) == 0)
		return(NULL)
	n = names(set)
	for (i in (1:length(set))) {
		if (n[i] == "method")
			ret = .Call(gstat_set_method, set[[i]])
		else 
			ret = .Call(gstat_set_set, n[i], set[[i]])
	}
	invisible(ret)
}

gstat.load.merge <- function(obj) {

	if (is.character(obj$merge) && length(obj$merge) == 2)
		obj$merge = list(c(obj$merge[1], 1, obj$merge[2], 1))

	if (!is.list(obj$merge))
		stop("merge argument should be list or character vector of lenght 2")

	ret = NULL
	for (i in 1:length(obj$merge)) {
		m = obj$merge[[i]]
		if (is.character(m) && length(m) == 4) {
			id = match(m[c(1,3)], names(obj$data)) - 1 # name ->> id
			if (any(is.na(id)))
				stop(paste("could not match all ids:", m[c(1,3)]))
			col = as.integer(m[c(2,4)]) - 1
			if (any(is.na(col)) || any(col < 0))
				stop("merge: parameters should be positive integers")
			ret = .Call(gstat_set_merge, id[1], col[1], id[2], col[2])
		} else stop(
			"list elements of merge should be lenght 4 character vectors")
	}
	ret
}

library(sp)
library(gstat)

data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y

mg = meuse.grid
gridded(mg) = FALSE
mg= mg[1500,]
krige(log(zinc)~1,meuse,mg,vgm(1, "Exp", 300, anis=c(0,0.01)),
	vdist=FALSE, maxdist=1000, nmax = 9)

## IGNORE_RDIFF_BEGIN
krige(log(zinc)~1,meuse,mg,vgm(1, "Exp", 300, anis=c(0,0.01)),
	vdist=TRUE, maxdist=1000, nmax = 9)
## IGNORE_RDIFF_END

# $Id: pcb.R,v 1.9 2008-02-01 22:39:44 edzer Exp $
# FIGURE 1:
library(sf)
library(stars)
library(gstat)
library(ggplot2)
data(pcb)
pcb = st_as_sf(pcb, coords = c("x", "y"), remove = FALSE)
data(ncp.grid)
ncp.grid = st_as_stars(ncp.grid)
wsv = read_sf(system.file("external/ncp.shp", package="gstat"))
classes = c(.2,1,2,5,10,20)
print(xyplot(y ~ x | as.factor(year), groups = sqrt(PCB138)/3, 
	data = as.data.frame(pcb),
    panel = function(x, y, groups, subscripts, ...) {
				 plot(st_geometry(wsv), col = grey(.5), add = TRUE)
                 panel.xyplot(x, y, cex = groups[subscripts], ...)
			},
    xlab = "x-coordinate", ylab = "y-coordinate",
    key = list(corner = c(0,0), x=0.8, y=0.25, points = list(pch = 1, col = 1,
        cex = sqrt(classes)/3), text = list(as.character(classes))),
	aspect = "iso", as.table = T,
	xlim = c(464000, 739000), ylim = c(5696500, 6131500),
	scales = list(draw = F)))

# FIGURE 2:
pcb$yf = as.factor(pcb$year)
pcb$int = rep(NA, dim(pcb)[1])
c = lm(log(PCB138)~-1+depth+yf,pcb)$coefficients
i = 2
for (f in levels(pcb$yf)) {
	pcb$int[pcb$yf == f] = c[i]
	i = i+1
}
print(xyplot(log(PCB138)~depth | as.factor(year), groups = int, 
	data = as.data.frame(pcb), 
    panel = function(x, y, groups, subscripts, ...) {
                # panel.grid(h=-1, v= 2)
                panel.xyplot(x, y)
				y = c(groups[subscripts][1], groups[subscripts][1] + -0.0641*45)
				llines(c(0, 45), y, lty = 2)
            },
    scales = list(
		y = list(at=log(c(.2, .5, 1, 2, 5, 10, 20)), 
		labels=c(".2",".5","1","2","5","10","20"), alternating = F)
	), xlab = "water depth", ylab = "PCB138", as.table = T)
)

# FIGURE 3:
# ps.options(width=2, height=2)
pcb$res=residuals(lm(log(PCB138)~year+depth, pcb))
v3 = variogram(res ~ year, pcb, dX=.1, bound=c(0,1000,3000,5000,(1:16)*10000))
print(plot(v3, model = vgm(.224,"Exp",17247,.08), plot.numbers = TRUE))

# FIGURE 4:
# next
g.pcb = NULL
merge = list(c("P1986", 2, "P1991", 2), c("P1986", 2, "P1996", 2),
		c("P1986", 2, "P2000", 2))
for (f in levels(pcb$yf)[c(1,4,6,7)])
    g.pcb= gstat(g.pcb, 
		paste("P", as.character(f), sep = ""), 
		log(PCB138)~depth, pcb[pcb$yf == f,],
		merge = merge)
g.pcb = gstat(g.pcb, model = vgm(.224,"Exp",17247,.08), fill.all=T)
v = variogram(g.pcb, cutoff=1e5)
#plot(v, model = fit.lmc(v, g))
#plot(v, model = g,plot.numbers = TRUE)
PCB.cor = matrix(NA, 4,4)
i = 1
for (x in levels(pcb$yf)[c(1,4,6,7)]) {
    j = 1
    for (y in levels(pcb$yf)[c(1,4,6,7)]) {
        A = log(pcb[pcb$yf == x,]$PCB138)
        B.tmp = krige(PCB138~1, pcb[pcb$yf == y,], 
			pcb[pcb$yf == x,], nmax = 1, set=list(debug=0))
        B = log(B.tmp$var1.pred)
        # print(paste(x, y, cor(A,B)))
        PCB.cor[i,j] = cor(A,B)
        j = j + 1
    }
    i = i + 1
}
years=c(1986,1991,1996,2000)
dimnames(PCB.cor)=list(years,years)
PCB.cor.ns = PCB.cor
PCB.cor = 0.5 * (PCB.cor + t(PCB.cor))
i = 1
for (x in levels(pcb$yf)[c(1,4,6,7)]) {
    j = 1
    for (y in levels(pcb$yf)[c(1,4,6,7)]) {
        if (j > i) {
            name = paste(paste("P", x, sep=""), paste("P", y, sep=""),sep = ".")
			print(name)
            g.pcb$model[[name]]["psill"] = 
				g.pcb$model[[name]]["psill"]  * PCB.cor[i,j]
        }
        j = j + 1
    }
    i = i + 1
}
print(plot(v, model = g.pcb, plot.numbers = FALSE))

print(PCB.cor.ns, digits=3)

print(PCB.cor, digits=3)

# FIGURE 5:
pcb.cok = predict(g.pcb, newdata = ncp.grid, debug.level = 0)
levs = c(.1,.2,.5,1,2,5,10,20)

spplot(pcb.cok[c(1,3,5,7)],
	as.table=T, col.regions = bpy.colors(7),
	at = log(levs),
	colorkey = list(at = log(levs), labels = as.character(levs),
		col = bpy.colors(7)), 
	layout = c(4,1)
)

spplot(pcb.cok[c(4,11,6,12,13,8,14,15,16,10)-2],
	skip=c(F,T,T,T,F,F,T,T,F,F,F,T,F,F,F,F),
	as.table=T, layout=c(4,4),
	asp="iso", col.regions = bpy.colors())

X = cbind(rep(1,7), c(1986, 1987, 1989, 1991, 1993, 1996, 2000))
X2=X[c(1,4,6,7),]
lambda = solve(t(X2) %*% X2) %*% t(X2)
dimnames(lambda) = list(NULL, c(1986, 1991, 1996, 2000))
print(lambda[2, ], digits=3)

# FIGURE 7:
pcb.contr = get.contr(pcb.cok, g.pcb, X=lambda[2, ])
# copy coordinates
#pcb.contr$x = pcb.cok$x
#pcb.contr$y = pcb.cok$y

pl1 = spplot(pcb.contr["beta.1"],
	main = "log-PCB138: change estimate",
	col.regions = bpy.colors(100))
pcb.contr$sig = pcb.contr$beta.1 / sqrt(pcb.contr$var.beta.1)
pl2 = spplot(pcb.contr["sig"],
	main = "log-PCB138 change/SE",
	col.regions = bpy.colors(100))
print(pl1, position=c(0,0,0.5,1), more = TRUE)
print(pl2, position=c(0.5,0,1,1), more = FALSE)

cat("source:\n\nEdzer J. Pebesma, Richard N.M. Duin (2005) Spatio-temporal mapping of\nsea floor sediment pollution in the North Sea.  In: Ph. Renard, and\nR. Froidevaux, eds. Proceedings GeoENV 2004 -- Fifth European Conference\non Geostatistics for Environmental Applications; Springer.\n")

# for validity of covariance functions on the sphere, see also:
# DOI 10.1007/s11004-011-9344-7
# http://mypage.iu.edu/~srobeson/Pubs/variogram_sphere_mathgeo_2011.pdf

suppressPackageStartupMessages(library(sp))
library(gstat)

if (require(rgdal, quietly = TRUE) && require(fields, quietly = TRUE)) {
data(meuse)
coordinates(meuse) = ~x+y
proj4string(meuse) = CRS("+init=epsg:28992")
meuse.ll = spTransform(meuse, CRS("+proj=longlat +ellps=WGS84"))
meuse.ll[1:10,]
variogram(log(zinc)~1, meuse.ll)

cloud1 = variogram(log(zinc)~1, meuse, cloud=T, cutoff=6000)
cloud2 = variogram(log(zinc)~1, meuse.ll, cloud=T, cutoff=6)

plot(cloud1$dist/1000, cloud2$dist, xlab="Amersfoort, km", ylab = "Long/lat")
abline(0,1)

  data(ozone2)
  oz = SpatialPointsDataFrame(ozone2$lon.lat, 
		  data.frame(t(ozone2$y)), 
		  proj4string=CRS("+proj=longlat +ellps=WGS84"))
  variogram(X870731~1,oz[!is.na(oz$X870731),])
  utm16 = CRS("+proj=utm +zone=16")
  oz.utm = spTransform(oz, utm16)
  variogram(X870731~1,oz.utm[!is.na(oz$X870731),])

# Timothy Hilton, r-sig-geo, Sept 14, 2008:

foo <-
structure(list(z = c(-1.95824831109744, -1.9158901643563, 4.22211761150161,
3.23356929459598, 1.12038389231868, 0.34613850821113, 1.12589932643631,
23.517912251617, 3.0519158690268, 3.20261431141517, -2.10947106854739
), lon = c(-125.29228, -82.1556, -98.524722, -99.948333, -104.691741,
-79.420833, -105.100533, -88.291867, -72.171478, -121.556944,
-89.34765), lat = c(49.87217, 48.2167, 55.905833, 56.635833,
53.916264, 39.063333, 48.307883, 40.0061, 42.537756, 44.448889,
46.242017)), .Names = c("z", "lon", "lat"), row.names = c(NA,
-11L), class = "data.frame")

coordinates(foo) <- ~lon+lat

proj4string(foo) <- CRS('+proj=longlat +ellps=WGS84')

vg.foo <- variogram(z~1, foo, cloud=TRUE, cutoff=1e10)

cat('==========\nvariogram:\n')
print(head(vg.foo))

cat('==========\nspDistsN1 Distances:\n')
print(spDistsN1(coordinates(foo), coordinates(foo)[1,], longlat=TRUE))
}

# $Id: plot.variogramCloud.q,v 1.7 2007-10-18 10:13:13 edzer Exp $

"plot.variogramCloud" <-
function (x, identify = FALSE, digitize = FALSE, 
	xlim = c(0, max(x$dist)), ylim, # = c(0, max(x$gamma)), 
	xlab = "distance", ylab = "semivariance", keep = FALSE, ...) 
{
    if (identify || digitize) {
		if (missing(ylim)) ylim = c(0, max(x$gamma))
        dots = list(...)
		if ("log" %in% names(dots)) {
			log = dots$log
			if (grep("x", log) & xlim[1] == 0) 
				xlim[1] = min(x$dist)/2
			if (grep("y", log) & ylim[1] == 0) 
				ylim[1] = min(x$gamma)/2
		}
        plot(x$dist, x$gamma, xlim = xlim, ylim = ylim, xlab = xlab, 
            ylab = ylab, ...)
		.BigInt = attr(x, ".BigInt")
        head = floor(x$np %/% .BigInt) + 1
        tail = floor(x$np %%  .BigInt) + 1
		if (identify) {
			print("mouse-left identifies, mouse-right or Esc stops")
        	labs = paste(head, tail, sep = ",")
        	sel = identify(x$dist, x$gamma, labs, pos = keep)
			ret = data.frame(cbind(head, tail)[sel,, drop = FALSE])
		} else {
			print("mouse-left digitizes, mouse-right closes polygon")
			poly = locator(n = 512, type = "l")
			if (!is.null(poly))
				sel = point.in.polygon(x$dist, x$gamma, poly$x, poly$y)
			else stop("digitized selection is empty")
			ret = data.frame(cbind(head, tail)[sel == 1,,drop = FALSE])
		}
		class(ret) = c("pointPairs", "data.frame")
        if (keep) {
			if (identify) {
				attr(x, "sel") = sel
				attr(x, "text") = labs[sel$ind]
			} else  # digitize
				attr(x, "poly") = poly
			attr(x, "ppairs") = ret
			return(x)
		} else 
        	return(ret)
	} else {
		sel = attr(x, "sel")
		lab = attr(x, "text")
		poly = attr(x, "poly")
		if (!is.null(sel) && !is.null(lab)) {
			if (missing(ylim)) ylim = c(0, max(x$gamma))
        	plot(x$dist, x$gamma, xlim = xlim, ylim = ylim, xlab = xlab, 
            	ylab = ylab, ...)
			text(x$dist[sel$ind], x$gamma[sel$ind], labels=lab, pos= sel$pos)
		} else if (!is.null(poly)) {
			if (missing(ylim)) ylim = c(0, max(x$gamma))
        	plot(x$dist, x$gamma, xlim = xlim, ylim = ylim, xlab = xlab, 
            	ylab = ylab, ...)
			lines(poly$x, poly$y)
		} else {
        	x$np = rep(1, length(x$gamma))
        	plot.gstatVariogram(x, xlim = xlim, ylim = ylim, xlab = xlab, 
        	    ylab = ylab, ...)
		}
    }
}

library(sp)
library(gstat)
data(meuse)
variogram(log(zinc)~1, ~x+y, meuse)

coordinates(meuse) <- ~ x + y                               
variogram(log(zinc)~1, meuse)

ind=seq(1,155,2)
var1= meuse[ind,]
var2= meuse[-ind,]
g <- gstat(NULL, id = "lead", form = lead ~ 1, data=var1)
g <- gstat(g, id = "zinc", form = zinc ~ 1, data=var2)
v.cross <- variogram(g)
plot(v.cross)

# $Id: krige.q,v 1.15 2009-02-20 13:53:38 edzer Exp $

if (!isGeneric("krige"))
	setGeneric("krige", function(formula, locations, ...)
		standardGeneric("krige"))

"krige.locations" <-
function (formula, locations, data = sys.frame(sys.parent()), 
	newdata, model = NULL, ..., beta = NULL, nmax = Inf, nmin = 0, omax = 0,
	maxdist = Inf, block = numeric(0), nsim = 0, indicators = FALSE, 
	na.action = na.pass, debug.level = 1)
{
    g = gstat(formula = formula, locations = locations, data = data, 
		model = model, beta = beta, nmax = nmax, nmin = nmin, omax = omax,
		maxdist = maxdist, ...)
    predict(g, newdata = newdata, block = block, nsim = nsim,
		indicators = indicators, na.action = na.action, debug.level = debug.level)
}
setMethod("krige", c("formula", "formula"), krige.locations)

krige.spatial <- function(formula, locations, newdata, model = NULL, ..., 
	beta = NULL, nmax = Inf, nmin = 0, omax = 0, maxdist = Inf, 
	block = numeric(0), nsim = 0, indicators = FALSE, 
	na.action = na.pass, debug.level = 1)
{
	# locations = coordinates(arg2)
    g = gstat(formula = formula, # locations = locations, 
		data = locations, 
		model = model, beta = beta, nmax = nmax, nmin = nmin, omax = omax,
		maxdist = maxdist, ...)
    predict(g, newdata = newdata, block = block, nsim = nsim,
		indicators = indicators, na.action = na.action, debug.level = debug.level)
}
setMethod("krige", c("formula", "Spatial"), krige.spatial)

setMethod("krige", c("formula", "NULL"),
	function(formula, locations, newdata, ...) { # manual dispatch based on newdata:
		if (inherits(newdata, c("sf", "sfc", "stars")))
			krige.sf(formula, locations, newdata = newdata, ...)
		else
			krige.spatial(formula, locations, newdata = newdata, ...)
	}
)

krige.sf <- function(formula, locations, newdata, ..., nsim = 0) {
	if (!requireNamespace("sf", quietly = TRUE))
		stop("sf required: install that first") # nocov
	if (!requireNamespace("stars", quietly = TRUE))
		stop("stars required: install that first") # nocov
	crs = sf::st_crs(newdata)
	if (!is.null(locations)) {
		stopifnot(sf::st_crs(locations) == sf::st_crs(newdata))
		crs = sf::st_crs(locations)
		if (!isTRUE(sf::st_is_longlat(locations))) {
			sf::st_crs(locations) = sf::NA_crs_
			sf::st_crs(newdata) = sf::NA_crs_# to avoid problems not handled by sp...
		}
		locations = as(locations, "Spatial")
	}
	ret = krige(formula, locations, as(newdata, "Spatial"), ..., nsim = nsim)
	if (gridded(ret)) {
		st = stars::st_as_stars(ret)
		if (nsim > 0)
			st = sim_to_dimension(st, nsim)
		sf::st_set_crs(st, crs)
	} else
		sf::st_set_crs(sf::st_as_sf(ret), crs)
}
setMethod("krige", c("formula", "sf"), krige.sf)

sim_to_dimension = function(st, nsim) {
	nms = names(stars::st_dimensions(st))
	if (length(st) > nsim) {
		nvars = length(st) / nsim
		l = vector("list", nvars)
		vars = unique(sub("([^.]+)\\.[[:alnum:]]+$", "\\1", names(st)))
		for (i in 1:nvars) {
			range = seq((i-1) * nsim + 1, length.out = nsim)
			m = setNames(st[range], paste0("sim", 1:nsim))
			l[[i]] = setNames(stars::st_set_dimensions(merge(m), 
				names = c(nms, "sample")), vars[i])
		}
		do.call(c, l)
	} else {
		if (nsim > 1)
			setNames(stars::st_set_dimensions(merge(st), names = c(nms, "sample")), "var1")
		else
			st
	}
}

setMethod(krige, signature("formula", "ST"),
	function(formula, locations, newdata, model, ...) {
		krigeST(formula, locations, newdata, model,...) 
	}
)

if (!isGeneric("idw"))
	setGeneric("idw", function(formula, locations, ...)
		standardGeneric("idw"))

idw.locations <- function (formula, locations, data = sys.frame(sys.parent()), 
		newdata, nmax = Inf, nmin = 0, omax = 0, maxdist = Inf, 
		block = numeric(0), 
		na.action = na.pass, idp = 2.0, debug.level = 1) {
	krige(formula, locations, data, newdata, nmax = nmax, nmin = nmin,
		omax = omax, maxdist = maxdist, block = block, na.action = na.action,
		set = list(idp = idp), debug.level = debug.level)
}
setMethod("idw", c("formula", "formula"), idw.locations)

idw.spatial <- function (formula, locations, 
		newdata, nmax = Inf, nmin = 0, omax = 0, 
		maxdist = Inf, block = numeric(0), 
		na.action = na.pass, idp = 2.0, debug.level = 1) {
	krige(formula, locations, newdata, nmax = nmax, nmin = nmin, omax = omax,
		maxdist = maxdist, block = block, na.action = na.action,
		set = list(idp = idp), debug.level = debug.level, model = NULL)
}
setMethod("idw", c("formula", "Spatial"), idw.spatial)

idw.sf <- function (formula, locations, 
		newdata, ..., idp = 2.0) {

	if (!requireNamespace("sf", quietly = TRUE))
		stop("sf required: install that first") # nocov
	if (!requireNamespace("stars", quietly = TRUE))
		stop("stars required: install that first") # nocov

	ret = krige(formula, locations, newdata, ..., set = list(idp = idp), model = NULL)
	if (inherits(newdata, c("sf", "sfc")))
		sf::st_as_sf(ret)
	else if (inherits(newdata, "stars"))
		stars::st_as_stars(ret)
	else stop("newdata should be of class sf or stars")
}
setMethod("idw", c("formula", "sf"), idw.sf)

STx2SpatialPoints = function(x, multiplyTimeWith = 1.0) { 
	x = as(geometry(x), "STI")
	t1 = as.numeric(as.POSIXct(index(x@time)))
	t2 = as.numeric(x@endTime)
	time = multiplyTimeWith * (t1 + t2) / 2
	cc = cbind(coordinates(x), time)
	SpatialPoints(cc, proj4string = x@sp@proj4string) 
}

STxDF2SpatialPointsDataFrame = function(x, multiplyTimeWith = 1.0) { 
	pts = STx2SpatialPoints(geometry(x), multiplyTimeWith)
	SpatialPointsDataFrame(pts, x@data)
}

SpatialPointsDataFrame2STxDF = function(x, class, tz = "", 
		origin = as.POSIXct("1970-01-01",tz=tz)) { 
	cc = coordinates(x)
	time = as.POSIXct(cc[,ncol(cc)], tz=tz, origin = origin)
	sp = SpatialPoints(cc[,-ncol(cc)], proj4string = x@sp@proj4string)
	st = as(STI(sp, time), class)
	addAttrToGeom(STI(sp, time), x@data)
}

idw.ST <- function (formula, locations, 
		newdata, nmax = Inf, nmin = 0, omax = 0, 
		maxdist = Inf, block = numeric(0), 
		na.action = na.pass, idp = 2.0, debug.level = 1, 
		multiplyTimeWith = 1.0) {
	stopifnot(ncol(coordinates(locations@sp)) == 2)
	ret = krige(formula, 
		STxDF2SpatialPointsDataFrame(locations, multiplyTimeWith),
		STx2SpatialPoints(newdata, multiplyTimeWith),
		nmax = nmax, nmin = nmin, omax = omax,
		maxdist = maxdist, block = block, na.action = na.action,
		set = list(idp = idp), debug.level = debug.level, model = NULL)
	SpatialPointsDataFrame2STxDF(ret, class(geometry(newdata)))
}
setMethod("idw", c("formula", "ST"), idw.ST)

# $Id: fit.variogram.q,v 1.10 2008-12-15 14:27:29 edzer Exp $

"fit.variogram" <-
function (object, model, fit.sills = TRUE, fit.ranges = TRUE, 
    fit.method = 7, debug.level = 1, warn.if.neg = FALSE, fit.kappa = FALSE) 
{
	cl = match.call()
    if (missing(object)) 
        stop("nothing to fit to")
	if (!inherits(object, "gstatVariogram") && !inherits(object, "variogramCloud"))
		stop("object should be of class gstatVariogram or variogramCloud")
	if (inherits(object, "variogramCloud"))
		object$np = rep(1, nrow(object))
	if (length(unique(object$id)) > 1)
		stop("to use fit.variogram, variogram object should be univariable")
    if (missing(model)) 
        stop("no model to fit")
	if (is(model, "variogramModelList")) {
		ret = lapply(model, function(x) fit.variogram(object, x, fit.sills = fit.sills, 
			fit.ranges = fit.ranges, fit.method = fit.method, debug.level = debug.level, 
			warn.if.neg = warn.if.neg, fit.kappa = fit.kappa))
		sse = sapply(ret, function(x) attr(x, "SSErr"))
		return(ret[[which.min(sse)]])
	}
    if (!inherits(model, "variogramModel"))
        stop("model should be of class variogramModel (use vgm)")
    if (fit.method == 5)
    	stop("use function fit.variogram.reml() to use REML")
    if (length(fit.sills) < length(model$model)) 
        fit.sills = rep(fit.sills, length(model$model))
    if (length(fit.ranges) < length(model$model)) 
        fit.ranges = rep(fit.ranges, length(model$model))
	if (fit.method == 7 && any(object$dist == 0))
		stop("fit.method 7 will not work with zero distance semivariances; use another fit.method value")
	if (any(is.na(model$psill)) || any(is.na(model$range)))
		model = vgm_fill_na(model, object)
    fit.ranges = fit.ranges & !(model$model %in% c("Nug", "Err")) # no ranges to fit for Nug/Err
	if (isTRUE(fit.kappa))
		fit.kappa = seq(0.3, 5, 0.1)
	if (any(model$model %in% c("Mat", "Ste")) && length(fit.kappa) > 1) {
		f = function(x, o, m) {
			m[m$model %in% c("Mat", "Ste"), "kappa"] = x
			fit.variogram(o, m, fit.kappa = FALSE, 
				fit.method = fit.method, debug.level = debug.level) # fits range
		}
		ret = lapply(fit.kappa, f, object, model)
		return(ret[[ which.min(sapply(ret, function(x) attr(x, "SSErr"))) ]])
	}
	initialRange = model$range
    .Call(gstat_init, as.integer(debug.level))
    .Call(gstat_load_ev, object$np, object$dist, object$gamma)
    load.variogram.model(model)
    ret = .Call(gstat_fit_variogram, as.integer(fit.method), 
        as.integer(fit.sills), as.integer(fit.ranges))
    .Call(gstat_exit, 0)
    model$psill = ret[[1]]
    model$range = ret[[2]]
	attr(model, "singular") = as.logical(ret[[3]]);
	attr(model, "SSErr") = ret[[4]]
	direct = attr(object, "direct")
	if (!is.null(direct)) {
		id = unique(object$id)
		if (any(direct[direct$id == id, "is.direct"]) && any(model$psill < 0)) {
			if (warn.if.neg)
				warning("partial sill or nugget fixed at zero value")
			fit.sills = model$psill > 0
			model$psill[model$psill < 0] = 0.0
			model$range = initialRange
			return(fit.variogram(object, model, fit.sills = fit.sills, fit.ranges =
				fit.ranges, fit.method = fit.method, debug.level = debug.level,
				warn.if.neg = warn.if.neg, fit.kappa = fit.kappa))
		}
	}
	if (attr(model, "singular") && debug.level) {
		rat = mean(object$gamma) / mean(object$dist) 
		if (rat > 1e6 || rat < 1e-6)
			print("a possible solution MIGHT be to scale semivariances and/or distances")
	}
	attr(model, "call") = cl
    model
}

vgm_fill_na = function(model, obj) {
	if (any(is.na(model$range))) {
    	model[model$model %in% c("Nug", "Err"), "range"] = 0
    	model[!model$model %in% c("Nug", "Err"), "range"] = max(obj$dist) / 3
	}
	if (any(model$model %in% "Nug") && is.na(model[model$model == "Nug","psill"]))
    	model[model$model == "Nug", "psill"] = mean(head(obj$gamma, 3))
	na_sills = is.na(model[model$model != "Nug", "psill"])
	if (any(na_sills)) {
		n = length(model[model$model != "Nug",]$psill)
    	model[model$model != "Nug", "psill"] = mean(tail(obj$gamma, 5)) / n
	}
	model
}

# $Id: gstat.cv.q,v 1.9 2009-10-30 16:11:21 edzer Exp $

"gstat.cv" <-
function (object, nfold = nrow(object$data[[1]]$data), remove.all = FALSE, 
	verbose = interactive(), all.residuals = FALSE, ...) 
{
	if (!inherits(object, "gstat")) 
		stop("first argument should be of class gstat")
	var1 = object$data[[1]]
	data = var1$data
	formula = var1$formula
	if (all.residuals) {
		nc = length(object$data)
		ret = data.frame(matrix(NA, nrow(data), nc))
	} else {
		cc = coordinates(data)
		rownames(cc) = NULL
		df = data.frame(matrix(as.numeric(NA), nrow(data), 2))
		ret = SpatialPointsDataFrame(cc, df, proj4string = data@proj4string)
	}
	if (missing(nfold)) 
		nfold = 1:nrow(data)
	if (length(nfold) == nrow(data))
		fold = nfold
	else if (nfold < nrow(data)) 
		fold = sample(nfold, nrow(data), replace = TRUE)
	else fold = 1:nrow(data)

	if (all.residuals || (remove.all && length(object$data) > 1)) {
		all.data = list()
		for (v in 1:length(object$data))
			all.data[[v]] = object$data[[v]]$data
	}

	if (verbose)
		pb <- txtProgressBar(1,length(unique(fold)),style=3)

	for (i in sort(unique(fold))) {
		if (verbose)
			setTxtProgressBar(pb, i) 
		sel = which(fold == i)
		object$data[[1]]$data = data[-sel, ]
		if (remove.all && length(object$data) > 1) {
			for (v in 2:length(object$data)) {
				varv = object$data[[v]]
				varv$data = all.data[[v]]
				#atv = gstat.formula(varv$formula, varv$data)$locations
				#at1 = gstat.formula(formula, data[sel, ])$locations
				atv = coordinates(varv$data)
				at1 = coordinates(data[sel,])
				cc = rbind(atv, at1)
				rownames(cc) = NULL # as there will be duplicates
				all = SpatialPoints(cc, proj4string = data@proj4string)
				zd = zerodist(all)
				skip = zd[, 1]
				object$data[[v]]$data = varv$data[-skip, ]
			}
		}
		x = predict(object, newdata = data[sel, ], ...)
		if (all.residuals) {
			for (i in 1:length(object$data)) {
				var.i = object$data[[i]]
				data.i = all.data[[i]]
				formula.i = var.i$formula
				observed = gstat.formula(formula.i, data.i)$y[sel]
				pred.name = paste(names(object$data)[i], "pred", sep = ".")
				residual = as.numeric(observed - x[[pred.name]])
				ret[sel, i] = residual
			}
		} else {
			ret[[1]][sel] = x[[1]]
			ret[[2]][sel] = x[[2]]
		}
	}
	if (verbose)
		cat("\n")

	if (! all.residuals) {
		names(ret) = names(x)[1:2]
		ret$observed = gstat.formula(formula, data)$y
		pred.name = paste(names(object$data)[1], "pred", sep = ".")
		ret$residual = ret$observed - ret[[pred.name]]
		var.name = paste(names(object$data)[1], "var", sep = ".")
		ret$zscore = ret$residual/sqrt(ret[[var.name]])
		ret$fold = fold
	} else
		names(ret) = names(object$data)

	if (!is.null(object$locations))
		ret = as.data.frame(ret)

	ret
}

# $Id: gstat.q,v 1.28 2009-11-02 21:33:17 edzer Exp $

"cross.name" <- function(id1, id2) {
    paste(id1, id2, sep = ".")
}

"gstat" <-
function (g, id, formula, locations,
	data = NULL, model = NULL, beta,
	nmax = Inf, nmin = 0, omax = 0, maxdist = Inf, force = FALSE,
	dummy = FALSE, set, fill.all = FALSE, fill.cross = TRUE,
	variance = "identity", weights = NULL, merge, degree = 0, vdist = FALSE,
	lambda = 1.0) 
{
	call = match.call()
	if (!missing(locations) && inherits(locations, "formula")) {
		if (!is.null(data))
			coordinates(data) = locations
		# locations = NULL
	} else if (missing(data) && !missing(locations) && 
			(is(locations, "Spatial") || inherits(locations, "sf"))) {
		data = locations
		locations = NULL
	}
	if (fill.all) {
	# fill all variogram models
		if (missing(g) || is.null(model))
			stop("fill.all assumes object g and model are supplied")
        g.names = names(g$data)
		for (i in 1:length(g.names)) {
           	g$model[[paste(g.names[i])]] = model
			if (fill.cross) {
				for (j in (i+1):length(g.names))
           			g$model[[cross.name(g.names[i], g.names[j])]] = model
			}
		}
        return(g)
	} 
    if (!missing(g) && inherits(g, "gstat") && !missing(id) && 
        !missing(model) && missing(formula) && missing(locations)) {
		# here, only direct or cross variogram model is defined
        g.names = names(g$data)
		if (length(id) == 2) {
           	m1 = match(id[1], g.names)
           	m2 = match(id[2], g.names)
           	if (is.na(m1)) 
               	stop("first id does not match available data")
           	if (is.na(m1)) 
               	stop("second id does not match available data")
           	nm = cross.name(g.names[min(m1, m2)], g.names[max(m1, m2)])
        } else if (length(id) == 1) {
			m1 = match(id, g.names)
        	if (is.na(m1)) 
           		stop("id does not match available data")
			nm = g.names[m1]
		} else
			stop("id should have length 1 or 2")
        g$model[[nm]] = model
        return(g)
    }
	if (!inherits(formula, "formula"))
        stop("argument formula should be of class formula")
    #if (!inherits(locations, "formula") && !has.coordinates(data))
    #	stop("argument locations should be of class formula or matrix")
	if (inherits(data, "sf"))
		data = as(data, "Spatial")
    if (missing(beta) || is.null(beta)) 
        beta = numeric(0)
	vfn = pmatch(variance, c("identity", "mu", "mu(1-mu)", "mu^2", "mu^3"))
	if (is.na(vfn))
		stop("unknown value for variance function")
	if (vfn > 1 && length(beta) == 0)
		stop("non-identity variance function only allowed if beta is supplied")
    if (missing(g) || is.null(g)) {
        g = list()
        g[["data"]] = list()
        g[["model"]] = list()
    } else if (!dummy && inherits(g$data[[1]], "Spatial") && !identical(g$data[[1]]$data@proj4string@projargs, data@proj4string@projargs))
		stop("data items in gstat object have different coordinate reference systems")
    if (missing(id)) 
        id = paste("var", length(g$data) + 1, sep = "")
    g$data[[id]] = list(formula = formula, # locations = locations, 
        data = data, has.intercept = attr(terms(formula), "intercept"),
		beta = beta, nmax = nmax, nmin = nmin, omax = omax,
		maxdist = maxdist, force = force,
		dummy = dummy, vfn = vfn, weights = weights, degree = degree, 
		vdist = vdist, lambda = lambda)
    g$model[[id]] = model
	if (!missing(locations))
		g$locations = locations
    if (!missing(set)) {
        if (!is.list(set)) 
            stop("argument set should be a list")
        g$set = set
    }
	if (!missing(merge))
		g$merge = merge
	g$call = call
    class(g) = c("gstat", "list")
    g
}

"[.gstat" <- function(x, ids) { 
	if (is.numeric(ids)) {
		if (min(ids) < 1 || max(ids) > length(names(x$data)))
			stop("selection index(es) out of bound")
		ids = names(x$data)[ids]
	} else if (any(is.na(match(ids, names(x$data)))))
		stop("selected ids do not match those of gstat object")
	g = list()
	g$data = x$data[ids]
	if (length(ids) > 1) {
		ids.cross = NULL
		for (i in 2:length(ids))
			for (j in 1:(i-1))
				ids.cross = c(ids.cross, cross.name(ids[j], ids[i]))
		g$model = x$model[c(ids, ids.cross)]
	} else
		g$model = x$model[ids]
	if (!is.null(x$set))
		g$set = x$set
	if (!is.null(g$merge))
		g$merge = x$merge
    class(g) = c("gstat", "list")
	g
}

## Benedikt Gräler (52North), 2018-05-25:
## circulant embedding following: Davies, Tilman M., and David Bryant. "On
## circulant embedding for Gaussian random fields in R." Journal of Statistical
## Software 55.9 (2013): 1-21.
## See i.e. the suplementary files at (retreived 2018-05-25): 
## https://www.jstatsoft.org/index.php/jss/article/downloadSuppFile/v055i09/v55i09.R


# extend the grid 
# input: SpatialGrid/SpatialPixels/GridTopology
# output: extended grid of class GridTopology
ceExtGrid <- function(grid, ext=2) {
  if (!inherits(grid, "GridTopology")) {
    stopifnot(gridded(grid))
    
    grid <- grid@grid
  }
  
  GridTopology(grid@cellcentre.offset,
               grid@cellsize,
               grid@cells.dim*ext)
}

# only for comaprission following the above paper
# expand and wrap a grid on a torus + calc distances
# input: SpatialGrid
# output: distance matrix
ceWrapOnTorusCalcDist <- function(grid, ext=2) {
  grid <- ceExtGrid(grid, ext)
  
  rangeXY <- grid@cellsize * grid@cells.dim
  
  MN.ext <- prod(grid@cells.dim)
  gridCoords <- coordinates(grid)
  
  mmat.ext <- matrix(rep(gridCoords[, 1], MN.ext), MN.ext, MN.ext)
  nmat.ext <- matrix(rep(gridCoords[, 2], MN.ext), MN.ext, MN.ext)
  
  mmat.diff <- mmat.ext - t(mmat.ext)
  nmat.diff <- nmat.ext - t(nmat.ext)
  
  mmat.torus <- pmin(abs(mmat.diff), rangeXY[1] - abs(mmat.diff))
  nmat.torus <- pmin(abs(nmat.diff), rangeXY[2] - abs(nmat.diff))
  
  sqrt(mmat.torus^2 + nmat.torus^2)
}

## FFT preparation with only first row of cov-matrix
ceWrapOnTorusCalcCovRow1 <- function(grid, vgmModel, ext=2) {
  grid <- ceExtGrid(grid, ext)
  
  stopifnot("variogramModel" %in% class(vgmModel))
  
  rangeXY <- grid@cellsize * grid@cells.dim
  
  cenX <- seq(from = grid@cellcentre.offset[1],
              by = grid@cellsize[1],
              length.out = grid@cells.dim[1])
  cenY <- seq(from = grid@cellcentre.offset[2],
              by = grid@cellsize[2],
              length.out = grid@cells.dim[2])
  
  m.diff.row1 <- abs(cenX[1] - cenX)
  m.diff.row1 <- pmin(m.diff.row1, rangeXY[1] - m.diff.row1)
  
  n.diff.row1 <- abs(cenY[1] - cenY)
  n.diff.row1 <- pmin(n.diff.row1, rangeXY[2] - n.diff.row1)
  
  cent.ext.row1 <- expand.grid(m.diff.row1, n.diff.row1)
  D.ext.row1 <- matrix(sqrt(cent.ext.row1[, 1]^2 + cent.ext.row1[, 2]^2), 
                       grid@cells.dim[1], 
                       grid@cells.dim[2])
  
  variogramLine(vgmModel, dist_vector = D.ext.row1, covariance = T)
}

# simulate GRF with given covariance structure using fft
# @input
#   covMatRow1: the first row of the covariance matrix for the fft
#   n: number of simulations
#   cells.dim: the original dimrensions of the grid to clip from the larger embedded simulation
#   grid.index: grid.index of a SpatialPixels object to select the right pixels from the larger square-grid
# @output
#   matrix where each column holds one simulated GRF corresponding to cells.dim (and grid.index if appropriate)
ceSim <- function(covMatRow1, n=1, cells.dim, grid.index) {
  d <- dim(covMatRow1)
  dp <- prod(d)
  sdp <- sqrt(dp)
  prefix <- sqrt(Re(fft(covMatRow1, TRUE)))
  
  simFun <- function(x) {
    std <- rnorm(dp)
    realz <- prefix * (fft(matrix(std, d[1], d[2]))/sdp)
    as.numeric(Re(fft(realz, TRUE)/sdp)[1:cells.dim[1], 1:cells.dim[2]])
  }
  
  simFunGridIndex <- function(x) {
    std <- rnorm(dp)
    realz <- prefix * (fft(matrix(std, d[1], d[2]))/sdp)
    as.numeric(Re(fft(realz, TRUE)/sdp)[1:cells.dim[1], 1:cells.dim[2]])[grid.index]
  }
  
  if (missing(grid.index))
    do.call(cbind, lapply(1:n, simFun))
  else
    do.call(cbind, lapply(1:n, simFunGridIndex))
}

# computes the covariance matrixes and weights once, applied to series of
# variables/simulations where each variable/simulation is stored in one column of
# the multiVarMatrix copied from krige0 to avoid repeted calls to krige with
# multiple, identical inversions of the weights matrix

krigeMultiple <- function(formula, from, to, model, multiVarMatrix) {
  lst = extractFormula(formula, from, to)
  X = lst$X
  x0 = lst$x0
  ll = (!is.na(is.projected(from)) && !is.projected(from))
  s = coordinates(from)
  s0 = coordinates(to)
  
  V = variogramLine(model,
                    dist_vector = spDists(s, s, ll),
                    covariance = TRUE)
  v0 = variogramLine(model,
                     dist_vector = spDists(s, s0, ll),
                     covariance = TRUE)
  
  skwts = CHsolve(V, cbind(v0, X))
  ViX = skwts[, -(1:nrow(s0))]
  skwts = skwts[, 1:nrow(s0)]
  
  idPredFun <- function(sim) {
    sim <- matrix(sim, ncol = 1)
    beta = solve(t(X) %*% ViX, t(ViX) %*% sim)
    x0 %*% beta + t(skwts) %*% (sim - X %*% beta)
  }
  
  apply(multiVarMatrix, 2, idPredFun)
}

### pubic function ###
# @input: 
# formula: definition of the dependent variable
# data:    optional Spatial*DataFrame for conditional simulation
# newdata: SpatialGrid or SpatialPixels
# model:   variogram model of the GRF
# n:       number of desired simulations
# ext:     extension degree of the circulant embedding, default to 2
# @output
# SpatialPixels or SpatailGridDataFrame with (additional) n columns holding one (un)conditional simulation each

krigeSimCE <- function(formula, data, newdata, model, n = 1, ext = 2) {
  stopifnot(is(model, "variogramModel"))
  stopifnot(gridded(newdata))
  if (!missing(data))
    stopifnot(identical(data@proj4string@projargs, newdata@proj4string@projargs))
  
  varName <- all.vars(formula[[2]])
  
  condSim <- TRUE
  if (missing(data)) {
    condSim <- FALSE
    message("[No data provided: performing unconditional simulation.]")
  } else {
    message("[Performing conditional simulation.]")
  }
  
  # prepare covariance matrix
  covMat <- ceWrapOnTorusCalcCovRow1(newdata, model, ext = ext)
  
  # simulate
  sims <- ceSim(covMat, n, newdata@grid@cells.dim, newdata@grid.index)
  colnames(sims) <- paste0(varName, ".sim", 1:n)
  
  # bind simulations to newdata geometry
  if (!condSim) {
    if ("data" %in% slotNames(newdata))
      newdata@data <- cbind(newdata@data, sims)
    else
      addAttrToGeom(newdata, as.data.frame(sims))
    return(newdata)
  }
  
  # function call ends here if no data has been provided -> unconditional case
  
  ## conditioning
  # interpolate the observations to the simulation grid
  obsMeanField <- krige(formula, data, newdata, model)
  
  # interpolate to observation locations from the simulated grids for each simulation
  simMeanObsLoc <- krigeMultiple(as.formula(paste0("var1.pred ~", formula[[3]])),
                                 obsMeanField, data, model, sims)
  
  # interpolate from kriged mean sim at observed locations back to the grid for mean surface of the simulations
  simMeanFields <- krigeMultiple(as.formula(paste0(varName, "~", formula[[3]])),
                                 data, newdata, model, simMeanObsLoc)
  
  # add up the mean field and the corrected data
  sims <- obsMeanField@data$var1.pred + sims - simMeanFields
  
  # bind simulations to newdata geometry
  if ("data" %in% slotNames(newdata)) {
    newdata@data <- cbind(newdata@data, sims)
    return(newdata)
  }
  
  addAttrToGeom(newdata, as.data.frame(sims))
}

###
# Note: to avoid the smoothing effect, the irregular observation locations could also be independently simulated by e.g. their surrounding grid values
###

## circulant embedding for ST-ocvariance functions with grids along one spatial and one temporal axis


# inputs: 
#   hDiscrete = c(hStep, hn): spatial step width and number of steps
#   tDiscrete = c(tStep, tn): temporal step length and number of steps

# CAVE: hDiscrete and tDiscrete must have the correct spatial and temporal metrics

ceWrapSpaceTimeOnTorusCalcCovRow1 <- function(hDiscrete, tDiscrete, vgmStModel, ext=2, turningLayers=TRUE) {
  stopifnot(is(vgmStModel)  == "StVariogramModel")
  
  hDiscrete[2] <- hDiscrete[2]*ext
  tDiscrete[2] <- tDiscrete[2]*ext
  
  rangeST <- c(prod(hDiscrete), prod(tDiscrete))
  
  cenX <- seq(from = 0, by = hDiscrete[1], length.out = hDiscrete[2])
  cenY <- seq(from = 0, by = tDiscrete[1], length.out = tDiscrete[2])
  
  m.diff.row1 <- abs(cenX[1] - cenX)
  m.diff.row1 <- pmin(m.diff.row1, rangeST[1] - m.diff.row1)
  
  n.diff.row1 <- abs(cenY[1] - cenY)
  n.diff.row1 <- pmin(n.diff.row1, rangeST[2] - n.diff.row1)
  
  cent.ext.row1 <- expand.grid(m.diff.row1, n.diff.row1)
  D.ext.row1 <- matrix(sqrt(cent.ext.row1[, 1]^2 + cent.ext.row1[, 2]^2), 
                       hDiscrete[2], 
                       tDiscrete[2])
  colnames(cent.ext.row1) <- c("spacelag", "timelag")
  
  if (turningLayers) {
    return(matrix(tbOperator(vgmStModel, dist_grid = cent.ext.row1)$gamma,
                  hDiscrete[2], tDiscrete[2]))
  } 
  
  matrix(variogramSurface(vgmStModel, dist_grid = cent.ext.row1, 
                          covariance = TRUE)$gamma,
         hDiscrete[2], tDiscrete[2])
}


     

options(digits=6)
# illustrates the use of merge, for merging parameters accross variables:
# Z1=m+e1(s)
# Z2=m+e2(s)
# Z1 and Z2 each have a different variogram, but share the parameter m
# see documentation of gstat() function
library(gstat)
d1 = data.frame(x=c(0,2),y=c(0,0),z=c(0,1))
d2 = data.frame(x=c(0,2),y=c(2,2),z=c(4,5))
g = gstat(NULL,"d1", z~1,~x+y,d1,model=vgm(1, "Exp", 1))
g = gstat(g,"d2", z~1,~x+y,d2,model=vgm(1, "Exp", 1), merge=c("d1","d2"))
g = gstat(g, c("d1", "d2"), model = vgm(0.5, "Exp", 1))
predict(g, data.frame(x=1,y=1), debug = 32)

# Z1 and Z2 share a regression slope:
g = gstat(NULL,"d1", z~x,~x+y,d1,model=vgm(1, "Exp", 1))
g = gstat(g,"d2", z~x,~x+y,d2,model=vgm(1, "Exp", 1), 
	merge=list(c("d1",2,"d2",2)))
g = gstat(g, c("d1", "d2"), model = vgm(0.5, "Exp", 1))
predict(g, data.frame(x=1,y=1), debug = 32)

# $Id: map.to.lev.q,v 1.2 2006-02-10 19:01:07 edzer Exp $

"map.to.lev" <-
function (data, xcol = 1, ycol = 2, zcol = c(3, 4), ns = names(data)[zcol]) 
{
    len = nrow(data)
    d = matrix(nrow = len * length(zcol), ncol = 3)
    xnames = NULL
    if (length(ns) > 1 && length(ns) != length(zcol)) 
        stop("names should have length 1 or equal to length of zcol")
    nr = 1
    for (i in zcol) {
        if (length(ns) == 1) 
            nm = rep(paste(ns, nr), len)
        else nm = rep(ns[nr], len)
        range = (1 + (nr - 1) * len):(nr * len)
        d[range, ] = cbind(data[, xcol], data[, ycol], data[, 
            i])
        xnames = c(xnames, nm)
        nr = nr + 1
    }
	nms <- factor(xnames, levels = unique(xnames))
    d = data.frame(d, nms)
    names(d) = c("x", "y", "z", "name")
    d
}

# $Id: krige.cv.q,v 1.18 2009-10-30 16:11:21 edzer Exp $

if (!isGeneric("krige.cv"))
	setGeneric("krige.cv", function(formula, locations, ...)
		standardGeneric("krige.cv"))

krige.cv.locations = function (formula, locations, data = sys.frame(sys.frame(sys.parent())), 
	model = NULL, ..., beta = NULL, nmax = Inf, nmin = 0, maxdist = Inf, 
	nfold = nrow(data), verbose = interactive(), debug.level = 0) {

	gstat.cv(gstat(g = NULL, id = "var1", formula = formula, locations = 
		locations, data = data, model = model, beta = beta, nmax = nmax, 
		nmin = nmin, maxdist = maxdist, ...), nfold = nfold, verbose = verbose,
		debug.level = debug.level)
}
setMethod("krige.cv", c("formula", "formula"), krige.cv.locations)

krige.cv.spatial = function (formula, locations, model = NULL, ..., beta = NULL,
	nmax = Inf, nmin = 0, maxdist = Inf, nfold = nrow(locations), verbose = interactive(),
	debug.level = 0) {

	# data = locations 
	gstat.cv(gstat(g = NULL, id = "var1", formula = formula,
		data = locations, model = model, beta =
		beta, nmax = nmax, nmin = nmin, maxdist = maxdist,
		...), nfold = nfold, verbose = verbose, debug.level = debug.level) 
}
setMethod("krige.cv", c("formula", "Spatial"), krige.cv.spatial)

krige.cv.sf = function (formula, locations, model = NULL, ..., beta = NULL,
	nmax = Inf, nmin = 0, maxdist = Inf, nfold = nrow(locations), verbose = interactive(),
	debug.level = 0) {

	# data = locations 
	if (!requireNamespace("sf", quietly = TRUE))
		stop("sf required: install that first") # nocov

	sf::st_as_sf(gstat.cv(gstat(g = NULL, id = "var1", formula = formula,
		data = as(locations, "Spatial"), model = model, beta =
		beta, nmax = nmax, nmin = nmin, maxdist = maxdist,
		...), nfold = nfold, verbose = verbose, debug.level = debug.level))
}
setMethod("krige.cv", c("formula", "sf"), krige.cv.sf)

# $Id: gstat.formula.q,v 1.8 2007-06-08 06:45:52 edzer Exp $

"gstat.formula" <-
function (formula, data)
{
	# check for duplicated pixels; if yes coerce to SpatialPointsDataFrame:
	if (is(data, "SpatialPixels") && anyDuplicated(data@grid.index) != 0)
		gridded(data) = FALSE

    m = model.frame(terms(formula), as(data, "data.frame"), na.action = na.fail)
    Y = model.extract(m, "response")
    if (length(Y) == 0)
        stop("no response variable present in formula")
    Terms = attr(m, "terms")
    X = model.matrix(Terms, m)
    has.intercept = attr(Terms, "intercept")

	if (gridded(data))
		grid = gridparameters(data)
	else
		grid = numeric(0)

	xlevels = .getXlevels(Terms, m)

    list(y = Y, locations = coordinates(data), X = X, call = call,
        has.intercept = has.intercept, grid = as.double(unlist(grid)),
		xlevels = xlevels)
}

extractFormula = function(formula, data, newdata) {
  # extract y and X from data:
  m = model.frame(terms(formula), as(data, "data.frame"), na.action = na.fail)
  y = model.extract(m, "response")
  if (length(y) == 0)
    stop("no response variable present in formula")
  Terms = attr(m, "terms")
  X = model.matrix(Terms, m)
  
  # extract x0 from newdata:
  terms.f = delete.response(terms(formula))
  mf.f = model.frame(terms.f, newdata) #, na.action = na.action)
  x0 = model.matrix(terms.f, mf.f)
  list(y = y, X = X, x0 = x0)
}

idw0 = function(formula, data, newdata, y, idp = 2.0) {
  s = coordinates(data)
  s0 = coordinates(newdata)
  if (missing(y))
    y = extractFormula(formula, data, newdata)$y
  D = 1.0 / (spDists(s0, s) ^ idp)
  sumD = apply(D, 1, sum)
  D %*% y / sumD
}

CHsolve = function(A, b) {
  # solves A x = b for x if A is PD symmetric
  #A = chol(A, LINPACK=TRUE) -> deprecated
  A = chol(A) # but use pivot=TRUE?
  backsolve(A, forwardsolve(A, b, upper.tri = TRUE, transpose = TRUE))
}

krige0 <- function(formula, data, newdata, model, beta, y, ..., 
                   computeVar = FALSE, fullCovariance = FALSE) {
  
  if (inherits(data, "ST"))
  	stopifnot(identical(data@sp@proj4string@projargs, newdata@sp@proj4string@projargs))
  else
  	stopifnot(identical(data@proj4string@projargs, newdata@proj4string@projargs))
  lst = extractFormula(formula, data, newdata)
  X = lst$X
  x0 = lst$x0
  if (missing(y))
    y = lst$y
  ll = (!is.na(is.projected(data)) && !is.projected(data))
  s = coordinates(data)
  s0 = coordinates(newdata)
  if (is(model, "variogramModel")) {
    require(gstat)
    V = variogramLine(model, dist_vector = spDists(s, s, ll),
                      covariance = TRUE)
    v0 = variogramLine(model, dist_vector = spDists(s, s0, ll),
                       covariance = TRUE)
    c0 = variogramLine(model, dist_vector = c(0), covariance = TRUE)$gamma
  } else {
    V = model(data, data, ...)
    v0 = model(data, newdata, ...)
    if (computeVar) {
      if (is(newdata, "SpatialLines") || is(newdata, "SpatialPolygons"))
        stop("varying target support (SpatialLines, SpatialPolygons) is not implemented")
      c0 = as.numeric(model(newdata[1, drop=FALSE],
                            newdata[1, drop=FALSE]))
      # ?check this: provide TWO arguments, so model(x,y) can target
      # eventually Y, instead of measurements Z=Y+e
      # with e measurement error term e
    }
  }
  if (!missing(beta)) { # sk:
    skwts = CHsolve(V, v0)
    if (computeVar)
      var <- c0 - apply(v0*skwts, 2, sum)
  } else { # ok/uk -- need to estimate beta:
    skwts = CHsolve(V, cbind(v0, X))
    ViX = skwts[,-(1:nrow(s0))]
    skwts = skwts[,1:nrow(s0)]
    beta = solve(t(X) %*% ViX, t(ViX) %*% y)
    if (computeVar) { 
      Q = t(x0) - t(ViX) %*% v0
      var <- c0 - apply(v0*skwts, 2, sum) + 
        apply(Q * CHsolve(t(X) %*% ViX, Q), 2, sum)
    }
  }
  pred = x0 %*% beta + t(skwts) %*% (y - X %*% beta)
  if (computeVar) {
    if (fullCovariance) {
      corMat <- cov2cor(variogramLine(model, 
                                      dist_vector = spDists(s0, s0), covariance = TRUE))
      var <- corMat*matrix(sqrt(var) %x% sqrt(var), 
                           nrow(corMat), ncol(corMat))
    }
    list(pred = pred, var = var)
  } else
    pred
}

# define variogram model FUNCTION that can deal with x and y
# being of class SpatialPolygons OR SpatialPoints; SpatialGrid/Pixels are coerced to SpatialPolygons
vgmArea = function(x, y = x, vgm, ndiscr = 16, verbose = FALSE, covariance = TRUE) {
  if (gridded(x))
    x = as(x, "SpatialPolygons")
  if (gridded(y))
    y = as(y, "SpatialPolygons")
  stopifnot(is(x, "SpatialPolygons") || is(x, "SpatialPoints"))
  stopifnot(is(y, "SpatialPolygons") || is(y, "SpatialPoints"))
  stopifnot(is(vgm, "variogramModel"))
  nx = length(x)
  ny = length(y)
  V = matrix(NA, nx, ny)
  if (verbose)
    pb = txtProgressBar(style = 3, max = nx)
  for (i in 1:nx) {
    if (is(x, "SpatialPolygons"))
      px = spsample(x[i,], ndiscr, "regular", offset = c(.5,.5))
    else
      px = x[i,]
    for (j in 1:ny) {
      if (is(y, "SpatialPolygons"))
        py = spsample(y[j,], ndiscr, "regular", offset = c(.5,.5))
      else
        py = y[j,]
      D = spDists(px, py)
      D[D == 0] = 1e-10
      V[i,j] = mean(variogramLine(vgm, dist_vector = D, 
                                  covariance = covariance))
    }
    if (verbose)
      setTxtProgressBar(pb, i)
  }
  if (verbose)
    close(pb)
  V
}

# $Id: variogram.gstat.q,v 1.9 2007-04-06 11:29:58 edzer Exp $

"variogram.gstat" = function (object, ...) {
	if (!inherits(object, "gstat"))
		stop("first argument should be of class gstat")
	y = list()
	locations = list()
	X = list()
	beta = list()
	grid = list()
	projected = TRUE
	for (i in seq(along.with = object$data)) {
		d = object$data[[i]]
		beta[[i]] = d$beta
		if (i > 1 && !identical(object$data[[1]]$data@proj4string@projargs, d$data@proj4string@projargs))
			stop("data items in gstat object have different coordinate reference systems")
		raw = gstat.formula(d$formula, d$data)
		y[[i]] = raw$y
		locations[[i]] = raw$locations
		X[[i]] = raw$X
		grid[[i]] = raw$grid
		if (is(d$data, "Spatial"))
			projected = is.projected(d$data)
		if (d$degree != 0)
			stop("degree != 0: residual variograms wrt coord trend using degree not supported")
	}
	names(y) = names(locations) = names(X) = names(object$data)
	# call variogram.default() next:
	variogram(y, locations, X, trend.beta = beta, grid = grid, g = object, ...,
		projected = projected)
}

# $Id: show.vgms.q,v 1.6 2008-12-15 14:27:29 edzer Exp $

"show.vgms" <-
function(min = 1e-12 * max, max = 3, n = 50, sill = 1, range = 1,
	models = as.character(vgm()$short[c(1:17)]), nugget = 0, kappa.range = 0.5,
	plot = TRUE, ..., as.groups = FALSE) 
{

	zero.range.models = c("Nug", "Int", "Lin", "Err")
	# print(models)
	L = max(length(sill), length(range), length(nugget), length(models), length(kappa.range))
	sill = rep(sill, length.out = L)
	range = rep(range, length.out = L)
	nugget = rep(nugget, length.out = L)
	i = 0
	if (length(kappa.range) > 1) { # loop over kappa values for Matern model:
		if (missing(models))
			models = "Mat"
		stopifnot(models == "Mat" || models == "Ste" || models == "Exc")
		data = matrix(NA, n * length(kappa.range), 2)
		v.level = rep("", n * length(kappa.range))
		for (kappa in kappa.range) {
			v = vgm(sill[i+1], models, range[i+1], nugget = nugget[i+1], kappa = kappa)
			x = variogramLine(v, 0, 1, 0)
			data[(i*n+1), ] = as.matrix(x)
			x = variogramLine(v, max, n - 1, min)
			data[(i*n+2):((i+1)*n), ] = as.matrix(x)
			m.name = paste("vgm(", sill[i+1], ",\"", models, "\",", range, sep = "")
			if (nugget[i+1] > 0)
				m.name = paste(m.name, ",nugget=", nugget[i+1], sep = "")
			m.name = paste(m.name, ",kappa=", kappa, ")", sep = "")
			v.level[(i*n+1):((i+1)*n)] = rep(m.name, n)
			i =  i + 1
		}
	} else {
		models = rep(models, length.out = L)
		data = matrix(NA, n * length(models), 2)
		v.level = rep("", n * length(models))
		for (m in models) {
			this.range = ifelse(!is.na(pmatch(m, zero.range.models)), 0, range[i+1])
			v = vgm(sill[i+1], m, this.range, nugget = nugget[i+1], kappa = kappa.range)
			x = variogramLine(v, 0, 1, 0)
			data[(i*n+1), ] = as.matrix(x)
			x = variogramLine(v, max, n - 1, min)
			data[(i*n+2):((i+1)*n), ] = as.matrix(x)
			m.name = paste("vgm(", sill[i+1], ",\"", m, "\",", this.range, sep = "")
			if (nugget[i+1] > 0)
				m.name = paste(m.name, ",nugget=", nugget[i+1], sep = "")
			m.name = paste(m.name, ")", sep = "")
			v.level[(i*n+1):((i+1)*n)] = rep(m.name, n)
			i = i + 1
		}
	}
	dframe = data.frame(semivariance = data[,2], distance = data[,1], 
		model = factor(v.level, levels = unique(v.level)))
	vgm.panel = function(x,y, ...) {
		n = length(x)
		lpoints(x[1],y[1])
		llines(x[2:n],y[2:n])
	}
	vgm.panel2 = function(x, y, subscripts, groups, ...) {
		lpoints(0, 0, col = 1)
		panel.superpose(x, y, subscripts, groups, ...)
	}
	if (!plot)
		dframe
	else {
		if (as.groups) {
			model = 0 # avoid NOTE on cran check
			xyplot(semivariance ~ distance, groups = model, dframe[dframe$distance > 0,], 
				panel = vgm.panel2, as.table = TRUE, auto.key = TRUE, type = 'l', ...)
		} else
			xyplot(semivariance ~ distance | model, dframe,
				panel = vgm.panel, as.table = TRUE, ...)
	}
}

# try bivariate cokriging; cross validate first variable
library(sp)
data(meuse)
library(gstat)
g=gstat(NULL, "log-zinc", log(zinc)~1,  ~x+y, meuse, nmax=10)
g=gstat(g, "log-lead", log(lead)~1,     ~x+y, meuse, nmax=10)
g=gstat(g, "log-copper", log(copper)~1, ~x+y, meuse, nmax=10)
v=variogram(g)
g=gstat(g, model=vgm(1, "Sph", 1000), fill.all=T)
g=fit.lmc(v, g)
g
set.seed(13131)
summary(gstat.cv(g, remove.all=TRUE, nfold=5))
summary(gstat.cv(g, remove.all=FALSE, nfold=5))

options(digits = 4)
set.seed(1131)
d = data.frame(x=runif(50),y=runif(50),z=runif(50),v=rnorm(50))
library(gstat)
xx = krige.cv(v~1,~x+y+z,d,model=vgm(1,"Exp",1), verbose=F, set=list(debug=0))
summary(xx)

%% Document started, Sat Jul  3 19:30:52 CEST 2004, my 37th birthday,
%% while being stuck for 24 hours at Philadelphia airport, on my way 
%% back from the joint TIES/Accuracy 2004 symposium in Portland, ME.
%% Continued, Oct 28, during the Apmosphere mid-term review. Oh, shame.

% \VignetteIndexEntry{ The pairwise relative semivariogram }

\documentclass[a4paper]{article}
\usepackage[colorlinks=true,urlcolor=blue]{hyperref}
\usepackage{alltt}

\newcommand{\code}[1]{{\tt #1}}

\SweaveOpts{echo=TRUE}

\title{The pairwise relative semivariogram}

\author{ \includegraphics[width=.4\columnwidth]{ifgi-logo_int}\\
\href{mailto:edzer.pebesma@uni-muenster.de}{Edzer Pebesma}
}
\date{\small Aug 29, 2011 }

\begin{document}
\maketitle

\section{Introduction}
The general relative variogram (Deutsch and Journel, 1997) is defined
as
$$
\gamma(h) = \frac{1}{2N_h} \sum_{i=1}^{N_h} 
\left(\frac{2(Z(s_i)-Z(s_i+h))}{Z(s_i)+Z(s_i+h)}\right)^2.
$$
It is claimed to reveal spatial structure (correlation) better when
data are skewed and/or clustered. The \code{cluster.dat} data set
used in this vignette, from the GSLIB distribution\footnote{F77
source code for Linux, downloaded Aug 28, 2011 from
\code{http://www.gslib.com/}}, seems to confirm this.

From version 1.02 on, R package \code{gstat} provides computation
of the {\em pairwise relative semivariogram}. The following code
provides an example and verification of the computation using direct
R code and using the GSLIB program \code{gamv}.

The following code imports the \code{cluster.dat} data from GSLIB,
which has been converted to have a single-line header containing
column names, packaged with the R gstat package, and converts it
into a \code{SpatialPointsDataFrame} object:
<<>>=
library(gstat)
cluster = read.table(system.file("external/cluster.txt", package="gstat"),
	header = TRUE)
summary(cluster)
library(sp)
coordinates(cluster) = ~X+Y
@
The following commands specify a sequence of lag boundaries that
correspond to the GSLIB conventions, and compute a regular variogram
using these boundaries:
<<>>=
bnd = c(0,2.5,7.5,12.5,17.5,22.5,27.5,32.5,37.5,42.5,47.5,52.5)
variogram(Primary~1, cluster, boundaries = bnd)
@

To compute the relative pairwise variogram, the logical argument
\code{PR} ({\em pairwise relative}) needs to be set to \code{TRUE}:
<<>>=
variogram(Primary~1, cluster, boundaries=bnd, PR = TRUE)
@
Figure \ref{fig:vgm} shows the two variograms, as plots, side by side
\begin{figure}
<<echo=FALSE,fig=TRUE>>=
pl1 = plot(variogram(Primary~1, cluster, boundaries=bnd, PR = FALSE))
pl2 = plot(variogram(Primary~1, cluster, boundaries=bnd, PR = TRUE))
print(pl1, split = c(1,1,2,1), more = TRUE)
print(pl2, split = c(2,1,2,1), more = FALSE)
@
\caption{Regular variogram (left) and pairwise relative variogram
(right) for the GSLIB data set \code{cluster.dat}.}
\label{fig:vgm}
\end{figure}

\section{Verification with plain R code}

The following R code reproduces the relative pairwise semivariogram
values for the first three lags, i.e. 0-2.5, 2.5-7.5 and 7.5-12.5.
<<>>=
z = cluster$Primary
d = spDists(cluster)
zd = outer(z, z, "-")
zs = outer(z, z, "+")
pr = (2 * zd / zs )^2
prv = as.vector(pr)
dv = as.vector(d)
mean(prv[dv > 0 & dv < 2.5])/2
mean(prv[dv > 2.5 & dv < 7.5])/2
mean(prv[dv > 7.5 & dv < 12.5])/2
@

\section{Verification with GSLIB}

In a verification with the GSLIB (Deutsch and Journel, 1997) code
of \code{gamv}, the following file was used:
\begin{alltt}
                  Parameters for GAMV
                  *******************

START OF PARAMETERS:
../data/cluster.dat               \\file with data
1   2   0                         \\   columns for X, Y, Z coordinates
1   3                             \\   number of varables,column numbers
-1.0e21     1.0e21                \\   trimming limits
gamv.out                          \\file for variogram output
10                                \\number of lags
5.0                               \\lag separation distance
2.5                               \\lag tolerance
1                                 \\number of directions
0.0  90.0 50.0   0.0  90.0  50.0  \\azm,atol,bandh,dip,dtol,bandv
0                                 \\standardize sills? (0=no, 1=yes)
2                                 \\number of variograms
1   1   1                         \\tail var., head var., variogram type
1   1   6                         \\tail var., head var., variogram type
\end{alltt}
Running this program with these parameters gave the following output:
\begin{alltt}
Semivariogram           tail:Primary      head:Primary       direction  1 
   1         .000       .00000      280        4.35043        4.35043
   2        1.528     58.07709      298        8.62309        8.62309
   3        5.473     54.09188     1248        5.41315        5.41315
   4       10.151     48.85144     1978        4.42758        4.42758
   5       15.112     40.08909     2498        4.25680        4.25680
   6       20.033     42.45081     2296        3.74311        3.74311
   7       25.020     48.60365     2734        4.09575        4.09575
   8       29.996     46.88879     2622        4.15950        4.15950
   9       34.907     44.36890     2170        3.77190        3.77190
  10       39.876     47.34666     1808        4.54173        4.54173
  11       44.717     38.72725     1222        5.15251        5.15251
  12       49.387     30.67908      438        4.56539        4.56539
Pairwise Relative       tail:Primary      head:Primary       direction  1 
   1         .000       .00000      280        4.35043        4.35043
   2        1.528       .36084      298        8.62309        8.62309
   3        5.473       .63071     1248        5.41315        5.41315
   4       10.151       .83764     1978        4.42758        4.42758
   5       15.112       .77691     2498        4.25680        4.25680
   6       20.033       .87746     2296        3.74311        3.74311
   7       25.020       .89610     2734        4.09575        4.09575
   8       29.996       .90023     2622        4.15950        4.15950
   9       34.907       .96043     2170        3.77190        3.77190
  10       39.876       .90554     1808        4.54173        4.54173
  11       44.717       .75545     1222        5.15251        5.15251
  12       49.387       .82268      438        4.56539        4.56539
\end{alltt}
As can be seen, the values in the third column (semivariogram for
the first section, pairwise relative semivariogram for the second)
correspond to the output generated by \code{variogram} of package
\code{gstat}.  Two differences with respect to the gstat output are:
\begin{itemize}
\item for the first lag with distance zero, GSLIB reports that
the semivariance value is zero based on 280 point pairs; 
\item the number of point pairs in GSLIB is double the number reported
by gstat. 
\end{itemize}

The ground for these differences seems that the GSLIB \code{gamv}
uses a single routine for computing variograms as well as
cross variograms and cross covariances. For cross variograms or
covariograms, considering two variables $Z_a$ and $Z_b$ each having
$N$ observations, the $N^2$ point pairs $Z_a(s_i),Z_b(s_i+h)$ and
$Z_a(s_i+h),Z_b(s_i)$ need to be evaluated, and all contribute
information. 

For direct (non-cross) variograms or covariograms, $Z_a=Z_b$
and the $N^2$ pairs considered contain the $N$ trivial pairs
$(Z(s_i)-Z(s_i))^2=0$, which contribute no information, as well as
all duplicate pairs, i.e. in addition to $(Z(s_i)-Z(s_i+h))^2$, the
identical pair $(Z(s_i+h)-Z(s_i))^2$ is also considered. This leads
to correct variogram value estimates, but incorrect unique point pair
numbers.  (Data set \code{cluster} contains $N=140$ observations.)

In contrast, \code{gstat} considers (and reports) only the number
of unique pairs for each lag.

\section*{References}
\begin{itemize}
\item Deutsch, C.V., A.G. Journel, 1997.  GSLIB: Geostatistical
Software Library and User's Guide, second edition.  Oxford University
Press.

\end{itemize}

\end{document}

%% Document started, Sat Jul  3 19:30:52 CEST 2004, my 37th birthday,
%% while being stuck for 24 hours at Philadelphia airport, on my way 
%% back from the joint TIES/Accuracy 2004 symposium in Portland, ME.
%% Continued, Oct 28, during the Apmosphere mid-term review. Oh, shame.

\documentclass[a4paper]{article}
\usepackage[colorlinks=true,urlcolor=blue]{hyperref}

\newcommand{\code}[1]{{\tt #1}}

\SweaveOpts{echo=TRUE}

\title{Introduction to Spatio-Temporal Variography }
% \VignetteIndexEntry{ Introduction to Spatio-Temporal Variography }

\author{ \includegraphics[width=.4\columnwidth]{ifgi-logo_int}\\
\href{mailto:edzer.pebesma@uni-muenster.de}{Edzer Pebesma},
\href{mailto:ben.graeler@uni-muenster.de}{Benedikt Gr\"{a}ler}
}
\date{\small \today }

\begin{document}
\setkeys{Gin}{width=0.9\textwidth}

\maketitle

\section{Introduction}

Since \code{gstat} package version 1.0-0, a dependency of gstat on
the R package \code{spacetime} was introduced, allowing the code in
\code{gstat} to exploit spatio-temporal data structures from that
package. This vignette describes the possibilities and limitations
of the package for spatio-temporal geostatistics.

To understand some of the possibilities and limitations, some
knowledge of the history of the software is needed. The original
\code{gstat} software (Pebesma and Wesseling, 1998) was a standalone
computer {\em program} written in around 25,000 lines of C code, and
would do geostatistical modelling, prediction and simulation. The
\code{gstat} R package (Pebesma, 2004) consisted mostly of an R
interface to this C code, together with convenience functions to
use R's modelling interface (formula's, see \code{?lm}) and graphic
capabilities (trellis graphics in package \code{lattice} to show
cross variogram as matrix plots; interaction with variogram clouds
using base plots).

Starting 2003, a group of programmers developed a set of classes
and methods for dealing with spatial data in R (points, lines,
polygons, grids), which was supported by the publications
of the well-known ASDAR book (Bivand et al. 2008; see also
\url{https://asdar-book.org/}) and helped convergence in the user
community, with in 2011 over 2000 subscribers on the {\tt r-sig-geo}
mailing list.  Package \code{gstat} was one of the first packages
that adopted and benefited from these classes.

To realize a particular idea, writing code in C typically takes
about 10-20 times as long as writing it in R. C code can be more
efficient, gives more control over memory usage, but is also more
error prone--mistakes in C code make an R session crash, something
that is hard to do when writing R code.

The original C code of \code{gstat} (Pebesma and Wesseling, 1998)
provides all kriging varieties (universal, ordinary, simple;
univariable, or multivariable as in cokriging) for two- or
three-dimensional data. When the spatial domain is constrained to
two dimensions (and this might cover over 99\% of the use cases!),
the third dimension might be used to represent time. As such, the
{\em metric} variogram model, which allows for geometric anisotropy
definition in three dimensions, can be used for spatio-temporal
kriging. When defining the three-dimensional variogram as the
sum of 2 or more nested variogram (summed) models, one can choose
anisotropy coefficients for a single model such that this model is
{\em effectively} zero in some directions, e.g. in space {\em or}
in time; this allows one to approximate the so-called space-time sum
model. It should be noted that at the C code there is no knowledge
whether a third dimension represents space, or time. As such,
particular characteristics of time cannot be taken care of.

Since the second half of 2010, the development of an R package
\code{spacetime} started. It provides methods and classes for
spatio-temporal data, and builds on the spatial data classes in
\code{sp} and time series classes in \code{xts}. This document will
explain how data in this form, and methods provided by this package,
can be used for spatio-temporal geostatistics.

We will work with a data set with air quality (PM10) measurements over
germany, taken from rural background stations available in the data
sets provided by the European Environmental Agency.
<<>>=
library(spacetime)
rm(list = ls())
data(air)
ls()
@

\section{Variography}

\subsection{Temporal autocorrelation and cross correlation}

We will look into a subset of the data, ranging from 2005 to 2010, and
remove stations that have only missing values in this period:
<<>>=
if (!exists("rural"))
	rural = STFDF(stations, dates, data.frame(PM10 = as.vector(air)))
rr = rural[,"2005::2010"]
unsel = which(apply(as(rr, "xts"), 2, function(x) all(is.na(x))))
r5to10 = rr[-unsel,]
summary(r5to10)
@
Next, we will (rather arbitrarily) select four stations, which have the
following labels:
<<>>=
rn = row.names(r5to10@sp)[4:7]
rn
@

In the following, autocorrelation functions are computed and plotted.
The resulting plot is shown in Figure~\ref{fig:acf}.
<<fig=FALSE,eval=FALSE>>=
par(mfrow=c(2,2))
# select 4, 5, 6, 7
for (i in rn) {
  acf(na.omit(r5to10[i,]$PM10), main = i)
}
par(mfrow=c(1,1))
@

\begin{figure}[hbt]
\begin{center}
<<fig=TRUE>>=
par(mfrow=c(2,2))
# select 4, 5, 6, 7
rn = row.names(r5to10@sp)[4:7]
for (i in rn) {
  x <- as.numeric(na.omit(r5to10[i, ]))
  acf(x, main = i)
}
@
\end{center}
\caption{Autocorrelations for PM10; time lag unit in days.}
\label{fig:acf}
\end{figure}

Auto- and cross correlations can be computed when a multivariate time
series object is passed to {\tt acf}: 
<<fig=FALSE,echo=TRUE,eval=FALSE>>=
acf(na.omit(as(r5to10[rn,], "xts")))
@
The resulting plot is shown in Figure~\ref{fig:ccf}.
\begin{figure}[hbt]
\begin{center}
<<fig=TRUE,echo=FALSE>>=
acf(na.omit(as(r5to10[rn,], "xts")))
@
\end{center}
\caption{autocorrelations (diagonal) and cross correlations (off-diagonal) for
the four stations selected; time lag unit in days. }
\label{fig:ccf}
\end{figure}
From these graphs one should be able to observe the following
\begin{itemize}
\item autocorrelations for lag 0 are always 1
\item cross correlations for lag 0 are not always 1
\item cross correlations can be asymmetric, meaning that when
$\rho_{AB}(h)$ is the correlation between $Z(s_A,t)$ and $Z(s_B,t+h)$,
$$\rho_{AB}(h) = \rho_{BA}(-h) \ne \rho_{AB}(-h)$$
with $s_A$ and $s_B$ the two stations between which a cross correlation
is computed, and $h$ the (directional!) lag between the series.
\end{itemize}
The plot further more shows that for these four stations the
asymmetry is not very strong, but that cross correlations are fairly
strong and of a similar form of autocorrelations.

This kind of plot does not work very well in layouts of e.g. 10 x 10
sub-plots; {\tt acf} automatically chooses 4 x 4 as the maximum a
single plot. To try this out, do a 7 x 7 plot
<<eval=FALSE>>=
acf(na.omit(as(r5to10[4:10,], "xts")))
@
and note that here we see in the last figure (DESH \& DESN04) a pair
of plots with nearly no cross correlation. This might have to do with
the spatial distance between these two stations:
<<>>=
library(sp)
print(spDists(r5to10[4:10,]@sp), digits=3)
@
(What is the spatial distance between stations DESH and DESN04?)

\subsection{Spatial correlation, variograms}

In the next steps, we will sample 100 time instances randomly,
<<>>=
rs = sample(dim(r5to10)[2], 100)
@
we select these instances as a {\tt SpatialPointsDataFrame} and add a
time index to them. After this we bind them together in a single
{\tt SpatialPointsDataFrame} which has a time index {\tt ti}:
<<>>=
lst = lapply(rs, function(i) { x = r5to10[,i]; x$ti = i; rownames(x@coords) = NULL; x} )
pts = do.call(rbind, lst)
@
Then, we can compute the pooled variogram
<<>>=
library(gstat)
v = variogram(PM10~ti, pts[!is.na(pts$PM10),], dX=0)
@
and plot it (Figure~\ref{fig:vgm}):
<<eval=FALSE>>=
# plot(v, fit.variogram(v, vgm(1, "Exp", 200, 1)))
vmod = fit.variogram(v, vgm(100, "Exp", 200))
plot(v, vmod)
@
\begin{figure}[hbt]
\begin{center}
<<fig=TRUE,echo=FALSE>>=
# plot(v, fit.variogram(v, vgm(1, "Exp", 200, 1)))
vmod = fit.variogram(v, vgm(100, "Exp", 200))
print(plot(v, vmod))
@
\end{center}
\caption{sample spatial variogram, averaged over 100 randomly
chosen time steps}
\label{fig:vgm}
\end{figure}
The fitted model is this:
<<>>=
vmod
@
One should note that the fit is rather poor, and not forget that
we only have 53 stations selected. The time resolution is rich
(1862 days) but the number of stations is small:
<<>>=
dim(r5to10)
@

We can fit a spatio-temporal variogram the usual way, by passing an
object of class {\tt STFDF} (Pebesma, 2012):
<<eval=FALSE>>=
vv = variogram(PM10~1, r5to10, width=20, cutoff = 200, tlags=0:5)
@
Alternatively, if 
this takes too long, a temporal subset can be taken, e.g.
using the first 200 days:
<<eval=FALSE>>=
vv = variogram(PM10~1, r5to10, width=20, cutoff = 200, tlags=0:5)
@
taking random days from the full period will lead to the a wrong
assumption that every time index increment reflect a constant
lag increase. As an alternative, we will here load the precomputed 
S/T variogram:
<<>>=
data(vv)
@
% remove the model column to keep text and figures in line.
<<echo=FALSE>>=
vv <- vv[c("np", "dist", "gamma", "id", "timelag", "spacelag")]
@
Plotting this object can be done in several ways, two 2D-plots are 
shown in Figure~\ref{fig:map} and a 3D wireplot is shown in 
Figure~\ref{fig:wire}:
<<eval=FALSE>>=
plot(vv)
plot(vv, map = FALSE)
@
\begin{figure}[hbt]
\begin{center}
<<fig=TRUE,echo=FALSE>>=
print(plot(vv), split = c(1,1,1,2), more = TRUE)
print(plot(vv, map = FALSE), split = c(1,2,1,2))
@
\end{center}
\caption{Spatio-temporal sample variogram map (top) and sample 
variograms for each time lag (bottom); both figures depict the 
information of object {\tt vv}.}
\label{fig:map}
\end{figure}

\subsection{Fitting a spatio-temporal variogram model}

At first, we try to fit a metric model with spatio-temporal anisotropy:
<<>>==
metricVgm <- vgmST("metric",
                   joint=vgm(50,"Exp",100,0),
                   stAni=50)

metricVgm <- fit.StVariogram(vv, metricVgm)
@
As numerical criterion to judge the goodness of fit of model and 
sample variogram, the root-mean-squared-difference between the 
surfaces can be obtained by:
<<>>=
attr(metricVgm, "optim")$value
@

The final model can be plotted with the sample variogram (Figure~\ref{fig:mm}):

<<eval=FALSE>>=
plot(vv, metricVgm)
@

\begin{figure}[hbt]
\begin{center}
<<fig=TRUE,echo=FALSE,width=8, height=4>>=
print(plot(vv, metricVgm))
@
\end{center}
\caption{Sample variogram map (left) and fitted metric model (right).}
\label{fig:mm}
\end{figure}

\pagebreak 

Now, let us try to fit and plot a separable model (Figure~\ref{fig:sm}):
<<>>==
sepVgm <- vgmST("separable",
                space=vgm(0.9,"Exp", 123, 0.1),
                time =vgm(0.9,"Exp", 2.9, 0.1),
                sill=100)

sepVgm <- fit.StVariogram(vv, sepVgm, method = "L-BFGS-B",
                          lower = c(10,0,0.01,0,1),
                          upper = c(500,1,20,1,200))
@

To compare this model with the previous one, we look at the optimized 
root-mean-squared-differences between the two surfaces and plot sample
and both models:
<<>>=
attr(sepVgm, "optim")$value
plot(vv, list(sepVgm, metricVgm))
@

\begin{figure}[hbt]
\begin{center}
<<fig=TRUE,echo=FALSE,width=8,height=4>>=
print(plot(vv, list(sepVgm, metricVgm)))
@
\end{center}
\caption{Sample variogram map (left), fitted separable model (middle)
and fittted metric model (right).}
\label{fig:sm}
\end{figure}
A wireframe (3D) plot of sample variogram and fitted variogram models can be obtained e.g. by
<<eval=FALSE>>=
library(lattice)
plot(vv, list(sepVgm, metricVgm), all=T, wireframe=T, zlim=c(0,120),
     zlab=NULL,
     xlab=list("distance (km)", rot=30),
     ylab=list("time lag (days)", rot=-35),
     scales=list(arrows=F, z = list(distance = 5)))
@
which is shown in Figure~\ref{fig:wire}. Further spatio-temporal model
definitions can be found in the help pages of {\tt fit.StVariogram} 
and {\tt variogramSurface}. The demo {\tt stkrige} presents further 
examples and illustrates an interactive 3D-plot of sample variogram 
and the fitted variogram model. 
\begin{figure}[hbt]
\begin{center}
<<fig=TRUE,echo=FALSE,width=8,height=8>>=
library(lattice)
print(plot(vv, list(sepVgm, metricVgm), all=T, wireframe=T, zlim=c(0,120),
           zlab=NULL,
           xlab=list("distance (km)", rot=30),
           ylab=list("time lag (days)", rot=-35),
           scales=list(arrows=F, z = list(distance = 5))))
@
\end{center}
\caption{Wireframe plots of sample and fitted space-time variograms.}
\label{fig:wire}
\end{figure}

\clearpage 

\section{Spatio-temporal prediction}

The vignette in package \code{spacetime} gives an example of using
the gstat function \code{krigeST} for spatio-temporal kriging of
the Irish wind data. The \code{krigeST} function uses global kriging,
but only needs to invert the purely spatial and purely time covariance
matrices in the separable case.

For more generic spatio-temporal kriging where space is
two-dimensional, one could use \code{krige}, defining the
observations and prediction locations as three-dimensional data sets,
see for an example
<<eval=FALSE>>=
demo(gstat3D)
@
It needs to be pointed out that in that case, the time (typically
the third dimension) needs to be numeric, and three-dimensional
anisotropy needs to be defined properly (see \code{?vgm}).

In case the data set is too large for global kriging, one could
try to use local kriging, and select data within some distance, or
by specifying \code{nmax} (the nearest $n$ observations).  In both
cases, it is advisable to transform time such that one can use an
{\em isotropic} variogram model in the three dimensions, as only
in that case the nearest $n$ observations correspond to the $n$
most correlated observations. \code{krigeST} provides a solution 
where a \code{bufferNmax}-times larger neighbourhood is evaluated 
within the covariance model and the strongest correlated \code{nmax}
neighbours are selected.

An additional consideration is that in space-time, observations may
not be regularly spaced. In some cases, the nearest $n$ observations
may come from a single measurement location, which may lead to sharp
jumps/boundaries in the interpolated values. This might be solved
by using larger neighbourhoods, or by setting the \code{omax}
in \code{krige} or \code{gstat} calls to the neighbourhood size
to select {\em per octant} (this should be combined with specifying
\code{maxdist}).

%\section{Spatio-temporal simulation}

\section*{References}
\begin{itemize}
\item Bivand, R., E. Pebesma and V. Gomez-Rubio, 2008. Applied
Spatial Data Analysis with R. Springer.

\item Cressie, N.A.C., 1993. Statistics for Spatial Data. Wiley.

\item Cressie, N. and C. Wikle, 2011. Statistics for Spatio-temporal
Data. Wiley.

\item Pebesma, E., 2012. spacetime: Spatio-Temporal Data
in R. Journal of Statistical Software, volume 51, issue 7;
\href{https://www.jstatsoft.org/v51/i07/}{1-30}.

\item Pebesma, E.J., Wesseling, C.G., 1998. Gstat, a program for
geostatistical modelling, prediction and simulation. Computers \&
Geosciences, 24 (1), pp. 17--31.

\item Pebesma, E.J., 2004.  Multivariable geostatistics
in S: the gstat package.  Computers \& Geosciences
\href{http://www.sciencedirect.com/science/journal/00983004}{30:
683-691}

\item Ver Hoef, J.M., Cressie, N.A.C, 1993. Multivariable Spatial
Prediction.  Mathematical Geology, 25 (2), pp. 219--240.
\end{itemize}

\end{document}

%% Document started, Sat Jul  3 19:30:52 CEST 2004, my 37th birthday,
%% while being stuck for 24 hours at Philadelphia airport, on my way 
%% back from the joint TIES/Accuracy 2004 symposium in Portland, ME.
%% Continued, Oct 28, during the Apmosphere mid-term review. Oh, shame.

% \VignetteIndexEntry{ The meuse data set: a tutorial for the gstat R package }

\documentclass[a4paper]{article}
\usepackage{hyperref}

\newcommand{\code}[1]{{\tt #1}}

\SweaveOpts{echo=TRUE}

\title{The meuse data set: a brief tutorial\\
for the {\tt gstat} R package }
\author{\href{mailto:edzer.pebesma@uni-muenster.de}{Edzer Pebesma}}
\date{\today}

\begin{document}
\maketitle

\section{Introduction}
The \code{meuse} data set provided by package \code{sp} is a
data set comprising of four heavy metals measured in the top soil
in a flood plain along the river Meuse, along with a handful of
covariates. The process governing heavy metal distribution seems
that polluted sediment is carried by the river, and mostly deposited
close to the river bank, and areas with low elevation. This document
shows a geostatistical analysis of this data set. The data set was
introduced by Burrough and McDonnell, 1998.

This tutorial introduced the functionality of the R package \code{gstat},
used in conjunction with package \code{sp}. Package \code{gstat} provides
a wide range of univariable and multivariable geostatistical modelling,
prediction and simulation functions, where package \code{sp} provides
general purpose classes and methods for defining, importing/exporting
and visualizing spatial data.

\section{R geostatistics packages}
Package \code{gstat} (Pebesma, 2004) is an R package that provides basic
functionality for univariable and multivariable geostatistical analysis,
including 
\begin{itemize}
\item variogram modelling, residual variogram modelling, and cross variogram
modelling using fitting of parametric models to sample variograms
\item geometric anisotropy specfied for each partial variogram model
\item restricted maximum likelihood fitting of partial sills 
\item variogram and cross variogram maps
\item simple, ordinary, universal and external drift (co)kriging
\item (sequential) Gaussian (co)simulation equivalents for each of
the kriging varieties
\item indicator (co)kriging and sequential indicator (co)simulation
\item kriging in a local or global neighbourhood 
\item block (co)kriging or simulation for each of the varieties, for
rectangular or irregular blocks
\end{itemize}
Other geostatistical packages for R usually lack part of these options
(e.g. block kriging, local kriging, or cokriging) but provide others:
e.g. package \code{geoR} and \code{geoRglm} (by Paulo Ribeiro and Ole
Christensen) provide the model-based geostatistics framework described
in Diggle et al. (1998), package \code{fields} (Doug Nychka and others)
provides thin plate spline interpolation, covariance functions for
spherical coordinates (unprojected data), and routines for spatial
sampling design optimization.

\section{Spatial data frames}
As an example, we will look at the meuse data set, which is a
regular data frame that comes with package \code{gstat} (remove the
88 from the colour strings to make a plot without alpha transparency
on windows or X11):

<<fig=FALSE>>=
library(sp)
data(meuse)
class(meuse)
names(meuse)
coordinates(meuse) = ~x+y
class(meuse)
summary(meuse)
coordinates(meuse)[1:5,]
bubble(meuse, "zinc", 
	col=c("#00ff0088", "#00ff0088"), main = "zinc concentrations (ppm)")
@

% the following is needed because lattice plots (bubble wraps xyplot) do not
% show without an explicit print; in order not to confuse users, we hide this:
<<fig=TRUE,echo=FALSE>>=
print(bubble(meuse, "zinc", 
	col=c("#00ff0088", "#00ff0088"), main = "zinc concentrations (ppm)")
)
@

and note the following: 
\begin{enumerate}
\item the function \code{coordinates}, when assigned (i.e. on
the left-hand side of an \verb|=| or \verb|<-| sign), promotes the
\code{data.frame} meuse into a \code{SpatialPointsDataFrame}, which knows about
its spatial coordinates; coordinates may be specified by a formula,
a character vector, or a numeric matrix or data frame with the actual
coordinates
\item the function \code{coordinates}, when not assigned, {\em retrieves}
the spatial coordinates from a \code{SpatialPointsDataFrame}.
\item the two plotting functions used, \code{plot} and \code{bubble}
assume that the $x$- and $y$-axis are the spatial coordinates.
\end{enumerate}

\section{Spatial data on a regular grid}

<<fig=TRUE,echo=TRUE>>=
data(meuse.grid)
summary(meuse.grid)
class(meuse.grid)
coordinates(meuse.grid) = ~x+y
class(meuse.grid)
gridded(meuse.grid) = TRUE
class(meuse.grid)
image(meuse.grid["dist"])
title("distance to river (red = 0)")
library(gstat)
zinc.idw = idw(zinc~1, meuse, meuse.grid)
class(zinc.idw)
spplot(zinc.idw["var1.pred"], main = "zinc inverse distance weighted interpolations")
@

<<fig=TRUE,echo=FALSE>>=
print(spplot(zinc.idw["var1.pred"], main = "zinc inverse distance weighted interpolations"))
@

If you compare the bubble plot of zinc measurements with the map with
distances to the river, it becomes evident that the larger concentrations
are measured at locations close to the river. This relationship can be
linearized by log-transforming the zinc concentrations, and taking the
square root of distance to the river:

<<fig=TRUE,echo=TRUE>>=
plot(log(zinc)~sqrt(dist), meuse)
abline(lm(log(zinc)~sqrt(dist), meuse))
@

\section{Variograms }

Variograms are calculated using the function \code{variogram}, which
takes a formula as its first argument: \verb|log(zinc)~1| means that we
assume a constant trend for the variable log(zinc).

<<fig=FALSE>>=
lzn.vgm = variogram(log(zinc)~1, meuse)
lzn.vgm
lzn.fit = fit.variogram(lzn.vgm, model = vgm(1, "Sph", 900, 1))
lzn.fit
plot(lzn.vgm, lzn.fit)
@

<<fig=TRUE,echo=FALSE>>=
print(plot(lzn.vgm, lzn.fit))
@

Instead of the constant mean, denoted by \verb|~1|, we can specify a
mean function, e.g. using \verb|~sqrt(dist)| as a predictor variable:

<<fig=FALSE>>=
lznr.vgm = variogram(log(zinc)~sqrt(dist), meuse)
lznr.fit = fit.variogram(lznr.vgm, model = vgm(1, "Exp", 300, 1))
lznr.fit
plot(lznr.vgm, lznr.fit)
@

<<fig=TRUE,echo=FALSE>>=
print(plot(lznr.vgm, lznr.fit))
@

In this case, the variogram of residuals with respect to a fitted mean
function are shown. Residuals were calculated using ordinary least
squares.

\section{Kriging}

<<fig=FALSE>>=
lzn.kriged = krige(log(zinc)~1, meuse, meuse.grid, model = lzn.fit)
spplot(lzn.kriged["var1.pred"])
@

<<fig=TRUE,echo=FALSE>>=
print(spplot(lzn.kriged["var1.pred"]))
@

\section{Conditional simulation}
<<fig=FALSE>>=
lzn.condsim = krige(log(zinc)~1, meuse, meuse.grid, model = lzn.fit, 
    nmax = 30, nsim = 4)
spplot(lzn.condsim, main = "four conditional simulations")
@

<<fig=TRUE,echo=FALSE>>=
print(spplot(lzn.condsim, main = "four conditional simulations"))
@

For UK/residuals:

<<fig=FALSE>>=
lzn.condsim2 = krige(log(zinc)~sqrt(dist), meuse, meuse.grid, model = lznr.fit, 
    nmax = 30, nsim = 4)
spplot(lzn.condsim2, main = "four UK conditional simulations")
@

<<fig=TRUE,echo=FALSE>>=
print(spplot(lzn.condsim2, main = "four UK conditional simulations"))
@

\section{Directional variograms}
The following command calculates a directional sample variogram, where
directions are binned by direction angle alone. For two point pairs,
$Z(s)$ and $Z(s+h)$, the separation vector is $h$, and it has a direction.
Here, we will classify directions into four direction intervals:

<<fig=FALSE>>=
lzn.dir = variogram(log(zinc)~1, meuse, alpha = c(0, 45, 90, 135))
lzndir.fit = vgm(.59, "Sph", 1200, .05, anis = c(45, .4))
plot(lzn.dir, lzndir.fit, as.table = TRUE)
@

<<fig=TRUE,echo=FALSE>>=
print(plot(lzn.dir, lzndir.fit, as.table = TRUE))
@

Looking at directions between 180 and 360 degrees will repeat the
image shown above, because the variogram is a symmetric measure:
$(Z(s)-Z(s+h))^2=(Z(s+h)-Z(s))^2$.

The first plot gives the variogram in the zero direction, which is
North; 90 degrees is East. By default, point pairs are assigned to the
directional variorgram panel with their nearest direction, so North
contains everything between -22.5 and 22.5 degrees (North-West to
North-East). After classifying by direction, point pairs are binned by
separation distance class, as is done in the usual omnidirectional case.

In the figure, the partial sill, nugget and model type of the model are
equal to those of the omnidirectional model fitted above; the range
is that in the direction with the largest range (45$^o$), and the
anisotropy ratio, the range in the 135 direction and the range in the
45 direction, estimated ``by eye'' by comparing the 45 and 135 degrees
sample variograms. Gstat does not fit anisotropy parameters automatically.

We do not claim that the model fitted here is ``best'' in some way; in
order to get to a better model we may want to look at more directions,
other directions (e.g. try {\tt alpha = c(22, 67, 112, 157) }), and to
variogram maps (see below). More elaborate approaches may use directions
in three dimensions, and want to further control the direction tolerance
(which may be set such that direction intervals overlap).

For the residual variogram from the linear regression model using
\code{sqrt(dist)} as covariate, the directional dependence is much
less obvious; the fitted model here is the fitted isotropic model
(equal in all directions).

<<fig=FALSE>>=
lznr.dir = variogram(log(zinc)~sqrt(dist), meuse, alpha = c(0, 45, 90, 135))
plot(lznr.dir, lznr.fit, as.table = TRUE)
@

<<fig=TRUE,echo=FALSE>>=
print(plot(lznr.dir, lznr.fit, as.table = TRUE))
@

\section{Variogram maps}

Another means of looking at directional dependence in semivariograms
is obtained by looking at variogram maps. Instead of classifying
point pairs $Z(s)$ and $Z(s+h)$ by direction and distance class {\em
separately}, we can classify them {\em jointly}. If $h=\{x,y\}$ be the
two-dimentional coordinates of the separation vector, in the variogram
map the semivariance contribution of each point pair $(Z(s)-Z(s+h))^2$
is attributed to the grid cell in which $h$ lies. The map is centered
around $(0,0)$, as $h$ is geographical distance rather than geographical
location. Cutoff and width correspond to some extent to map extent
and cell size; the semivariance map is point symmetric around $(0,0)$,
as $\gamma(h)=\gamma(-h)$.

<<fig=FALSE>>=
vgm.map = variogram(log(zinc)~sqrt(dist), meuse, cutoff = 1500, width = 100, 
	map = TRUE)
plot(vgm.map, threshold = 5)
@

<<fig=TRUE,echo=FALSE>>=
print(plot(vgm.map, threshold = 5))
@

The threshold assures that only semivariogram map values based on at
least 5 point pairs are shown, removing too noisy estimation.

% The plot is plagued by one or two extreme values, corresponding to cells
% with very small number of point pairs, which should be removed.

\section{Cross variography}

Fitting a linear model of coregionalization.

<<fig=FALSE>>=
g = gstat(NULL, "log(zn)", log(zinc)~sqrt(dist), meuse)
g = gstat(g, "log(cd)", log(cadmium)~sqrt(dist), meuse)
g = gstat(g, "log(pb)", log(lead)~sqrt(dist), meuse)
g = gstat(g, "log(cu)", log(copper)~sqrt(dist), meuse)
v = variogram(g)
g = gstat(g, model = vgm(1, "Exp", 300, 1), fill.all = TRUE)
g.fit = fit.lmc(v, g)
g.fit
plot(v, g.fit)
vgm.map = variogram(g, cutoff = 1500, width = 100, map = TRUE)
plot(vgm.map, threshold = 5, col.regions = bpy.colors(), xlab = "", ylab = "")
@

<<fig=TRUE,echo=FALSE>>=
print(plot(v, g.fit))
@

<<fig=TRUE,echo=FALSE>>=
print(plot(vgm.map, threshold = 5, col.regions = bpy.colors(), ylab = "", xlab = ""))
@

\section*{References}
\begin{itemize}
% \item Abrahamsen, P., F. Espen Benth, 2001. Kriging with inequality
% constraints.  Mathematical Geology 33 (6), 719--744.

% \item Bivand, R.S., 2003. Approaches to classes for spatial data in R.
% In: K.~Hornik \& F.~Leisch (Eds.), Proceedings of the 3rd International
% Workshop on Distributed Statistical Computing (DSC 2003) March 20--22,
% Vienna, Austria. ISSN 1609-395X; available from [1].
\item Burrough, P.A., R.A. McDonnell, 1998.  Principles of Geographical
Information Systems, 2nd Edition. Oxford University Press.

\item Diggle, P.J., J.A. Tawn, R.A. Moyeed, 1998. Model-based
geostatistics. Applied Statistics 47(3), pp 299-350.

% \item Pebesma, E.J., Wesseling, C.G., 1998. Gstat, a program for
% geostatistical modelling, prediction and simulation. Computers \&
% Geosciences, 24 (1), pp. 17--31.

\item Pebesma, E.J., 2004.  Multivariable geostatistics
in S: the gstat package.  Computers \& Geosciences
\href{http://dx.doi.org/10.1016/j.cageo.2004.03.012}{30: 683-691}.

% \item Ver Hoef, J.M., Cressie, N.A.C, 1993. Multivariable Spatial
% Prediction.  Mathematical Geology, 25 (2), pp. 219--240.

\item Wackernagel, H., 1998. Multivariate Geostatistics; an introduction
with applications, $2^{\mbox{nd}}$ edn., Springer, Berlin, 291 pp.

\end{itemize}

\end{document}

# g = gstat(NULL, "log.zinc", log(zinc)~1, meuse)
# g = gstat(g, "log.zinc.res", log(zinc)~sqrt(dist), meuse)
# lplot(vgm.map[["map"]], c("log.zinc", "log.zinc.res"))

% vim:syntax=tex

% $Id: jura.Rd,v 1.3 2007-03-08 09:35:49 edzer Exp $
\name{jura}
\alias{jura}
\alias{prediction.dat}
\alias{validation.dat}
\alias{transect.dat}
\alias{juragrid.dat}
\alias{jura.grid}
\alias{jura.pred}
\alias{jura.val}
\title{Jura data set}
\description{
The jura data set from Pierre Goovaerts' book (see references
below). It contains four \code{data.frame}s: prediction.dat, validation.dat
and transect.dat and juragrid.dat, and three \code{data.frame}s with
consistently coded land use and rock type factors, as well as geographic
coordinates. The examples below show how to transform these into
spatial (sp) objects in a local coordinate system and in geographic
coordinates, and how to transform to metric coordinate reference
systems.
}
\format{
  The \code{data.frames} prediction.dat and validation.dat contain the following fields:
  \describe{
	\item{Xloc}{ X coordinate, local grid km }
	\item{Yloc}{ Y coordinate, local grid km }
	\item{Landuse}{ see book and below }
	\item{Rock}{ see book and below }
	\item{Cd}{ mg cadmium \eqn{\mbox{kg}^{-1}}{kg^-1} topsoil }
	\item{Co}{ mg cobalt \eqn{\mbox{kg}^{-1}}{kg^-1} topsoil }
	\item{Cr}{ mg chromium \eqn{\mbox{kg}^{-1}}{kg^-1} topsoil }
	\item{Cu}{ mg copper \eqn{\mbox{kg}^{-1}}{kg^-1} topsoil }
	\item{Ni}{ mg nickel \eqn{\mbox{kg}^{-1}}{kg^-1} topsoil }
	\item{Pb}{ mg lead \eqn{\mbox{kg}^{-1}}{kg^-1} topsoil }
	\item{Zn}{ mg zinc \eqn{\mbox{kg}^{-1}}{kg^-1} topsoil }
  }
  The \code{data.frame} juragrid.dat only has the first four fields.
  In addition the \code{data.frame}s jura.pred, jura.val and jura.grid also
  have inserted third and fourth fields giving geographic coordinates:
  \describe{
  	\item{long}{ Longitude, WGS84 datum }
	\item{lat}{ Latitude, WGS84 datum }
  } 
} 
\usage{
data(jura)
}
\author{ Data preparation by David Rossiter (dgr2@cornell.edu) 
and Edzer Pebesma; georeferencing by David Rossiter }
\references{ 
Goovaerts, P. 1997. Geostatistics for Natural Resources Evaluation. Oxford
Univ. Press, New-York, 483 p. Appendix C describes (and gives) the Jura
data set.

Atteia, O., Dubois, J.-P., Webster, R., 1994, Geostatistical analysis of
soil contamination in the Swiss Jura: Environmental Pollution 86, 315-327

Webster, R., Atteia, O., Dubois, J.-P., 1994, Coregionalization of trace
metals in the soil in the Swiss Jura: European Journal of Soil Science
45, 205-218

}
\note{
The points data sets were obtained from
http://home.comcast.net/~pgoovaerts/book.html, which seems to be
no longer available; the grid data were kindly provided by Pierre
Goovaerts.

The following codes were used to convert \code{prediction.dat}
and \code{validation.dat} to \code{jura.pred} and \code{jura.val}
(see examples below):

Rock Types: 1: Argovian, 2: Kimmeridgian, 3: Sequanian, 4: Portlandian,
5: Quaternary.

Land uses: 1: Forest, 2: Pasture (Weide(land), Wiese, Grasland),
3: Meadow (Wiese, Flur, Matte, Anger), 4: Tillage (Ackerland,
bestelltes Land)

Points 22 and 100 in the validation set
(\code{validation.dat[c(22,100),]}) seem not to lie exactly on the
grid originally intended, but are kept as such to be consistent with
the book.

Georeferencing was based on two control points in the Swiss grid system
shown as Figure 1 of Atteia et al. (see above) and further points digitized
on the tentatively georeferenced scanned map. RMSE 2.4 m. Location of points
in the field was less precise.
}

\keyword{datasets}
\examples{
data(jura)
summary(prediction.dat)
summary(validation.dat)
summary(transect.dat)
summary(juragrid.dat)

# the following commands were used to create objects with factors instead
# of the integer codes for Landuse and Rock:
\dontrun{
  jura.pred = prediction.dat
  jura.val = validation.dat
  jura.grid = juragrid.dat

  jura.pred$Landuse = factor(prediction.dat$Landuse, 
	labels=levels(juragrid.dat$Landuse))
  jura.pred$Rock = factor(prediction.dat$Rock, 
	labels=levels(juragrid.dat$Rock))
  jura.val$Landuse = factor(validation.dat$Landuse, 
	labels=levels(juragrid.dat$Landuse))
  jura.val$Rock = factor(validation.dat$Rock, 
	labels=levels(juragrid.dat$Rock))
}

# the following commands convert data.frame objects into spatial (sp) objects
#   in the local grid:
require(sp)
coordinates(jura.pred) = ~Xloc+Yloc
coordinates(jura.val) = ~Xloc+Yloc
coordinates(jura.grid) = ~Xloc+Yloc
gridded(jura.grid) = TRUE

# the following commands convert the data.frame objects into spatial (sp) objects
#   in WGS84 geographic coordinates
# example is given only for jura.pred, do the same for jura.val and jura.grid
# EPSG codes can be found by searching make_EPSG()
jura.pred <- as.data.frame(jura.pred)
coordinates(jura.pred) = ~ long + lat
proj4string(jura.pred) = CRS("+init=epsg:4326")
}

% $Id: hscat.Rd,v 1.3 2008-02-04 10:06:44 edzer Exp $
\name{hscat}
\alias{hscat}
\title{
Produce h-scatterplot
}
\description{
Produces h-scatterplots, where point pairs having specific separation distances
are plotted. This function is a wrapper around xyplot.
}
\usage{
hscat(formula, data, breaks, pch = 3, cex = .6, mirror = FALSE, 
	variogram.alpha = 0, as.table = TRUE,...)
}
\arguments{
\item{formula}{ specifies the dependent variable }
\item{data}{ data where the variable in formula is resolved }
\item{breaks}{ distance class boundaries }
\item{pch}{ plotting symbol }
\item{cex}{ plotting symbol size }
\item{mirror}{ logical; duplicate all points mirrored along x=y?
(note that correlations are those of the points plotted) }
\item{variogram.alpha}{ parameter to be passed as alpha parameter to
\link{variogram}; if alpha is specified it will only affect xyplot by
being passed through ...}
\item{as.table}{logical; if \code{TRUE}, panels plot top-to-bottom }
\item{...}{ parameters, passed to variogram and xyplot }
}
\value{
an object of class trellis; normally the h scatter plot
}
\author{ Edzer Pebesma }
\note{
Data pairs are plotted once, so the h-scatterplot are not symmetric.
}

\references{ 
Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.
}
\examples{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
hscat(log(zinc)~1, meuse, c(0, 80, 120, 250, 500, 1000))
}

\keyword{models}

% $Id: walker.Rd,v 1.2 2006-02-10 19:03:27 edzer Exp $
\name{walker}
\alias{walker}
\alias{walker.exh}
\title{Walker Lake sample and exhaustive data sets}
\description{
This is the Walker Lake data sets (sample and exhaustive
data set), used in Isaaks and Srivastava's Applied Geostatistics.
}
\format{
  This data frame contains the following columns:
  \describe{
  \item{Id}{Identification Number}
  \item{X}{Xlocation in meter}
  \item{Y}{Ylocation in meter}
  \item{V}{V variable, concentration in ppm}
  \item{U}{U variable, concentration in ppm}
  \item{T}{T variable, indicator variable}
  }
}
\usage{
data(walker)
}
\references{ 
Applied Geostatistics
by Edward H. Isaaks, R. Mohan Srivastava;
Oxford University Press.
}
\note{ This data sets was obtained from the data sets on
ai-geostats (link no longer functioning) }
\keyword{datasets}
\examples{
library(sp)
data(walker)
summary(walker)
summary(walker.exh)
}

% $Id: spplot.vcov.Rd,v 1.2 2007-11-16 12:59:47 edzer Exp $
\name{spplot.vcov}
\alias{spplot.vcov}
\title{ Plot map matrix of prediction error variances and covariances }
\description{
Plot map matrix of prediction error variances and covariances
}
\usage{
spplot.vcov(x, ...)
}
\arguments{
\item{x}{ Object of class SpatialPixelsDataFrame or SpatialGridDataFrame,
resulting from a krige call with multiple variables (cokriging }
\item{...}{remaining arguments passed to spplot }
}
\value{ The plotted object, of class trellis; see \code{spplot} in
package \pkg{sp}. }
\author{ Edzer Pebesma }
\keyword{dplot}

% $Id: sic2004.Rd,v 1.11 2006-02-10 19:03:27 edzer Exp $
\name{sic2004}
\alias{sic2004}
\alias{sic.train}
\alias{sic.pred}
\alias{sic.grid}
\alias{sic.test}
\alias{sic.val}
\title{ Spatial Interpolation Comparison 2004 data set: Natural Ambient Radioactivity }
\description{ The text below was copied from the original sic2004
event, which is no longer online available.

The variable used in the SIC 2004 exercise is natural ambient
radioactivity measured in Germany. The data, provided kindly by the
German Federal Office for Radiation Protection (BfS), are gamma dose rates
reported by means of the national automatic monitoring network (IMIS).

In the frame of SIC2004,  a rectangular area was used to select 1008
monitoring stations (from a total of around 2000 stations). For these
1008 stations, 11 days of measurements have been randomly selected during
the last 12 months and the average daily dose rates calculated for each
day. Hence, we ended up having 11 data sets.

Prior information (sic.train): 10 data sets of 200 points that are
identical for what concerns the locations of the monitoring stations
have been prepared. These locations have been randomly selected (see
Figure 1). These data sets differ only by their Z values since each set
corresponds to 1 day of measurement made during the last 14 months. No
information will be provided on the date of measurement.  These 10 data
sets (10 days of measurements) can be used as prior information to tune
the parameters of the mapping algorithms. No other information will be
provided about these sets. Participants are free of course to gather
more information about the variable in the literature and so on.

The 200 monitoring stations above were randomly taken from a larger set
of 1008 stations. The remaining 808 monitoring stations have a topology
given in sic.pred.  Participants to SIC2004 will have to estimate the
values of the variable taken at these 808 locations.  

The SIC2004 data (sic.val, variable dayx):
The exercise consists in using 200 measurements made on a 11th day (THE
data of the exercise) to estimate the values observed at the remaining
808 locations (hence the question marks as symbols in the maps shown
in Figure 3). These measurements will be provided only during two weeks
(15th of September until 1st of October 2004) on a web page restricted
to the participants. The true values observed at these 808 locations
will be released only at the end of the exercise to allow participants
to write their manuscripts (sic.test, variables dayx and joker).

In addition, a joker data set was released (sic.val, variable joker),
which contains an anomaly. The anomaly was generated by a simulation
model, and does not represent measured levels.

}

\format{
  The data frames contain the following columns:
  \describe{
   \item{record}{this integer value is the number (unique value) of
   the monitoring station chosen by us.}
   \item{x}{X-coordinate of the monitoring station indicated in meters}
   \item{y}{Y-coordinate of the monitoring station indicated in meters}
   \item{day01}{mean gamma dose rate measured during 24 hours, at day01. Units
are nanoSieverts/hour}
   \item{day02}{same, for day 02}
   \item{day03}{...}
   \item{day04}{...}
   \item{day05}{...}
   \item{day06}{...}
   \item{day07}{...}
   \item{day08}{...}
   \item{day09}{...}
   \item{day10}{...}
   \item{dayx}{ the data observed at the 11-th day}
   \item{joker}{ the joker data set, containing an anomaly not present
   in the training data}
  }
}
\note{
the data set sic.grid provides a set of points on a regular grid (almost
10000 points) covering the area; this is convenient for interpolation;
see the function \code{makegrid} in package sp.

The coordinates have been projected around a point located in the
South West of Germany. Hence, a few coordinates have negative values as
can be guessed from the Figures below.
}

\usage{
data(sic2004) # 
}
\author{ 
Data: the German Federal Office for Radiation Protection (BfS),
\url{https://www.bfs.de/EN/home/home_node.html}, 
data provided by Gregoire Dubois, R compilation
by Edzer Pebesma.  }

\keyword{datasets}
\examples{
data(sic2004) 
# FIGURE 1. Locations of the 200 monitoring stations for the 11 data sets. 
# The values taken by the variable are known.
plot(y~x,sic.train,pch=1,col="red", asp=1)

# FIGURE 2. Locations of the 808 remaining monitoring stations at which 
# the values of the variable must be estimated.
plot(y~x,sic.pred,pch="?", asp=1, cex=.8) # Figure 2

# FIGURE 3. Locations of the 1008 monitoring stations (exhaustive data sets). 
# Red circles are used to estimate values located at the questions marks
plot(y~x,sic.train,pch=1,col="red", asp=1)
points(y~x, sic.pred, pch="?", cex=.8)

}

\name{variogramST}
\alias{variogramST}
\title{Calculate Spatio-Temporal Sample Variogram}
\description{
Calculates the sample variogram from spatio-temporal data.
}
\usage{
variogramST(formula, locations, data, ..., tlags = 0:15,  cutoff,
	width = cutoff/15, boundaries = seq(0, cutoff, width),
	progress = interactive(), pseudo = TRUE, assumeRegular = FALSE, 
	na.omit = FALSE, cores = 1)
}
\arguments{
\item{formula}{formula, specifying the dependent variable.}
\item{locations}{A STFDF or STSDF containing the variable; kept for 
  compatibility reasons with variogram, either \code{locations} or \code{data} 
  must be provided.}
\item{data}{A \code{\link[spacetime:STFDF-class]{STFDF}}, \code{\link[spacetime:STSDF-class]{STSDF}} or \code{\link[spacetime:STIDF]{STIDF}} containing the variable.}
\item{...}{any other arguments that will be passed to the underlying 
  \code{\link{variogram}} function. In case of using data of type \code{\link[spacetime:STIDF-class]{STIDF}}, the argument \code{tunit} is recommended (and only used in the case of STIDF) to set the temporal unit of the \code{tlags}. Additionally, \code{twindow} can be passed to control the temporal window used for temporal distance calculations. This builds on the property of xts being ordered and only the next \code{twindow} instances are considered. This avoids the need of huge temporal distance matrices. The default uses twice the number as the average difference goes into the temporal cutoff.}
\item{tlags}{ integer; time lags to consider or in case \code{data} is of class \code{\link[spacetime:STIDF-class]{STIDF}} the actual temporal boundaries with time unit given by \code{tunit} otherwise the same unit as \code{\link{diff}} on the index of the time slot will generate is assumed.}
\item{cutoff}{ spatial separation distance up to which point pairs
are included in semivariance estimates; as a default, the
length of the diagonal of the box spanning the data is divided by three. }
\item{width}{ the width of subsequent distance intervals into which
data point pairs are grouped for semivariance estimates, by default the 
  \code{cutoff} is divided into 15 equal lags.}
\item{boundaries}{ numerical vector with distance interval upper boundaries; 
values should be strictly increasing }
\item{progress}{ logical; if TRUE, show text progress bar }
\item{pseudo}{ integer; use pseudo cross variogram for computing
time-lagged spatial variograms? -1: find out from coordinates -- if they
are equal then yes, else no; 0: no; 1: yes. }
\item{assumeRegular}{logical; whether the time series should be assumed regular. 
  The first time step is assumed to be representative for the whole series. Note,
  that temporal lags are considered by index, and no check is made whether pairs
  actually have the desired separating distance.}
\item{na.omit}{shall all \code{NA} values in the spatio-temporal variogram be dropped? In case where complete rows or columns in the variogram consists of \code{NA} only, \code{plot} might produce a distorted picture.}
\item{cores}{number of cores to use in parallel}
}

\value{The spatio-temporal sample variogram contains besides the fields 
  \code{np}, \code{dist} and \code{gamma} the spatio-temporal fields, 
  \code{timelag}, \code{spacelag} and \code{avgDist}, the first of which indicates the time lag 
  used, the second and third different spatial lags. \code{spacelag} is the midpoint in the spatial
  lag intervals as passed by the parameter \code{boundaries}, whereas \code{avgDist} is the average 
  distance between the point pairs found in a distance interval over all temporal lags (i.e. the 
  averages of the values \code{dist} per temporal lag.) To compute variograms for space lag \eqn{h} and
  time lag \eqn{t}, the pseudo cross variogram \eqn{(Z_i(s)-Z_{i+t}(s+h))^2} is averaged over all time 
  lagged observation sets \eqn{Z_i} and \eqn{Z_{i+t}} available (weighted by the number of pairs involved).
}

\references{ 
Cressie, N.A.C., 1993, Statistics for Spatial Data, Wiley.

Cressie, N., C. Wikle, 2011, Statistics for Spatio-temporal Data, Wiley.

Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.
}
\author{ Edzer Pebesma, Benedikt Graeler }

\seealso{
\code{\link{plot.StVariogram}},
for variogram models: \code{\link{vgmST}},
to fit a spatio-temporal variogram model to a spatio-temporal sample variogram: 
\code{\link{fit.StVariogram}}
}
\examples{
# The following spatio-temporal variogram has been calcualted through
# vv = variogram(PM10~1, r5to10, width=20, cutoff = 200, tlags=0:5)
# in the vignette "st".

data(vv)
str(vv)
plot(vv)
}

\keyword{models}

% $Id: variogramLine.Rd,v 1.3 2008-08-19 07:27:02 edzer Exp $
\name{vgmArea}
\alias{vgmArea}
\title{ point-point, point-area or area-area semivariance }
\description{ Compute point-point, point-area or area-area variogram values from point model }
\usage{
vgmArea(x, y = x, vgm, ndiscr = 16, verbose = FALSE, covariance = TRUE)
}
\arguments{
\item{x}{ object of class \link[sp]{SpatialPoints} or \link[sp]{SpatialPolygons}}
\item{y}{ object of class \link[sp]{SpatialPoints} or \link[sp]{SpatialPolygons}}
\item{vgm}{ variogram model, see \link{vgm}}
\item{ndiscr}{ number of points to discretize an area, using \link[sp]{spsample}}
\item{verbose}{ give progress bar }
\item{covariance}{ logical; compute covariances, rather than semivariances? }
}
\value{ semivariance or covariance matrix of dimension \code{length(x)} x \code{lenght(y)}}

\author{ Edzer Pebesma }

\examples{
library(sp)
demo(meuse, ask = FALSE, echo = FALSE)
vgmArea(meuse[1:5,], vgm = vgm(1, "Exp", 1000)) # point-point
vgmArea(meuse[1:5,], meuse.area, vgm = vgm(1, "Exp", 1000)) # point-area
}

\keyword{models}

% $Id: vgm.panel.Rd,v 1.3 2008-10-30 13:47:05 edzer Exp $
\name{vgm.panel.xyplot}
\alias{vgm.panel.xyplot}
\alias{panel.pointPairs}
\title{ panel functions for most of the variogram plots through lattice }
\description{
Variogram plots contain symbols and lines; more control over them can
be gained by writing your own panel functions, or extending the ones
described here; see examples.
}
\usage{
vgm.panel.xyplot(x, y, subscripts, type = "p", pch = plot.symbol$pch, 
    col, col.line = plot.line$col, col.symbol = plot.symbol$col, 
    lty = plot.line$lty, cex = plot.symbol$cex, ids, lwd = plot.line$lwd, 
    model = model, direction = direction, labels, shift = shift, mode = mode, ...) 
panel.pointPairs(x, y, type = "p", pch = plot.symbol$pch, col, col.line = 
	plot.line$col, col.symbol = plot.symbol$col, lty = plot.line$lty, 
	cex = plot.symbol$cex, lwd = plot.line$lwd, pairs = pairs, 
	line.pch = line.pch, ...) 
}
\arguments{
\item{x}{ x coordinates of points in this panel}
\item{y}{ y coordinates of points in this panel}
\item{subscripts }{ subscripts of points in this panel}
\item{type}{ plot type: "l" for connected lines }
\item{pch}{ plotting symbol }
\item{col}{ symbol and line color (if set) }
\item{col.line}{ line color }
\item{col.symbol}{ symbol color }
\item{lty}{ line type for variogram model }
\item{cex}{ symbol size }
\item{ids}{ gstat model ids }
\item{lwd}{ line width }
\item{model}{ variogram model }
\item{direction}{ direction vector \code{c(dir.horizontal, dir.ver)}}
\item{labels}{ labels to plot next to points }
\item{shift}{ amount to shift the label right of the symbol }
\item{mode}{ to be set by calling function only }
\item{line.pch}{ symbol type to be used for point of selected point pairs, e.g.
to highlight point pairs with distance close to zero }
\item{pairs}{ two-column matrix with pair indexes to be highlighted }
\item{...}{ parameters that get passed to \link[lattice]{lpoints} }
}
\value{
ignored; the enclosing function returns a plot of class \code{trellis}
}
\author{ Edzer Pebesma }
\seealso{ \link{plot.gstatVariogram}, \link{vgm}}
\examples{ 
library(sp)
data(meuse)
coordinates(meuse) <- c("x", "y")
library(lattice)
mypanel = function(x,y,...) { 
	vgm.panel.xyplot(x,y,...)
	panel.abline(h=var(log(meuse$zinc)), color = 'red')
}
plot(variogram(log(zinc)~1,meuse), panel = mypanel)
}
\keyword{models}

% $Id: pcb.Rd,v 1.8 2009-07-03 12:10:55 edzer Exp $
\name{pcb}
\alias{pcb}
\title{PCB138 measurements in sediment at the NCP, the Dutch part of the North Sea}
\description{
PCB138 measurements in sediment at the NCP, which is the 
Dutch part of the North Sea
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{year}{ measurement year }
    \item{x}{ x-coordinate; UTM zone 31 }
    \item{y}{ y-coordinate; UTM zone 31 }
    \item{coast}{ distance to coast of the Netherlands, in km.}
    \item{depth}{ sea water depth, m. }
    \item{PCB138}{ PCB-138, measured on the sediment fraction smaller than
	63 \eqn{\mu}{mu}, in \eqn{\mu g/kg}{mu g/kg} dry matter; BUT SEE NOTE BELOW }
	\item{yf}{ year; as factor}
  }
}

\usage{
data(pcb)
}
\note{ A note of caution: The PCB-138 data are provided only to be able
to re-run the analysis done in Pebesma and Duin (2004; see references
below). If you want to use these data for comparison with PCB measurements
elsewhere, or if you want to compare them to regulation standards, or
want to use these data for any other purpose, you should first contact
\url{mailto:basisinfodesk@rikz.rws.minvenw.nl}.  The reason for this is
that several normalisations were carried out that are not reported here,
nor in the paper below. }

\references{ 
Pebesma, E. J., and Duin, R. N. M. (2005). Spatial patterns of temporal
change in North Sea sediment quality on different spatial scales. In
P. Renard, H. Demougeot-Renard and R. Froidevaux (Eds.), Geostatistics for
Environmental Applications: Proceedings of the Fifth European Conference
on Geostatistics for Environmental Applications (pp. 367-378): Springer.
}

\seealso{\link{ncp.grid}}
\keyword{datasets}
\examples{
data(pcb)
library(lattice)
xyplot(y~x|as.factor(yf), pcb, aspect = "iso")
# demo(pcb)
}

\name{tull}
\alias{tull}
\alias{TULLNREG}
\alias{tull36}
\alias{Chlorid92}
\encoding{utf-8}
\title{Südliche Tullnerfeld data set}
\description{
The Südliche Tullnerfeld is a part of the Danube river basin in
central Lower Austria and due to its homogeneous aquifer well suited
for a model-oriented geostatistical analysis. It contains 36 official
water quality measurement stations, which are irregularly spread over the
region. 
}
\format{
  The data frames contain the following columns:
  \describe{
  \item{x}{X location in meter}
  \item{y}{Y location in meter}
\item{S411}{Station name}
\item{S429}{Station name}
\item{S849}{Station name}
\item{S854}{Station name}
\item{S1502}{Station name}
\item{S1584}{Station name}
\item{S1591}{Station name}
\item{S2046}{Station name}
\item{S2047}{Station name}
\item{S2048}{Station name}
\item{S2049}{Station name}
\item{S2051}{Station name}
\item{S2052}{Station name}
\item{S2053}{Station name}
\item{S2054}{Station name}
\item{S2055}{Station name}
\item{S2057}{Station name}
\item{S2058}{Station name}
\item{S2059}{Station name}
\item{S2060}{Station name}
\item{S2061}{Station name}
\item{S2062}{Station name}
\item{S2063}{Station name}
\item{S2064}{Station name}
\item{S2065}{Station name}
\item{S2066}{Station name}
\item{S2067}{Station name}
\item{S2070}{Station name}
\item{S2071}{Station name}
\item{S2072}{Station name}
\item{S2128}{Station name}
\item{S5319}{Station name}
\item{S5320}{Station name}
\item{S5321}{Station name}
\item{S5322}{Station name}
\item{S5323}{Station name}
  }
}
\usage{
data(tull)
}
\references{ 
Werner G. Müller, Collecting Spatial Data, 3rd edition.
Springer Verlag, Heidelberg, 2007
}
\note{
This data set was obtained on May 6, 2008 from
\url{ http://www.ifas.jku.at/e5361/index_ger.html }.
The author of the book that uses it is found at:
\url{ http://www.ifas.jku.at/e2571/e2604/index_ger.html }
}
\keyword{datasets}
\examples{
data(tull)

# TULLNREG = read.csv("TULLNREG.csv")

# I modified tulln36des.csv, such that the first line only contained: x,y
# resulting in row.names that reflect the station ID, as in
# tull36 = read.csv("tulln36des.csv")

# Chlorid92 was read & converted by:
#Chlorid92=read.csv("Chlorid92.csv")
#Chlorid92$Datum = as.POSIXct(strptime(Chlorid92$Datum, "%d.%m.%y"))

summary(tull36)
summary(TULLNREG)
summary(Chlorid92)

# stack & join data to x,y,Date,Chloride form:
cl.st = stack(Chlorid92[-1])
names(cl.st) = c("Chloride", "Station")
cl.st$Date = rep(Chlorid92$Datum, length(names(Chlorid92))-1)
cl.st$x = tull36[match(cl.st[,"Station"], row.names(tull36)), "x"]
cl.st$y = tull36[match(cl.st[,"Station"], row.names(tull36)), "y"]
# library(lattice)
# xyplot(Chloride~Date|Station, cl.st)
# xyplot(y~x|Date, cl.st, asp="iso", layout=c(16,11))
summary(cl.st)
plot(TULLNREG, pch=3, asp=1)
points(y~x, cl.st, pch=16)
}

% $Id: coalash.Rd,v 1.2 2007-03-21 15:14:25 edzer Exp $
\name{coalash}
\alias{coalash}
\title{Coal ash samples from a mine in Pennsylvania}
\description{
Data obtained from Gomez and Hazen (1970, Tables 19 and 20) on coal ash
for the Robena Mine Property in Greene County Pennsylvania.
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{x}{a numeric vector; x-coordinate; reference unknown }
    \item{y}{a numeric vector; x-coordinate; reference unknown }
	\item{coalash}{the target variable}
  } 
} 
\usage{
data(coalash)
}
\author{ unknown; R version prepared by Edzer Pebesma; data obtained from
\url{http://homepage.divms.uiowa.edu/~dzimmer/spatialstats/}, Dale Zimmerman's
course page }
\references{ N.A.C. Cressie, 1993, Statistics for Spatial Data,
Wiley. 

Gomez, M. and Hazen, K. (1970). Evaluating sulfur and ash
distribution in coal seems by statistical response surface regression
analysis. U.S. Bureau of Mines Report RI 7377.

see also fields manual: 
\url{https://www.image.ucar.edu/GSP/Software/Fields/fields.manual.coalashEX.Krig.shtml}
}
\note{ data are also present in package fields, as coalash.  }
\keyword{datasets}
\examples{
data(coalash)
summary(coalash)
}

\name{krigeSTSimTB}
\alias{krigeSTSimTB}

\title{conditional/unconditional spatio-temporal simulation}
\description{
conditional/unconditional spatio-temporal simulation based on turning bands
}
\usage{
krigeSTSimTB(formula, data, newdata, modelList, nsim, progress = TRUE, 
             nLyrs = 500, tGrid = NULL, sGrid = NULL, ceExt = 2, nmax = Inf)
}
\arguments{
  \item{formula}{the formula of the kriging predictor}
  \item{data}{conditioning data}
  \item{newdata}{locations in space and time where the simulation is carried out}
  \item{modelList}{the spatio-temporal variogram (from \code{\link{vgmST}}) defining the spatio-temporal covariance structure of the simulated Gaussian random field}
  \item{nsim}{number of simulations}
  \item{progress}{boolean; whether the progress should be shown in progress bar}
  \item{nLyrs}{number of layers used in the turning bands approach (default = 500) }
  \item{tGrid}{optional explicit temporal griding that shall be used}
  \item{sGrid}{optional explicit spatial griding that shall be used}
  \item{ceExt}{expansion in the circulant embedding, defaults to 2}
  \item{nmax}{number of nearest neighbours that shall e used, defaults to 'Inf' meaning all available points are used}
}
\value{a spatio-temporal data frame with \code{nSim} simulations}
\references{
Turning bands

Lantuejoul, C. (2002) Geostatistical Simulation: Models and Algorithms. Springer.

Matheron, G. (1973). The intrinsic random functions and their applications. Adv. Appl. Probab., 5, 439-468.

Strokorb, K., Ballani, F., and Schlather, M. (2014) Tail correlation functions of max-stable processes: Construction principles, recovery and diversity of some mixing max-stable processes with identical TCF. Extremes, Submitted.

Turning layers

Schlather, M. (2011) Construction of covariance functions and unconditional simulation of random fields. In Porcu, E., Montero, J.M. and Schlather, M., Space-Time Processes and Challenges Related to Environmental Problems. New York: Springer.
}

\author{Benedikt Graeler}

\seealso{\code{\link{krigeSimCE}}}
\examples{
# see demo('circEmbeddingMeuse')
}

% $Id: krigeTg.Rd,v 1.10 2009-08-17 14:38:28 edzer Exp $
\name{krigeTg}
\docType{methods}
\alias{krigeTg}
\title{ TransGaussian kriging using Box-Cox transforms }
\description{
TransGaussian (ordinary) kriging function using Box-Cox transforms  
}
\usage{
krigeTg(formula, locations, newdata, model = NULL, ...,
	nmax = Inf, nmin = 0, maxdist = Inf, block = numeric(0),
	nsim = 0, na.action = na.pass, debug.level = 1,
	lambda = 1.0)
}
\arguments{
 \item{formula}{ formula that defines the dependent variable as a linear
  model of independent variables; suppose the dependent variable has name
  \code{z}, for ordinary and use a formula like \code{z~1}; the dependent
  variable should be NOT transformed.
  }
 \item{locations}{ object of class \code{Spatial}, with observations }
 \item{newdata}{ Spatial object with prediction/simulation locations; 
  the coordinates should have names as defined in \code{locations} }
 \item{model}{ variogram model of the TRANSFORMED dependent variable, 
  see \link{vgm}, or \link{fit.variogram}}
 \item{nmax}{ for local kriging: the number of nearest observations that
  should be used for a kriging prediction or simulation, where nearest
  is defined in terms of the space of the spatial locations. By default,
  all observations are used }
 \item{nmin}{ for local kriging: if the number of nearest observations
  within distance \code{maxdist} is less than \code{nmin}, a missing 
  value will be generated; see maxdist }
 \item{maxdist}{ for local kriging: only observations within a distance
  of \code{maxdist} from the prediction location are used for prediction
  or simulation; if combined with \code{nmax}, both criteria apply }
 \item{block}{ does not function correctly, afaik }
 \item{nsim}{ does not function correctly, afaik }
 \item{na.action}{ function determining what should be done with missing
  values in 'newdata'.  The default is to predict 'NA'.  Missing values 
  in coordinates and predictors are both dealt with. }
 \item{lambda}{value for the Box-Cox transform}
 \item{debug.level}{debug level, passed to \link[gstat]{predict}; use -1 
 to see progress in percentage, and 0 to suppress all printed information }
 \item{\dots}{ other arguments that will be passed to \link{gstat}}
}
\details{
Function \code{krigeTg} uses transGaussian kriging as explained in
\url{https://www.math.umd.edu/~bnk/bak/Splus/kriging.html}. 

As it uses the R/gstat krige function to derive everything, it needs in
addition to ordinary kriging on the transformed scale a simple kriging
step to find m from the difference between the OK and SK prediction
variance, and a kriging/BLUE estimation step to obtain the estimate
of \eqn{\mu}{mu}.

For further details, see \link{krige} and \link[gstat]{predict}.
}

\value{
an SpatialPointsDataFrame object containing the fields:
\code{m} for the m (Lagrange) parameter for each location;
\code{var1SK.pred} the \eqn{c_0 C^{-1}}{c0 Cinv} correction obtained by
\code{muhat} for the mean estimate at each location;
\code{var1SK.var} the simple kriging variance;
\code{var1.pred} the OK prediction on the transformed scale;
\code{var1.var} the OK kriging variance on the transformed scale;
\code{var1TG.pred} the transGaussian kriging predictor;
\code{var1TG.var} the transGaussian kriging variance, obtained by
\eqn{\phi'(\hat{\mu},\lambda)^2 \sigma^2_{OK}}{phi'(muhat, lambda)^2 * var1.var}

}

\references{ N.A.C. Cressie, 1993, Statistics for Spatial Data,
Wiley. 

}

\author{ Edzer Pebesma }

\seealso{ \link{gstat}, \link[gstat]{predict} }

\examples{
\donttest{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y
v = vgm(1, "Exp", 300)
x1 = krigeTg(zinc~1,meuse,meuse.grid,v, lambda=1) # no transform
x2 = krige(zinc~1,meuse,meuse.grid,v)
summary(x2$var1.var-x1$var1TG.var)
summary(x2$var1.pred-x1$var1TG.pred)
lambda = -0.25
m = fit.variogram(variogram((zinc^lambda-1)/lambda ~ 1,meuse), vgm(1, "Exp", 300))
x = krigeTg(zinc~1,meuse,meuse.grid,m,lambda=-.25)
spplot(x["var1TG.pred"], col.regions=bpy.colors())
summary(meuse$zinc)
summary(x$var1TG.pred)
}
}
\keyword{ models }

% $Id: fit.variogram.Rd,v 1.6 2006-02-10 19:03:27 edzer Exp $
\name{fit.variogram}
\alias{fit.variogram}
\title{ Fit a Variogram Model to a Sample Variogram }
\description{
Fit ranges and/or sills from a simple or nested variogram model 
to a sample variogram }
\usage{
fit.variogram(object, model, fit.sills = TRUE, fit.ranges = TRUE,
	fit.method = 7, debug.level = 1, warn.if.neg = FALSE, fit.kappa = FALSE)
}
\arguments{
\item{object}{ sample variogram, output of \link{variogram} }
\item{model}{ variogram model, output of \link{vgm}; see Details below
for details on how \code{NA} values in \code{model} are initialised. }
\item{fit.sills}{ logical; determines whether the partial sill
coefficients (including nugget variance) should be fitted; or logical
vector: determines for each partial sill parameter whether it should be
fitted or fixed.  }
\item{fit.ranges}{ logical; determines whether the range coefficients
(excluding that of the nugget component) should be fitted; or logical
vector: determines for each range parameter whether it should be fitted
or fixed.  }
\item{fit.method}{ fitting method, used by gstat. The default method uses
weights $N_h/h^2$ with $N_h$ the number of point pairs and $h$ the
distance. This criterion is not supported by theory, but by practice. 
For other values of \code{fit.method}, see details. }
\item{debug.level}{ integer; set gstat internal debug level }
\item{warn.if.neg}{ logical; if TRUE a warning is issued whenever
a sill value of a direct variogram becomes negative }
\item{fit.kappa}{ logical; if \code{TRUE}, a sequence of 0.3, 0.4,...,5 will
be searched for optimal fit; alternatively another sequence can be given to
this argument }
}
\value{
returns a fitted variogram model (of class \code{variogramModel}).

This is a \code{data.frame} with two attributes: (i) \code{singular}
a logical attribute that indicates whether the non-linear fit
converged (FALSE), or ended in a singularity (TRUE), and (ii)
\code{SSErr} a numerical attribute with the (weighted) sum of
squared errors of the fitted model. See Notes below.

}

\note{ If fitting the range(s) is part of the job of this function,
the results may well depend on the starting values, given in
argument \code{model}, which is generally the case for non-linear
regression problems. This function uses internal C code, which
uses Levenberg-Marquardt.

If for a direct (i.e. not a cross) variogram a sill parameter
(partial sill or nugget) becomes negative, fit.variogram is called
again with this parameter set to zero, and with a FALSE flag to
further fit this sill. This implies that the search does not move
away from search space boundaries.

On singular model fits: If your variogram turns out to be a flat,
horizontal or sloping line, then fitting a three parameter model such
as the exponential or spherical with nugget is a bit heavy: there's an
infinite number of possible combinations of sill and range (both very
large) to fit to a sloping line. In this case, the returned, singular
model may still be useful: just try and plot it. Gstat converges when
the parameter values stabilize, and this may not be the case. Another
case of singular model fits happens when a model that reaches the sill
(such as the spherical) is fit with a nugget, and the range parameter
starts, or converges to a value smaller than the distance of the second
sample variogram estimate. In this case, again, an infinite number of
possibilities occur essentially for fitting a line through a single
(first sample variogram) point. In both cases, fixing one or more of
the variogram model parameters may help you out.

The function will accept anisotropic sample variograms as input. It will
fit a model for a given direction interval if the sample variogram 
only includes this direction. It is not possible to fit 
a multiple direction model to each direction of the sample variogram, 
in this case the model will be fitted to an average of all directions.
}

\details{
If any of the initial parameters of \code{model} are \code{NA},
they are given default values as follows. The range parameter
is given one third of the maximum value of \code{object$dist}.
The nugget value is given the mean value of the first three values
of \code{object$gamma}.  The partial sill is given the mean of the
last five values of \code{object$gamma}.

Values for \code{fit.method} are 1: weights equal to
$N_j$; 2: weights equal to $N_j/((gamma(h_j))^2)$; 5 (ignore, use
\link{fit.variogram.reml}); 6: unweighted (OLS); 7: $N_j/(h_j^2)$. 
}

\references{
Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.
}

\author{ Edzer Pebesma }
\seealso{
\link{variogram},
\link{vgm}
}

\examples{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
vgm1 <- variogram(log(zinc)~1, meuse)
fit.variogram(vgm1, vgm(1, "Sph", 300, 1))
fit.variogram(vgm1, vgm("Sph"))

# optimize the value of kappa in a Matern model, using ugly <<- side effect:
f = function(x) attr(m.fit <<- fit.variogram(vgm1, vgm(,"Mat",nugget=NA,kappa=x)),"SSErr")
optimize(f, c(0.1, 5))
plot(vgm1, m.fit)
# best fit from the (0.3, 0.4, 0.5. ... , 5) sequence:
(m <- fit.variogram(vgm1, vgm("Mat"), fit.kappa = TRUE))
attr(m, "SSErr")
}

\keyword{models}

% $Id: plot.gstatVariogram.Rd,v 1.14 2008-02-19 10:01:22 edzer Exp $
\name{plot.gstatVariogram}
\alias{plot.gstatVariogram}
\alias{plot.variogramMap}
\alias{plot.StVariogram}
\title{
Plot a sample variogram, and possibly a fitted model
}
\description{
Creates a variogram plot
}
\usage{
\method{plot}{gstatVariogram}(x, model = NULL, ylim, xlim, xlab = "distance", 
	ylab = attr(x, "what"), panel = vgm.panel.xyplot, multipanel = TRUE, 
	plot.numbers = FALSE, scales, ids = x$id, group.id = TRUE, skip, 
	layout, ...)
\method{plot}{variogramMap}(x, np = FALSE, skip, threshold, ...)
\method{plot}{StVariogram}(x, model = NULL, ..., col = bpy.colors(), xlab, ylab, 
	map = TRUE, convertMonths = FALSE, as.table = TRUE, wireframe = FALSE, 
	diff = FALSE, all = FALSE)
}
\arguments{
\item{x}{ object obtained from the method 
\link{variogram}, possibly containing directional or cross variograms,
space-time variograms and variogram model information }
\item{model}{in case of a single variogram: a variogram model, as 
obtained from \link{vgm} or \link{fit.variogram}, to be drawn as 
a line in the variogram plot; in case of a set of variograms and
cross variograms: a list with variogram models; in the spatio-temporal case,
a single or a list of spatio-temporal models that will be plotted next to each
other for visual comparison.}
\item{ylim}{ numeric; vector of length 2, limits of the y-axis}
\item{xlim}{ numeric; vector of length 2, limits of the x-axis}
\item{xlab}{ character; x-axis label }
\item{ylab}{ character; y-axis label }
\item{panel}{ panel function }
\item{multipanel}{ logical; if TRUE, directional variograms are plotted in
different panels, if FALSE, directional variograms are plotted in the same
graph, using color, colored lines and symbols to distinguish them }
\item{plot.numbers}{ logical or numeric; if TRUE, plot number of point pairs next to
each plotted semivariance symbol, if FALSE these are omitted. 
If numeric, TRUE is assumed and the value is passed as the relative distance to be used between symbols
and numeric text values (default 0.03). }
\item{scales}{ optional argument that will be passed to \code{\link[lattice]{xyplot}} in
case of the plotting of variograms and cross variograms; use the value
\code{list(relation = "same")} if y-axes need to share scales }
\item{ids}{ ids of the data variables and variable pairs }
\item{group.id}{ logical; control for directional multivariate variograms:
if TRUE, panels divide direction and colors indicate variables
(ids), if FALSE panels divide variables/variable pairs and colors
indicate direction}
\item{skip}{ logical; can be used to arrange panels, see \code{\link[lattice]{xyplot}}}
\item{layout}{ integer vector; can be used to set panel layout: c(ncol,nrow) }
\item{np}{ logical (only for plotting variogram maps); if TRUE, plot number of point pairs, if FALSE plot
semivariances }
\item{threshold}{semivariogram map values based on fewer point pairs
than threshold will not be plotted}
\item{\dots}{ any arguments that will be passed to the panel plotting functions
(such as \code{auto.key} in examples below) }
\item{col}{colors to use}
\item{map}{logical; if TRUE, plot space-time variogram map}
\item{convertMonths}{logical; if TRUE, \code{yearmon} time lags will
be unit converted and plotted as (integer) months, and no longer match the
numeric representation of \code{yearmon}, which has years as unit }
\item{as.table}{controls the plotting order for multiple panels, see \code{\link[lattice]{xyplot}} for details.}
\item{wireframe}{logical; if TRUE, produce a wireframe plot}
\item{diff}{logical; if TRUE, plot difference between model and sample variogram; ignores \code{all}.}
\item{all}{logical; if TRUE, plot sample and model variogram(s) in single wireframes.}
}
\value{
returns (or plots) the variogram plot
}
\details{Please note that in the spatio-temporal case the levelplot and wireframe plots use the spatial distances averaged for each time lag \code{avgDist}. For strongly varying spatial locations over time, please check the distance columns \code{dist} and \code{avgDist} of the spatio-temporal sample variogram. The \code{lattice::cloud} function is one option to plot irregular 3D data.}

\author{ Edzer Pebesma }
\note{ currently, plotting models and/or point pair numbers is not
supported when a variogram is both directional and multivariable; also,
three-dimensional directional variograms will probably not be displayed
correctly. }
\seealso{
\link{variogram},
\link{fit.variogram}, 
\link{vgm}
\link{variogramLine},
}
\examples{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
vgm1 <- variogram(log(zinc)~1, meuse)
plot(vgm1)
model.1 <- fit.variogram(vgm1,vgm(1,"Sph",300,1))
plot(vgm1, model=model.1)
plot(vgm1, plot.numbers = TRUE, pch = "+")
vgm2 <- variogram(log(zinc)~1, meuse, alpha=c(0,45,90,135))
plot(vgm2)
# the following demonstrates plotting of directional models:
model.2 <- vgm(.59,"Sph",926,.06,anis=c(0,0.3))
plot(vgm2, model=model.2)
 
g = gstat(NULL, "zinc < 200", I(zinc<200)~1, meuse)
g = gstat(g, "zinc < 400", I(zinc<400)~1, meuse)
g = gstat(g, "zinc < 800", I(zinc<800)~1, meuse)
# calculate multivariable, directional variogram:
v = variogram(g, alpha=c(0,45,90,135))
plot(v, group.id = FALSE, auto.key = TRUE) # id and id pairs panels
plot(v, group.id = TRUE, auto.key = TRUE)  # direction panels

# variogram maps:
plot(variogram(g, cutoff=1000, width=100, map=TRUE),
    main = "(cross) semivariance maps")
plot(variogram(g, cutoff=1000, width=100, map=TRUE), np=TRUE,
    main = "number of point pairs")
}

\keyword{dplot}

% $Id: ossfim.Rd,v 1.3 2006-02-10 19:03:27 edzer Exp $
\name{ossfim}
\alias{ossfim}
\title{ Kriging standard errors as function of grid spacing and block size}
\description{ Calculate, for a given variogram model, ordinary block
kriging standard errors as a function of sampling spaces and block sizes }
\usage{
ossfim(spacings = 1:5, block.sizes = 1:5, model, nmax = 25, debug = 0)
}
\arguments{
\item{spacings}{range of grid (data) spacings to be used}
\item{block.sizes}{ range of block sizes to be used}
\item{model}{variogram model, output of \code{vgm}}
\item{nmax}{set the kriging neighbourhood size}
\item{debug}{debug level; set to 32 to see a lot of output}
}

\value{ data frame with columns \code{spacing} (the grid spacing),
\code{block.size} (the block size), and \code{kriging.se} (block kriging
standard error) }

\references{ 
Burrough, P.A., R.A. McDonnell (1999) Principles of Geographical
Information Systems. Oxford University Press (e.g., figure 10.11 on
page 261)

Burgess, T.M., R. Webster, A.B. McBratney (1981) Optimal interpolation and
isarithmic mapping of soil properties. IV Sampling strategy.  The journal
of soil science 32(4), 643-660.

McBratney, A.B., R. Webster (1981) The design of optimal sampling schemes
for local estimation and mapping of regionalized variables: 2 program
and examples. Computers and Geosciences 7: 335-365.
}

\author{ Edzer Pebesma }
\note{ The idea is old, simple, but still of value. If you want to map
a variable with a given accuracy, you will have to sample it. Suppose
the variogram of the variable is known. Given a regular sampling scheme,
the kriging standard error decreases when either (i) the data spacing
is smaller, or (ii) predictions are made for larger blocks. This function
helps quantifying this relationship. Ossfim probably refers to ``optimal
sampling scheme for isarithmic mapping''.
}

\seealso{
\link{krige}
}
\examples{
\dontrun{
x <- ossfim(1:15,1:15, model = vgm(1,"Exp",15))
library(lattice)
levelplot(kriging.se~spacing+block.size, x, 
  main = "Ossfim results, variogram 1 Exp(15)")
}
# if you wonder about the decrease in the upper left corner of the graph,
# try the above with nmax set to 100, or perhaps 200.
}
\keyword{models}

% $Id: krige.cv.Rd,v 1.18 2009-10-30 16:11:21 edzer Exp $
\name{krige.cv}
\docType{methods}
\alias{krige.cv}
\alias{krige.cv.spatial}
\alias{krige.cv.locations}
\alias{gstat.cv}
\alias{krige.cv,formula,formula-method}
\alias{krige.cv,formula,Spatial-method}
\alias{krige.cv,formula,sf-method}
\title{ (co)kriging cross validation, n-fold or leave-one-out }
\description{ Cross validation functions for simple, ordinary or universal
point (co)kriging, kriging in a local neighbourhood.  }
\usage{
gstat.cv(object, nfold, remove.all = FALSE, verbose = interactive(), 
	all.residuals = FALSE, ...)
krige.cv(formula, locations, ...)
krige.cv.locations(formula, locations, data, model = NULL, ..., beta = NULL, 
	nmax = Inf, nmin = 0, maxdist = Inf, nfold = nrow(data), 
	verbose = interactive(), debug.level = 0)
krige.cv.spatial(formula, locations, model = NULL, ..., beta = NULL, 
	nmax = Inf, nmin = 0, maxdist = Inf, nfold = nrow(locations), 
	verbose = interactive(), debug.level = 0)
}
\arguments{
  \item{object}{ object of class gstat; see function \link{gstat}}
  \item{nfold}{ integer; if larger than 1, then apply n-fold cross validation; 
  if \code{nfold} equals \code{nrow(data)} (the default), apply leave-one-out cross 
  validation; if set to e.g. 5, five-fold cross validation is done. To specify the
  folds, pass an integer vector of length \code{nrow(data)} with fold indexes. }
  \item{remove.all}{ logical; if TRUE, remove observations at cross validation
  locations not only for the first, but for all subsequent variables as well }
  \item{verbose}{ logical; if FALSE, progress bar is suppressed }
  \item{all.residuals}{ logical; if TRUE, residuals for all variables are
  returned instead of for the first variable only}
  \item{\dots}{ other arguments that will be passed to \link[gstat]{predict}
  in case of \code{gstat.cv}, or to \link{gstat} in case of \code{krige.cv}}
  \item{formula}{ formula that defines the dependent variable as a linear
  model of independent variables; suppose the dependent variable has name
  \code{z}, for ordinary and simple kriging use the formula \code{z~1};
  for simple kriging also define \code{beta} (see below); for universal
  kriging, suppose \code{z} is linearly dependent on \code{x} and \code{y},
  use the formula \code{z~x+y}}
  \item{locations}{ data object deriving from class \code{Spatial} or \code{sf} }
  \item{data}{ data frame (deprecated); should contain the dependent variable, independent
  variables, and coordinates; only to be provided if \code{locations} is a formula}
  \item{model}{ variogram model of dependent variable (or its residuals), 
  defined by a call to \link{vgm} or \link{fit.variogram}}
  \item{beta}{ only for simple kriging (and simulation based on simple
  kriging); vector with the trend coefficients (including intercept);
  if no independent variables are defined the model only contains an
  intercept and this should be the simple kriging mean }
  \item{nmax}{ for local kriging: the number of nearest observations that
  should be used for a kriging prediction or simulation, where nearest
  is defined in terms of the space of the spatial locations. By default,
  all observations are used }
  \item{nmin}{ for local kriging: if the number of nearest observations
  within distance \code{maxdist} is less than \code{nmin}, a missing 
  value will be generated; see maxdist }
  \item{maxdist}{ for local kriging: only observations within a distance
  of \code{maxdist} from the prediction location are used for prediction
  or simulation; if combined with \code{nmax}, both criteria apply }
  \item{debug.level}{ print debugging information; 0 suppresses
  debug information }
}

\section{Methods}{
\describe{
\item{formula = "formula", locations = "formula"}{ 
locations specifies which coordinates in \code{data} refer to spatial coordinates
}
\item{formula = "formula", locations = "Spatial"}{ 
Object locations knows about its own spatial locations
}
}}

\details{ Leave-one-out cross validation (LOOCV) visits a data point, and
predicts the value at that location by leaving out the observed value,
and proceeds with the next data point. (The observed value is left
out because kriging would otherwise predict the value itself.) N-fold
cross validation makes a partitions the data set in N parts. For all
observation in a part, predictions are made based on the remaining N-1
parts; this is repeated for each of the N parts. N-fold cross validation
may be faster than LOOCV. }

\value{ data frame containing the coordinates of \code{data} or those
of the first variable in \code{object}, and columns of prediction and
prediction variance of cross validated data points, observed values,
residuals, zscore (residual divided by kriging standard error), and fold.

If \code{all.residuals} is true, a data frame with residuals for all
variables is returned, without coordinates. }

\author{ Edzer Pebesma }
\note{ Leave-one-out cross validation seems to be much faster in plain
(stand-alone) gstat, apparently quite a bit of the effort is spent moving 
data around from R to gstat. }

\seealso{ \link{krige}, \link{gstat}, \link[gstat]{predict} }

\examples{
library(sp)
data(meuse)
coordinates(meuse) <- ~x+y
m <- vgm(.59, "Sph", 874, .04)
# five-fold cross validation:
x <- krige.cv(log(zinc)~1, meuse, m, nmax = 40, nfold=5)
bubble(x, "residual", main = "log(zinc): 5-fold CV residuals")

# multivariable; thanks to M. Rufino:
meuse.g <- gstat(id = "zn", formula = log(zinc) ~ 1, data = meuse)
meuse.g <- gstat(meuse.g, "cu", log(copper) ~ 1, meuse)
meuse.g <- gstat(meuse.g, model = vgm(1, "Sph", 900, 1), fill.all = TRUE)
x <- variogram(meuse.g, cutoff = 1000)
meuse.fit = fit.lmc(x, meuse.g)
out = gstat.cv(meuse.fit, nmax = 40, nfold = 5) 
summary(out)
out = gstat.cv(meuse.fit, nmax = 40, nfold = c(rep(1,100), rep(2,55))) 
summary(out)
# mean error, ideally 0:
mean(out$residual)
# MSPE, ideally small
mean(out$residual^2)
# Mean square normalized error, ideally close to 1
mean(out$zscore^2)
# correlation observed and predicted, ideally 1
cor(out$observed, out$observed - out$residual)
# correlation predicted and residual, ideally 0
cor(out$observed - out$residual, out$residual)
}
\keyword{ models }

% $Id: ncp.grid.Rd,v 1.5 2007-11-16 12:59:47 edzer Exp $
\name{ncp.grid}
\alias{ncp.grid}
\title{Grid for the NCP, the Dutch part of the North Sea}
\description{
Gridded data for the NCP (Nederlands Continentaal Plat, the Dutch
part of the North Sea), for a 5 km x 5 km grid; stored as data.frame.
}
\format{
  This data frame contains the following columns:
  \describe{
   \item{x}{x-coordinate, UTM zone 31}
   \item{y}{y-coordinate, UTM zone 31}
   \item{depth}{sea water depth, m.}
   \item{coast}{distance to the coast of the Netherlands, in km.}
   \item{area}{identifier for administrative sub-areas}
  }
}
\usage{
data(ncp.grid)
}
\author{Dutch National Institute for Coastal and Marine Management (RIKZ);
data compiled for R by Edzer Pebesma }
\seealso{\link{fulmar}}
\keyword{datasets}
\examples{
data(ncp.grid)
summary(ncp.grid)
}

% $Id: plot.pointPairs.Rd,v 1.5 2006-12-12 20:44:07 edzer Exp $
\name{plot.pointPairs}
\alias{plot.pointPairs}
\title{
Plot a point pairs, identified from a variogram cloud
}
\description{
Plot a point pairs, identified from a variogram cloud
}
\usage{
\method{plot}{pointPairs}(x, data, xcol = data$x, ycol = data$y, xlab = "x coordinate", 
ylab = "y coordinate", col.line = 2, line.pch = 0, main = "selected point pairs", ...) 
}
\arguments{
\item{x}{ object of class "pointPairs", obtained from the function 
\link{plot.variogramCloud}, containing point pair indices }
\item{data}{ data frame to which the indices refer (from which the
variogram cloud was calculated) }
\item{xcol}{ numeric vector with x-coordinates of data }
\item{ycol}{ numeric vector with y-coordinates of data }
\item{xlab}{ x-axis label }
\item{ylab}{ y-axis label }
\item{col.line}{ color for lines connecting points }
\item{line.pch}{ if non-zero, symbols are also plotted at the middle of
line segments, to mark lines too short to be visible on the plot;
the color used is \code{col.line}; the value passed to this argument
will be used as plotting symbol (pch) }
\item{main}{ title of plot }
\item{...}{ arguments, further passed to \code{xyplot}}
}
\value{
plots the data locations, with lines connecting the point pairs identified
(and refered to by indices in) x
}
\author{ Edzer Pebesma }
\seealso{
\link{plot.variogramCloud}
}
\examples{
### The following requires interaction, and is therefore outcommented
#data(meuse)
#coordinates(meuse) = ~x+y
#vgm1 <- variogram(log(zinc)~1, meuse, cloud = TRUE)
#pp <- plot(vgm1, id = TRUE)
### Identify the point pairs
#plot(pp, data = meuse) # meuse has x and y as coordinates
}
\keyword{dplot}

\name{variogramSurface}
\alias{variogramSurface}
\title{Semivariance values for a given spatio-temporal variogram model}
\description{
Generates a surface of semivariance values given a spatio-temporal variogram model (one of separable, productSum, sumMetric, simpleSumMetric or metric)
}
\usage{
variogramSurface(model, dist_grid, covariance = FALSE)
}

\arguments{
  \item{model}{
  A spatio-temporal variogram model generated through \code{\link{vgmST}} or \code{\link{fit.StVariogram}}.
}
  \item{dist_grid}{
  A data.frame with two columns: \code{spacelag} and \code{timelag}.
}
  \item{covariance}{
Whether the covariance should be computed instead of the variogram (default: FALSE).
}
}
\value{
A data.frame with columns \code{spacelag}, \code{timelag} and \code{gamma}.
}
\author{
Benedikt Graeler
}
\seealso{
See \code{\link{variogramLine}} for the spatial version and \code{\link{fit.StVariogram}} for the estimation of spatio-temporal variograms.
}
\examples{
separableModel <- vgmST("separable", 
                        space=vgm(0.86, "Exp", 476, 0.14),
                        time =vgm(   1, "Exp",   3, 0),
                        sill=113)

data(vv)

if(require(lattice)) {
plot(vv, separableModel, wireframe=TRUE, all=TRUE)
}

# plotting of sample and model variogram
plot(vv, separableModel)

}
\keyword{models}
\keyword{spatio-temporal}

% $Id: variogramLine.Rd,v 1.3 2008-08-19 07:27:02 edzer Exp $
\name{variogramLine}
\alias{variogramLine}
\alias{getGammas}
\title{ Semivariance Values For a Given Variogram Model }
\description{ Generates a semivariance values given a variogram model }
\usage{
variogramLine(object, maxdist, n = 200, min = 1.0e-6 * maxdist, 
	dir = c(1,0,0), covariance = FALSE, ..., dist_vector, debug.level = 0)
}
\arguments{
\item{object}{ variogram model for which we want semivariance function values }
\item{maxdist}{ maximum distance for which we want semivariance values }
\item{n}{ number of points }
\item{min}{ minimum distance; a value slightly larger than zero is usually
used to avoid the discontinuity at distance zero if a nugget component is
present }
\item{dir}{ direction vector: unit length vector pointing the direction in 
x (East-West), y (North-South) and z (Up-Down) }
\item{covariance}{logical; if TRUE return covariance values, otherwise
return semivariance values }
\item{...}{ignored}
\item{dist_vector}{numeric vector or matrix with distance values}
\item{debug.level}{gstat internal debug level}
}
\value{
a data frame of dimension (\code{n} x 2), with columns distance and gamma
(semivariances or covariances), or in case \code{dist_vector} is a matrix, a
conforming matrix with semivariance/covariance values is returned.
}
\note{variogramLine is used to generate data for plotting a variogram model.}
\author{ Edzer Pebesma }
\seealso{ \link{plot.gstatVariogram}}

\examples{
variogramLine(vgm(5, "Exp", 10, 5), 10, 10)
# anisotropic variogram, plotted in E-W direction:
variogramLine(vgm(1, "Sph", 10, anis=c(0,0.5)), 10, 10)
# anisotropic variogram, plotted in N-S direction:
variogramLine(vgm(1, "Sph", 10, anis=c(0,0.5)), 10, 10, dir=c(0,1,0))
variogramLine(vgm(1, "Sph", 10, anis=c(0,0.5)), dir=c(0,1,0), dist_vector = 0.5)
variogramLine(vgm(1, "Sph", 10, anis=c(0,0.5)), dir=c(0,1,0), dist_vector = c(0, 0.5, 0.75))
}

\keyword{models}

% $Id: fit.variogram.reml.Rd,v 1.4 2009-02-20 13:53:38 edzer Exp $
\name{fit.variogram.reml}
\alias{fit.variogram.reml}
\title{ REML Fit Direct Variogram Partial Sills to Data }
\description{
Fit Variogram Sills to Data, using REML (only for direct variograms;
not for cross variograms)
}
\usage{
fit.variogram.reml(formula, locations, data, model, debug.level = 1, set, degree = 0)
}
\arguments{
\item{formula}{formula defining the response vector and (possible) 
regressors; in case of absence of regressors, use e.g. \code{z~1}}
\item{locations}{ spatial data locations; a formula with the
coordinate variables in the right hand (dependent variable) side. }
\item{data}{data frame where the names in formula and locations 
are to be found}
\item{model}{variogram model to be fitted, output of \code{vgm}}
\item{debug.level}{debug level; set to 65 to see the iteration trace and
log likelihood}
\item{set}{additional options that can be set; use \code{set=list(iter=100)}
to set the max. number of iterations to 100.  }
\item{degree}{order of trend surface in the location, between 0 and 3}
}

\value{ an object of class "variogramModel"; see \link{fit.variogram} }

\references{ 
Christensen, R. Linear models for multivariate, Time Series,
and Spatial Data, Springer, NY, 1991. 

Kitanidis, P., Minimum-Variance Quadratic Estimation of Covariances of
Regionalized Variables, Mathematical Geology 17 (2), 195--208, 1985 }

\author{ Edzer Pebesma }
\note{
This implementation only uses REML fitting of sill parameters. For each
iteration, an \eqn{n \times n}{n x n} matrix is inverted, with $n$ the number of
observations, so for large data sets this method becomes 
demanding. I guess there is much more to likelihood variogram fitting in 
package \code{geoR}, and probably also in \code{nlme}.
}
\seealso{
\link{fit.variogram},
}
\examples{
library(sp)
data(meuse)
fit.variogram.reml(log(zinc)~1, ~x+y, meuse, model = vgm(1, "Sph", 900,1))
}
\keyword{models}

\name{krigeSimCE}
\alias{krigeSimCE}
\title{Simulation based on circulant embedding}
\description{Simulating a conditional/unconditional Gaussian random field via kriging and circulant embedding}
\usage{
krigeSimCE(formula, data, newdata, model, n = 1, ext = 2)
}

\arguments{
  \item{formula}{ the formula of the kriging predictor}
  \item{data}{ spatial data frame that conditions the simulation}
  \item{newdata}{locations in space where the Gaussian random field shall be simulated}
  \item{model}{a vgm model that defines the spatial covariance structure}
  \item{n}{number of simulations}
  \item{ext}{extension factor of the circulant embedding, defaults to 2}
}
\value{A spatial data frame as defined in \code{newdata} with \code{n} simulations.}

\references{
Davies, Tilman M., and David Bryant: "On circulant embedding for Gaussian random fields in R." 
Journal of Statistical Software 55.9 (2013): 1-21.
See i.e. the supplementary files at (retrieved 2018-05-25): 
https://www.jstatsoft.org/index.php/jss/article/downloadSuppFile/v055i09/v55i09.R
}

\author{Benedikt Graeler}

\seealso{\code{\link{krigeSTSimTB}}}

\examples{# see demo('circEmbeddingMeuse')}

\name{estiStAni}
\alias{estiStAni}

\title{
Estimation of the spatio-temporal anisotropy
}
\description{
Estimation of the spatio-temporal anisotropy without an underlying spatio-temporal model. Different methods are implemented using a linear model to predict the temporal gamma values or the ratio of the ranges of a spatial and temporal variogram model or a spatial variogram model to predict the temporal gamma values or the spatio-temporal anisotropy value as used in a metric spatio-temporal variogram. 
}
\usage{
estiStAni(empVgm, interval, method = "linear", spatialVgm, 
          temporalVgm, s.range=NA, t.range=NA)
}

\arguments{
  \item{empVgm}{
  An empirical spatio-temporal variogram.
}
  \item{interval}{
  A search interval for the optimisation of the spatio-temporal anisotropy parameter
}
  \item{method}{
  A character string determining the method to be used (one of \code{linear}, \code{range}, \code{vgm} or \code{metric}, see below for details)
}
  \item{spatialVgm}{
  A spatial variogram definition from the call to \code{\link{vgm}}. The model is optimised based on the pure spatial values in \code{empVgm}.
}
  \item{temporalVgm}{
  A temporal variogram definition from the call to \code{\link{vgm}}. The model is optimised based on the pure temporal values in \code{empVgm}.}
  \item{s.range}{
  A spatial cutoff value applied to the empirical variogram \code{empVgm}.  
  }
  \item{t.range}{
  A temporal cutoff value applied to the empirical variogram \code{empVgm}.  
  }
}
\details{
\describe{
  \item{linear}{
  A linear model is fitted to the pure spatial gamma values based on the spatial distances. An optimal scaling is searched to stretch the temporal distances such that the linear model explains best the pure temporal gamma values. This assumes (on average) a linear relationship between distance and gamma, hence it is advisable to use only those pairs of pure spatial (pure temporal) distance and gamma value that show a considerable increase (i.e. drop all values beyond the range by setting values for \code{s.range} and \code{t.range}).
  }
  \item{range}{
  A spatial and temporal variogram model is fitted to the pure spatial and temporal gamma values respectively. The spatio-temporal anisotropy estimate is the ratio of the spatial range over the temporal range.
  }
  \item{vgm}{
  A spatial variogram model is fitted to the pure spatial gamma values. An optimal scaling is used to stretch the temporal distances such that the spatial variogram model explains best the pure temporal gamma values.
  }
  \item{metric}{
  A metric spatio-temporal variogram model is fitted with \code{joint} component according to the defined spatial variogram \code{spatialVgm}. The starting value of \code{stAni} is the mean of the \code{interval} parameter (see \code{\link{vgmST}} for the metric variogram definition). The spatio-temporal anisotropy as estimated in the spatio-temporal variogram is returned. Note that the parameter \code{interval} is only used to set the starting value. Hence, the estimate might exceed the given interval.
  }
}
}
\value{A scalar representing the spatio-temporal anisotropy estimate.}

\note{Different methods might lead to very different estimates. All but the \code{linear} approach are sensitive to the variogram model selection.}

\author{Benedikt Graeler}

\examples{
data(vv)

estiStAni(vv, c(10, 150))
estiStAni(vv, c(10, 150), "vgm", vgm(80, "Sph", 120, 20))
}

% $Id: krige.Rd,v 1.25 2010-01-12 12:15:48 edzer Exp $
\name{krigeST}
\docType{methods}
\alias{krigeST}
\alias{krige,formula,ST-method} % calling krigeST
\alias{krigeSTTg}
% \alias{idw,formula,ST-method}

\title{ Ordinary global Spatio-Temporal Kriging}
\description{
Function for ordinary global and local and trans Gaussian spatio-temporal kriging on point support
}
\usage{
krigeST(formula, data, newdata, modelList, beta, y, ...,
        nmax = Inf, stAni = NULL,
        computeVar = FALSE,	fullCovariance = FALSE,
        bufferNmax=2, progress=TRUE)
krigeSTTg(formula, data, newdata, modelList, y, nmax=Inf, stAni=NULL,
                      bufferNmax=2, progress=TRUE, lambda = 0)
}
\arguments{
 \item{formula}{ formula that defines the dependent variable as a linear
  model of independent variables; suppose the dependent variable has name
  \code{z}, for ordinary and simple kriging use the formula \code{z~1};
  for simple kriging also define \code{beta} (see below); for universal
  kriging, suppose \code{z} is linearly dependent on \code{x} and \code{y},
  use the formula \code{z~x+y}}
 \item{data}{ ST object: should contain the dependent variable and independent
  variables.}
 \item{newdata}{ ST object with prediction/simulation locations in
 space and time; should contain attribute columns with the independent variables
 (if present).}
 \item{modelList}{ object of class \code{StVariogramModel}, created by
 \code{\link{vgmST}} - see below or the function \code{\link{vgmAreaST}} for area-to-point kriging. For the general kriging case: a list with named elements: \code{space}, \code{time}
 and/or \code{joint} depending 
  on the spatio-temporal covariance family, and an entry \code{stModel}. 
  Currently 
  implemented families that may be used for \code{stModel} 
  are \code{separable}, \code{productSum}, \code{metric}, 
  \code{sumMetric} and \code{simpleSumMetric}. 
  See the examples section in \code{\link{fit.StVariogram}} or 
  \code{\link{variogramSurface}} for details on how to define spatio-temporal 
  covariance models. \code{krigeST} will look for a "temporal unit" attribute
  in the provided modelList in order to adjust the temporal scales.}
 \item{y}{matrix; to krige multiple fields in a single step, pass data 
  as columns of matrix \code{y}. This will ignore the value of the 
  response in \code{formula}.}
 \item{beta}{The (known) mean for simple kriging.}
 \item{nmax}{The maximum number of neighbouring locations for a spatio-temporal local neighbourhood}
 \item{stAni}{a spatio-temporal anisotropy scaling assuming a metric spatio-temporal space. Used only for the selection of the closest neighbours. This scaling needs only to be provided in case the model does not have a stAni parameter, or if a different one should be used for the neighbourhood selection. Mind the correct spatial unit. Currently, no coordinate conversion is made for the neighbourhood selection (i.e. Lat and Lon require a spatio-temporal anisotropy scaling in degrees per second).}
 \item{\dots}{ further arguments used for instance to pass the model into vgmAreaST for area-to-point kriging}
 \item{computeVar}{logical; if TRUE, prediction variances will be returned}
 \item{fullCovariance}{logical; if FALSE a vector with prediction 
  variances will be returned, if TRUE the full covariance matrix of 
  all predictions will be returned}
  \item{bufferNmax}{factor with which nmax is multiplied for an extended search radius (default=2). Set to 1 for no extension of the search radius.}
  \item{progress}{whether a progress bar shall be printed for local spatio-temporal kriging; default=TRUE}
  \item{lambda}{The value of lambda used in the box-cox transformation.}
}
\details{
 Function \code{krigeST} is a R implementation of the kriging function from 
 \link{gstat} using spatio-temporal covariance models following the 
 implementation of \code{\link{krige0}}. Function \code{krigeST} offers some 
 particular methods for ordinary spatio-temporal (ST) kriging. In particular,
 it does not support block kriging or kriging in a distance-based
 neighbourhood, and does not provide simulation.

 If \code{data} is of class \code{sftime}, then \code{newdata} MUST be
 of class \code{stars} or \code{sftime}, i.e. mixing form old-style
 classes (package spacetime) and new-style classes (sf, stars, sftime)
 is not supported.
}

\value{
 An object of the same class as \code{newdata} (deriving from 
 \code{\link[spacetime:ST-class]{ST}}). Attributes columns contain prediction and prediction 
 variance.
}

\references{ 
Benedikt Graeler, Edzer Pebesma, Gerard Heuvelink.  Spatio-Temporal
Geostatistics using gstat.  The R Journal 8(1), 204--218.
\url{https://journal.r-project.org/articles/RJ-2016-014/index.html}

N.A.C. Cressie, 1993, Statistics for Spatial Data,
Wiley. 

Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.  }
\author{ Edzer Pebesma, Benedikt Graeler }

\seealso{ \code{\link{krige0}}, \code{\link{gstat}}, \code{\link[gstat]{predict}}, \code{\link{krigeTg}} }

\examples{
library(sp)
library(spacetime)
sumMetricVgm <- vgmST("sumMetric",
                      space = vgm( 4.4, "Lin", 196.6,  3),
                      time  = vgm( 2.2, "Lin",   1.1,  2),
                      joint = vgm(34.6, "Exp", 136.6, 12),
                      stAni = 51.7)

data(air)
suppressWarnings(proj4string(stations) <- CRS(proj4string(stations)))
rural = STFDF(stations, dates, data.frame(PM10 = as.vector(air)))

rr <- rural[,"2005-06-01/2005-06-03"]
rr <- as(rr,"STSDF")

x1 <- seq(from=6,to=15,by=1)
x2 <- seq(from=48,to=55,by=1)

DE_gridded <- SpatialPoints(cbind(rep(x1,length(x2)), rep(x2,each=length(x1))), 
                            proj4string=CRS(proj4string(rr@sp)))
gridded(DE_gridded) <- TRUE
DE_pred <- STF(sp=as(DE_gridded,"SpatialPoints"), time=rr@time)
DE_kriged <- krigeST(PM10~1, data=rr, newdata=DE_pred,
                     modelList=sumMetricVgm)
gridded(DE_kriged@sp) <- TRUE
stplot(DE_kriged)
}
\keyword{ models }

% $Id: gstat-internal.Rd,v 1.6 2008-03-10 10:36:04 edzer Exp $
\name{gstat-internal}
\alias{load.variogram.model}
\alias{gstat.formula}
\alias{gstat.formula.predict}
\alias{gstat.debug}
\alias{gstat.set}
\alias{cross.name}
\alias{gstat.load.set}

\title{Gstat Internal Functions}
\description{gstat internal functions}
\author{Edzer Pebesma}
\note{these functions are not meant to be called by users directly}
\keyword{internal}

% $Id: show.vgms.Rd,v 1.7 2008-12-15 14:27:29 edzer Exp $
\name{show.vgms}
\alias{show.vgms}
\title{
Plot Variogram Model Functions
}
\description{
Creates a trellis plot for a range of variogram models, possibly with nugget;
and optionally a set of Matern models with varying smoothness.
}
\usage{
show.vgms(min = 1e-12 * max, max = 3, n = 50, sill = 1, range = 1,
    models = as.character(vgm()$short[c(1:17)]), nugget = 0, kappa.range = 0.5,
	plot = TRUE, ..., as.groups = FALSE)
}
\arguments{
\item{min}{ numeric; start distance value for semivariance calculation 
beyond the first point at exactly zero  }
\item{max}{ numeric; maximum distance for semivariance calculation 
and plotting }
\item{n}{ integer; number of points to calculate distance values }
\item{sill}{ numeric; (partial) sill(s) of the variogram model }
\item{range}{ numeric; range(s) of the variogram model }
\item{models}{ character; variogram model(s) to be plotted }
\item{nugget}{ numeric; nugget component(s) for variogram models }
\item{kappa.range}{ numeric; if this is a vector with more than one
element, only a range of Matern models is plotted with these kappa 
values }
\item{plot}{ logical; if TRUE, a plot is returned with the models
specified; if FALSE, the data prepared for this plot is returned }
\item{...}{ passed on to the call to xyplot }
\item{as.groups}{ logical; if TRUE, different models are plotted with
different lines in a single panel, else, in one panel per model }
}
\value{
returns a (Trellis) plot of the variogram models requested; see
examples. I do currently have strong doubts about the ``correctness''
of the ``Hol'' model. The ``Spl'' model does seem to need a very
large range value (larger than the study area?) to be of some value.

If plot is FALSE, a data frame with the data prepared to plot
is being returned.
}
\author{ Edzer Pebesma }
\note{ the \code{min} argument is supplied because the variogram
function may be discontinuous at distance zero, surely when a positive
nugget is present. }
\seealso{
\link{vgm}, \link{variogramLine},
}
\examples{
show.vgms()
show.vgms(models = c("Exp", "Mat", "Gau"), nugget = 0.1)
# show a set of Matern models with different smoothness:
show.vgms(kappa.range = c(.1, .2, .5, 1, 2, 5, 10), max = 10)
# show a set of Exponential class models with different shape parameter:
show.vgms(kappa.range = c(.05, .1, .2, .5, 1, 1.5, 1.8, 1.9, 2), models = "Exc", max = 10)
# show a set of models with different shape parameter of M. Stein's representation of the Matern:
show.vgms(kappa.range = c(.01, .02, .05, .1, .2, .5, 1, 2, 5, 1000), models = "Ste", max = 2)

}

\keyword{dplot}

% $Id: variogram.Rd,v 1.23 2009-11-02 21:33:17 edzer Exp $
\name{variogram}
\alias{variogram}
\alias{variogram.gstat}
\alias{variogram.formula}
\alias{variogram.default}
\alias{print.gstatVariogram}
\alias{print.variogramCloud}
\alias{as.data.frame.variogramCloud}
\title{ Calculate Sample or Residual Variogram or Variogram Cloud }
\description{
Calculates the sample variogram from data, or in case of a linear model
is given, for the residuals, with options for directional, robust,
and pooled variogram, and for irregular distance intervals.

In case spatio-temporal data is provided, the function \code{\link{variogramST}}
is called with a different set of parameters.
}
\usage{
\method{variogram}{gstat}(object, ...)
\method{variogram}{formula}(object, locations = coordinates(data), data, ...)
\method{variogram}{default}(object, locations, X, cutoff, width = cutoff/15,
	alpha = 0, beta = 0, tol.hor = 90/length(alpha), tol.ver =
	90/length(beta), cressie = FALSE, dX = numeric(0), boundaries =
	numeric(0), cloud = FALSE, trend.beta = NULL, debug.level = 1,
	cross = TRUE, grid, map = FALSE, g = NULL, ..., projected = TRUE, 
	lambda = 1.0, verbose = FALSE, covariogram = FALSE, PR = FALSE, 
	pseudo = -1)
\method{print}{gstatVariogram}(x, ...)
\method{print}{variogramCloud}(x, ...)
}
\arguments{
\item{object}{object of class \code{gstat}; in this form, direct
and cross (residual) variograms are calculated for all variables and
variable pairs defined in \code{object}; in case of \code{variogram.formula},
formula defining the response vector and (possible) 
regressors, in case of absence of regressors, use e.g. \code{z~1};
in case of \code{variogram.default}: list with for each variable 
the vector with responses (should not be called directly)
}
\item{data}{data frame where the names in formula are to be found}
\item{locations}{ spatial data locations.  For variogram.formula: a
formula with only the coordinate variables in the right hand (explanatory
variable) side e.g. \code{~x+y}; see examples.

For variogram.default: list with coordinate matrices, each with the number
of rows matching that of corresponding vectors in y; the number of columns
should match the number of spatial dimensions spanned by the data (1
(x), 2 (x,y) or 3 (x,y,z)).  }
\item{...}{any other arguments that will be passed to
\link{variogram.default} (ignored)}
\item{X}{ (optional) list with for each variable the matrix with
regressors/covariates; the number of rows should match that of the
correspoding element in y, the number of columns equals the number of
regressors (including intercept) }
\item{cutoff}{ spatial separation distance up to which point pairs
are included in semivariance estimates; as a default, the
length of the diagonal of the box spanning the data is divided by three. }
\item{width}{ the width of subsequent distance intervals into which
data point pairs are grouped for semivariance estimates }
\item{alpha}{ direction in  plane (x,y), in positive degrees clockwise
from positive y (North): alpha=0 for direction North (increasing y),
alpha=90 for direction East (increasing x); optional a vector of
directions in (x,y) }
\item{beta}{ direction in z, in positive degrees up from the (x,y) plane; }
optional a vector of directions
\item{tol.hor}{ horizontal tolerance angle in degrees }
\item{tol.ver}{ vertical tolerance angle in degrees }
\item{cressie}{ logical; if TRUE, use Cressie''s robust variogram estimate;
if FALSE use the classical method of moments variogram estimate }
\item{dX}{ include a pair of data points $y(s_1),y(s_2)$ taken at
locations $s_1$ and $s_2$ for sample variogram calculation only when
$||x(s_1)-x(s_2)|| < dX$ with and $x(s_i)$ the vector with regressors at
location $s_i$, and $||.||$ the 2-norm.  This allows pooled estimation of
within-strata variograms (use a factor variable as regressor, and dX=0.5),
or variograms of (near-)replicates in a linear model (addressing point
pairs having similar values for regressors variables) }
\item{boundaries}{ numerical vector with distance interval upper boundaries; 
values should be strictly increasing }
\item{cloud}{ logical; if TRUE, calculate the semivariogram cloud }
\item{trend.beta}{vector with trend coefficients, in case they are
known. By default, trend coefficients are estimated from the data.}
\item{debug.level}{ integer; set gstat internal debug level }
\item{cross}{ logical or character; if FALSE, no cross variograms are computed
when object is of class \code{gstat} and has more than one variable; if
TRUE, all direct and cross variograms are computed; if
equal to "ST", direct and cross variograms are computed for all pairs 
involving the first (non-time lagged) variable; if equal to "ONLY",
only cross variograms are computed (no direct variograms). }
\item{formula}{formula, specifying the dependent variable and possible covariates}
\item{x}{ object of class \code{variogram} or \code{variogramCloud}
to be printed}
\item{grid}{ grid parameters, if data are gridded (not to be called
directly; this is filled automatically) }
\item{map}{ logical; if TRUE, and \code{cutoff} and \code{width}
are given, a variogram map is returned. This requires package
sp. Alternatively, a map can be passed, of class SpatialDataFrameGrid
(see sp docs) }
\item{g}{ NULL or object of class gstat; may be used to pass settable
parameters and/or variograms; see example }
\item{projected}{logical; if FALSE, data are assumed to be unprojected,
meaning decimal longitude/latitude. For projected data, Euclidian
distances are computed, for unprojected great circle distances
(km). In \code{variogram.formula} or \code{variogram.gstat}, for data
deriving from class Spatial, projection is detected automatically using
\code{is.projected}}
\item{lambda}{test feature; not working (yet)}
\item{verbose}{logical; print some progress indication}
\item{pseudo}{ integer; use pseudo cross variogram for computing
time-lagged spatial variograms? -1: find out from coordinates -- if they
are equal then yes, else no; 0: no; 1: yes. }
\item{covariogram}{logical; compute covariogram instead of variogram?}
\item{PR}{logical; compute pairwise relative variogram (does NOT check
whether variable is strictly positive)}
}

\value{ If map is TRUE (or a map is passed), a grid map is returned
containing the (cross) variogram map(s). See package sp.

In other cases, an object of class "gstatVariogram" with the 
following fields:
\item{np}{the number of point pairs for this estimate; 
in case of a \code{variogramCloud} see below}
\item{dist}{the average distance of all point pairs considered
for this estimate}
\item{gamma}{the actual sample variogram estimate}
\item{dir.hor}{the horizontal direction}
\item{dir.ver}{the vertical direction}
\item{id}{the combined id pair}

If cloud is TRUE: an object of class \code{variogramCloud}, with the field
\code{np} encoding the numbers of the point pair that contributed to a
variogram cloud estimate, as follows. The first point is found by 1 + the
integer division of np by the \code{.BigInt} attribute of the returned
object, the second point by 1 + the remainder of that division. 
\link{as.data.frame.variogramCloud} returns no \code{np} field,
but does the decoding into:
\item{left}{for variogramCloud: data id (row number) of one of 
the data pair}
\item{right}{for variogramCloud: data id (row number) of the other 
data in the pair}

In case of a spatio-temporal variogram is sought see \code{\link{variogramST}} 
for details.
}
\note{ \code{variogram.default} should not be called by users directly,
as it makes many assumptions about the organization of the data, that
are not fully documented (but of course, can be understood from reading
the source code of the other \code{variogram} methods) 

Successfully setting \code{gridded() <- TRUE} may trigger a branch that
will fail unless dx and dy are identical, and not merely similar
to within machine epsilon.  }

\references{ 
Cressie, N.A.C., 1993, Statistics for Spatial Data, Wiley.

Cressie, N., C. Wikle, 2011, Statistics for Spatio-temporal Data, Wiley.

Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.
}
\author{ Edzer Pebesma }
\seealso{
\link{print.gstatVariogram},
\link{plot.gstatVariogram},
\link{plot.variogramCloud};
for variogram models: \link{vgm},
to fit a variogram model to a sample variogram: 
\link{fit.variogram}
\code{\link{variogramST}} for details on the spatio-temporal sample variogram.
}
\examples{
library(sp)
data(meuse)
# no trend:
coordinates(meuse) = ~x+y
variogram(log(zinc)~1, meuse)
# residual variogram w.r.t. a linear trend:
variogram(log(zinc)~x+y, meuse)
# directional variogram:
variogram(log(zinc)~x+y, meuse, alpha=c(0,45,90,135))
variogram(log(zinc)~1, meuse, width=90, cutoff=1300)

# GLS residual variogram:
v = variogram(log(zinc)~x+y, meuse)
v.fit = fit.variogram(v, vgm(1, "Sph", 700, 1))
v.fit
set = list(gls=1)
v
g = gstat(NULL, "log-zinc", log(zinc)~x+y, meuse, model=v.fit, set = set)
variogram(g)

if (require(sf)) {
  proj4string(meuse) = CRS("+init=epsg:28992")
  meuse.ll = sf::st_transform(sf::st_as_sf(meuse), sf::st_crs("+proj=longlat +datum=WGS84"))
# variogram of unprojected data, using great-circle distances, returning km as units
  print(variogram(log(zinc) ~ 1, meuse.ll))
}

}
\keyword{models}

% $Id: meuse.alt.Rd,v 1.6 2006-02-10 19:03:27 edzer Exp $
\name{meuse.alt}
\alias{meuse.alt}
\title{Meuse river altitude data set}
\description{
This data set gives a point set with altitudes, digitized from the
1:10,000 topographical map of the Netherlands.
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{x}{a numeric vector; x-coordinate (m) in RDM (Dutch topographical map coordinates) }
    \item{y}{a numeric vector; y-coordinate (m) in RDM (Dutch topographical map coordinates)}
	\item{alt}{altitude in m. above NAP (Dutch zero for sea level)}
  }
}
\usage{
data(meuse.alt)
}
\seealso{\link{meuse.all}}
\keyword{datasets}
\examples{
data(meuse.alt)
library(lattice)
xyplot(y~x, meuse.alt, aspect = "iso")
}

% $Id: vgm.Rd,v 1.16 2008-10-09 14:25:20 edzer Exp $
\name{vgm}
\alias{vgm}
\alias{print.variogramModel}
\alias{plot.variogramModel}
\alias{as.vgm.variomodel}
\title{
Generate, or Add to Variogram Model
}
\description{
Generates a variogram model, or adds to an existing model. 
\code{print.variogramModel} prints the essence of a variogram  model.
}
\usage{
vgm(psill = NA, model, range = NA, nugget, add.to, anis, kappa = 0.5, ..., covtable,
	Err = 0)
\method{print}{variogramModel}(x, ...)
\method{plot}{variogramModel}(x, cutoff, ..., type = 'l')
as.vgm.variomodel(m)
}
\arguments{
\item{psill}{ (partial) sill of the variogram model component, or model: see Details }
\item{model}{ model type, e.g. "Exp", "Sph", "Gau", or "Mat". Can be a character vector of model 
types combined with c(), e.g. c("Exp", "Sph"), in which case the best fitting is returned. 
Calling vgm() without a model argument returns a data.frame with available models. }
\item{range}{ range parameter of the variogram model component; in case of anisotropy: major range }
\item{kappa}{ smoothness parameter for the Matern class of variogram 
models }
\item{nugget}{ nugget component of the variogram (this basically adds a nugget
compontent to the model); if missing, nugget component is omitted }
\item{add.to}{ the variogram model to which we want to add a component (structure) }
\item{anis}{ anisotropy parameters: see notes below }
\item{x}{ a variogram model to print or plot}
\item{...}{ arguments that will be passed to \code{print}, e.g. \code{digits}
(see examples), or to \code{variogramLine} for the plot method }
\item{covtable}{ if model is \code{Tab}, instead of model parameters a one-dimensional 
covariance table can be passed here. See covtable.R in tests directory, and example below. }
\item{Err}{ numeric; if larger than zero, the measurement error variance
component that will not be included to the kriging equations, i.e. kriging
will now smooth the process Y instead of predict the measured Z, 
where Z=Y+e, and Err is the variance of e}
\item{m}{ object of class \code{variomodel}, see \pkg{geoR}}
\item{cutoff}{maximum distance up to which variogram values are computed}
\item{type}{plot type}
}
\value{
If a single model is passed, an object of class \code{variogramModel}
extending \code{data.frame}.

In case a vector ofmodels is passed, an object of class
\code{variogramModelList} which is a list of \code{variogramModel}
objects.

When called without a model argument, a data.frame with available models
is returned, having two columns: short (abbreviated names, to be used
as model argument: "Exp", "Sph" etc) and long (with some description).

as.vgm.variomodel tries to convert an object of class variomodel (geoR)
to vgm.
}
\author{ Edzer Pebesma }
\details{ If only the first argument (\code{psill}) is given a
\code{character} value/vector indicating one or more models, as in \code{vgm("Sph")},
then this taken as a shorthand form of \code{vgm(NA,"Sph",NA,NA)},
i.e. a spherical variogram with nugget and unknown parameter values;
see examples below. Read \link{fit.variogram} to find out how
\code{NA} variogram parameters are given initial values for a fitting
a model, based on the sample variogram. Package \code{automap}
gives further options for automated variogram modelling. }

\note{ 
Geometric anisotropy can be modelled for each individual simple model
by giving two or five anisotropy parameters, two for two-dimensional
and five for three-dimensional data. In any case, the range defined
is the range in the direction of the strongest correlation, or the
major range. Anisotropy parameters define which direction this is (the
main axis), and how much shorter the range is in (the) direction(s)
perpendicular to this main axis.

In two dimensions, two parameters define an anisotropy ellipse, say
\code{anis = c(30, 0.5)}. The first parameter, \code{30}, refers to
the main axis direction: it is the angle for the principal direction
of continuity (measured in degrees, clockwise from positive Y, i.e. North).
The second parameter, \code{0.5}, is the anisotropy ratio, the ratio
of the minor range to the major range (a value between 0 and 1).  So,
in our example, if the range in the major direction (North-East) is 100,
the range in the minor direction (South-East) is 0.5 x 100 = 50.

In three dimensions, five values should be given in the form \code{anis
= c(p,q,r,s,t)}.  Now, $p$ is the angle for the principal direction of
continuity (measured in degrees, clockwise from Y, in direction of X),
$q$ is the dip angle for the principal direction of continuity (measured
in positive degrees up from horizontal), $r$ is the third rotation angle
to rotate the two minor directions around the principal direction defined
by $p$ and $q$. A positive angle acts counter-clockwise while looking
in the principal direction. Anisotropy ratios $s$ and $t$ are the ratios
between the major range and each of the two minor ranges. The anisotropy code
was taken from GSLIB. Note that in \url{http://www.gslib.com/sec_gb.html}
it is reported that this code has a bug. Quoting from this
site: ``The third angle in all GSLIB programs operates in the opposite
direction than specified in the GSLIB book.  Explanation - The books
says (pp27) the angle is measured clockwise when looking toward
the origin (from the postive principal direction), but it should be
counter-clockwise. This is a documentation error. Although rarely used,
the correct specification of the third angle is critical if used.''

(Note that \code{anis = c(p,s)} is equivalent to \code{anis = c(p,0,0,s,1)}.)

The implementation in gstat for 2D and 3D anisotropy was taken from
the gslib (probably 1992) code.  I have seen a paper where it is argued
that the 3D anisotropy code implemented in gslib (and so in gstat)
is in error, but I have not corrected anything afterwards.
}
\references{

Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.

Deutsch, C.V. and Journel, A.G., 1998.  GSLIB: Geostatistical software
library and user's guide, second edition, Oxford University Press.

For the validity of variogram models on the sphere, see Huang,
Chunfeng, Haimeng Zhang, and Scott M. Robeson. On the validity
of commonly used covariance and variogram functions on the
sphere. Mathematical Geosciences 43.6 (2011): 721-733.

}
\seealso{
\link{show.vgms} to view the available models,
\link{fit.variogram},
\link{variogramLine},
\link{variogram} for the sample variogram.
}
\examples{
vgm()
vgm("Sph")
vgm(NA, "Sph", NA, NA)
vgm(, "Sph") # "Sph" is second argument: NO nugget in this case
vgm(10, "Exp", 300)
x <- vgm(10, "Exp", 300)
vgm(10, "Nug", 0)
vgm(10, "Exp", 300, 4.5)
vgm(10, "Mat", 300, 4.5, kappa = 0.7)
vgm( 5, "Exp", 300, add.to = vgm(5, "Exp", 60, nugget = 2.5))
vgm(10, "Exp", 300, anis = c(30, 0.5))
vgm(10, "Exp", 300, anis = c(30, 10, 0, 0.5, 0.3))
# Matern variogram model:
vgm(1, "Mat", 1, kappa=.3)
x <- vgm(0.39527463, "Sph", 953.8942, nugget = 0.06105141)
x
print(x, digits = 3);
# to see all components, do
print.data.frame(x)
vv=vgm(model = "Tab",  covtable = 
	variogramLine(vgm(1, "Sph", 1), 1, n=1e4, min = 0, covariance = TRUE))
vgm(c("Mat", "Sph"))
vgm(, c("Mat", "Sph")) # no nugget
} 
\keyword{models}

% $Id: oxford.Rd,v 1.5 2006-02-10 19:03:27 edzer Exp $
\name{oxford}
\alias{oxford}
\title{Oxford soil samples}
\description{
Data: 126 soil augerings on a 100 x 100m square grid, with 6 columns
and 21 rows. Grid is oriented with long axis North-north-west to
South-south-east Origin of grid is South-south-east point, 100m outside
grid.

Original data are part of a soil survey carried out by P.A. Burrough in
1967. The survey area is located on the chalk downlands on the Berkshire
Downs in Oxfordshire, UK. Three soil profile units were recognised on
the shallow Rendzina soils; these are Ia - very shallow, grey calcareous
soils less than 40cm deep over chalk;  Ct - shallow to moderately deep,
grey-brown calcareous soils  on calcareous colluvium, and Cr: deep,
moderately acid, red-brown clayey soils.  These soil profile classes
were registered at every augering.

In addition, an independent landscape soil map was made by interpolating
soil boundaries between these soil types, using information from the
changes in landform.  Because the soil varies over short distances,
this field mapping caused some soil borings to receive a different
classification from the classification based on the point data.

Also registered at each auger point were the site elevation (m), the
depth to solid chalk rock (in cm) and the depth to lime in cm. Also,
the percent clay content, the Munsell colour components of VALUE and
CHROMA , and the lime content of the soil (as tested using HCl)  were
recorded for the top two soil layers (0-20cm and 20-40cm).

Samples of topsoil taken as a bulk sample within a circle of radius 2.5m
around each sample point were used for the laboratory determination of Mg
(ppm), OM1 \%, CEC as mequ/100g air dry soil, pH, P as ppm and K (ppm).

}
\format{
This data frame contains the following columns:
\describe{
\item{PROFILE}{profile number}
\item{XCOORD}{x-coordinate, field, non-projected}
\item{YCOORD}{y-coordinate, field, non-projected}
\item{ELEV}{elevation, m.}
\item{PROFCLASS}{soil class, obtained by classifying the soil profile at the
sample site }
\item{MAPCLASS}{soil class, obtained by looking up the site location in the soil map}
\item{VAL1}{Munsell colour component VALUE, 0-20 cm}
\item{CHR1}{Munsell colour component CHROMA, 20-40 cm}
\item{LIME1}{Lime content (tested using HCl), 0-20 cm}
\item{VAL2}{Munsell colour component VALUE, 0-20 cm}
\item{CHR2}{Munsell colour component CHROMA, 20-40 cm}
\item{LIME2}{Lime content (tested using HCl), 20-40 cm}
\item{DEPTHCM}{soil depth, cm }
\item{DEP2LIME}{depth to lime, cm }
\item{PCLAY1}{percentage clay, 0-20 cm}
\item{PCLAY2}{percentage clay, 20-40 cm}
\item{MG1}{Magnesium content (ppm), 0-20 cm}
\item{OM1}{organic matter (\%), 0-20 cm}
\item{CEC1}{CES as mequ/100g air dry soil, 0-20 cm}
\item{PH1}{pH, 0-20 cm}
\item{PHOS1}{Phosphorous, 0-20 cm, ppm}
\item{POT1}{K (potassium), 0-20 cm, ppm}
}
}
\usage{
data(oxford)
}
\author{ P.A. Burrough; compiled for R by Edzer Pebesma }
\references{ 
P.A. Burrough, R.A. McDonnell, 1998. Principles of Geographical Information
Systems. Oxford University Press.
}
\note{ \code{oxford.jpg}, in the gstat package external directory (see
example below), shows an image of the soil map for the region }
\keyword{datasets}
\examples{
data(oxford)
summary(oxford)
# open the following file with a jpg viewer:
system.file("external/oxford.jpg", package="gstat")
}

% $Id: image.Rd,v 1.10 2007-11-16 12:59:47 edzer Exp $
\name{image}
\alias{image.data.frame}
\alias{image}
\alias{xyz2img}
\title{
Image Gridded Coordinates in Data Frame
}
\description{
Image gridded data, held in a data frame, keeping the
right aspect ratio for axes, and the right cell shape
}
\usage{
\method{image}{data.frame}(x, zcol = 3, xcol = 1, ycol = 2, asp = 1, ...)
xyz2img(xyz, zcol = 3, xcol = 1, ycol = 2, tolerance = 10 * .Machine$double.eps)
}
\arguments{
\item{x}{ data frame (or matrix) with x-coordinate,
y-coordinate, and z-coordinate in its columns }
\item{zcol}{ column number or name of z-variable }
\item{xcol}{ column number or name of x-coordinate }
\item{ycol}{ column number or name of y-coordinate }
\item{asp}{ aspect ratio for the x and y axes }
\item{...}{ arguments, passed to image.default }
\item{xyz}{data frame (same as \code{x})}
\item{tolerance}{ maximum allowed deviation for coordinats from being
exactly on a regularly spaced grid }
}
\value{
\link{image.data.frame} plots an image from gridded data, organized
in arbritrary order, in a data frame. It uses \link{xyz2img} and
\link{image.default} for this. In the S-Plus version, \link{xyz2img}
tries to make an image object with a size such that it will plot with
an equal aspect ratio; for the R version, image.data.frame uses the
\code{asp=1} argument to guarantee this.

\link{xyz2img} returns a list with components: \code{z}, a matrix
containing the z-values; \code{x}, the increasing coordinates of the
rows of \code{z}; \code{y}, the increasing coordinates of the columns
of \code{z}. This list is suitable input to \link{image.default}.
}
\note{
I wrote this function before I found out about \code{levelplot},
a Lattice/Trellis function that lets you control the aspect ratio by
the \code{aspect} argument, and that automatically draws a legend, and
therefore I now prefer levelplot over \code{image}. Plotting points
on a levelplots is probably done with providing a panel function and
using \code{lpoints}.

(for S-Plus only -- ) it is hard (if not impossible) to get exactly right
cell shapes (e.g., square for a square grid) without altering the size of
the plotting region, but this function tries hard to do so by extending
the image to plot in either x- or y-direction.  The larger the grid, the
better the approximation. Geographically correct images can be obtained
by modifiying \code{par("pin")}. Read the examples, image a 2 x 2 grid,
and play with \code{par("pin")} if you want to learn more about this.
}
\author{ Edzer Pebesma }
\examples{
library(sp)
data(meuse)
data(meuse.grid)
g <- gstat(formula=log(zinc)~1,locations=~x+y,data=meuse,model=vgm(1,"Exp",300))
x <- predict(g, meuse.grid)
image(x, 4, main="kriging variance and data points")
points(meuse$x, meuse$y, pch = "+")
}

\keyword{dplot}

% $Id: hscat.Rd,v 1.3 2008-02-04 10:06:44 edzer Exp $
\name{progress}
\alias{get_gstat_progress}
\alias{set_gstat_progress}
\title{ Get or set progress indicator }
\description{ Get or set progress indicator }
\usage{
get_gstat_progress()
set_gstat_progress(value)
}
\arguments{
\item{value}{ logical }
}
\value{
return the logical value indicating whether progress bars should be given
}
\author{ Edzer Pebesma }

\examples{
set_gstat_progress(FALSE)
get_gstat_progress()
}
\keyword{models}

\name{vgmST}
\alias{vgmST}
\title{Constructing a spatio-temporal variogram}
\description{
Constructs a spatio-temporal variogram of a given type checking for a minimal set of parameters.
}
\usage{
vgmST(stModel, ..., space, time, joint, sill, k, nugget, stAni, temporalUnit)
}
\arguments{
  \item{stModel}{A string identifying the spatio-temporal variogram model (see details below). Only the string before an optional "_" is used to identify the model. This mechanism can be used to identify different fits of the same model (\code{separable_A} and \code{separable_B} will be interpreted as separable models, but carry different names).}
  \item{\dots}{unused, but ensure an exact match of the following parameters.}
  \item{space}{A spatial variogram.}
  \item{time}{A temporal variogram.}
  \item{joint}{A joint spatio-temporal variogram.}
  \item{sill}{A joint spatio-temporal sill.}
  \item{k}{The weighting of the product in the product-sum model.}
  \item{nugget}{A joint spatio-temporal nugget.}
  \item{stAni}{A spatio-temporal anisotropy; the number of space units equivalent to one time unit.}
  \item{temporalUnit}{length one character vector, indicating the temporal unit (like secs)}
}

\details{
The different implemented spatio-temporal variogram models have the following required parameters (see as well the example section)
\describe{
\item{separable:}{A variogram for \code{space} and \code{time} each and a joint spatio-temporal \code{sill} (variograms may have a separate nugget effect, but their joint sill will be 1) generating the call
  \preformatted{vgmST("separable", space, time, sill)}}
\item{productSum:}{A variogram for \code{space} and \code{time} each, and the weighting of product \code{k} generating the call
  \preformatted{vgmST("productSum", space, time, k)}}
\item{sumMetric:}{A variogram (potentially including a nugget effect) for \code{space}, \code{time} and \code{joint} each and a spatio-temporal anisotropy ratio \code{stAni} generating the call
  \preformatted{vgmST("sumMetric", space, time, joint, stAni)}}
\item{simpleSumMetric:}{A variogram (without nugget effect) for \code{space}, \code{time} and \code{joint} each, a joint spatio-temporal \code{nugget} effect and a spatio-temporal anisotropy ratio \code{stAni} generating the call
  \preformatted{vgmST("simpleSumMetric", space, time, joint, nugget, stAni)}}
\item{metric:}{A spatio-temporal \code{joint} variogram (potentially including a nugget effect) and \code{stAni} generating the call
  \preformatted{vgmST("metric", joint, stAni)}}}
}


\value{
Returns an S3 object  of class \code{StVariogramModel}.
}
\author{
Benedikt Graeler
}

\seealso{
\code{\link{fit.StVariogram}} for fitting, \code{\link{variogramSurface}} to plot the variogram and \code{\link{extractParNames}} to better understand the parameter structure of spatio-temporal variogram models.
}
\examples{
# separable model: spatial and temporal sill will be ignored
# and kept constant at 1-nugget respectively. A joint sill is used.
separableModel <- vgmST("separable", 
                        space=vgm(0.9,"Exp", 147, 0.1),
                        time =vgm(0.9,"Exp", 3.5, 0.1),
                        sill=40)

# product sum model: spatial and temporal nugget will be ignored and kept
# constant at 0. Only a joint nugget is used.
prodSumModel <- vgmST("productSum",
                      space=vgm(39, "Sph", 343, 0),
                      time= vgm(36, "Exp",   3, 0), 
                      k=15)

# sum metric model: spatial, temporal and joint nugget will be estimated
sumMetricModel <- vgmST("sumMetric",
                        space=vgm( 6.9, "Lin", 200, 3.0),
                        time =vgm(10.3, "Lin",  15, 3.6),
                        joint=vgm(37.2, "Exp",  84,11.7),
                        stAni=77.7)
                       
# simplified sumMetric model, only a overall nugget is fitted. The spatial, 
# temporal and jont nuggets are set to 0.
simpleSumMetricModel <- vgmST("simpleSumMetric",
                              space=vgm(20,"Lin", 150, 0),
                              time =vgm(20,"Lin", 10,  0),
                              joint=vgm(20,"Exp", 150, 0),
                              nugget=1, stAni=15)

# metric model
metricModel <- vgmST("metric",
                     joint=vgm(60, "Exp", 150, 10),
                     stAni=60)
}

\keyword{models}

% $Id: sic97.Rd,v 1.2 2008-10-30 13:47:05 edzer Exp $
\name{sic97}
\alias{sic_obs}
\alias{sic_full}
\alias{demstd}
\title{ Spatial Interpolation Comparison 1997 data set: Swiss Rainfall}
\description{
The text below is copied from the data item at ai-geostats,
(link no longer working).
}

\format{
  The data frames contain the following columns:
  \describe{
   \item{ID}{this integer value is the number (unique value) of
   the monitoring station}
   \item{rainfall}{ rainfall amount, in 10th of mm }
  }
}
\note{
See the pdf that accompanies the original file for a description of the data.
The .dxf file with the Swiss border is not included here.
}

\usage{
data(sic97) # 
}
\author{ 
Gregoire Dubois and others.
}

\keyword{datasets}
\examples{
data(sic97) 
image(demstd)
points(sic_full, pch=1)
points(sic_obs, pch=3)
}

% $Id: meuse.all.Rd,v 1.5 2006-02-10 19:03:27 edzer Exp $
\name{meuse.all}
\alias{meuse.all}
\title{Meuse river data set -- original, full data set}
\description{
This data set gives locations and top soil heavy metal concentrations
(ppm), along with a number of soil and landscape variables, collected in
a flood plain of the river Meuse, near the village Stein. Heavy metal
concentrations are bulk sampled from an area of approximately 15 m
x 15 m.
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{sample}{sample number}
    \item{x}{a numeric vector; x-coordinate (m)
	in RDM (Dutch topographical map coordinates) }
    \item{y}{a numeric vector; y-coordinate (m)
	in RDM (Dutch topographical map coordinates)}
	\item{cadmium}{topsoil cadmium concentration, ppm.; note that zero cadmium
	values in the original data set have been shifted to 0.2 (half the lowest 
	non-zero value) }
	\item{copper}{topsoil copper concentration, ppm. }
	\item{lead}{topsoil lead concentration, ppm. }
	\item{zinc}{topsoil zinc concentration, ppm. } 
	\item{elev}{relative elevation}
	\item{om}{organic matter, as percentage }
	\item{ffreq}{flooding frequency class}
	\item{soil}{soil type}
	\item{lime}{lime class}
	\item{landuse}{landuse class}
	\item{dist.m}{distance to river Meuse (metres), as obtained during
	the field survey}
	\item{in.pit}{logical; indicates whether this is a sample taken in
	a pit}
	\item{in.meuse155}{logical; indicates whether the sample is part of
	the \code{meuse} (i.e., filtered) data set; in addition to the samples
	in a pit, an sample (139) with outlying zinc content was removed }
	\item{in.BMcD}{logical; indicates whether the sample is used as part
	of the subset of 98 points in the various interpolation examples of
	Burrough and McDonnell}
  } 
} 
\usage{
data(meuse.all)
}
\author{ The actual field data were collected by Ruud van Rijn and
Mathieu Rikken; data compiled for R by Edzer Pebesma }
\references{ 
P.A. Burrough, R.A. McDonnell, 1998. Principles of Geographical Information
Systems. Oxford University Press.

}
\note{ \code{sample} refers to original sample number.  Eight samples
were left out because they were not indicative for the metal content of
the soil. They were taken in an old pit. One sample contains an outlying
zinc value, which was also discarded for the meuse (155) data set.
}
\seealso{\link{meuse.alt}}
\keyword{datasets}
\examples{
data(meuse.all)
summary(meuse.all)
}

% $Id: fit.variogram.gls,v 1.4 2009-02-20 13:53:38 edzer Exp $
\name{fit.variogram.gls}
\alias{fit.variogram.gls}
\title{ GLS fitting of variogram parameters }
\description{
Fits variogram parameters (nugget, sill, range) to variogram cloud,
using GLS (generalized least squares) fitting. Only for direct variograms.
}
\usage{
fit.variogram.gls(formula, data, model, maxiter = 30, 
		eps = .01, trace = TRUE, ignoreInitial = TRUE, cutoff = Inf,
		plot = FALSE)
}
\arguments{
\item{formula}{formula defining the response vector and (possible) 
regressors; in case of absence of regressors, use e.g. \code{z~1}}
\item{data}{object of class Spatial}
\item{model}{variogram model to be fitted, output of \code{vgm}}
\item{maxiter}{maximum number of iterations}
\item{eps}{ convergence criterium }
\item{trace}{ logical; if TRUE, prints parameter trace}
\item{ignoreInitial}{ logical; 
if FALSE, initial parameter are taken from model; 
if TRUE, initial values of model are
ignored and taken from variogram cloud: 
nugget: \code{mean(y)/2}, sill: \code{mean(y)/2}, range \code{median(h0)/4}
with \code{y} the semivariance cloud value and \code{h0} the distances }
\item{cutoff}{maximum distance up to which point pairs are taken into
consideration}
\item{plot}{logical; if TRUE, a plot is returned with variogram cloud and
fitted model; else, the fitted model is returned.}
}

\value{ an object of class "variogramModel"; see \link{fit.variogram}; if
\code{plot} is TRUE, a plot is returned instead. }

\references{ 
Mueller, W.G., 1999: Least-squares fitting from the variogram
cloud. Statistics and Probability Letters, 43, 93-98.

Mueller, W.G., 2007: Collecting Spatial Data. Springer, Heidelberg.
}

\author{ Edzer Pebesma }
\note{
Inspired by the code of Mihael Drinovac, which was again inspired by
code from Ernst Glatzer, author of package vardiag.
}
\seealso{
\link{fit.variogram},
}
\examples{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
\dontrun{
fit.variogram.gls(log(zinc)~1, meuse[1:40,], vgm(1, "Sph", 900,1))
}
}
\keyword{models}

% $Id: krige.Rd,v 1.25 2010-01-12 12:15:48 edzer Exp $
\name{krige}
\docType{methods}
\alias{krige}
\alias{krige0}
%\alias{krigeST}
\alias{krige.locations}
\alias{krige.spatial}
\alias{idw}
\alias{idw0}
\alias{idw.locations}
\alias{idw.spatial}
\alias{krige-methods}
\alias{idw-methods}
\alias{krige,formula,formula-method}
\alias{krige,formula,Spatial-method}
\alias{krige,formula,sf-method}
\alias{krige,formula,NULL-method}
\alias{idw,formula,formula-method}
\alias{idw,formula,Spatial-method}
\alias{idw,formula,sf-method}
\alias{idw,formula,ST-method} % move to krigeST as well?
% \alias{krige,formula,ST-method} % calling krigeST

\title{ Simple, Ordinary or Universal, global or local, Point or Block Kriging,
or simulation. }

\description{
Function for simple, ordinary or universal kriging (sometimes called
external drift kriging), kriging in a local neighbourhood, point kriging
or kriging of block mean values (rectangular or irregular blocks), and
conditional (Gaussian or indicator) simulation equivalents for all kriging
varieties, and function for inverse distance weighted interpolation. 
For multivariable prediction, see \link{gstat} and \link[gstat]{predict}
}
\usage{
krige(formula, locations, ...)
krige.locations(formula, locations, data, newdata, model, ..., beta, nmax
= Inf, nmin = 0, omax = 0, maxdist = Inf, block, nsim = 0, indicators = FALSE,
na.action = na.pass, debug.level = 1)
krige.spatial(formula, locations, newdata, model, ..., beta, nmax
= Inf, nmin = 0, omax = 0, maxdist = Inf, block, nsim = 0, indicators = FALSE,
na.action = na.pass, debug.level = 1)
krige0(formula, data, newdata, model, beta, y, ..., computeVar = FALSE,
	fullCovariance = FALSE)
idw(formula, locations, ...)
idw.locations(formula, locations, data, newdata, nmax = Inf, 
	nmin = 0, omax = 0, maxdist = Inf, block, na.action = na.pass, idp = 2.0,
	debug.level = 1)
idw.spatial(formula, locations, newdata, nmax = Inf, nmin = 0, 
    omax = 0, maxdist = Inf, block = numeric(0), na.action = na.pass, idp = 2.0,
	debug.level = 1)
idw0(formula, data, newdata, y, idp = 2.0)
}
\arguments{
 \item{formula}{ formula that defines the dependent variable as a linear
  model of independent variables; suppose the dependent variable has name
  \code{z}, for ordinary and simple kriging use the formula \code{z~1};
  for simple kriging also define \code{beta} (see below); for universal
  kriging, suppose \code{z} is linearly dependent on \code{x} and \code{y},
  use the formula \code{z~x+y}}
  \item{locations}{ object of class \code{Spatial} or \code{sf}, or (deprecated)
  formula defines the spatial data locations (coordinates) such as \code{~x+y}}
 \item{data}{ data frame: should contain the dependent variable, independent
  variables, and coordinates, should be missing if locations contains data. }
 \item{newdata}{ object of class \code{Spatial}, \code{sf} or \code{stars} with prediction/simulation 
  locations; should contain attributes with the independent variables (if present).  }
 \item{model}{ variogram model of dependent variable (or its residuals), 
  defined by a call to \link{vgm} or \link{fit.variogram}; for \code{krige0}
  also a user-supplied covariance function is allowed (see example below) }
 \item{beta}{ for simple kriging (and simulation based on simple
  kriging): vector with the trend coefficients (including intercept);
  if no independent variables are defined the model only contains an
  intercept and beta should be the simple kriging mean }
 \item{nmax}{ for local kriging: the number of nearest observations that
  should be used for a kriging prediction or simulation, where nearest
  is defined in terms of the space of the spatial locations. By default,
  all observations are used }
 \item{nmin}{ for local kriging: if the number of nearest observations
  within distance \code{maxdist} is less than \code{nmin}, a missing 
  value will be generated; see maxdist }
 \item{omax}{ see \link{gstat} }
 \item{maxdist}{ for local kriging: only observations within a distance
  of \code{maxdist} from the prediction location are used for prediction
  or simulation; if combined with \code{nmax}, both criteria apply }
 \item{block}{ block size; a vector with 1, 2 or 3 values containing
  the size of a rectangular in x-, y- and z-dimension respectively
  (0 if not set), or a data frame with 1, 2 or 3 columns, containing
  the points that discretize the block in the x-, y- and z-dimension
  to define irregular blocks relative to (0,0) or (0,0,0)---see also the details 
  section of \link[gstat]{predict}. By default, predictions or simulations 
  refer to the support of the data values. }
 \item{nsim}{ integer; if set to a non-zero value, conditional simulation
  is used instead of kriging interpolation. For this, sequential Gaussian
  or indicator simulation is used (depending on the value of 
  \code{indicators}), following a single random path through the data.  }
 \item{indicators}{ logical, only relevant if \code{nsim} is non-zero; if
  TRUE, use indicator simulation; else use Gaussian simulation }
 \item{na.action}{ function determining what should be done with missing
  values in 'newdata'.  The default is to predict 'NA'.  Missing values 
  in coordinates and predictors are both dealt with. }
 \item{debug.level}{debug level, passed to \link[gstat]{predict}; use -1 
 to see progress in percentage, and 0 to suppress all printed information }
 \item{\dots}{ for krige: arguments that will be passed to \link{gstat};
 for \code{krige0}: arguments that will be passe to \code{model}}
 \item{idp}{numeric; specify the inverse distance weighting power}
 \item{y}{matrix; to krige multiple fields in a single step, pass data 
 as columns of matrix \code{y}. This will ignore the value of the 
 response in \code{formula}.}
 \item{computeVar}{logical; if TRUE, prediction variances will be returned}
 \item{fullCovariance}{logical; if FALSE a vector with prediction 
 variances will be returned, if TRUE the full covariance matrix of 
 all predictions will be returned}
}
\section{Methods}{
\describe{
\item{formula = "formula", locations = "formula"}{ 
locations specifies which coordinates in \code{data} refer to spatial coordinates
}
\item{formula = "formula", locations = "Spatial"}{ 
Object locations knows about its own spatial locations
}
\item{formula = "formula", locations = "NULL"}{ used in case of unconditional simulations;
newdata needs to be of class Spatial }
}}
\details{
Function \code{krige} is a simple wrapper method around \link{gstat}
and \link[gstat]{predict} for univariate kriging prediction and conditional
simulation methods available in gstat. For multivariate prediction or
simulation, or for other interpolation methods provided by gstat (such as
inverse distance weighted interpolation or trend surface interpolation)
use the functions \link{gstat} and \link[gstat]{predict} directly.

Function \code{idw} performs just as \code{krige} without a model being
passed, but allows direct specification of the inverse distance weighting
power. Don't use with predictors in the formula.

For further details, see \link[gstat]{predict}.
}

\value{
if \code{locations} is not a formula, object of the same class as
\code{newdata} (deriving from \code{Spatial}); else a data frame
containing the coordinates of \code{newdata}. Attributes columns
contain prediction and prediction variance (in case of kriging) or the
\code{abs(nsim)} columns of the conditional Gaussian or indicator
simulations 

\code{krige0} and \code{idw0} are alternative functions with reduced
functionality and larger memory requirements; they return numeric vectors
(or matrices, in case of multiple dependent) with predicted values only;
in case \code{computeVar} is TRUE, a list with elements \code{pred} and
\code{var} is returned, containing predictions, and (co)variances (depending
on argument \code{fullCovariance}).
}

\references{ N.A.C. Cressie, 1993, Statistics for Spatial Data,
Wiley. 

Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.
}
\author{ Edzer Pebesma }
\note{  
Daniel G. Krige is a South African scientist who was a mining engineer
when he first used generalised least squares prediction with spatial
covariances in the 50's. George Matheron coined the term \code{kriging}
in the 60's for the action of doing this, although very similar approaches
had been taken in the field of meteorology. Beside being Krige's name,
I consider "krige" to be to "kriging" what "predict" is to "prediction".
}

\seealso{ \link{gstat}, \link[gstat]{predict} }

\examples{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
data(meuse.grid)
gridded(meuse.grid) = ~x+y
m <- vgm(.59, "Sph", 874, .04)
# ordinary kriging:
x <- krige(log(zinc)~1, meuse, meuse.grid, model = m)
spplot(x["var1.pred"], main = "ordinary kriging predictions")
spplot(x["var1.var"],  main = "ordinary kriging variance")
# simple kriging:
x <- krige(log(zinc)~1, meuse, meuse.grid, model = m, beta = 5.9)
# residual variogram:
m <- vgm(.4, "Sph", 954, .06)
# universal block kriging:
x <- krige(log(zinc)~x+y, meuse, meuse.grid, model = m, block = c(40,40))
spplot(x["var1.pred"], main = "universal kriging predictions")

# krige0, using user-defined covariance function and multiple responses in y:
# exponential variogram with range 500, defined as covariance function:
v = function(x, y = x) { exp(-spDists(coordinates(x),coordinates(y))/500) }
# krige two variables in a single pass (using 1 covariance model):
y = cbind(meuse$zinc,meuse$copper,meuse$lead,meuse$cadmium)
x <- krige0(zinc~1, meuse, meuse.grid, v, y = y)
meuse.grid$zinc = x[,1]
spplot(meuse.grid["zinc"], main = "zinc")
meuse.grid$copper = x[,2]
spplot(meuse.grid["copper"], main = "copper")

# the following has NOTHING to do with kriging, but --
# return the median of the nearest 11 observations:
x = krige(zinc~1, meuse, meuse.grid, set = list(method = "med"), nmax = 11)
# get 25%- and 75%-percentiles of nearest 11 obs, as prediction and variance:
x = krige(zinc~1, meuse, meuse.grid, nmax = 11, 
	set = list(method = "med", quantile = 0.25))
# get diversity (# of different values) and mode from 11 nearest observations:
x = krige(zinc~1, meuse, meuse.grid, nmax = 11, set = list(method = "div"))
}
\keyword{ models }

% $Id: fit.lmc.Rd,v 1.4 2007-11-16 12:59:35 edzer Exp $
\name{fit.lmc}
\alias{fit.lmc}
\title{ Fit a Linear Model of Coregionalization to a Multivariable Sample Variogram }
\description{
Fit a Linear Model of Coregionalization to a Multivariable Sample Variogram;
in case of a single variogram model (i.e., no nugget) this is equivalent to
Intrinsic Correlation
}
\usage{
fit.lmc(v, g, model, fit.ranges = FALSE, fit.lmc = !fit.ranges, 
correct.diagonal = 1.0, ...) 
}
\arguments{
\item{v}{ multivariable sample variogram, output of \link{variogram} }
\item{g}{ gstat object, output of \link{gstat} }
\item{model}{ variogram model, output of \link{vgm}; if supplied 
this value is used as initial value for each fit }
\item{fit.ranges}{ logical; determines whether the range coefficients
(excluding that of the nugget component) should be fitted; or logical
vector: determines for each range parameter of the variogram model
whether it should be fitted or fixed. }
\item{fit.lmc}{ logical; if TRUE, each coefficient matrices of partial
sills is guaranteed to be positive definite }
\item{correct.diagonal}{ multiplicative correction factor to be applied
to partial sills of direct variograms only; the default value, 1.0, does
not correct. If you encounter problems with singular covariance matrices
during cokriging or cosimulation, you may want to try to increase this
to e.g. 1.01 }
\item{...}{ parameters that get passed to \link{fit.variogram} }
}
\value{
returns an object of class \code{gstat}, with fitted variograms;
}
\author{ Edzer Pebesma }
\note{ This function does not use the iterative procedure proposed by
M. Goulard and M. Voltz (Math. Geol., 24(3): 269-286; reproduced in
Goovaerts' 1997 book) but uses simply two steps: first, each variogram
model is fitted to a direct or cross variogram; next each of the partial
sill coefficient matrices is approached by its in least squares sense
closest positive definite matrices (by setting any negative eigenvalues
to zero). 

The argument \code{correct.diagonal} was introduced by experience: by
zeroing the negative eigenvalues for fitting positive definite partial
sill matrices, apparently still perfect correlation may result, leading
to singular cokriging/cosimulation matrices. If someone knows of a more
elegant way to get around this, please let me know.
}
\seealso{ \link{variogram}, \link{vgm}, \link{fit.variogram},
\code{demo(cokriging)} }
\keyword{models}

% $Id: map.to.lev.Rd,v 1.4 2006-02-10 19:03:27 edzer Exp $
\name{map.to.lev}
\alias{map.to.lev}
\title{ rearrange data frame for plotting with levelplot }
\description{ rearrange data frame for plotting with levelplot }
\usage{
map.to.lev(data, xcol = 1, ycol = 2, zcol = c(3, 4), ns = names(data)[zcol])
}
\arguments{
\item{data}{ data frame, e.g. output from \link{krige} or \link[gstat]{predict} }
\item{xcol}{ x-coordinate column number }
\item{ycol}{ y-coordinate column number }
\item{zcol}{ z-coordinate column number range }
\item{ns}{names of the set of z-columns to be viewed}
}
\value{
data frame with the following elements:
\item{x}{ x-coordinate for each row}
\item{y}{ y-coordinate for each row} 
\item{z}{ column vector with each of the elements in columns \code{zcol}
of \code{data} stacked } 
\item{name}{ factor; name of each of the stacked \code{z} columns } 
}
\seealso{ 
\link{image.data.frame}, \link{krige};
for examples see \link[gstat]{predict};
\code{levelplot} in package lattice.
}
\keyword{dplot}

% $Id: get.contr.Rd,v 1.6 2009-02-20 13:53:38 edzer Exp $
\name{get.contr}
\alias{get.contr}
\title{ Calculate contrasts from multivariable predictions }
\description{ Given multivariable predictions and prediction (co)variances,
calculate contrasts and their (co)variance }
\usage{ get.contr(data, gstat.object, X, ids = names(gstat.object$data)) }
\arguments{
\item{data}{data frame, output of \link[gstat]{predict} }
\item{gstat.object}{object of class \code{gstat}, used to
extract ids; may be missing if \code{ids} is used }
\item{X}{ contrast vector or matrix; the number of variables in
\code{gstat.object} should equal the number of elements in \code{X}
if \code{X} is a vector, or the number of rows in \code{X} if \code{X}
is a matrix. }
\item{ids}{ character vector with (selection of) id names, present in data }
}

\details{ 
From data, we can extract the \eqn{n \times 1}{n x 1} vector with multivariable
predictions, say $y$, and its \eqn{n \times n}{n x n} covariance matrix $V$. Given
a contrast matrix in $X$, this function computes the contrast vector 
$C=X'y$ and its variance $Var(C)=X'V X$.
}

\value{
a data frame containing for each row in \code{data} the generalized
least squares estimates (named beta.1, beta.2, ...), their
variances (named var.beta.1, var.beta.2, ...) and covariances
(named cov.beta.1.2, cov.beta.1.3, ...)
}

\author{ Edzer Pebesma }
\seealso{\link[gstat]{predict}}


\keyword{ models }

% $Id: gstat.Rd,v 1.23 2009-11-02 21:33:17 edzer Exp $
\name{gstat}
\alias{gstat}
\alias{print.gstat}
\alias{[.gstat}
\title{ Create gstat objects, or subset it }
\description{
Function that creates gstat objects; objects that hold all the information
necessary for univariate or multivariate geostatistical prediction
(simple, ordinary or universal (co)kriging), or its conditional or
unconditional Gaussian or indicator simulation equivalents. Multivariate
gstat object can be subsetted. }
\usage{
gstat(g, id, formula, locations, data, model = NULL, beta,
	nmax = Inf, nmin = 0, omax = 0, maxdist = Inf, force = FALSE,
	dummy = FALSE, set, fill.all = FALSE,
	fill.cross = TRUE, variance = "identity", weights = NULL, merge, 
	degree = 0, vdist = FALSE, lambda = 1.0)
\method{print}{gstat}(x, ...)
}
\arguments{
  \item{g}{ gstat object to append to; if missing, a new gstat object
  is created }
  \item{id}{ identifier of new variable; if missing, \code{varn} is used with
    \code{n} the number for this variable. If a cross variogram is entered,
    \code{id} should be a vector with the two \code{id} values , e.g. 
    \code{c("zn", "cd")}, further only supplying arguments \code{g} 
    and \code{model}. It is advisable not to use expressions, such
	as \code{log(zinc)}, as identifiers, as this may lead to complications 
	later on. }
  \item{formula}{ formula that defines the dependent variable as a linear
  model of independent variables; suppose the dependent variable has name
  \code{z}, for ordinary and simple kriging use the formula \code{z~1};
  for simple kriging also define \code{beta} (see below); for universal
  kriging, suppose \code{z} is linearly dependent on \code{x} and \code{y},
  use the formula \code{z~x+y}}
  \item{locations}{ formula with only independent variables that define the
  spatial data locations (coordinates), e.g. \code{~x+y}; if \code{data}
  has a \code{coordinates} method to extract its coordinates this argument 
  can be ignored (see package sp for classes for point or grid data). }
  \item{data}{ data frame; contains the dependent variable, independent
  variables, and locations. }
  \item{model}{ variogram model for this \code{id}; defined by a call to 
  \link{vgm}; see argument \code{id} to see how cross variograms are entered }
  \item{beta}{ for simple kriging (and simulation based on simple
  kriging): vector with the trend coefficients (including intercept);
  if no independent variables are defined the model only contains an
  intercept and this should be the expected value; for cross 
  variogram computations: mean parameters to be used instead of the 
  OLS estimates }
  \item{nmax}{ for local kriging: the number of nearest observations that
  should be used for a kriging prediction or simulation, where nearest
  is defined in terms of the space of the spatial locations }
  \item{nmin}{ for local kriging: if the number of nearest observations
  within distance \code{maxdist} is less than \code{nmin}, a missing 
  value will be generated, unless \code{force==TRUE}; see \code{maxdist} }
  \item{omax}{ maximum number of observations to select per octant (3D) or
  quadrant (2D); only relevant if \code{maxdist} has been defined as well }
  \item{maxdist}{ for local kriging: only observations within a distance
  of \code{maxdist} from the prediction location are used for prediction
  or simulation; if combined with \code{nmax}, both criteria apply }
  \item{force}{ for local kriging, force neighbourhood selection: in case
  \code{nmin} is given, search beyond \code{maxdist} until \code{nmin}
  neighbours are found. A missing value is returned if this is not possible. }
  \item{dummy}{ logical; if TRUE, consider this data as a dummy variable
  (only necessary for unconditional simulation) }
  \item{set}{ named list with optional parameters to be passed to
  gstat (only \code{set} commands of gstat are allowed, and not all of
  them may be relevant; see the manual for gstat stand-alone, URL below ) }
  \item{x}{ gstat object to print }
  \item{fill.all}{ logical; if TRUE, fill all of the direct variogram and,
  depending on the value of \code{fill.cross} also all cross
  variogram model slots in \code{g} with the given variogram model }
  \item{fill.cross}{ logical; if TRUE, fill all of the cross variograms, if
  FALSE fill only all direct variogram model slots in \code{g} with the 
  given variogram model (only if \code{fill.all} is used)}
  \item{variance}{ character; variance function to transform to non-stationary
  covariances; "identity" does not transform, other options are "mu" (Poisson)
  and "mu(1-mu)" (binomial) }
  \item{weights}{ numeric vector; if present, covariates are present,
  and variograms are missing weights are passed to OLS prediction routines
  resulting in WLS; if variograms are given, weights should be 1/variance, 
  where variance specifies location-specific measurement error; see references 
  section below }
  \item{merge}{ either character vector of length 2, indicating two ids 
  that share a common mean; the more general gstat merging of any two
  coefficients across variables is obtained when a list is passed, with
  each element a character vector of length 4, in the form 
  \code{c("id1", 1,"id2", 2)}. This merges the first parameter 
  for variable \code{id1} to the second of variable \code{id2}.}
  \item{degree}{order of trend surface in the location, between 0 and 3}
  \item{vdist}{logical; if TRUE, instead of Euclidian distance 
  variogram distance is used for selecting the nmax nearest neighbours,
  after observations within distance maxdist (Euclidian/geographic) have been 
  pre-selected }
  \item{lambda}{test feature; doesn't do anything (yet)}
  \item{...}{ arguments that are passed to the printing of variogram
  models only}
}
\details{ to print the full contents of the object \code{g} returned,
use \code{as.list(g)} or \code{print.default(g)} }
\value{
an object of class \code{gstat}, which inherits from \code{list}.
Its components are: 
 \item{data}{list; each element is a list with the \code{formula}, 
  \code{locations}, \code{data}, \code{nvars}, \code{beta}, etc., for a 
  variable}
 \item{model}{list; each element contains a variogram model; names are
  those of the elements of \code{data}; cross variograms have names of
  the pairs of data elements, separated by a \code{.} (e.g.: 
  \code{var1.var2}})
 \item{set}{list; named list, corresponding to set \code{name}=\code{value};
 gstat commands (look up the set command in the gstat manual for a full list)}
}
\references{
Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.

for kriging with known, varying measurement errors (\code{weights}), see e.g.
Delhomme, J.P.  Kriging in the hydrosciences.  Advances in Water
Resources, 1(5):251-266, 1978; see also the section Kriging with known
measurement errors in the gstat user's manual.
}
\author{ Edzer Pebesma }
\note{ 
The function currently copies the data objects into the gstat object, so
this may become a large object. I would like to copy only the name of the
data frame, but could not get this to work. Any help is appreciated.  

Subsetting (see examples) is done using the \code{id}'s of the variables,
or using numeric subsets. Subsetted gstat objects only contain cross
variograms if (i) the original gstat object contained them and (ii) the
order of the subset indexes increases, numerically, or given the order
they have in the gstat object.

The merge item may seem obscure. Still, for colocated cokriging, it is
needed. See texts by Goovaerts, Wackernagel, Chiles and Delfiner, or
look for standardised ordinary kriging in the 1992 Deutsch and Journel
or Isaaks and Srivastava. In these cases, two variables share a common
mean parameter. Gstat generalises this case: any two variables may share
any of the regression coefficients; allowing for instance analysis of
covariance models, when variograms were left out (see e.g. R. Christensen's
``Plane answers'' book on linear models). The tests directory of the
package contains examples in file merge.R. There is also \code{demo(pcb)}
which merges slopes across years, but with year-dependent intercept.

}

\seealso{ \link[gstat]{predict}, \link{krige} }

\examples{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
# let's do some manual fitting of two direct variograms and a cross variogram
g <- gstat(id = "ln.zinc", formula = log(zinc)~1, data = meuse)
g <- gstat(g, id = "ln.lead", formula = log(lead)~1, data = meuse)
# examine variograms and cross variogram:
plot(variogram(g))
# enter direct variograms:
g <- gstat(g, id = "ln.zinc", model = vgm(.55, "Sph", 900, .05))
g <- gstat(g, id = "ln.lead", model = vgm(.55, "Sph", 900, .05))
# enter cross variogram:
g <- gstat(g, id = c("ln.zinc", "ln.lead"), model = vgm(.47, "Sph", 900, .03))
# examine fit:
plot(variogram(g), model = g$model, main = "models fitted by eye")
# see also demo(cokriging) for a more efficient approach
g["ln.zinc"]
g["ln.lead"]
g[c("ln.zinc", "ln.lead")]
g[1]
g[2]

# Inverse distance interpolation with inverse distance power set to .5:
# (kriging variants need a variogram model to be specified)
data(meuse.grid)
gridded(meuse.grid) = ~x+y
meuse.gstat <- gstat(id = "zinc", formula = zinc ~ 1, data = meuse, 
	nmax = 7, set = list(idp = .5))
meuse.gstat
z <- predict(meuse.gstat, meuse.grid)
spplot(z["zinc.pred"])
# see demo(cokriging) and demo(examples) for further examples, 
# and the manuals for predict and image

# local universal kriging
gmeuse <- gstat(id = "log_zinc", formula = log(zinc)~sqrt(dist), data = meuse)
# variogram of residuals
vmeuse.res <- fit.variogram(variogram(gmeuse), vgm(1, "Exp", 300, 1))
# prediction from local neighbourhoods within radius of 170 m or at least 10 points
gmeuse <- gstat(id = "log_zinc", formula = log(zinc)~sqrt(dist),
	data = meuse, maxdist=170, nmin=10, force=TRUE, model=vmeuse.res)
predmeuse <- predict(gmeuse, meuse.grid)
spplot(predmeuse)

}
\keyword{ models }

\name{vgmAreaST}
\alias{vgmAreaST}
\title{
Function that returns the covariances for areas
}
\description{
Function that returns the covariances for areas based on spatio-temporal point variograms for use in the spatio-temporal area-to-point kriging
}
\usage{
vgmAreaST(x, y = x, model, ndiscrSpace = 16, verbose = FALSE, covariance = TRUE)
}
\arguments{
  \item{x}{spatio-temporal data frame}
  \item{y}{spatio-temporal data frame}
  \item{model}{spatio-temporal variogram model for point support}
  \item{ndiscrSpace}{number of discretisation in space}
  \item{verbose}{Boolean: default to FALSE, set to TRUE for debugging}
  \item{covariance}{Boolean: whether the covariance shall be evaluated, currently disfunction and set to TRUE}
}
\value{The covariance between 'x'  and 'y'.}
\author{Benedikt Graeler}

\seealso{\code{\link{vgmArea}}}
\examples{
# see demo('a2pinST')
}

% $Id: fulmar.Rd,v 1.7 2008-07-03 11:49:08 edzer Exp $
\name{fulmar}
\alias{fulmar}
\title{Fulmaris glacialis data}
\description{ Airborne counts of Fulmaris glacialis during the
Aug/Sept 1998 and 1999 flights on the Dutch (Netherlands) part of
the North Sea (NCP, Nederlands Continentaal Plat). }
\format{
  This data frame contains the following columns:
  \describe{
    \item{year}{year of measurement: 1998 or 1999}
	\item{x}{x-coordinate in UTM zone 31} 
	\item{y}{y-coordinate in UTM zone 31}
	\item{depth}{sea water depth, in m}
	\item{coast}{distance to coast of the Netherlands, in km.}
	\item{fulmar}{observed density (number of birds per square km)}
  }
}
\usage{
data(fulmar)
}
\author{ Dutch National Institute for Coastal and Marine Management (RIKZ) }
\seealso{\link{ncp.grid}

E.J. Pebesma, R.N.M. Duin, P.A. Burrough, 2005. Mapping Sea Bird
Densities over the North Sea: Spatially Aggregated Estimates and Temporal
Changes. Environmetrics 16, (6), p 573-587.
}
\keyword{datasets}
\examples{
data(fulmar)
summary(fulmar)
\dontrun{
demo(fulmar)
}
}

\name{extractPar}
\alias{extractPar}
\alias{extractParNames}

\title{
Extracting parameters and their names from a spatio-temporal variogram model
}
\description{
All spatio-temporal variogram models have a different set of parameters. These functions extract the parameters and their names from the spatio-temporal variogram model. Note, this function is as well used to pass the parameters to the optim function. The arguments lower and upper passed to optim should follow the same structure. 
}
\usage{
extractPar(model)
extractParNames(model)
}

\arguments{
  \item{model}{a spatio-temporal variogram model from \code{\link{vgmST}}}
}

\value{
A named numeric vector of parameters or a vector of characters holding the parameters' names.
}

\author{
Benedikt Graeler
}


\seealso{
\code{\link{fit.StVariogram}} and \code{\link{vgmST}}
}

\examples{
sumMetricModel <- vgmST("sumMetric",
                        space=vgm(30, "Sph", 200,  6),
                        time =vgm(30, "Sph",  15,  7),
                        joint=vgm(60, "Exp",  84, 22),
                        stAni=100)

extractPar(sumMetricModel)
extractParNames(sumMetricModel)
}

% $Id: predict.gstat.Rd,v 1.18 2007-11-16 12:59:47 edzer Exp $
\name{predict}
\alias{predict}
\alias{predict.gstat}
\title{ Multivariable Geostatistical Prediction and Simulation }
\description{ The function provides the following prediction methods:
simple, ordinary, and universal kriging, simple, ordinary, and universal
cokriging, point- or block-kriging, and conditional simulation equivalents
for each of the kriging methods.  }
\usage{
\method{predict}{gstat}(object, newdata, block = numeric(0), nsim = 0, 
	indicators = FALSE, BLUE = FALSE, debug.level = 1, mask, 
	na.action = na.pass, sps.args = list(n = 500, type = "regular",
	offset = c(.5, .5)), ...)
}
\arguments{
\item{object}{ object of class \code{gstat}, see \link{gstat} and
  \link{krige}}
\item{newdata}{ data frame with prediction/simulation locations; should 
  contain columns with the independent variables (if present) and the
  coordinates with names as defined in \code{locations}; or: polygons, 
  see below}
\item{block}{ block size; a vector with 1, 2 or 3 values containing
  the size of a rectangular in x-, y- and z-dimension respectively
  (0 if not set), or a data frame with 1, 2 or 3 columns, containing
  the points that discretize the block in the x-, y- and z-dimension
  to define irregular blocks relative to (0,0) or (0,0,0)---see also
  the details section below. By default, predictions
  or simulations refer to the support of the data values. }
\item{nsim}{ integer; if set to a non-zero value, conditional simulation
  is used instead of kriging interpolation. For this, sequential Gaussian
  or indicator simulation is used (depending on the value of 
  \code{indicators}), following a single random path through the data.  }
\item{indicators}{ logical; only relevant if \code{nsim} is non-zero; if
 TRUE, use indicator simulation, else use Gaussian simulation }
\item{BLUE}{ logical; if TRUE return the BLUE trend estimates only, 
if FALSE return the BLUP predictions (kriging) }
\item{debug.level}{ integer; set gstat internal debug level, see below
for useful values. If set to -1 (or any negative value), a progress counter
is printed }
\item{mask}{ not supported anymore -- use na.action; 
logical or numerical vector; pattern with valid values
in newdata (marked as TRUE, non-zero, or non-NA); if mask is specified,
the returned data frame will have the same number and order of rows 
in newdata, and masked rows will be filled with NA's. }
\item{na.action}{ function determining what should be done with missing
values in 'newdata'.  The default is to predict 'NA'.  Missing values 
in coordinates and predictors are both dealt with. }
\item{sps.args}{ when newdata is of class \code{SpatialPolygons} 
or \code{SpatialPolygonsDataFrame} this
argument list gets passed to \link[sp]{spsample} to
control the discretizing of polygons }
\item{...}{ ignored (but necessary for the S3 generic/method consistency) }
}

\details{ When a non-stationary (i.e., non-constant) mean is used, both
for simulation and prediction purposes the variogram model defined should
be that of the residual process, not that of the raw observations.

For irregular block kriging, coordinates should discretize the area
relative to (0), (0,0) or (0,0,0); the coordinates in newdata should give
the centroids around which the block should be located. So, suppose the
block is discretized by points (3,3) (3,5) (5,5) and (5,3), we should
pass point (4,4) in newdata and pass points (-1,-1) (-1,1) (1,1) (1,-1)
to the block argument. Although passing the uncentered block and (0,0)
as newdata may work for global neighbourhoods, neighbourhood selection
is always done relative to the centroid values in newdata.

If newdata is of class \link[sp]{SpatialPolygons}  or
\link[sp]{SpatialPolygonsDataFrame}, then the block
average for each of the polygons or polygon sets is
calculated, using \link[sp]{spsample} to discretize the
polygon(s). Argument \code{sps.args} controls the parameters
used for \link[sp]{spsample}. The "location" with respect to
which neighbourhood selection is done is for each polygon the
SpatialPolygons polygon label point; if you use local neighbourhoods
you should check out where these points are---it may be well
outside the polygon itself.

The algorithm used by gstat for simulation random fields is the
sequential simulation algorithm. This algorithm scales well to large or
very large fields (e.g., more than $10^6$ nodes). Its power lies in using
only data and simulated values in a local neighbourhood to approximate the
conditional distribution at that location, see \code{nmax} in \link{krige}
and \link{gstat}. The larger \code{nmax}, the better the approximation,
the smaller \code{nmax}, the faster the simulation process. For selecting
the nearest \code{nmax} data or previously simulated points, gstat uses
a bucket PR quadtree neighbourhood search algorithm; see the reference
below.

For sequential Gaussian or indicator simulations, a random path through
the simulation locations is taken, which is usually done for sequential
simulations. The reason for this is that the local approximation of the
conditional distribution, using only the \code{nmax} neareast observed
(or simulated) values may cause spurious correlations when a regular
path would be followed. Following a single path through the locations,
gstat reuses the expensive results (neighbourhood selection and solution
to the kriging equations) for each of the subsequent simulations when
multiple realisations are requested.  You may expect a considerable speed
gain in simulating 1000 fields in a single call to \link[gstat]{predict},
compared to 1000 calls, each for simulating a single field.

The random number generator used for generating simulations is the native
random number generator of the environment (R, S); fixing randomness by
setting the random number seed with \code{set.seed()} works.

When mean coefficient are not supplied, they are generated as well
from their conditional distribution (assuming multivariate normal,
using the generalized least squares BLUE estimate and its estimation
covariance); for a reference to the algorithm used see Abrahamsen and
Benth, Math. Geol. 33(6), page 742 and leave out all constraints.

Memory requirements for sequential simulation: let n be the product of
the number of variables, the number of simulation locations, and the
number of simulations required in a single call.  the gstat C function
\code{gstat_predict} requires a table of size n * 12 bytes to pass the
simulations back to R, before it can free n * 4 bytes. Hopefully, R does
not have to duplicate the remaining n * 8 bytes when the coordinates
are added as columns, and when the resulting matrix is coerced to
a \code{data.frame}.

Useful values for \code{debug.level}: 0: suppres any output except
warning and error messages; 1: normal output (default): short data report,
program action and mode, program progress in \%, total execution time;
2: print the value of all global variables, all files read and written,
and include source file name and line number in error messages; 4: print
OLS and WLS fit diagnostics; 8: print all data after reading them; 16:
print the neighbourhood selection for each prediction location; 32:
print (generalised) covariance matrices, design matrices, solutions,
kriging weights, etc.; 64: print variogram fit diagnostics (number
of iterations and variogram model in each iteration step) and order
relation violations (indicator kriging values before and after order
relation correction); 512: print block (or area) discretization data
for each prediction location. To combine settings, sum their respective
values. Negative values for \code{debug.level} are equal to positive,
but cause the progress counter to work.

For data with longitude/latitude coordinates (checked by
\code{is.projected}), gstat uses great circle distances in km to compute
spatial distances. The user should make sure that the semivariogram
model used is positive definite on a sphere.

}

\value{
a data frame containing the coordinates of \code{newdata}, and columns
of prediction and prediction variance (in case of kriging) or the
columns of the conditional Gaussian or indicator simulations 
}

\references{ 
N.A.C. Cressie, 1993, Statistics for Spatial Data, Wiley. 

Pebesma, E.J., 2004. Multivariable geostatistics in S: the gstat package.
Computers and Geosciences, 30: 683-691.
}
\author{ Edzer Pebesma }

\seealso{\link{gstat}, \link{krige}}

\examples{
# generate 5 conditional simulations
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
v <- variogram(log(zinc)~1, meuse)
m <- fit.variogram(v, vgm(1, "Sph", 300, 1))
plot(v, model = m)
set.seed(131)
data(meuse.grid)
gridded(meuse.grid) = ~x+y
sim <- krige(formula = log(zinc)~1, meuse, meuse.grid, model = m, 
	nmax = 10, beta = 5.9, nsim = 5) # for speed -- 10 is too small!!
# show all 5 simulation
spplot(sim)

# calculate generalised least squares residuals w.r.t. constant trend:
g <- gstat(NULL, "log.zinc", log(zinc)~1, meuse, model = m)
blue0 <- predict(g, newdata = meuse, BLUE = TRUE)
blue0$blue.res <- log(meuse$zinc) - blue0$log.zinc.pred
bubble(blue0, zcol = "blue.res", main = "GLS residuals w.r.t. constant")

# calculate generalised least squares residuals w.r.t. linear trend:
m <- fit.variogram(variogram(log(zinc)~sqrt(dist.m), meuse), 
	vgm(1, "Sph", 300, 1))
g <- gstat(NULL, "log.zinc", log(zinc)~sqrt(dist.m), meuse, model = m)
blue1 <- predict(g, meuse, BLUE = TRUE)
blue1$blue.res <- log(meuse$zinc) - blue1$log.zinc.pred
bubble(blue1, zcol = "blue.res", 
	main = "GLS residuals w.r.t. linear trend")

# unconditional simulation on a 100 x 100 grid
xy <- expand.grid(1:100, 1:100)
names(xy) <- c("x","y")
gridded(xy) = ~x+y
g.dummy <- gstat(formula = z~1, dummy = TRUE, beta = 0,
	model = vgm(1,"Exp",15), nmax = 10) # for speed -- 10 is too small!!
yy <- predict(g.dummy, xy, nsim = 4)
# show one realisation:
spplot(yy[1])
# show all four:
spplot(yy)

}

\keyword{ models }

\name{vv}
\alias{vv}
\title{Precomputed variogram for PM10 in data set air}
\description{
Precomputed variogram for PM10 in data set air
}
\format{
data set structure is explained in \link{variogramST}.
}
\usage{
data(vv)
}
\examples{
\dontrun{
# obtained by:
library(spacetime)
library(gstat)
data(air)
suppressWarnings(proj4string(stations) <- CRS(proj4string(stations)))
rural = STFDF(stations, dates, data.frame(PM10 = as.vector(air)))
rr = rural[,"2005::2010"]
unsel = which(apply(as(rr, "xts"), 2, function(x) all(is.na(x))))
r5to10 = rr[-unsel,]
vv = variogram(PM10~1, r5to10, width=20, cutoff = 200, tlags=0:5)
}
}

\name{fit.StVariogram}
\alias{fit.StVariogram}
\title{Fit a spatio-temporal sample variogram to a sample variogram}
\description{
Fits a spatio-temporal variogram of a given type to spatio-temporal sample variogram.
}
\usage{
fit.StVariogram(object, model, ...,  method = "L-BFGS-B",
	lower, upper, fit.method = 6, stAni=NA, wles)
}
\arguments{
  \item{object}{The spatio-temporal sample variogram. Typically output from \code{\link{variogramST}}}
  \item{model}{The desired spatio-temporal model defined through \code{\link{vgmST}}.}
  \item{\dots}{further arguments passed to \code{\link{optim}}. \code{\link{extractParNames}} provides the parameter structure of spatio-temporal variogram models that help to provide sensible \code{upper} and \code{lower} limits.}
  \item{lower}{Lower limits used by optim. If missing, the smallest well defined values are used (mostly near 0).}
  \item{upper}{Upper limits used by optim. If missing, the largest well defined values are used (mostly \code{Inf}).}
  \item{method}{fit method, pass to \code{\link{optim}}}
  \item{fit.method}{an integer between 0 and 13 determine the fitting routine (i.e. weighting of the squared residuals in the LSE). Values 0 to 6 correspond with the pure spatial version (see \code{\link{fit.variogram}}). See the details section for the meaning of the other values (partly experimental).}
  \item{stAni}{The spatio-temporal anisotropy that is used in the weighting. Might be missing if the desired spatio-temporal variogram model does contain a spatio-temporal anisotropy parameter (this might cause bad convergence behaviour). The default is \code{NA} and will be understood as identity (1 temporal unit = 1 spatial unit). As this only in very few cases a valid assumption, a warning is issued.}
  \item{wles}{Should be missing; only for backwards compatibility, \code{wles = TRUE} corresponds to \code{fit.method = 1} and \code{wles = FALSE} corresponds to \code{fit.method = 6}.}
}

\details{
The following list summarizes the meaning of the \code{fit.method} argument which is essential a weighting of the squared residuals in the least-squares estimation. Please note, that weights based on the models gamma value might fail to converge properly due to the dependence of weights on the variogram estimate:
\describe{
  \item{\code{fit.method = 0}}{no fitting, however the MSE between the provided variogram model and sample variogram surface is calculated.}
  \item{\code{fit.method = 1}}{Number of pairs in the spatio-temporal bin: \eqn{N_j}{N[j]}}
  \item{\code{fit.method = 2}}{Number of pairs in the spatio-temporal bin divided by the square of the current variogram model's value: \eqn{N_j/\gamma(h_j, u_j)^2}{N[j]/gamma(h[j],u[j])^2}}
  \item{\code{fit.method = 3}}{Same as \code{fit.method = 1} for compatibility with \code{\link{fit.variogram}} but as well evaluated in R.}
  \item{\code{fit.method = 4}}{Same as \code{fit.method = 2} for compatibility with \code{\link{fit.variogram}} but as well evaluated in R.}
  \item{\code{fit.method = 5}}{Reserved for REML for compatibility with \code{\link{fit.variogram}}, not yet implemented.}
  \item{\code{fit.method = 6}}{No weights.}
  \item{\code{fit.method = 7}}{Number of pairs in the spatio-temporal bin divided by the square of the bin's metric distance. If \code{stAni} is not specified, the model's  parameter is used to calculate the metric distance across space and time: \eqn{N_j/(h_j^2 + {\rm stAni}^2\cdot u_j^2)}{N[j]/(h[j]^2+ stAni^2*u[j]^2)}}
  \item{\code{fit.method = 8}}{Number of pairs in the spatio-temporal bin divided by the square of the bin's spatial distance. \eqn{N_j/h_j^2}{N[j]/h[j]^2}. Note that the 0 distances are replaced by the smallest non-zero distances to avoid division by zero.}
  \item{\code{fit.method = 9}}{Number of pairs in the spatio-temporal bin divided by the square of the bin's temporal distance. \eqn{N_j/u_j^2}{N[j]/u[j]^2}. Note that the 0 distances are replaced by the smallest non-zero distances to avoid division by zero.}
  \item{\code{fit.method = 10}}{Reciprocal of the square of the current variogram model's value: \eqn{1/\gamma(h_j,u_j)^2}{1/gamma(h[j],u[j])^2}}
  \item{\code{fit.method = 11}}{Reciprocal of the square of the bin's metric distance. If \code{stAni} is not specified, the model's  parameter is used to calculate the metric distance across space and time: \eqn{1/(h_j^2 + {\rm stAni}^2\cdot u_j^2)}{1/(h[j]^2+ stAni^2*u[j]^2)}}
  \item{\code{fit.method = 12}}{Reciprocal of the square of the bin's spatial distance. \eqn{1/h_j^2}{1/h[j]^2}. Note that the 0 distances are replaced by the smallest non-zero distances to avoid division by zero.}
  \item{\code{fit.method = 13}}{Reciprocal of the square of the bin's temporal distance. \eqn{1/u_j^2}{1/u[j]^2}. Note that the 0 distances are replaced by the smallest non-zero distances to avoid division by zero.}
}
See also Table 4.2 in the gstat manual for the original spatial version.
}

\value{
Returns a spatio-temporal variogram model, as S3 class StVariogramModel. It carries the temporal and spatial unit as attributes \code{"temporal unit"} and \code{"spatial unit"} in order to allow \code{\link{krigeST}} to adjust for different units. The units are obtained from the provided empirical variogram. Further attributes are the optim output \code{"optim.output"} and the always not weighted mean squared error \code{"MSE"}.
}

\author{
Benedikt Graeler
}

\seealso{
\code{\link{fit.variogram}} for the pure spatial case. \code{\link{extractParNames}} helps to understand the parameter structure of spatio-temporal variogram models.
}
\examples{
# separable model: spatial and temporal sill will be ignored
# and kept constant at 1-nugget respectively. A joint sill is used.
\dontrun{
separableModel <- vgmST("separable", 
                        method = "Nelder-Mead", # no lower & upper needed
                        space=vgm(0.9,"Exp", 123, 0.1),
                        time =vgm(0.9,"Exp", 2.9, 0.1),
                        sill=100)

data(vv)
separableModel <- fit.StVariogram(vv, separableModel,
                                  method="L-BFGS-B",
                                  lower=c(10,0,0.01,0,1),
                                  upper=c(500,1,20,1,200))
plot(vv, separableModel)
} # dontrun
}

\keyword{models}

% $Id: plot.variogramCloud.Rd,v 1.6 2006-12-12 20:44:07 edzer Exp $
\name{plot.variogramCloud}
\alias{plot.variogramCloud}
\title{
Plot and Identify Data Pairs on Sample Variogram Cloud
}
\description{
Plot a sample variogram cloud, possibly with identification of
individual point pairs
}
\usage{
\method{plot}{variogramCloud}(x, identify = FALSE, digitize = FALSE, xlim, ylim, xlab, ylab, 
	keep = FALSE, ...)
}
\arguments{
\item{x}{ object of class \code{variogramCloud}}
\item{identify}{ logical; if TRUE, the plot allows identification of
a series of individual point pairs that correspond to individual variogram
cloud points (use left mouse button to select; right mouse button ends) }
\item{digitize}{ logical; if TRUE, select point pairs by digitizing a
region with the mouse (left mouse button adds a point, right mouse button
ends) }
\item{xlim}{ limits of x-axis }
\item{ylim}{ limits of y-axis }
\item{xlab}{ x axis label }
\item{ylab}{ y axis label }
\item{keep}{ logical; if TRUE and \code{identify} is TRUE, the labels
identified and their position are kept and glued to object x, which is
returned. Subsequent calls to plot this object will now have the labels
shown, e.g. to plot to hardcopy }
\item{...}{ parameters that are passed through to \link{plot.gstatVariogram}
(in case of identify = FALSE) or to plot (in case of identify = TRUE) }
}
\value{
If \code{identify} or \code{digitize} is TRUE, a data frame of class
\code{pointPairs} with in its rows the point pairs identified (pairs of
row numbers in the original data set); if identify is F, a plot of the
variogram cloud, which uses \link{plot.gstatVariogram}

If in addition to \code{identify}, \code{keep} is also TRUE, an object
of class \code{variogramCloud} is returned, having attached to it attributes
"sel" and "text", which will be used in subsequent calls to plot.variogramCloud
with \code{identify} set to FALSE, to plot the text previously identified.

If in addition to \code{digitize}, \code{keep} is also TRUE, an object of
class \code{variogramCloud} is returned, having attached to it attribute
"poly", which will be used in subsequent calls to plot.variogramCloud
with \code{digitize} set to FALSE, to plot the digitized line.

In both of the \code{keep = TRUE} cases, the attribute \code{ppairs} of
class \code{pointPairs} is present, containing the point pairs identified.
}
\author{ Edzer Pebesma }
\seealso{
\link{variogram},
\link{plot.gstatVariogram},
\link{plot.pointPairs},
\link{identify},
\link{locator}
}
\examples{
library(sp)
data(meuse)
coordinates(meuse) = ~x+y
plot(variogram(log(zinc)~1, meuse, cloud=TRUE))
## commands that require interaction:
# x <- variogram(log(zinc)~1, loc=~x+y, data=meuse, cloud=TRUE)
# plot(plot(x, identify = TRUE), meuse)
# plot(plot(x, digitize = TRUE), meuse)
}

\keyword{dplot}

\name{wind}
\alias{wind}
\alias{wind.loc}
\title{Ireland wind data, 1961-1978}
\description{
Daily average wind speeds for 1961-1978 at 12 synoptic meteorological 
stations in the Republic of Ireland (Haslett and raftery 1989).
Wind speeds are in knots (1 knot = 0.5418 m/s), at each of the 
stations in the order given in Fig.4 of Haslett and Raftery (1989, see below)
}
\format{
data.frame \code{wind} contains the following columns:
\describe{
\item{year}{year, minus 1900}
\item{month}{month (number) of the year}
\item{day }{day}
\item{RPT}{average wind speed in knots at station RPT}
\item{VAL}{average wind speed in knots at station VAL}
\item{ROS}{average wind speed in knots at station ROS}
\item{KIL}{average wind speed in knots at station KIL}
\item{SHA}{average wind speed in knots at station SHA}
\item{BIR}{average wind speed in knots at station BIR}
\item{DUB}{average wind speed in knots at station DUB}
\item{CLA}{average wind speed in knots at station CLA}
\item{MUL}{average wind speed in knots at station MUL}
\item{CLO}{average wind speed in knots at station CLO}
\item{BEL}{average wind speed in knots at station BEL}
\item{MAL}{average wind speed in knots at station MAL}
}
data.frame \code{wind.loc} contains the following columns:
\describe{
\item{Station}{Station name}
\item{Code }{Station code}
\item{Latitude}{Latitude, in DMS, see examples below}
\item{Longitude}{Longitude, in DMS, see examples below}
\item{MeanWind}{mean wind for each station, metres per second }
}
}
\usage{
data(wind)
}
\author{ Adrian Raftery; imported to R by Edzer Pebesma }
\references{ 
These data were analyzed in detail in the following article:

   Haslett, J. and Raftery, A. E. (1989). Space-time Modelling with
   Long-memory Dependence: Assessing Ireland's Wind Power Resource
   (with Discussion). Applied Statistics 38, 1-50.

and in many later papers on space-time analysis, for example:

   Tilmann Gneiting, Marc G. Genton, Peter Guttorp: Geostatistical
   Space-Time Models, Stationarity, Separability and Full symmetry.
   Ch. 4 in: B. Finkenstaedt, L. Held, V. Isham, Statistical Methods
   for Spatio-Temporal Systems.
}
\note{ This data set comes with the following message:
``Be aware that the dataset is 532494 bytes long (thats over half a
Megabyte).  Please be sure you want the data before you request it.''

The data were obtained on Oct 12, 2008, from:
http://www.stat.washington.edu/raftery/software.html
The data are also available from statlib.

Locations of 11 of the stations (ROS, Rosslare has
been thrown out because it fits poorly the spatial
correlations of the other stations) were obtained from:
http://www.stat.washington.edu/research/reports/2005/tr475.pdf

Roslare lat/lon was obtained from google maps, location Roslare. The mean
wind value for Roslare comes from Fig. 1 in the original paper.

Haslett and Raftery proposed to use a sqrt-transform to stabilize the variance.
}
\keyword{datasets}
\examples{
data(wind)
summary(wind)
wind.loc
library(sp) # char2dms
wind.loc$y = as.numeric(char2dms(as.character(wind.loc[["Latitude"]])))
wind.loc$x = as.numeric(char2dms(as.character(wind.loc[["Longitude"]])))
coordinates(wind.loc) = ~x+y

\dontrun{
# fig 1:
library(maps)
library(mapdata)
map("worldHires", xlim = c(-11,-5.4), ylim = c(51,55.5))
points(wind.loc, pch=16)
text(coordinates(wind.loc), pos=1, label=wind.loc$Station)
}

wind$time = ISOdate(wind$year+1900, wind$month, wind$day)
# time series of e.g. Dublin data:
plot(DUB~time, wind, type= 'l', ylab = "windspeed (knots)", main = "Dublin")

# fig 2:
#wind = wind[!(wind$month == 2 & wind$day == 29),]
wind$jday = as.numeric(format(wind$time, '\%j'))
windsqrt = sqrt(0.5148 * as.matrix(wind[4:15]))
Jday = 1:366
windsqrt = windsqrt - mean(windsqrt)
daymeans = sapply(split(windsqrt, wind$jday), mean)
plot(daymeans ~ Jday)
lines(lowess(daymeans ~ Jday, f = 0.1))

# subtract the trend:
meanwind = lowess(daymeans ~ Jday, f = 0.1)$y[wind$jday]
velocity = apply(windsqrt, 2, function(x) { x - meanwind })

# match order of columns in wind to Code in wind.loc:
pts = coordinates(wind.loc[match(names(wind[4:15]), wind.loc$Code),])

# fig 3, but not really yet...
dists = spDists(pts, longlat=TRUE)
corv = cor(velocity)
sel = !(as.vector(dists) == 0)
plot(as.vector(corv[sel]) ~ as.vector(dists[sel]),
	xlim = c(0,500), ylim = c(.4, 1), xlab = "distance (km.)", 
	ylab = "correlation") 
# plots all points twice, ignores zero distance 
# now really get fig 3:
ros = rownames(corv) == "ROS"
dists.nr = dists[!ros,!ros]
corv.nr = corv[!ros,!ros]
sel = !(as.vector(dists.nr) == 0)
plot(as.vector(corv.nr[sel]) ~ as.vector(dists.nr[sel]), pch = 3,
	xlim = c(0,500), ylim = c(.4, 1), xlab = "distance (km.)", 
	ylab = "correlation") 
# add outlier:
points(corv[ros,!ros] ~ dists[ros,!ros], pch=16, cex=.5)
xdiscr = 1:500
# add correlation model:
lines(xdiscr, .968 * exp(- .00134 * xdiscr))
}
